<?php
    class Backup_ClientController extends Application_Admin_ControllerAction
    {
        public function init()
        {
            parent::init();
            $this->breadcrumbs->add('System Maintenance', $this->getUrl('index', 'index', 'system'));
            $this->breadcrumbs->add('Backup Manager', $this->getUrl('index', 'index', 'backup'));
            $this->breadcrumbs->add('Client Backups', $this->getUrl('index'));
        }

        public function preDispatch()
        {
            $request = $this->getRequest();

            if ($request->getActionName() != 'no-support') {
                try {
                    Application_Loader::AssertCanLoadClass('Module_Backup_Dump_Manager');
                }
                catch (Exception $ex) {
                    $this->_forward('no-support');
                }
            }

            parent::preDispatch();
        }

        public function noSupportAction()
        {

        }

        public function indexAction()
        {
            $this->view->dumps   = Module_Backup_Dump_Manager::GetAvailableDumps();
            $this->view->clients = Application_Client::GetClients();
            $this->view->brands  = Application_Brand::GetBrands();
        }

        public function exportAction()
        {
            $request  = $this->getRequest();
            $clientId = (int) $request->getPost('client_id');

            $client = new Application_Client();

            if (!$client->load($clientId)) {
                $this->_redirect($this->getUrl('index'));
            }

            try {
                Module_Backup_Dump_Manager::Export($client);
                $this->messenger->addMessage(
                    sprintf("Client `%s' exported", $client)
                );
            }
            catch (Exception $ex) {
                $this->messengerError(
                    sprintf("Error exporting client `%s': %s", $client, $ex->getMessage())
                );
            }


            $this->_redirect($this->getUrl('index'));
        }

        public function importAction()
        {
            $request  = $this->getRequest();
            $dump     = $request->getPost('dump');
            $target   = $request->getPost('target');

            if (preg_match('/^brand_(\d+)$/', $target, $matches)) {
                $id = (int) $matches[1];
                $obj = new Application_Brand();
            }
            else {
                $id = (int) $target;
                $obj = new Application_Client();
            }

            $obj->load($id);

            try {
                $report = Module_Backup_Dump_Manager::Import($dump, $obj);
                $client = $report->getClient();

                $msg = 'Client backup imported';
                $url = $this->getUrl('view', 'index', 'clients') . '?id=' . $client->getId();

                $this->messenger->addMessage($msg);
            }
            catch (Exception $ex) {
                $msg = 'Unable to import client backup: ' . $ex->getMessage();
                $url = $this->getUrl('index');

                $this->messengerError($msg);
            }

            $this->_redirect($url);
        }

        public function downloadAction()
        {
            $request = $this->getRequest();
            $dump    = $request->getQuery('dump');

            try {
                $fullpath = Module_Backup_Dump_Manager::GetPath($dump);

                $this->noRender();

                $options = array(
                    'filename' => basename($fullpath),
                    'attach'   => true
                );

                HTTP::Sendfile($fullpath, $options);
            }
            catch (Exception $ex) {
                $this->_redirect($this->getUrl('index'));
            }
        }

        public function deleteAction()
        {
            $request = $this->getRequest();
            $dump    = $request->getPost('dump');

            try {
                Module_Backup_Dump_Manager::Delete($dump);
                $msg = 'Client backup deleted';
            }
            catch (Exception $ex) {
                $msg = 'Unable to delete client backup';
            }

            $this->messenger->addMessage($msg);
            $this->_redirect($this->getUrl('index'));
        }

        public function uploadAction()
        {
            $request = $this->getRequest();

            try {
                Module_Backup_Dump_Manager::Upload('file');
                $this->messenger->addMessage('Client backup uploaded');
            }
            catch (Exception $ex) {
                $this->messengerError($ex->getMessage());
            }

            $this->_redirect($this->getUrl('index'));
        }
    }
?>