<?php
    class Brands_IndexController extends Application_Admin_ControllerAction
    {
        public function init()
        {
            parent::init();
            $this->breadcrumbs->addStep('Brands', $this->getUrl('index'));
        }

        public function indexAction()
        {
            $this->view->brands = Application_Brand::GetBrands();
        }

        public function viewAction()
        {
            $request  = $this->getRequest();
            $brand_id = (int) $request->getQuery('id');

            $brand = new Application_Brand();

            if (!$brand->load($brand_id)) {
                $this->_redirect($this->getUrl('index'));
            }

            $this->breadcrumbs->add($brand);
            $this->view->brand      = $brand;
            $this->view->autoconfig = $brand->profile->httpd_conf_generate;
        }

        public function editAction()
        {
            $request  = $this->getRequest();
            $brand_id = (int) $request->getQuery('id');

            $fp = new Application_Brand_Editor_Admin($brand_id);

            if ($request->getPost('cancel')) {
                if ($fp->obj->saved()) {
                    $this->_redirect($this->getUrl('view') . '?id=' . $fp->obj->getId());
                }
                else {
                    $this->_redirect($this->getUrl());
                }
            }

            if ($request->getPost('submit') && $fp->process($request)) {
                $this->messenger->addMessage(sprintf('Brand %s saved', $fp->obj));
                $this->_redirect($this->getUrl('view') . '?id=' . $fp->obj->getId());
            }

            if ($fp->obj->saved()) {
                $title = 'Edit Brand';
            }
            else {
                $title = 'Create Brand';
            }

            $steps = array();
            if ($fp->obj->saved()) {
                $this->breadcrumbs->add($fp->obj->get('brand_name'), $this->getUrl('view') . '?id=' . $fp->obj->getId());
            }

            $this->breadcrumbs->add($title);
            $this->view->fp = $fp;
        }

        public function deleteAction()
        {
            $request  = $this->getRequest();
            $clientId = (int) $request->getQuery('id');

            $brand = new Application_Brand();
            $brand->load($clientId);

            if (!$brand->saved()) {
                $this->_redirect($this->getUrl('index'));
            }

            if ($request->getPost('cancel')) {
                $this->_redirect($this->getUrl('view') . '?id=' . $brand->getId());
            }
            else if ($request->getPost('confirm')) {
                $brand->delete();

                $this->messenger->addMessage('Brand deleted');
                $this->_redirect($this->getUrl('index'));
            }

            $this->breadcrumbs->add($brand, $this->getUrl('view') . '?id=' . $brand->getId());
            $this->breadcrumbs->add('Delete Brand');

            $this->view->brand = $brand;
        }

        public function domainsAction()
        {
            $request = $this->getRequest();

            $brand_id   = (int) $request->getQuery('brand_id');
            $domain_id  = (int) $request->getQuery('id');
            $action     = $request->getQuery('action');
            $url        = $request->getQuery('url');
            $autoconfig = (bool) $request->getQuery('autoconfig');

            $brand = new Application_Brand();
            $brand->load($brand_id);

            if (!$brand->saved()) {
                throw new Exception('Invalid brand');
            }

            if ($action == 'add') {
                $brand->domainAdd($url);
                $brand->profile->httpd_conf_generate = $autoconfig;
                $brand->save();

                Application_HttpdConfig::Rebuild($brand);

                $this->messenger->addMessage('Domain added');
                $this->_redirect($this->getUrl('view') . '?id=' . $brand->getId());
            }
            else if ($action == 'delete') {
                $domain = new Application_Brand_Domain();
                if ($domain->load($domain_id)) {
                    $domain->delete();

                    Application_HttpdConfig::Rebuild($brand);

                    $this->messenger->addMessage('Domain deleted');
                    $this->_redirect($this->getUrl('view') . '?id=' . $brand->getId());
                }
            }

            $this->_redirect($this->getUrl());
        }

        public function settingsAction()
        {
            $request  = $this->getRequest();
            $brand_id = (int) $request->getQuery('id');
            $bundle   = $request->getQuery('bundle');

            $brand = new Application_Brand();
            if (!$brand->load($brand_id)) {
                $this->_redirect($this->getUrl());
            }

            $url = $this->getUrl('view') . '?id=' . $brand->getId();

            $fp = new Application_Brand_Settings_Editor_Admin($brand, $bundle);

            if ($request->getPost('cancel')) {
                $this->_redirect($url);
            }

            if ($request->getPost('submit') && $fp->process($request)) {
                $this->messenger->addMessage('Brand settings saved');
                $this->_redirect($this->getUrl('settings') . '?id=' . $brand->getId() . '&bundle=' . $bundle);
            }
            else if ($request->getPost('reset')) {
                $fp->reset();
                $this->messenger->addMessage('Brand settings reset');
                $this->_redirect($this->getUrl('settings') . '?id=' . $brand->getId() . '&bundle=' . $bundle);
            }

            $this->breadcrumbs->add($brand->get('brand_name'), $url);
            $this->breadcrumbs->add('Edit Brand Settings');

            $this->view->brand = $brand;
            $this->view->fp = $fp;
        }

        public function domainSortAction()
        {
            $request = $this->getRequest();
            $brandId = (int) $request->getQuery('brand');

            $brand = new Application_Brand();
            $brand->load($brandId);

            if (!$brand->saved()) {
                throw new Exception('Invalid brand');
            }

            Application_Brand_Domain::ProcessSortForm($brand, $request);
            $this->noRender();
        }
    }
?>