<?php
    class Brands_WidgetsController extends Application_Admin_ControllerAction
    {
        protected $_brand;

        public function init()
        {
            parent::init();
            $this->breadcrumbs->addStep('Brands', $this->getUrl('index', 'index'));
        }

        public function preDispatch()
        {
            parent::preDispatch();

            $request = $this->getRequest();
            $brand_id = $request->getQuery('id');

            $this->_brand = new Application_Brand();
            $this->_brand->load($brand_id);

            if (!$this->_brand->saved()) {
                $this->_redirect($this->getUrl('index', 'index'));
            }

            $this->breadcrumbs->addStep($this->_brand, $this->getUrl('view', 'index') . '?id=' . $this->_brand->getId());
            $this->breadcrumbs->addStep('Default Widget Layout', $this->getUrl('index') . '?id=' . $this->_brand->getId());
            $this->view->brand = $this->_brand;
        }

        public function indexAction()
        {
            $options = array(
                'addableOnly' => true,
                'assert'      => false
            );

            $widgets = array();

            foreach (Application_Cp_Widget_Manager::GetDrivers($options) as $driver) {
                $categories = ArrayTools::Clean($driver->getCategories());

                foreach ($categories as $category) {
                    if (!array_key_exists($category, $widgets)) {
                        $widgets[$category] = array();
                    }
                    $widgets[$category][] = $driver;
                }
            }

            $defaults = Application_Cp_Tab_Manager::GetDefaults($this->_brand);

            $this->view->defaults      = $defaults;
            $this->view->widgets       = $widgets;
            $this->view->layouts       = Application_Cp_Tab_Manager::GetCustomLayouts();
            $this->view->defaultLayout = Application_Cp_Tab_Manager::GetDefaultLayout();
            $this->view->templates     = Application_Cp_Tab_Manager::GetTemplates();
        }

        public function processAction()
        {
            $request = $this->getRequest();

            if ($request->isPost()) {
                try {
                    Application_Cp_Tab_Manager::ProcessDefaultTabs($this->_brand, $request);
                    $this->messenger->addMessage('Brand widgets set');
                }
                catch (Exception $ex)
                {
                    $this->messengerError($ex->getMessage());
                }
            }

            $this->_redirect($this->getUrl('index') . '?id=' . $this->_brand->getId());
        }
    }
?>