<?php
    class Clients_DomainsController extends Application_Admin_ControllerAction
    {
        public function init()
        {
            parent::init();
            $this->breadcrumbs->addStep('Clients', $this->getUrl('index', 'index'));
        }

        public function addAction()
        {
            $request    = $this->getRequest();
            $clientId   = (int) $request->getQuery('client');
            $url        = $request->getQuery('url');
            $autoconfig = (bool) $request->getQuery('autoconfig');

            $client = new Application_Client();
            $client->load($clientId);

            if (!$client->saved()) {
                throw new Exception('Invalid client');
            }

            try {
                $client->domainAdd($url);
                $client->profile->httpd_conf_generate = $autoconfig;
                $client->save();

                Application_HttpdConfig::Rebuild($client);

                $msg = sprintf('Domain %s added to client', $url);

                $this->messenger->addMessage($msg);
            }
            catch (Exception $ex) {
                $this->messengerError($ex->getMessage());
            }

            $this->_redirect($this->getUrl('view', 'index') . '?id=' . $clientId);
        }

        public function davAddAction()
        {
            $request  = $this->getRequest();
            $clientId = (int) $request->getQuery('client');
            $url      = $request->getQuery('url');

            $client = new Application_Client();
            $client->load($clientId);

            if (!$client->saved()) {
                throw new Exception('Invalid client');
            }

            try {
                $client->domainAddDav($url);

                Application_HttpdConfig::Rebuild($client);

                $msg = sprintf('DAV Domain %s added to client', $url);

                $this->messenger->addMessage($msg);
            }
            catch (Exception $ex) {
                $this->messengerError($ex->getMessage());
            }

            $this->_redirect($this->getUrl('view', 'index') . '?id=' . $clientId);
        }

        public function deleteAction()
        {
            $request  = $this->getRequest();
            $domainId = (int) $request->getQuery('id');

            $domain = new Application_Client_Domain();
            $domain->load($domainId);

            if (!$domain->saved()) {
                throw new Exception('Invalid domain');
            }

            if ($domain->isDav()) {
                throw new Exception('Domain is for DAV');
            }

            $client = new Application_Client();
            $client->load($domain->client_id);

            $url = $domain->url;
            $domain->delete();

            if ($client->saved()) {
                Application_HttpdConfig::Rebuild($client);
            }

            $this->messenger->addMessage(sprintf('Domain %s deleted', $url));
            $this->_redirect($this->getUrl('view', 'index') . '?id=' . $client->getId());
        }

        public function davDeleteAction()
        {
            $request  = $this->getRequest();
            $domainId = (int) $request->getQuery('id');

            $domain = new Application_Client_Domain();
            $domain->load($domainId);

            if (!$domain->saved()) {
                throw new Exception('Invalid domain');
            }

            if (!$domain->isDav()) {
                throw new Exception('Domain is not for DAV');
            }

            $client = new Application_Client();
            $client->load($domain->client_id);

            $url = $domain->url;
            $domain->delete();

            if ($client->saved()) {
                Application_HttpdConfig::Rebuild($client);
            }

            $this->messenger->addMessage(sprintf('DAV Domain %s deleted', $url));
            $this->_redirect($this->getUrl('view', 'index') . '?id=' . $client->getId());
        }

        public function sortAction()
        {
            $request  = $this->getRequest();
            $clientId = (int) $request->getQuery('client');

            $client = new Application_Client();
            $client->load($clientId);

            if (!$client->saved()) {
                throw new Exception('Invalid client');
            }

            Application_Client_Domain::ProcessSortForm($client, $request);
            $this->noRender();
        }
    }
?>