<?php
    class Clients_IndexController extends Application_Admin_ControllerAction
    {
        public function init()
        {
            parent::init();
            $this->breadcrumbs->addStep('Clients', $this->getUrl('index'));
        }

        public function indexAction()
        {
            $dr = new Application_Client_Renderer();
            $dr->run($this->getRequest());

            $this->view->dr = $dr;
        }

        public function viewAction()
        {
            $request   = $this->getRequest();
            $client_id = (int) $request->getQuery('id');

            $client = new Application_Client();

            if (!$client->load($client_id)) {
                $this->_redirect($this->getUrl());
            }

            $this->breadcrumbs->addStep($client);

            $this->view->client     = $client;
            $this->view->autoconfig = $client->profile->httpd_conf_generate;
            $this->view->licenses   = Application_License_Manager::GetClientLicenseSummary($client);
            $this->view->staticFilePath = Application_Client_Data::GetNamespacedPathForClient($client->getId(), 'assets/static');
        }

        public function editAction()
        {
            $request   = $this->getRequest();
            $client_id = (int) $request->getQuery('id');
            $brand_id  = (int) $request->getQuery('brand_id');

            $fp = new Application_Client_Editor_Admin($client_id, $brand_id);

            if ($request->getPost('cancel')) {
                if ($fp->obj->saved()) {
                    $this->_redirect($this->getUrl('view') . '?id=' . $fp->obj->getId());
                }
                else {
                    $this->_redirect($this->getUrl('index'));
                }
            }

            if ($request->getPost('submit') && $fp->process($this->getRequest())) {
                $this->messenger->addMessage('Client details saved');
                $this->_redirect($this->getUrl('view') . '?id=' . $fp->obj->getId());
            }

            if ($fp->obj->saved()) {
                $title = 'Edit Client';
            }
            else {
                $title = 'Create Client';
            }

            if ($fp->obj->saved()) {
                $this->breadcrumbs->addStep($fp->obj->get('client_name'), $this->getUrl('view') . '?id=' . $fp->obj->getId());
            }

            $this->breadcrumbs->addStep($title);
            $this->view->fp = $fp;
        }

        public function settingsAction()
        {
            $request   = $this->getRequest();
            $client_id = (int) $request->getQuery('id');
            $bundle    = $request->getQuery('bundle');

            $client = new Application_Client();
            if (!$client->load($client_id)) {
                $this->_redirect($this->getUrl());
            }

            $url = $this->getUrl('view') . '?id=' . $client->getId();

            $fp = new Application_Client_Settings_Editor_Admin($client, $bundle);

            if ($request->getPost('cancel')) {
                $this->_redirect($url);
            }

            if ($request->getPost('submit') && $fp->process($request)) {
                $this->messenger->addMessage('Client settings saved');
                $this->_redirect($this->getUrl('settings') . '?id=' . $client->getId() . '&bundle=' . $bundle);
            }
            else if ($request->getPost('reset')) {
                $fp->reset();
                $this->messenger->addMessage('Client settings reset');
                $this->_redirect($this->getUrl('settings') . '?id=' . $client->getId() . '&bundle=' . $bundle);
            }

            $this->breadcrumbs->addStep($client, $url);
            $this->breadcrumbs->addStep('Client Settings');

            $this->view->client = $client;
            $this->view->fp = $fp;
        }

        public function deleteAction()
        {
            $request  = $this->getRequest();
            $clientId = (int) $request->getQuery('id');

            $client = new Application_Client();
            $client->load($clientId);

            if (!$client->saved()) {
                $this->_redirect($this->getUrl('index'));
            }

            if ($request->getPost('cancel')) {
                $this->_redirect($this->getUrl('view') . '?id=' . $client->getId());
            }
            else if ($request->getPost('confirm')) {
                $client->enableDelete(true);
                $client->delete();

                $this->messenger->addMessage('Client deleted');
                $this->_redirect($this->getUrl('index'));
            }

            $this->breadcrumbs->add($client, $this->getUrl('view') . '?id=' . $client->getId());
            $this->breadcrumbs->add('Delete Client');

            $this->view->client = $client;
        }
    }
?>