<?php
    class Clients_LicensesController extends Application_Admin_ControllerAction
    {
        protected $_supported = false;

        public function init()
        {
            parent::init();
            $this->breadcrumbs->addStep('Clients', $this->getUrl('index', 'index'));
            $this->breadcrumbs->addStep('Licenses', $this->getUrl('index'));

            $request = $this->getRequest();

            $this->_supported = Application_License_Manager::IsSupported();
            $this->view->supported = $this->_supported;

            if (!$this->_supported && $request->getActionName() != 'index') {
                $this->messengerError('Licensing not supported');
                $this->_redirect($this->getUrl('index'));
                return;
            }
        }

        public function indexAction()
        {
            $this->view->licenses = Application_License_Manager::GetInstalledLicenses();
        }

        public function loadAction()
        {
            $request = $this->getRequest();
            $fp = new Application_License_Editor();

            if ($request->getPost('cancel')) {
                $this->_redirect($this->getUrl('index'));
            }
            else if ($request->getPost('submit') && $fp->process($this->getRequest())) {
                $this->messenger->addMessage('Licenses saved');

                if (count($fp->objs) == 1) {
                    $this->_redirect($this->getUrl('view') . '?license=' . $fp->objs[0]['filename']);
                }
                else {
                    $this->_redirect($this->getUrl('load'));
                }
            }

            $this->breadcrumbs->addStep('Load Licenses');
            $this->view->fp = $fp;
        }

        public function viewAction()
        {
            $request = $this->getRequest();

            try {
                $license = Application_License_Manager::GetLicenseProperties(
                    Application_License_Manager::GetLicensePath($request->getQuery('license'))
                );
            }
            catch (Exception $ex) {
                $this->messengerError($ex->getMessage());
                $this->_redirect($this->getUrl('index'));
            }

            $this->breadcrumbs->add('License: ' . $license['filename'], $this->getUrl('view') . '?license=' . $license['filename']);
            $this->view->license = $license;
        }

        public function deleteAction()
        {
            $request = $this->getRequest();
            $name    = Application_License_Manager::GetLicensePath($request->getQuery('license'));

            if ($request->getPost('cancel')) {
                $this->_redirect($this->getUrl('index'));
            }

            try {
                $license = Application_License_Manager::GetLicenseProperties($name);

                if ($request->getPost('confirm')) {
                    Application_License_Manager::Delete($license['filename']);

                    $this->messenger->addMessage('License deleted');
                    $this->_redirect($this->getUrl('index'));
                }
            }
            catch (Exception $ex) {
                $this->messengerError($ex->getMessage());
                $this->_redirect($this->getUrl('index'));
            }

            $this->breadcrumbs->add('License: ' . $license['filename'], $this->getUrl('view') . '?license=' . $license['filename']);
            $this->breadcrumbs->add('Delete License');
            $this->view->license = $license;
        }
    }
?>