<?php
    class System_DatabaseController extends Application_Admin_ControllerAction
    {
        public function init()
        {
            parent::init();

            $this->breadcrumbs->add('System Maintenance', $this->getUrl('index', 'index', 'system'));
            $this->breadcrumbs->add('Database Management', $this->getUrl('index', 'database', 'system'));
        }

        public function indexAction()
        {
            try {
                $report = Application_Upgrader_Database::getDifferences();
            }
            catch (Exception $ex) {
                $this->view->error = $ex->getMessage();
            }

            $this->view->report = $report;
        }

        public function createtableAction()
        {
            $request = $this->getRequest();
            $tables  = ArrayTools::clean($request->getPost('table'));

            try {
                $num = Application_Upgrader_Database::createTables($this->db, $tables);
                $msg = sprintf('%d table%s created', $num, $num != 1 ? 's' : '');

                $this->messenger->addMessage($msg);
            }
            catch (Exception $ex) {
                $this->messengerError($ex->getMessage());
            }

            $this->_redirect($this->getUrl('index'));
        }

        public function repairTableAction()
        {
            $request = $this->getRequest();
            $name    = $request->getPost('table');
            $action  = $request->getPost('action');

            $add     = $request->getPost('add');
            $repair  = $request->getPost('repair');
            $drop    = $request->getPost('drop');
            $pk      = $request->getPost('pk');
            $indexes = $request->getPost('indexes');

            if (!is_array($add)) {
                $add = array();
            }

            if (!is_array($repair)) {
                $repair = array();
            }

            if (!is_array($drop)) {
                $drop = array();
            }

            if (!is_array($pk)) {
                $pk = array();
            }

            if (!is_array($indexes)) {
                $indexes = array();
            }

            $tables = array_unique(array_merge(
                array_keys($add),
                array_keys($repair),
                array_keys($drop),
                array_keys($pk),
                array_keys($indexes)
            ));

            foreach ($tables as $name) {
                try {
                    $table = Application_Installer_Database::getTable($name);
                }
                catch (Exception $ex) {
                    $this->messengerError($ex->getMessage());
                    continue;
                }

                if ($table->isCorrect()) {
                    continue;
                }

                $msgs   = array();
                $errors = array();

                if (array_key_exists($name, $drop)) {
                    foreach ($drop[$name] as $column) {
                        try {
                            $table->dropField($column);
                            $msgs[] = sprintf("Column '%s' dropped", $column);
                        }
                        catch (Exception $ex) {
                            $errors[] = sprintf("Error dropping column '%s': %s", $column, $ex->getMessage());
                        }
                    }
                }

                if (array_key_exists($name, $add)) {
                    foreach ($add[$name] as $column) {
                        try {
                            $table->createField($column);
                            $msgs[] = sprintf("Column '%s' created", $column);
                        }
                        catch (Exception $ex) {
                            $errors[] = sprintf("Error creating column '%s': %s", $column, $ex->getMessage());
                        }
                    }
                }

                if (array_key_exists($name, $repair)) {
                    foreach ($repair[$name] as $column) {
                        try {
                            $table->repairField($column);
                            $msgs[] = sprintf("Column '%s' repaired", $column);
                        }
                        catch (Exception $ex) {
                            $errors[] = sprintf("Error repairing column '%s': %s", $column, $ex->getMessage());
                        }
                    }
                }

                if (array_key_exists($name, $pk)) {
                    try {
                        $table->rebuildPk();
                        $msgs[] = 'Primary key rebuilt';
                    }
                    catch (Exception $ex) {
                        $errors[] = sprintf("Error rebuilding primary key: %s", $ex->getMessage());
                    }
                }

                if (array_key_exists($name, $indexes)) {
                    foreach ($indexes[$name] as $column) {
                        try {
                            $table->createIndex($column);
                            $msgs[] = sprintf("Index '%s' created", $column);
                        }
                        catch (Exception $ex) {
                            $errors[] = sprintf("Error creating index '%s': %s", $column, $ex->getMessage());
                        }
                    }
                }

                if (count($msgs) > 0) {
                    $this->messenger->addMessage(sprintf("Successful update%s to '%s': %s", count($msgs) != 1 ? 's' : '', $name, join(', ', $msgs)));
                }

                if (count($errors) > 0) {
                    $this->messengerError(sprintf("Error%s updating '%s': %s", count($errors) != 1 ? 's' : '', $name, join(', ', $errors)));
                }
            }

            $this->_redirect($this->getUrl('index'));
        }
    }
?>