<?php
    class System_IndexController extends Application_Admin_ControllerAction
    {
        public function init()
        {
            parent::init();
            $this->breadcrumbs->addStep('System Maintenance', $this->getUrl('index'));
        }

        public function indexAction()
        {

        }

        public function summaryAction()
        {
            $report = Application_SystemStatus::GetStatusReport();

            $json = array();

            foreach ($report->getItems() as $item) {
                $json[] = array(
                    'url'      => $item->getFixUrl(),
                    'messages' => $item->getMessages(),
                    'critical' => $item->isCritical()
                );
            }

            $this->sendJson($json);
        }

        public function statusAction()
        {
            $this->breadcrumbs->add('Status Report');
            $this->view->report = Application_SystemStatus::GetStatusReport();
        }

        public function phpinfoAction()
        {
            $request = $this->getRequest();
            $type    = $request->getQuery('type');

            $types = array(
                INFO_ALL           => 'All Info',
                INFO_GENERAL       => 'General',
                INFO_CONFIGURATION => 'Configuration',
                INFO_MODULES       => 'Modules',
                INFO_ENVIRONMENT   => 'Environment',
                INFO_VARIABLES     => 'Variables'
            );

            if (!array_key_exists($type, $types)) {
                $type = INFO_ALL;
            }

            ob_start();
            phpinfo($type);
            $info = ob_get_clean();

            $this->view->info  = $info;
            $this->view->types = $types;
            $this->view->type  = $type;
        }

        public function disableMaintenanceAction()
        {
            $request = $this->getRequest();
            $referer = trim($request->getServer('HTTP_REFERER'));

            try {
                Application_SystemStatus::DisableMaintenanceMode();
                $this->messenger->addMessage('Maintenance mode disabled, Control Panel and client sites are active again.');

                if (strlen($referer) > 0) {
                    $url = $referer;
                }
                else {
                    $url = $this->getUrl('index');
                }
            }
            catch (Exception $ex) {
                $this->messengerError($ex->getMessage());
                $url = $this->getUrl('status');
            }

            $this->_redirect($url);
        }

        public function enableMaintenanceAction()
        {
            $request = $this->getRequest();
            $referer = trim($request->getServer('HTTP_REFERER'));

            Application_SystemStatus::EnableMaintenanceMode();
            $this->messengerError('Maintenance mode enabled, Control Panel and client sites are now disabled.');

            if (strlen($referer) > 0) {
                $url = $referer;
            }
            else {
                $url = $this->getUrl('index');
            }
            $this->_redirect($url);
        }
    }
?>