<?php
    class Upgrade_IndexController extends Application_Admin_ControllerAction
    {
        public function init()
        {
            parent::init();
            $this->breadcrumbs->add('System Maintenance', $this->getUrl('index', 'index', 'system'));
            $this->breadcrumbs->add('Package Management', $this->getUrl('index', 'index'));
        }

        public function indexAction()
        {
            $this->view->canAutoCheck = Application_Upgrader_Manager::CanAutoCheck();

            try {
                $this->view->username = Application_Upgrader_Manager::GetUsername();
            }
            catch (Exception $ex) {
                $this->view->username = '';
            }

            try {
                $this->view->password = Application_Upgrader_Manager::GetPassword();
            }
            catch (Exception $ex) {
                $this->view->password = '';
            }

            $this->view->channels  = Application_Upgrader_Manager::GetChannels();
            $this->view->installed = Application_Upgrader_Manager::GetRegisteredPackages();
        }

        public function settingsAction()
        {
            $request = $this->getRequest();

            if ($request->getPost('submit')) {
                Application_Upgrader_Manager::SetSettings(
                    $request->getPost('username'),
                    $request->getPost('password'),
                    (bool) $request->getPost('enabled'),
                    $request->getPost('channel')
                );

                $this->messenger->addMessage('Settings updated');
            }

            $this->_redirect($this->getUrl('index'));
        }

        public function summaryAction()
        {
            try {
                $json = Application_Upgrader_Manager::GetUpgradeSummary();
            }
            catch (Exception $ex) {
                $json = array(
                    'error'  => true,
                    'errnum' => $ex->getCode(),
                    'errmsg' => $ex->getMessage()
                );
            }

            $this->sendJson($json);
        }

        public function packagesAction()
        {
            $request  = $this->getRequest();
            $packages = ArrayTools::Clean($request->getPost('packages'));

            $this->breadcrumbs->add('Install Packages');
            $this->view->info = Application_Upgrader_Package::GetInstallInfo($packages);
        }

        public function packageInstalled($package)
        {
            ob_flush();
            flush();
        }

        public function packageInstallAction()
        {
            $this->noRender();

            $request  = $this->getRequest();
            $packages = ArrayTools::Clean($request->getPost('packages'));

            header('Content-type: text/html');

            $i = 0;

            echo '<style type="text/css">body { font-family : monospace; }</style>';
            echo "INSTALLING PACKAGES" . str_repeat(' ', 500) . "<br />";
            echo "-------------------" . str_repeat(' ', 500) . "<br />";
            echo str_repeat(' ', 500) . "<br />\n";

            try {
                $inMaintenance = Application_SystemStatus::InMaintenanceMode();

                Application_SystemStatus::EnableMaintenanceMode();

                $fp = fopen('php://output', 'w');
                Application_Upgrader_Manager::SetLoggerStream($fp);
                Application_Upgrader_Manager::SetLoggerLineFormat("%s<br />\n");
                Application_Upgrader_Manager::SetPackageInstallCallback(array($this, 'packageInstalled'));

                $installed = Application_Upgrader_Manager::InstallPackages($packages);

                if (count($installed) == 0) {
                    echo "No packages installed\n";
                }
                else {
                    echo "<div>Installed packages:</div>";
                    echo "<ul>";
                    foreach ($installed as $package) {
                        echo sprintf("<li>%s-%s</li>\n", $package->getName(), $package->getVersion());
                    }
                    echo "</ul>";
                }

                if (!$inMaintenance) {
                    Application_SystemStatus::DisableMaintenanceMode();
                }
            }
            catch (Exception $ex) {
                echo $ex->getMessage() . "\n";
            }

            echo '<p><a href="/upgrade">Package Management</a></p>';

            fclose($fp);
        }

        public function appAction()
        {
            $request = $this->getRequest();

            $this->breadcrumbs->add('Upgrade Recite');
            $this->view->info = Application_Upgrader_Manager::GetInstallInfo();
        }
    }
?>