<?php
    class Users_IndexController extends Application_Admin_ControllerAction
    {
        public function init()
        {
            parent::init();
            $this->breadcrumbs->addStep('Users', $this->getUrl('index', 'index', 'users'));
        }

        public function indexAction()
        {
            $dr = new Application_User_Renderer(new Application_User_Directory());
            $dr->run($this->getRequest());

            $this->view->dr = $dr;
        }

        public function viewAction()
        {
            $request = $this->getRequest();
            $user_id = (int) $request->getQuery('id');
            $action  = $request->getQuery('action');

            $user = new Application_User();
            if (!$user->loadForDirectory(null, $user_id)) {
                $this->_redirect($this->geturl());
            }

            switch ($action) {
                case 'roleadd':
                    $role_id = (int) $request->getPost('role_id');
                    if ($user->roleAdd($role_id)) {
                        $this->messenger->addMessage('Role added to user');
                        $this->_redirect($this->geturl('view') . '?id=' . $user->getId());
                    }

                    break;

                case 'roleremove':
                    $role_id = (int) $request->getQuery('role_id');
                    $user->roleRemove($role_id);

                    $this->messenger->addMessage('Role removed from user');
                    $this->_redirect($this->geturl('view') . '?id=' . $user->getId());
            }

            $clients = Application_Client::GetClients();

            $roles = array(
                0 => array(
                    'client' => 'Global Roles',
                    'roles'  => array()
                )
            );

            foreach ($clients as $client) {
                $roles[$client->getId()] = array(
                    'client' => $client,
                    'roles'  => array()
                );
            }

            $options = array(
                'directory_ids' => array(null, $user->directory_id)
            );

            foreach (Application_User_Role::GetRoles($options) as $role) {
                $clientId = (int) $role->client_id;

                if (array_key_exists($clientId, $roles)) {
                    $roles[$clientId]['roles'][$role->getId()] = $role;
                }
            }

            $this->breadcrumbs->add($user->username);
            $this->view->roles = $roles;
            $this->view->user = $user;
        }

        public function editAction()
        {
            $request  = $this->getRequest();
            $clientId = (int) $request->get('client');
            $userId   = (int) $request->get('id');
            $roleId   = (int) $request->get('role_id');

            $fp = new Application_User_Editor($userId, $roleId, $clientId);

            if ($request->getPost('cancel')) {
                if ($fp->obj->saved()) {
                    $this->_redirect($this->getUrl('view') . '?id=' . $fp->obj->getId());
                }
                else {
                    $this->_redirect($this->getUrl());
                }
            }

            if ($request->getPost('submit') && $fp->process($request)) {
                $this->messenger->addMessage('User details saved');
                $this->_redirect($this->getUrl('view') . '?id=' . $fp->obj->getId());
            }

            if ($fp->obj->saved()) {
                $title = 'Edit User';
            }
            else {
                $title = 'Create User';
            }

            if ($fp->obj->saved()) {
                $this->breadcrumbs->addStep($fp->obj->get('username'), $this->getUrl('view') . '?id=' . $fp->obj->getId());
            }

            $this->breadcrumbs->add($title);
            $this->view->fp = $fp;
        }

        public function deleteAction()
        {
            $request = $this->getRequest();
            $user_id = (int) $request->getQuery('id');

            $user = new Application_User();
            if (!$user->loadForDirectory(null, $user_id)) {
                $this->_redirect($this->getUrl());
            }

            if ($user->getId() == getUserId()) {
                $this->messengerError('You cannot delete your own account');
                $this->_redirect($this->getUrl('view') . '?id=' . $user->getId());
            }
            else if ($request->getPost('cancel')) {
                $this->_redirect($this->getUrl('view') . '?id=' . $user->getId());
            }
            else if ($request->getPost('confirm')) {
                $user->delete();

                $this->messenger->addMessage('User deleted');
                $this->_redirect($this->getUrl('index'));
            }

            $this->breadcrumbs->add($user->username, $this->getUrl('view') . '?id=' . $user->getId());
            $this->breadcrumbs->add('Delete User');
            $this->view->user = $user;
        }
    }
?>