<?php
    class Users_PermissionsController extends Application_Admin_ControllerAction
    {
        public function init()
        {
            parent::init();
        }

        public function manageAction()
        {
            $request = $this->getRequest();
            $fp = new Application_User_Permissions_Permission_Editor($request);
            $url = $this->getUrl('manage') . '?role_id=' . $fp->role->getId() . '&directory_id=' . $fp->directory->getId() . '&client_id=' . $fp->client->getId();

            if ($request->isXmlHttpRequest()) {
                $fp->process($request);
                $json = array(
                    'success' => !$fp->hasError(),
                    'errors'  => $fp->getErrors()
                );

                $this->sendJson($json);
                return;
            }
            else if ($request->getPost('submit')) {
                $fp->process($request);

                if (strlen($fp->filter) > 0) {
                    $url .= '&filter=' . $fp->filter;
                }

                $this->messenger->addMessage('Permission saved');
                $this->_redirect($url);
            }
            else if ($request->getPost('delete')) {
                $permission_ids = ArrayTools::clean($request->getPost('delete'));

                if (strlen($fp->filter) > 0) {
                    $url .= '&filter=' . $fp->filter;
                }

                foreach ($permission_ids as $permission_id => $null) {
                    $permission = new Application_User_Permissions_Permission();
                    if ($permission->load($permission_id)) {
                        $permission->delete();

                        $this->messenger->addMessage('Permission deleted');
                        $this->_redirect($url);
                    }
                }
            }

            if ($fp->client->saved()) {
                $this->breadcrumbs->add('Clients', $this->getUrl('index', 'index', 'clients'));
                $this->breadcrumbs->add($fp->client, $this->getUrl('view', 'index', 'clients') . '?id=' . $fp->client->getId());
            }
            else {
                $this->breadcrumbs->addStep('Users', $this->getUrl('index', 'index', 'users'));
                $this->breadcrumbs->addStep('User Directories', $this->getUrl('index', 'directories', 'users'));

                if ($fp->directory->saved()) {
                    $title = (string) $fp->directory;
                }
                else {
                    $title = 'Default user directory';
                }

                $this->breadcrumbs->add($title, $this->getUrl('view', 'directories') . '?id=' . $fp->directory->getId());
            }

            $this->breadcrumbs->add('Permissions');
            $this->view->fp = $fp;
        }

        public function deployAction()
        {
            $request   = $this->getRequest();
            $client_id = (int) $request->getPost('client_id');
            $filter    = $request->getPost('filter');
            $client    = new Application_Client();

            if (!$client->load($client_id)) {
                $this->_redirect($this->getUrl('index', 'index'));
            }

            $pm = Application_User_Permissions_Manager::GetInstance();
            $pm->deployDefaults($client);

            $url = $this->getUrl('manage') . '?client_id=' . $client->getId();

            if (strlen($filter) > 0) {
                $url .= '&filter=' . $filter;
            }

            $this->messenger->addMessage('Default permissions deployed');
            $this->_redirect($url);
        }
    }
?>