{include file='header.tpl' active='system'}

{status}
    This tool checks whether or not all database tables required by modules are
    installed and up-to-date.
{/status}

{if $report.errors|@count > 0}
    {status class='no'}
        {if $report.errors|@count == 1}
            {$report.errors.0|escape}
        {else}
            <ul>
                {foreach from=$report.errors item=error}
                    <li>{$error|escape}</li>
                {/foreach}
            </ul>
        {/if}
    {/status}
{elseif $report.missing|@count == 0 && $report.incorrect|@count == 0 && $report.extra|@count == 0}
    {if $report.unsupported|@count == 0}
        {status class='yes'}
            The database appears to be correct.
        {/status}
    {else}
        {status class='maybe'}
            The database appears to be correct, however the following checks are unable
            to be performed by this this database adapter ({$db|escape}):

            <ul>
                {foreach from=$report.unsupported item=item}
                    <li>{$item|escape}</li>
                {/foreach}
            </ul>
        {/status}
    {/if}
{else}
    {status class='no'}
        Please review the database differences below.
    {/status}
{/if}

{if $report.errors|@count == 0}
    {if $report.missing|@count > 0}
        <h2>Missing Tables</h2>

        <form method="post" action="{geturl target='system:database:createtable'}" onsubmit="return confirm('Click OK to create selected tables')">
            <table class="data full checkall" id="missing-tables">
                <tr>
                    <th><input type="checkbox" class="checkall" /></th>
                    <th class="title">Table</th>
                </tr>
                {foreach from=$report.missing item=row}
                    <tr>
                        <td class="value"><input type="checkbox" name="table[]" value="{$row.name|escape}" /></td>
                        <td>{$row.name|escape}</td>
                    </tr>
                {/foreach}
            </table>

            {status class='submit'}
                <input type="submit" value="Create Selected Tables" class="submit" />
            {/status}
        </form>
    {/if}

    {if $report.incorrect|@count > 0}
        <h2>Incorrect Tables</h2>

        {if $report.unsupported|@count > 0}
            {status class='no'}
                The following checks are not supported by this database adapter:
                <ul>
                    {foreach from=$report.unsupported item=item}
                        <li>{$item|escape}</li>
                    {/foreach}
                </ul>
            {/status}
        {/if}

        <form method="post" action="{geturl target='system:database:repair-table'}" onsubmit="return confirm('Click OK to repair the selected items')">
            <table class="data full">
                <tr>
                    <th class="title">Table</th>
                    <th class="title">
                        <label>
                            <input type="checkbox" name="add_all" value="1" />
                            Missing Columns
                        </label>
                    </th>
                    <th class="title">
                        <label>
                            <input type="checkbox" name="repair_all" value="1" />
                            Incorrect Columns
                        </label>
                    </th>
                    <th class="title">
                        <label>
                            <input type="checkbox" name="drop_all" value="1" />
                            Extra Columns
                        </label>
                    </th>
                    <th class="title">
                        <label>
                            <input type="checkbox" name="indexes_all" value="1" />
                            Indexes
                        </label>
                    </th>
                </tr>
                {foreach from=$report.incorrect key=table item=row}
                    <tr>
                        <td>{$table|escape}</td>
                        <td>
                            {foreach from=$row.missing item=column}
                                <div>
                                    <label>
                                        <input type="checkbox" name="add[{$table|escape}][]" class="add" value="{$column|escape}" />
                                        {$column|escape}
                                    </label>
                                </div>
                            {/foreach}
                        </td>
                        <td>
                            {foreach from=$row.incorrect key=column item=error}
                                <div>
                                    <label>
                                        <input type="checkbox" name="repair[{$table|escape}][]" class="repair" value="{$column|escape}" />
                                        {$column|escape}
                                        <span class="inline-options">({$error|escape})</span>
                                    </label>
                                </div>
                            {/foreach}
                        </td>
                        <td>
                            {foreach from=$row.extra item=column}
                                <div>
                                    <label>
                                        <input type="checkbox" name="drop[{$table|escape}][]" class="drop" value="{$column|escape}" />
                                        {$column|escape}
                                    </label>
                                </div>
                            {/foreach}
                        </td>
                        <td>
                            {if $row.pk|@count > 0}
                                <div>
                                    <label>
                                        <input type="checkbox" name="pk[{$table|escape}]" class="indexes" value="1" />
                                        Primary key incorrect
                                        <span class="inline-options">(Expected {$row.pk|@join:', '|escape})</span>
                                    </label>
                                </div>
                            {/if}
                            {foreach from=$row.indexes item=column}
                                <div>
                                    <label>
                                        <input type="checkbox" name="indexes[{$table|escape}][]" class="indexes" value="{$column|escape}" />
                                        Missing index
                                        <span class="inline-options">({$column|escape})</span>
                                    </label>
                                </div>
                            {/foreach}
                        </td>
                    </tr>
                {/foreach}
            </table>

            {status class='submit'}
                <input type="submit" value="Repair Selected Items" class="submit" />
            {/status}
        </form>

        {literal}
        <script type="text/javascript">
            jQuery(function($) {
                $([ 'add', 'repair', 'drop', 'indexes' ]).each(function(i, item) {
                    $(':checkbox[name=' + item + '_all]').bind('click', function(e) {
                        $(':checkbox.' + item).attr('checked', $(this).attr('checked'));
                    });
                });
            });
        </script>
        {/literal}
    {/if}

    {if $report.extra|@count > 0}
        <h2>Extra Tables</h2>

        <ul>
            {foreach from=$report.extra item=row}
                <li>{$row|escape}</li>
            {/foreach}
        </ul>
    {/if}
{/if}

{include file='footer.tpl'}
