{include file='header.tpl' active='system'}

{status}
    This page is used to manage web server configurations. You can choose either
    to auto-generate configurations that you can load into your web server, or
    you can build your own using the instructions on this page.
{/status}

<h2>Manual Configuration</h2>

<p>
    If managing web server configuration yourself, use the following paths
    as the web root for each respective domain:
</p>

<ul>
    <li>Control Panel web root: <code>{$fp->cpWebRoot}</code></li>
    <li>Client site web root: <code>{$fp->clientWebRoot|escape}</code></li>
    <li>WebDAV web root: <code>{$fp->davWebRoot|escape}</code></li>
</ul>

<h2>Auto-Generated Configurations</h2>

<ul>
    <li>Output directory: <code>{$fp->confDir|escape}</code></li>
    <li>Brand files: <code>brand-<em>[ID]</em>-<em>[NAME]</em>.conf</code></li>
    <li>Client files: <code>client-<em>[ID]</em>-<em>[NAME]</em>.conf</code></li>
    <li>Lock file: <code>{$fp->touchPath|escape}</code></li>
</ul>

<p>
    Files are auto-updated when a brand or client is updated (e.g. domain added or removed)
    and the lock file is touched (created or update).
</p>

<p>
    You can load all of these files in Apache HTTP server using the following directive in
    your <code>httpd.conf</code> file:
</p>

<div class="program-listing">
    <pre>{$fp->apacheSample|escape}</pre>
</div>

<h3>Select Brands and Clients</h3>

{if $fp->items|@count == 0}
    {status class='no'}
        There are no brands or clients to configure!
    {/status}
{else}
    <form method="post">
        <div>
            {status}
                Select which brands and clients to auto-generate configuration files for.
            {/status}

            {foreach from=$fp->items item=brand key=brandId}
                {if $brand.brand->saved()}
                    <h3>
                        <label>
                            <input type="checkbox" name="brand[]" value="{$brandId}"{if $fp->brands.$brandId} checked="checked"{/if} />
                            Brand: {$brand.brand|escape}
                        </label>
                        <span class="inline-options">
                            (<a href="{geturl target='brands:index:view'}?id={$brandId}">view brand</a>, <a href="{geturl target='system:webserver:brand'}?brand={$brandId}">view config</a>)
                        </span>
                    </h3>
                {else}
                    <h3>Brand: (None)</h3>
                {/if}

                {if $brand.clients|@count > 0}
                    <ul>
                        {foreach from=$brand.clients item=client key=clientId}
                            <li>
                                <label>
                                    <input type="checkbox" name="client[]" value="{$client->getId()}"{if $fp->clients.$clientId} checked="checked"{/if} />
                                    Client: {$client|escape}
                                </label>
                                <span class="inline-options">
                                    (<a href="{geturl target='clients:index:view'}?id={$clientId}">view client</a>, <a href="{geturl target='system:webserver:client'}?client={$clientId}">view config</a>)
                                </span>
                            </li>
                        {/foreach}
                    </ul>
                {/if}
            {/foreach}
        </div>

        {status class='submit'}
            <input type="submit" value="Save And Rebuild All" class="submit" name="submit" />
        {/status}
    </form>
{/if}

{include file='footer.tpl'}
