<?php
    class AuthController extends Application_Cp_Site_ControllerAction
    {
        public function indexAction()
        {
            $this->_forward('login');
        }

        public function loginAction()
        {
            if ($this->auth->hasIdentity()) {
                $this->_redirect('/');
            }

            $request = $this->getRequest();

            if ($request->isXmlHttpRequest()) {
                $this->addEvent('notloggedin.auth');
            }
            else {
                $fp = new Application_Cp_Login($request, $this->getUrl('login'), '/');

                if ($request->isPost() && $fp->process($request)) {
                    $this->_redirect($fp->redirect);
                }

                $this->breadcrumbs->add(translate('Log In'));
                $this->view->fp = $fp;
            }
        }

        public function logoutAction()
        {
            Application_Cp_Login::Logout();
            $this->_redirect($this->geturl('login'));
        }

        public function resetPasswordAction()
        {
            $request  = $this->getRequest();
            $complete = (bool) $request->getQuery('complete');

            if (!$complete) {
                $fp = new Application_Cp_Login_ResetPassword();

                if ($request->isPost() && $fp->process($request)) {
                    $this->_redirect($this->getUrl('reset-password') . '?complete=1');
                }

                $this->view->fp = $fp;
            }

            $this->breadcrumbs->add(translate('Reset Password'));
            $this->view->complete = $complete;
        }

        public function changePasswordAction()
        {
            $request  = $this->getRequest();
            $complete = (bool) $request->getQuery('complete');

            if (!$complete) {
                $fp = new Application_Cp_Login_ChangePassword($request);

                if ($request->isPost() && $fp->process($request)) {
                    $this->_redirect($this->getUrl('change-password') . '?complete=1');
                }

                $this->view->fp = $fp;
            }

            $this->breadcrumbs->add(translate('Change Password'));
            $this->view->complete = $complete;
        }

        public function chooseClientAction()
        {
            $request  = $this->getRequest();
            $redirect = $request->getQuery('redirect');

            if (strlen($redirect) == 0) {
                $redirect = $request->getServer('REQUEST_URI');
            }

            $exempt = array(
                $this->getUrl('choose-client'),
                $this->getUrl('login')
            );

            foreach ($exempt as $_url) {
                if (substr($redirect, 0, strlen($_url)) == $_url) {
                    $redirect = '';
                    break;
                }
            }

            if (strlen($redirect) == 0) {
                $redirect = '/';
            }

            $fp = new Application_Cp_Login_ClientChooser();

            if ($request->isPost()) {
                $fp->process($request);
            }

            if ($fp->hasSetClient()) {
                $this->_redirect($redirect);
            }

            if (count($fp->clients) == 0) {
                $title = 'Unable to Log You In';
            }
            else {
                $title = 'Choose Client';
            }

            $this->breadcrumbs->add(translate($title));
            $this->view->fp = $fp;
            $this->view->redirect = $redirect;
        }
    }
?>