<?php
    class SettingsController extends Application_Cp_Site_ControllerAction
    {
        public function preDispatch()
        {
            Application_User_Permissions_Query::BuildAndAssert('core:settings:editable');
            parent::preDispatch();
        }

        public function indexAction()
        {
            $theme = Application_Cp_Theme_Manager::GetActiveTheme();

            $dialog = new Application_Cp_Ui_Dialog_Form_SaveClose();
            $dialog->setSaveText('Save Settings')
                   ->setControllerScript(null, 'Recite_Settings')
                   ->setDimensions(800, 450);

            $this->breadcrumbs->add('Web Site Settings');
            $this->view->dialog = $dialog;

            $this->sendDialog($dialog);
        }

        public function bundlesAction()
        {
            $treeView = new Application_Settings_Cp_TreeView();
            $treeView->setRequest($this->getRequest());

            $this->noRender();
            $treeView->send($this->getResponse());
        }

        public function bundleAction()
        {
            $request    = $this->getRequest();
            $bundleName = $request->getQuery('bundle');

            $event = $this->addEvent('bundle.settings');

            try {
                $fp = new Application_Client_Settings_Editor_Cp($bundleName);

                $this->view->fp = $fp;

                $event->addMemo('bundle', array(
                    'name'  => $fp->bundle->getName(),
                    'title' => (string) $fp->bundle
                ));
                $event->addMemo('body', $this->view->render($this->_helper->viewRenderer->getViewScript()));
            }
            catch (Exception $ex) {
                $event->addMemo('error', true)
                      ->addMemo('errmsg', $ex->getMessage())
                      ->addMemo('body', $this->view->render('default/settings/lib/none.tpl'));
            }
        }

        public function saveSettingsAction()
        {
            $request    = $this->getRequest();
            $bundleName = $request->getPost('bundle');

            $fp = new Application_Client_Settings_Editor_Cp($bundleName);

            if ($request->isPost()) {
                if ($fp->process($request)) {
                    //$this->addStatusMessage(translate("Settings bundle '%s' updated", $fp->bundle), Application_Site_Response_StatusMessage::STATUS_SUCCESS);

                    $event = $this->addEvent('bundlesaved.settings');
                    $event->addMemo('bundle', array(
                        'name'  => $fp->bundle->getName(),
                        'title' => (string) $fp->bundle
                    ));
                }

                $this->addEvent('submitted.form')
                     ->addMemo('form', $fp->getResponseAsArray());
            }
        }

        public function customEditAction()
        {
            $request    = $this->getRequest();
            $setting_id = (int) $request->getQuery('id');
            $type       = $request->getQuery('type');

            $fp = new Application_Client_Settings_Custom_Editor($setting_id, $type);

            if ($request->isPost()) {
                $new = !$fp->obj->saved();

                if ($fp->process($request)) {
                    if ($new) {
                        $this->addStatusMessage(translate("Custom setting '%s' created", $fp->obj), Application_Site_Response_StatusMessage::STATUS_SUCCESS);
                        $this->addEvent('customsettingcreated.settings');
                    }
                    else {
                        $this->addStatusMessage(translate("Custom setting '%s' updated", $fp->obj), Application_Site_Response_StatusMessage::STATUS_SUCCESS);
                        $this->addEvent('customsettingmodified.settings');
                    }
                }

                $this->addEvent('submitted.form')
                     ->addMemo('form', $fp->getResponseAsArray());

                return;
            }

            $dialog = new Application_Cp_Ui_Dialog_Form();

            if ($fp->obj->saved()) {
                $dialog->setOkText('Save Custom Setting Changes');
                $this->breadcrumbs->add('Edit Custom Setting');
            }
            else {
                $dialog->setOkText('Create Custom Setting');
                $this->breadcrumbs->add('New Custom Setting');
            }

            $this->view->dialog = $dialog;
            $this->view->fp = $fp;

            $this->sendDialog($dialog);
        }

        public function customDeleteAction()
        {
            $request    = $this->getRequest();
            $setting_id = (int) $request->getPost('id');

            $setting = new Application_Client_Settings_Custom();
            $setting->load($setting_id);

            if (!$setting->saved()) {
                throw new Exception('Invalid custom setting');
            }

            $title = $setting;
            $setting->delete();

            $this->addStatusMessage(translate("Custom setting '%s' deleted", $title), Application_Site_Response_StatusMessage::STATUS_NOTICE);
            $this->addEvent('customsettingdeleted.settings')
                 ->addMemo('title', $title)
                 ->addMemo('id', $setting_id);
        }

        public function clearCacheAction()
        {
            Application_Cache::ClearAllCaches(Application::GetClient());

            $this->addStatusMessage(translate('Cache cleared'), Application_Site_Response_StatusMessage::STATUS_NOTICE);
            $this->addEvent('cachecleared.settings');
        }

        public function maintenanceEnableAction()
        {
            Application_SystemStatus::EnableItemMaintenanceMode(Application::GetClient());

            //$this->addStatusMessage(translate('Maintenance mode enabled'), Application_Site_Response_StatusMessage::STATUS_NOTICE);
            $this->addEvent('maintenanceenabled.settings');
        }

        public function maintenanceDisableAction()
        {
            Application_SystemStatus::DisableItemMaintenanceMode(Application::GetClient());

            $this->addStatusMessage(translate('Maintenance mode disabled'), Application_Site_Response_StatusMessage::STATUS_SUCCESS);
            $this->addEvent('maintenancedisabled.settings');
        }
    }
?>
