<?php
    class TabController extends Application_Cp_Site_ControllerAction
    {
        protected function assertPermissions()
        {
            Application_User_Permissions_Query_Item::BuildAndAssert('core:tabs:manageable');
        }

        public function init()
        {
            parent::init();

            Application_Cp_Tab_RequestHandler::SetView($this->view);
            Application_Cp_Tab_RequestHandler::SetRequest($this->getRequest());
            Application_Cp_Tab_RequestHandler::SetResponse($this->getResponse());
        }

        public function getStateAction()
        {
            $request  = $this->getRequest();
            $selected = (int) $request->getQuery('selected');

            if ($selected <= 0) {
                $selected = Application_User_Cp_Tab::GetActiveTabId();
            }

            $json = Application_Cp_Tab_RequestHandler::GetFullState($selected);

            $this->addEvent('fullstate.tabs')
                 ->addMemo('tabs', $json);
        }

        public function loadWidgetAction()
        {
            $request   = $this->getRequest();
            $response  = $this->getResponse();
            $driver    = $request->getQuery('driver');
            $container = $request->getQuery('container');

            try {
                $json = Application_Cp_Tab_RequestHandler::GetDriver($driver);

                $event = $this->addEvent('widgetdata.tabs');
                $event->addMemo('widget', $json)
                      ->addMemo('container', $container);
            }
            catch (Exception $ex) {
                $this->addStatusMessage($ex->getMessage(), Application_Site_Response_StatusMessage::STATUS_ERR);
            }
        }

        public function setWidgetParamsAction()
        {
            Application_Cp_Tab_RequestHandler::SetWidgetParams();

            $this->addEvent('widgetparamsaved.tabs');
        }

        public function jsAction()
        {
            $request = $this->getRequest();
            $tabId   = $request->getQuery('tab');

            $tab = new Application_User_Cp_Tab();
            $tab->loadForClientAndUser($this->client->getId(), $this->user->getId(), $tabId);

            if (!$tab->saved()) {
                $this->addEvent('tabdataerror.tabs');
                return;
            }
            Application_Cp_Tab_RequestHandler::SendTabJs($tab);
        }

        public function deleteWidgetAction()
        {
            $request  = $this->getRequest();
            $widgetId = (int) $request->getQuery('widget');

            $widget = new Application_User_Cp_Widget();
            $widget->loadWidget($this->client->getId(), $this->user->getId(), $widgetId);

            if ($widget->saved()) {
                $widget->delete();
                $this->addEvent('widgetdeleted.tabs')
                     ->addMemo('id', $widget->getDeletedId());
            }
            else {
                $this->addEvent('invalidwidget.tabs');
            }
        }

        public function getTabAction()
        {
            $request = $this->getRequest();
            $tabId   = (int) $request->getQuery('id');

            $tab = new Application_User_Cp_Tab();

            if ($tab->loadForClientAndUser($this->client->getId(), $this->user->getId(), $tabId)) {
                $json = Application_Cp_Tab_RequestHandler::GetTab($tab);

                Application_User_Cp_Tab::SetActiveTab($tab);

                $this->addEvent('tabdata.tabs')
                     ->addMemo('tab', $json);
            }
            else {
                $this->addEvent('tabdataerror.tabs');
            }
        }

        public function createAction()
        {
            $this->assertPermissions();

            $request = $this->getRequest();
            $fp = new Application_User_Cp_Tab_Editor();

            if ($request->isPost()) {
                if ($fp->process($request)) {
                    $json = Application_Cp_Tab_RequestHandler::GetTab($fp->obj);

                    $this->addStatusMessage(translate("Tab '%s' created", $fp->obj->getTitle()), Application_Site_Response_StatusMessage::STATUS_SUCCESS);
                    $this->addEvent('tabcreated.tabs')
                         ->addMemo('tab', $json);
                }

                $this->addEvent('submitted.form')
                     ->addMemo('form', $fp->getResponseAsArray());

                return;
            }

            $dialog = new Application_Cp_Ui_Dialog_Form();
            $dialog->setOkText('Save Tab');

            $this->breadcrumbs->add('Create New Tab');
            $this->view->dialog = $dialog;
            $this->view->fp = $fp;

            $this->sendDialog($dialog);
        }

        public function editAction()
        {
            $this->assertPermissions();

            $request = $this->getRequest();
            $tabId   = (int) $request->getParam('id');
            $fp      = new Application_User_Cp_Tab_Editor($tabId);

            if ($request->isPost()) {
                if ($fp->process($request)) {
                    $json = Application_Cp_Tab_RequestHandler::GetTab($fp->obj);

                    $this->addStatusMessage(translate("Tab '%s' updated", $fp->obj->getTitle()), Application_Site_Response_StatusMessage::STATUS_SUCCESS);
                    $this->addEvent('tabupdated.tabs')
                         ->addMemo('tab', $json);
                }

                $this->addEvent('submitted.form')
                     ->addMemo('form', $fp->getResponseAsArray());

                return;
            }

            $dialog = new Application_Cp_Ui_Dialog_Form();
            $dialog->setOkText('Save Tab');

            $this->breadcrumbs->add('Edit Tab');
            $this->view->dialog = $dialog;
            $this->view->fp = $fp;

            $this->sendDialog($dialog);
        }

        public function deleteAction()
        {
            $this->assertPermissions();

            $request = $this->getRequest();
            $tab_id  = (int) $request->getPost('id');
            $tab     = new Application_User_Cp_Tab();
            $tab->loadForClientAndUser($this->client->getId(), $this->user->getId(), $tab_id);

            if (!$tab->saved()) {
                throw new Exception('Invalid tab');
            }

            $title = $tab->getTitle();

            $tab->delete();

            $this->addStatusMessage(translate("Tab '%s' deleted", $title), Application_Site_Response_StatusMessage::STATUS_NOTICE);
            $this->addEvent('tabdeleted.tabs')
                 ->addMemo('id', $tab_id);
        }

        public function tabOrderAction()
        {
            $fp = new Application_User_Cp_Tab_Editor();

            try {
                $fp->saveOrder($this->getRequest());
                $this->addEvent('taborder.tabs');
            }
            catch (Exception $ex) {
                $this->addStatusMessage($ex->getMessage(), Application_Site_Response_StatusMessage::STATUS_ERR);
            }
        }

        public function widgetBrowserAction()
        {
            $request = $this->getRequest();
            $tab_id  = $request->getQuery('tab');

            $options = array(
                'addableOnly' => true,
                'checkPerms'  => true
            );

            $categories = Application_Cp_Widget_Manager::GetCategories($options);

            $dialog = new Application_Cp_Ui_Dialog_Info();
            $dialog->setOkText('Close');

            $this->breadcrumbs->add('Add Widgets');
            $this->view->dialog     = $dialog;
            $this->view->categories = $categories;
            $this->view->tab_id     = $tab_id;

            $this->sendDialog($dialog);
        }

        public function widgetListAction()
        {
            $request  = $this->getRequest();
            $category = Text::Sanitize($request->getPost('category'));

            $json = array();

            if (strlen($category) > 0) {
                $options = array(
                    'addableOnly' => true,
                    'categories'  => $category,
                    'checkPerms'  => true
                );

                foreach (Application_Cp_Widget_Manager::GetDrivers($options) as $driver) {
                    $json[] = array(
                        'name'  => $driver->getName(),
                        'title' => (string) $driver
                    );
                }
            }

            $this->sendJson($json);
        }

        public function widgetInfoAction()
        {
            $request = $this->getRequest();
            $name    = $request->getPost('driver');
            $driver  = Application_Cp_Widget_Manager::Factory($name);

            $driver->assertPermissions();

            $ret = array(
                'name'        => $driver->getName(),
                'title'       => (string) $driver,
                'categories'  => $driver->getCategories(),
                'description' => $driver->getDescription()
            );

            $this->sendJson($ret);
        }

        public function addWidgetAction()
        {
            $request = $this->getRequest();

            try {
                $driver = $request->getPost('driver');
                $tabId  = (int) $request->getPost('tab');
                $column = (int) $request->getPost('column');
                $params = array();

                foreach ($request->getPost() as $k => $v) {
                    if (preg_match('/^param_(.*)$/', $k, $matches)) {
                        $params[$matches[1]] = $v;
                    }
                }

                $tab = new Application_User_Cp_Tab();

                if (!$tab->loadForClientAndUser($this->client->getId(), $this->user->getId(), $tabId)) {
                    throw new Exception('Invalid tab');
                }

                $widget = Application_User_Cp_Widget::Create($tab, $driver, $params, $column);

                $json = Application_Cp_Tab_RequestHandler::GetWidget($widget);

                $this->addEvent('widgetadded.tabs')
                     ->addMemo('widget', $json)
                     ->addMemo('column', $widget->col)
                     ->addMemo('ranking', $widget->ranking);
            }
            catch (Exception $ex) {
                $this->addStatusMessage($ex->getMessage(), Application_Site_Response_StatusMessage::STATUS_ERR);
            }
        }

        public function loadSubWidgetAction()
        {
            $request   = $this->getRequest();
            $response  = $this->getResponse();
            $driver    = $request->getQuery('driver');
            $path      = $request->getQuery('path');

            try {
                $driver = Application_Cp_Widget_Manager::Factory($driver);

                $this->view->driver = $driver;
                $driver->setRequest($request);
                $driver->setResponse($response);
                $driver->setView($this->view);
                $driver->run($path);

                $body = $response->getBody();

                $event = $this->addEvent('subwidgetdata.tabs');
                $event->addMemo('body', $body)
                      ->addMemo('driver', $driver->getName());

                try {
                    $script = $driver->getScript($path);

                    if (!$script instanceof Application_Cp_Widget_Script) {
                        throw new Exception('Script is not an instance of Application_Cp_Widget_Script');
                    }

                    $event->addMemo('script', array(
                        'path' => $script->getWebPath(),
                        'name' => $script->getName()
                    ));
                }
                catch (Exception $ex)
                { }

                try {
                    $event->addMemo('css', $driver->getCssPaths($path));
                }
                catch (Exception $ex)
                { }
            }
            catch (Exception $ex) {
                $this->addStatusMessage($ex->getMessage(), Application_Site_Response_StatusMessage::STATUS_ERR);
            }
        }

        public function widgetOrderAction()
        {
            $request    = $this->getRequest();
            $tab_id     = (int) $request->getQuery('tab');
            $column     = (int) $request->getQuery('column');
            $widget_ids = ArrayTools::Clean($request->getPost('widget'), true);

            try {
                $tab = new Application_User_Cp_Tab();
                if (!$tab->loadForClientAndUser($this->client->getId(), $this->user->getId(), $tab_id)) {
                    throw new Exception('Invalid tab');
                }

                $tab->updateColumnOrder($column, $widget_ids);
                $this->addEvent('widgetorder.tabs');
            }
            catch (Exception $ex) {
                $this->addStatusMessage($ex->getMessage(), Application_Site_Response_StatusMessage::STATUS_ERR);
            }
        }

        public function suggestedAction()
        {
            $request = $this->getRequest();
            $tabId   = (int) $request->getQuery('tab');

            try {
                $tab = new Application_User_Cp_Tab();

                if (!$tab->loadForClientAndUser($this->client->getId(), $this->user->getId(), $tabId)) {
                    throw new Exception('Invalid tab');
                }

                foreach ($tab->getSuggestedWidgets() as $widget) {
                    $ret[] = array(
                        'n' => $widget->getName(),
                        't' => (string) $widget,
                        'd' => $widget->getDescription()
                    );
                }
            }
            catch (Exception $ex) {
                $ret = array();
            }

            $this->addEvent('suggestedwidgets.tabs')
                 ->addMemo('widgets', $ret);
        }
    }
?>