<?php
    class Application_Admin_Auth extends Zend_Controller_Plugin_Abstract
    {
        public static $role = 'sysadmin';

        protected $_sysAdminRole;

        private static $_defaultRoles = array('guest');

        private static $_authController = array(
            'module'     => 'default',
            'controller' => 'auth',
            'action'     => 'login'
        );

        protected $_done = false;

        public function __construct()
        {
            $this->auth = Zend_Auth::getInstance();
            $this->auth->setStorage(new Zend_Auth_Storage_Session('auth'));
            Zend_Registry::set('auth', $this->auth);

            $user = new Application_User();
            Zend_Registry::set('user', $user);

            $this->acl = new Zend_Acl();

            $this->acl->add(new Zend_Acl_Resource('default'));
            $this->acl->add(new Zend_Acl_Resource('install'));
            $this->acl->add(new Zend_Acl_Resource('webservices'));

            foreach (self::$_defaultRoles as $role) {
                $this->acl->addRole(new Zend_Acl_Role($role));
            }

            $this->_sysAdminRole = Application_User_Role::GetSysAdminRole();

            $this->acl->addRole(new Zend_Acl_Role($this->_sysAdminRole->title));

            $this->acl->allow('guest', 'default', 'auth');
            $this->acl->allow('guest', 'default', 'error');
            $this->acl->allow(self::$role);
            $this->acl->deny(null, 'install');
            $this->acl->deny(null, 'webservices');
        }

        public function preDispatch(Zend_Controller_Request_Abstract $request)
        {
            if ($this->_done) {
                return;
            }

            $this->_done = true;

            $resource = $request->module;

            if ($resource == 'webservices') {
                $config = array(
                    'accept_schemes' => 'digest',
                    'realm'          => Application_Auth::DIGEST_REALM,
                    'digest_domains' => '/webservices',
                    'nonce_timeout'  => 3600,
                );

                $adapter = new Zend_Auth_Adapter_Http($config);

                $digestResolver = new Application_Auth_DigestResolver($this->_sysAdminRole->getId());

                $adapter->setDigestResolver($digestResolver);
                $adapter->setRequest($this->getRequest());
                $adapter->setResponse($this->getResponse());

                $result = $adapter->authenticate();

                if (!$result->isValid()) {
                    $request->setModuleName('default');
                    $request->setControllerName('error');
                    $request->setActionName('error401');
                    return;
                }

                $user = $digestResolver->getUser();

                if (!$user->saved()) {
                    throw new Exception('Invalid user');
                }

                Zend_Registry::Set('user', $user);
                return;
            }

            $roles = array();

            try {
                if (!$this->auth->hasIdentity()) {
                    throw new Exception('Identity not found');
                }

                $identity = $this->auth->getIdentity();

                if (!is_array($identity) || !array_key_exists('user_id', $identity)) {
                    throw new Exception('Invalid identity data');
                }

                $user = Zend_Registry::get('user');
                $user->load($identity['user_id']);

                if (isset($identity['roles'])) {
                    $roleIds = array_keys($identity['roles']['ids']);

                    if (count($roleIds) > 0) {
                        $r = Application_User_Role::GetRoles(array(
                            'role_ids'      => $roleIds,
                            'directory_ids' => array(null)
                        ));

                        foreach ($r as $role) {
                            $roles[] = (string) $role;
                        }
                    }
                }
            }
            catch (Exception $ex) {
                $roles = array();
            }

            if (count($roles) == 0) {
                $roles = self::$_defaultRoles;
            }

            if (!$this->acl->has($resource)) {
                $resource = null;
            }

            $allowed = false;

            foreach ($roles as $role) {
                if (!$this->acl->hasRole($role)) {
                    continue;
                }

                if ($this->acl->isAllowed($role, $resource, $request->controller)) {
                    $allowed = true;
                    break;
                }
            }

            if (!$allowed) {
                // this line seemed to cause Apache to segfault when source protected
                //Application_Admin_Login::Logout();

                $request->setModuleName(self::$_authController['module']);
                $request->setControllerName(self::$_authController['controller']);
                $request->setActionName(self::$_authController['action']);
            }
        }
    }
?>