<?php
    class Application_Admin_Login extends FormProcessor
    {
        public static $validRoles = array('sysadmin');

        public function __construct($request, $loginUrl, $defaultUrl)
        {
            parent::__construct();

            $this->username = $request->getParam('username');
            if (strlen($this->username) == 0) {
                $this->username = $request->getCookie('lastUsername');
            }

            $this->redirect = $request->getParam('redirect');

            if (strlen($this->redirect) == 0) {
                $this->redirect = $request->getServer('REQUEST_URI');
            }

            if (strlen($this->redirect) == 0) {
                $this->redirect = $defaultUrl;
            }

            if (preg_match('|^' . preg_quote($loginUrl, '|') . '|', $this->redirect)) {
                $this->redirect = $defaultUrl;
            }
        }

        public function process(Zend_Controller_Request_Abstract $request)
        {
            $this->username = $this->sanitize($request->getPost('username'));
            $this->password = $this->sanitize($request->getPost('password'));

            if (strlen($this->username) == 0) {
                $this->addError('username', translate('Required field must not be blank'));
            }
            if (strlen($this->password) == 0) {
                $this->addError('password', translate('Required field must not be blank'));
            }

            if (!$this->hasError()) {
                $auth = Zend_Auth::getInstance();

                $adapter = new Application_Auth_DbAdapter();
                $adapter->setDetails($this->username, $this->password, self::$validRoles);

                $result = $auth->authenticate($adapter);
                if ($result->isValid()) {
                    $identity = $result->getIdentity();
                    setCookie('lastUsername', $identity['username'], time() + 31536000, '/');
                }
                else {
                    $this->addError('username', translate('Your login details were invalid'));
                }
            }

            return !$this->hasError();
        }

        public static function Logout()
        {
            $auth = Zend_Auth::getInstance();
            $auth->clearIdentity();

            Application_Auditor::Record('User logged out');
        }
    }
?>
