<?php
    /**
     * This class is used to record activity that occurs by users who
     * use Recite. This is designed to track everything in the admin site,
     * control panel and on client sites, regardless of the user.
     *
     * @author  Quentin Zervaas
     */
    class Application_Auditor
    {
        private static $_enabled = true;

        /**
         * Write a log message to audit log
         *
         * @param   string|Application_Auditor_Message  $msg    The message to save to the audit log
         */
        public static function Record($msg, $client_id = null)
        {
            // queue up the archive action 1% of the time
            if (rand(1, 100) == 1) {
                Application_Maintenance::Queue('Application_Auditor_Archiver');
            }

            if (!self::$_enabled) {
                return;
            }

            $client_id = (int) $client_id;

            if (!$msg instanceof Application_Auditor_Message) {
                $msg = new Application_Auditor_Message($msg);
            }

            if ($client_id > 0) {
                $msg->setClientId($client_id);
            }

            $entry = new Application_Auditor_Entry();

            $entry->ts_created = $msg->getTimestamp();
            $entry->site       = Application::GetSite();
            $entry->module     = $msg->getModule();
            $entry->message    = (string) $msg;
            $entry->memo       = $msg->getMemo();

            if (strlen($entry->message) == 0)
                return false;

            if (!IS_CLI) {
                $entry->remote_addr     = HTTP::GetRemoteAddr();
                $entry->http_user_agent = coalesce($_SERVER['HTTP_USER_AGENT']);
            }

            try {
                $entry->client_id = $msg->getClientId();
            }
            catch (Exception $ex) {
                try {
                    $client = Application::GetClient();
                    $entry->client_id = $client->getId();
                }
                catch (Exception $ex)
                { }
            }

            try {
                $entry->user_id = $msg->getUserId();
            }
            catch (Exception $ex) {
                if (Zend_Registry::isRegistered('user')) {
                    $user = Zend_Registry::get('user');

                    if ($user->saved())
                        $entry->user_id = $user->getId();
                    else if (Zend_Registry::isRegistered('user_hash'))
                        $entry->user_hash = Zend_Registry::get('user_hash');
                }
            }

            $entry->save(false);

            return true;
        }

        public static function Disable()
        {
            self::Enable(false);
        }

        public static function Enable($flag = true)
        {
            self::$_enabled = (bool) $flag;
        }

        public static function ClearForUser($userIds)
        {
            $userIds = ArrayTools::Clean($userIds, true);

            if (count($userIds) == 0) {
                return;
            }

            $db = Application::GetDb();

            $query = sprintf('update %s set user_id = null where user_id in (%s)', Application_Auditor_Entry::$table, $db->quote($userIds));
            $db->query($query);
        }

        public static function ClearForDirectory($directoryId)
        {
            $query = sprintf('update %s set user_id = null where user_id in (select user_id from %s where directory_id = %d)', Application_Auditor_Entry::$table, Application_User::$table, $directoryId);
            Application::GetDb()->query($query);
        }

        public static function ClearForClient($client_id)
        {
            // @todo    Should this delete the records or just clear them?
            $query = sprintf('update %s set client_id = null where client_id = %d', Application_Auditor_Entry::$table, $client_id);
            Application::GetDb()->query($query);
        }
    }
?>