<?php
    class Application_Auditor_Archiver extends Application_Maintenance_Abstract
    {
        public function __toString()
        {
            return 'Archive application auditor messages';
        }

        public function run($options)
        {
            $config = Application::GetConfig();

            if (!$config->debug->system_audit_archive) {
                return;
            }

            $db = self::GetDb();

            // get all records prior to this date
            $now      = time();
            $date     = mktime(0, 0, 0, date('n', $now) - 1, 1, date('Y', $now));
            $fullpath = self::GetArchiveFullpath($date - 1);

            $select = $db->select();
            $select->from(array('l' => Application_Auditor_Entry::$table), '*')
                   ->where('ts_created < ?', date('Y-m-d 00:00:00', $date))
                   ->order('l.ts_created')
                   ->order('l.entry_id');

            $result = $db->query($select);

            // ensure that there are rows to archive
            if ($result->rowCount() == 0) {
                return;
            }

            // create a csv with the retrieved records

            $fp = fopen($fullpath, 'w');

            $csv = new File_CSV();
            $csv->setStream($fp);

            while ($row = $result->fetch()) {
                $r = new File_CSV_Row();
                $r->add('id', $row->entry_id)
                  ->add('ts_created', $row->ts_created)
                  ->add('user_id', $row->user_id)
                  ->add('user_hash', $row->user_hash)
                  ->add('role_id', $row->role_id)
                  ->add('client_id', $row->client_id)
                  ->add('site', $row->site)
                  ->add('module', $row->module)
                  ->add('http_user_agent', $row->http_user_agent)
                  ->add('remote_addr', $row->remote_addr)
                  ->add('message', $row->message)
                  ->add('memo', $row->memo);

                $csv->add($r);
            }

            fclose($fp);

            // compress csv data

            chdir(dirname($fullpath));

            File_Zip::CreateFromDirectory(basename($fullpath), $fullpath . '.zip');
            @unlink($fullpath);

            // delete archived records
            $where = array(
                $db->quoteInto('ts_created < ?', date('Y-m-d 00:00:00', $date))
            );

            $db->delete(Application_Auditor_Entry::$table, $where);
        }

        public static function GetArchiveFullpath($ts)
        {
            $dir  = Zend_Registry::get('config')->paths->logs;
            $date = date('Y-m', $ts);

            $fullpath   = sprintf('%s/audit-log-%s.csv', $dir, $date);
            $compressed = $fullpath . '.gz';

            if (!file_exists($fullpath) && !file_exists($compressed)) {
                return $fullpath;
            }

            $i = 0;

            while ($i < 100) {
                $fullpath   = sprintf('%s/audit-log-%s-%d.csv', $dir, $date, ++$i);
                $compressed = $fullpath . '.gz';

                if (!file_exists($fullpath) && !file_exists($compressed)) {
                    return $fullpath;
                }
            }

            throw new Application_Maintenance_Exception('Unable to determine archive path');
        }
    }
?>
