<?php
    class Application_Auditor_Message
    {
        protected $_ts;
        protected $_msgs = array();
        protected $_module;
        protected $_client_id;
        protected $_user_id;
        protected $_memo = array();

        public function __construct($msg = null, $module = null)
        {
            $this->setTimestamp(time());
            $this->add($msg);

            $this->setModule($module);
        }

        public function record()
        {
            Application_Auditor::Record($this);
        }

        public function __toString()
        {
            return join("\n", $this->_msgs);
        }

        public function setTimestamp($ts)
        {
            $this->_ts = $ts;
            return $this;
        }

        public function getTimestamp()
        {
            return $this->_ts;
        }

        public function add($msg)
        {
            $msg = trim($msg);

            if (strlen($msg) == 0)
                return;

            $this->_msgs[] = $msg;

            return $this;
        }

        public function getMessages()
        {
            return $this->_msgs;
        }

        public function setModule($module)
        {
            if ($module instanceof Application_Module) {
                $module = $module->getName();
            }

            if (strlen($module) > 0) {
                $this->_module = (string) $module;
            }

            return $this;
        }

        public function getModule()
        {
            return $this->_module;
        }

        public function setClientId($id)
        {
            $this->_client_id = $id;
            return $this;
        }

        public function getClientId()
        {
            if (is_null($this->_client_id))
                throw new Exception('Client ID not set');

            return $this->_client_id;
        }

        public function setUserId($id)
        {
            $this->_user_id = $id;
            return $this;
        }

        public function getUserId()
        {
            if (is_null($this->_user_id))
                throw new Exception('User ID not set');

            return $this->_user_id;
        }

        public function addMemo($key, $value)
        {
            if (!is_null($value)) {
                if (array_key_exists($key, $this->_memo)) {
                    $this->_memo[$key] = array($this->_memo[$key]);
                    $this->_memo[$key][] = $value;
                }
                else
                    $this->_memo[$key] = $value;
            }
            return $this;
        }

        public function getMemo()
        {
            return $this->_memo;
        }
    }
?>