<?php
    class Application_Auditor_Renderer extends DataRenderer
    {
        public function __construct()
        {
            parent::__construct(Application::GetDb());

            $this->getLimitOption()->setDefault(50);

            $this->addOption('order')
                 ->setValues(array('createdd' => 'Date Created (Newest to Oldest)',
                                   'created'  => 'Date Created (Oldest to Newest)'))
                 ->setDefault('createdd');


            $option = $this->addOption('site')->addValue('', 'All');

            foreach (Application::$sites as $k => $v)
                $option->addValue($k, $v);

            $option = $this->addOption('modules')->addValue('', 'All');

            foreach (Application_Module_Manager::GetModules() as $module) {
                $option->addValue($module->getName(), (string) $module);
            }

            $option = $this->addOption('client')->addValue('', 'All');

            foreach (Application_Client::GetClients() as $client) {
                $option->addValue($client->getId(), sprintf('%d. %s', $client->getId(), $client));
            }
        }

        public function getData(&$arr, DataRenderer_Pager $pager)
        {
            $options = $this->getOptions();
            $options['limit']  = (int) $pager->limit;
            $options['offset'] = (int) $pager->offset;

            switch ($this->getSetting('order')) {
                case 'created':
                    $order = array('l.ts_created', 'l.entry_id');
                    break;

                case 'createdd':
                default:
                    $order = array('l.ts_created desc', 'l.entry_id desc');
            }

            $options['order'] = $order;

            $arr = Application_Auditor_Entry::GetEntries($options);
        }

        public function getDataCount()
        {
            return Application_Auditor_Entry::GetEntriesCount($this->getOptions());
        }

        protected function getOptions()
        {
            $ret = array();

            if (strlen($this->getSetting('site')) > 0)
                $ret['sites'] = $this->getSetting('site');

            if (strlen($this->getSetting('modules')) > 0)
                $ret['modules'] = $this->getSetting('modules');

            $client_id = (int) $this->getSetting('client');

            if ($client_id > 0)
                $ret['client_ids'] = $client_id;

            return $ret;
        }
    }
?>
