<?php
    class Application_Auth_DbAdapter implements Zend_Auth_Adapter_Interface
    {
        protected $_tableName = 'users';
        protected $_identityColumn = 'username';
        protected $_identity = null;
        protected $_credentialColumn = 'password';
        protected $_credential = null;
        protected $_directory_id = null;
        protected $_roles = array();

        public function __construct($directory_id = null)
        {
            $this->_directory_id = $directory_id;
        }

        public function setDetails($identity, $credential, $roles = array())
        {
            $this->_identity   = trim($identity);

            $this->_credential = Application_Auth::GetAuthHash($identity, $credential);

            if (!is_array($roles)) {
                $roles = array($roles);
            }

            foreach ($roles as $role) {
                $role = trim($role);
                if (strlen($role) > 0)
                    $this->_roles[$role] = $role;
            }

            return $this;
        }

        public function authenticate()
        {
            $userAlreadyLoggedIn = Zend_Auth::getInstance()->hasIdentity();

            $exception = null;

            $db = Application::GetDb();

            if (is_null($db->getConnection())) {
                $exception = 'Database not connected';
            }
            else if (strlen($this->_identity) == 0) {
                $exception = 'A value for the identity was not provided';
            }
            else if (strlen($this->_credential) == 0) {
                $exception = 'A value for the credential was not provided';
            }

            if (strlen($exception) > 0) {
                throw new Zend_Auth_Adapter_Exception($exception);
            }

            $select = $db->select();
            $select->distinct()
                   ->from(array('u' => $this->_tableName))
                   ->where(sprintf('lower(u.%s) = lower(?)', $this->_identityColumn), $this->_identity)
                   ->where('u.status = ?', Application_User::STATUS_ACTIVE)
                   ->where(sprintf('lower(u.%s) = lower(?)', $this->_credentialColumn), $this->_credential);

            if ($this->_directory_id > 0) {
                $select->where('u.directory_id = ?', $this->_directory_id);
            }
            else {
                $select->where('u.directory_id is null');
            }

            if (count($this->_roles) > 0) {
                $select->joinInner(array('utr' => $this->_tableName . '_to_roles'), 'u.user_id = utr.user_id', array())
                       ->joinInner(array('ur'  => $this->_tableName . '_roles'), 'utr.role_id = ur.role_id', array())
                       ->where('ur.title in (?)', $this->_roles);

                if ($this->_directory_id > 0)
                    $select->where('ur.directory_id = ?', $this->_directory_id);
                else
                    $select->where('u.directory_id is null');
            }

            $result = $db->query($select);

            if ($result->rowCount() > 1) {
                return new Zend_Auth_Result(Zend_Auth_Result::FAILURE_IDENTITY_AMBIGUOUS, $this->_identity, array('Multiple users found'));
            }

            if ($row = $result->fetch()) {
                $user = new Application_User();
                $user->load($row->user_id);

                if ($user->saved()) {
                    if ($userAlreadyLoggedIn) {
                        foreach (Zend_Session::getIterator() as $namespace) {
                            Zend_Session::namespaceUnset($namespace);
                        }

                        Zend_Session::regenerateId();
                    }

                    $user->loginSuccess();
                    $identity = $user->getPublicIdentityData();

                    return new Zend_Auth_Result(Zend_Auth_Result::SUCCESS, $identity, array('Authentication successful'));
                }
            }

            Application_User::LoginFailure($this->_identity);
            return new Zend_Auth_Result(Zend_Auth_Result::FAILURE_CREDENTIAL_INVALID, $this->_identity, array('Invalid login'));
        }
    }
?>