<?php
    class Application_Auth_DigestResolver implements Zend_Auth_Adapter_Http_Resolver_Interface
    {
        protected $_roleIds = array();
        protected $_clientId = -1;
        protected $_user;

        public function __construct($roleIds = array())
        {
            $this->_roleIds = ArrayTools::Clean($roleIds, true);
        }

        public function setClientId($clientId)
        {
            $this->_clientId = (int) $clientId;
            return $this;
        }

        public function resolve($username, $realm)
        {
            $this->_user = null;

            if (empty($username)) {
                /**
                 * @see Zend_Auth_Adapter_Http_Resolver_Exception
                 */
                require_once 'Zend/Auth/Adapter/Http/Resolver/Exception.php';
                throw new Zend_Auth_Adapter_Http_Resolver_Exception('Username is required');
            } else if (!ctype_print($username) || strpos($username, ':') !== false) {
                /**
                 * @see Zend_Auth_Adapter_Http_Resolver_Exception
                 */
                require_once 'Zend/Auth/Adapter/Http/Resolver/Exception.php';
                throw new Zend_Auth_Adapter_Http_Resolver_Exception('Username must consist only of printable characters, '
                                                                  . 'excluding the colon');
            }
            if (empty($realm)) {
                /**
                 * @see Zend_Auth_Adapter_Http_Resolver_Exception
                 */
                require_once 'Zend/Auth/Adapter/Http/Resolver/Exception.php';
                throw new Zend_Auth_Adapter_Http_Resolver_Exception('Realm is required');
            } else if (!ctype_print($realm) || strpos($realm, ':') !== false) {
                /**
                 * @see Zend_Auth_Adapter_Http_Resolver_Exception
                 */
                require_once 'Zend/Auth/Adapter/Http/Resolver/Exception.php';
                throw new Zend_Auth_Adapter_Http_Resolver_Exception('Realm must consist only of printable characters, '
                                                                  . 'excluding the colon.');
            }

            try {
                if (count($this->_roleIds) > 0) {
                    $options = array(
                        'role_ids'  => $this->_roleIds,
                        'limit'     => 1,
                        'usernames' => array($username)
                    );

                    foreach (Application_User::GetUsers($options) as $user) {
                        $this->_user = $user;
                        break;
                    }
                }
                else if ($this->_clientId > 0) {
                    $user = new Application_User();
                    $user->loadControlPanelUserByUsername($this->_clientId, $username, Application_User::STATUS_ACTIVE);

                    if (!$user->saved()) {
                        throw new Exception('Invalid user');
                    }

                    $this->_user = $user;
                }
                else {
                    throw new Exception('Must specify client or roles');
                }
            }
            catch (Exception $ex) {
                return false;
            }

            return $this->_user->password;
        }

        public function getUser()
        {
            if (!$this->_user instanceof Application_User) {
                throw new Exception('User not set');
            }

            return $this->_user;
        }
    }
?>
