<?php
    class Application_Brand_Exporter
    {
        public static function GetXml(Application_Brand $brand)
        {
            if (!$brand->saved()) {
                throw new Exception('Invalid brand');
            }

            $doc = new DOMDocument('1.0', 'utf-8');
            $doc->formatOutput = true;

            $root = $doc->createElement('brand');
            $root->setAttribute('xmlns', 'http://www.recite.com.au/schema/backup/brand');
            $root->setAttribute('version', Application::COMPAT_VERSION);
            $doc->appendChild($root);

            $info = $doc->createElement('brandInfo');
            $root->appendChild($info);

            $info->appendChild($doc->createElement('brandName', (string) $brand));


            $domains = $brand->getDomains();
            if (count($domains) > 0) {
                $domainsNode = $doc->createElement('domains');

                foreach ($domains as $domain) {
                    $domainNode = $doc->createElement('domain');
                    $domainNode->appendChild($doc->createElement('url', $domain->url));
                    $domainsNode->appendChild($domainNode);
                }

                $root->appendChild($domainsNode);
            }

            $modules = Application_Module_Manager::GetModules($brand);
            if (count($modules) > 0) {
                $modulesNode = $doc->createElement('modules');

                foreach ($modules as $module) {
                    $modulesNode->appendChild($doc->createElement('module', $module->getName()));
                }

                $root->appendChild($modulesNode);
            }

            $drivers = Application_Driver_Manager::GetDrivers($brand);
            if (count($drivers) > 0) {
                $driversNode = $doc->createElement('drivers');

                foreach ($drivers as $driver) {
                    $driversNode->appendChild($doc->createElement('driver', $driver->getFullName()));
                }

                $root->appendChild($driversNode);
            }

            $bundles = Application_Settings_Manager::GetBundles($brand, array('all' => true));

            if (count($bundles) > 0) {
                $node = $doc->createElement('settings');

                foreach ($bundles as $bundle) {
                    $bundle->xmlExport($brand, $doc, $node, 'bundle');
                }

                $root->appendChild($node);
            }

            $tabs = Application_Cp_Tab_Manager::GetDefaults($brand);

            if (count($tabs) > 0) {
                $tabsNode = $doc->createElement('tabs');
                $root->appendChild($tabsNode);

                foreach ($tabs as $tab) {
                    $tabNode = $doc->createElement('tab');
                    $tabsNode->appendChild($tabNode);

                    $titleNode = $doc->createElement('title');
                    $titleNode->appendChild($doc->createCDATASection($tab['label']));
                    $tabNode->appendChild($titleNode);

                    for ($i = 0; $i < strlen($tab['layout']); $i++) {
                        $span = (int) $tab['layout'][$i];

                        $columnNode = $doc->createElement('column');
                        $tabNode->appendChild($columnNode);

                        if ($span > 1) {
                            $columnNode->setAttribute('span', $span);
                        }

                        if (array_key_exists($i, $tab['columns'])) {
                            foreach ($tab['columns'][$i] as $widget) {
                                $widgetNode = $doc->createElement('widget');
                                $widgetNode->setAttribute('name', $widget);
                                $columnNode->appendChild($widgetNode);
                            }
                        }
                    }
                }
            }

            return $doc->saveXml();
        }
    }
?>
