<?php
    class Application_Brand_Importer
    {
        public static function ProcessUpload($key)
        {
            if (!array_key_exists($key, $_FILES)) {
                throw new Exception('Unable to find upload information');
            }

            $file = $_FILES[$key];

            if ($file['error'] != UPLOAD_ERR_OK) {
                throw new Exception(sprintf('File not uploaded (code %d', $file['error']));
            }

            return self::ImportFromFile($file['tmp_name']);
        }

        public static function ImportFromFile($fullpath)
        {
            if (!file_exists($fullpath)) {
                throw new Exception('File not found');
            }

            if (!is_file($fullpath)) {
                throw new Exception('Path is not a file');
            }

            if (!is_readable($fullpath)) {
                throw new Exception('Unable to read file');
            }

            $xml = new SimpleXMLElement($fullpath, LIBXML_NOCDATA, true);

            $db = Application::GetDb();
            $db->begin();

            $title = trim($xml->brandInfo->brandName);

            if (strlen($title) == 0) {
                throw new Exception('Brand title not found');
            }

            // create brand

            $brand = new Application_Brand();
            $brand->brand_name = $title;
            $brand->save();

            // set modules and drivers

            $modules = array();

            foreach ($xml->modules->module as $node) {
                $modules[] = trim($node);
            }

            $drivers = array();

            foreach ($xml->drivers->driver as $node) {
                $drivers[] = trim($node);
            }

            Application_Module_Manager::SetModulesAndDrivers($brand, $modules, $drivers);

            // set settings

            foreach ($xml->settings->bundle as $node) {
                Application_Settings_Bundle::XmlImport($brand, $node);
            }


            // set tabs/widgets
            $widgets = array();

            foreach ($xml->tabs->tab as $node) {
                $row = array(
                    'label'   => trim($node->title),
                    'layout'  => '',
                    'columns' => array()
                );

                foreach ($node->column as $columnNode) {
                    if (isset($columnNode['span'])) {
                        $span = max(1, (int) $columnNode);
                    }
                    else {
                        $span = 1;
                    }

                    $row['layout'] .= $span;

                    $column = array();

                    foreach ($columnNode->widget as $widgetNode) {
                        $column[] = trim($widgetNode['name']);
                    }

                    $row['columns'][] = $column;
                }

                $widgets[] = $row;
            }

            Application_Cp_Tab_Manager::SetDefaultTabs($brand, $widgets);

            $db->commit();

            return $brand;
        }
    }
?>
