<?php
    class Application_Brand_Settings_Editor_Admin extends FormProcessor
    {
        public $brand;
        public $bundles;
        public $bundle;
        public $fieldLists = array();

        public function __construct(Application_Brand $brand, $bundleName)
        {
            $this->brand = $brand;

            $options = array(
                'all' => true
            );

            $this->bundles = Application_Settings_Manager::GetBundles($brand, $options);
            $this->bundleName = (string) $bundleName;

            if (!array_key_exists($bundleName, $this->bundles)) {
                return;
            }
            $this->bundle = $this->bundles[$bundleName];

            foreach ($this->bundle->getSections() as $k => $section) {
                $this->fieldLists[$k] = $section->getFieldList();

                foreach ($this->fieldLists[$k] as $field) {
                    if ($brand->profile->has($field->getName())) {
                        $this->set($field->getName(), $brand->getSetting($field->getName()));
                    }
                    else {
                        $this->set($field->getName(), $field->get());
                    }
                }
            }
        }

        public function process(Zend_Controller_Request_Abstract $request)
        {
            foreach ($this->fieldLists as $fieldList) {
                $processor = $fieldList->process($request);

                foreach ($processor->getFieldList() as $field) {
                    $this->brand->setSetting($field->getName(), $field->get());
                    $this->set($field->getName(), $field->get());
                }

                foreach ($processor->getErrors() as $error) {
                    $field = $error['field'];

                    switch ($error['error']) {
                        case Profile_Field_FieldList_FormProcessor::ERROR_EMPTY:
                            $this->addError($field->getName(), sprintf('%s is required', $field->getTitle()));
                            break;
                        case Profile_Field_FieldList_FormProcessor::ERROR_INVALID:
                            $this->addError($field->getName(), sprintf('%s is invalid', $field->getTitle()));
                            $this->set($field->getName(), $error['value']);
                            break;
                        default:
                            $this->addError($field, 'Error in field');
                    }
                }
            }

            if (!$this->hasError()) {
                $this->brand->profile->save();

                $message = new Application_Auditor_Message('Brands: Settings updated');
                $message->addMemo('brand_id', $this->brand->getId())->record();
            }

            return !$this->hasError();
        }

        public function reset()
        {
            Application_Settings_Manager::DeployBundle($this->brand, $this->bundle);
            $this->brand->save();
        }
    }
?>
