<?php
    class Application_Cache
    {
        const LOCK_FILE = 'nocache.lock';
        protected static $_caches = array();

        public static function HandleRequest()
        {
            if (isset($_SERVER['RECITE_DISABLE_CACHE']) && $_SERVER['RECITE_DISABLE_CACHE']) {
                return;
            }

            if ($_SERVER['REQUEST_METHOD'] == 'POST') {
                return;
            }

            require_once('HTTP/URL.php');

            $uri = HTTP_URL::GenerateCurrentUrl();
            $cacheId = md5($uri);

            $cache = self::GetUrlCache();

            $data = $cache->load($cacheId);

            if (!$data) {
                return;
            }

            require_once('HTTP.php');

            if (array_key_exists('fullpath', $data)) {
                $options = array(
                    'filename' => $data['filename'],
                    'mime'     => null
                );

                if (isset($data['mime'])) {
                    $options['mime'] = $data['mime'];
                }

                HTTP::SendFile(
                    $data['fullpath'],
                    $options
                );
            }
            else if (array_key_exists('data', $data)) {
                $options = array(
                    'filename' => $data['filename'],
                    'mime'     => null
                );

                if (isset($data['mime'])) {
                    $options['mime'] = $data['mime'];
                }

                require_once('Application/Client/Site/Response.php');
                $response = new Application_Client_Site_Response();

                HTTP::SendDataAsFile($response, $data['data'], $options);

                $response->sendResponse();
            }

            exit;
        }

        public static function GetCache($name, $frontendDriver = null, Application_Cache_Options $cacheOptions = null)
        {
            if (!array_key_exists($name, self::$_caches) || !is_null($cacheOptions)) {
                try {
                    if (!is_null($cacheOptions) && $cacheOptions->useGlobalCache()) {
                        throw new Exception('Global cache');
                    }

                    $clientId = getClientId();
                }
                catch (Exception $ex) {
                    $clientId = 0;
                }

                $options = array(
                    'frontend' => array(
                        'lifetime' => null,
                        'automatic_serialization' => true
                    ),
                    'backend' => array(
                        'cache_dir' => sprintf('%s/client-%d/%s', self::GetCacheDir(), $clientId, $name)
                    )
                );

                $options['frontend']['caching'] = self::CachingIsEnabled();

                if ($cacheOptions instanceof Application_Cache_Options) {
                    foreach ($cacheOptions->getFrontendOptionsArray() as $k => $v) {
                        $options['frontend'][$k] = $v;
                    }

                    if ($cacheOptions->forceCaching()) {
                        $options['frontend']['caching'] = true;
                    }
                }

                if (!$frontendDriver instanceof Zend_Cache_Core && strlen($frontendDriver) == 0) {
                    $frontendDriver = 'Core';
                }

                require_once('File.php');
                File::CreateDirStructure($options['backend']['cache_dir']);

                if (is_null($cacheOptions)) {
                    self::$_caches[$name] = Zend_Cache::Factory($frontendDriver, 'File', $options['frontend'], $options['backend']);
                }
                else {
                    require_once('Zend/Cache.php');
                    return Zend_Cache::Factory($frontendDriver, 'File', $options['frontend'], $options['backend']);
                }
            }

            return self::$_caches[$name];
        }

        public static function GetUrlCache()
        {
            require_once('Application/Cache/Options.php');
            $options = new Application_Cache_Options();
            $options->setUseGlobalCache(true);

            return self::GetCache('urls', null, $options);
        }

        public static function GetCacheDir()
        {
            require_once('Application.php');

            $ret = Application::getDataDir() . '/tmp/cache';

            if (!file_exists($ret)) {
                File::CreateDirStructure($ret);
            }

            return $ret;
        }

        public static function ClearClientCaches($clientId)
        {
            $clientId = (int) $clientId;

            if ($clientId <= 0) {
                return;
            }

            try {
                self::ClearOpCache();
                self::ClearSmartyTemplates();
                File::RmdirRecursive(sprintf('%s/client-%d', self::GetCacheDir(), $clientId));
            }
            catch (Exception $ex) {
            }

            self::GetUrlCache()->clean(
                Zend_Cache::CLEANING_MODE_MATCHING_ANY_TAG,
                array('client_' . $clientId)
            );
        }

        public static function ClearAllCaches()
        {
            try {
                self::ClearOpCache();
                self::ClearSmartyTemplates();
                File::RmdirRecursive(self::GetCacheDir());
            }
            catch (Exception $ex) {
            }
        }

        public static function ClearOpCache()
        {
            if( function_exists( 'opcache_reset' ) ){
                opcache_reset();
            }
        }

        public static function ClearSmartyTemplates()
        {
            File::RmdirRecursive( Application::getTmpDir() . '/templates_c' );

            try{
                $client_id = Application::GetClientId();
                if( !empty( $client_id ) ){
                    File::EmptyDir( Application_Client_Data::GetNamespacedPathForClient($client_id, 'templates/compiled') );
                }
            }
            catch(Exception $e){}
        }

        public static function CachingIsEnabled()
        {
            $path = self::GetLockFullpath();

            return !file_exists($path);
        }

        public static function EnableCaching($flag = true)
        {
            if (!$flag) {
                return self::DisableCaching();
            }

            @unlink(self::GetLockFullpath());
            self::ClearAllCaches();
        }

        public static function DisableCaching()
        {
            @touch(self::GetLockFullpath());
        }

        protected function GetLockFullpath()
        {
            return Application::GetDataDir() . '/' . self::LOCK_FILE;
        }
    }
?>
