<?php
    class Application_Cache_Options
    {
        const LIFETIME_UNLIMITED = 0;

        protected $_maxLifetime = -1;
        protected $_cancel      = false;
        protected $_force       = false;
        protected $_tags        = array();
        protected $_masterFile  = null;
        protected $_globalCache = false;

        public function __construct()
        { }

        public function setUseGlobalCache($flag)
        {
            $this->_globalCache = (bool) $flag;
            return $this;
        }

        public function useGlobalCache()
        {
            return $this->_globalCache;
        }

        public function setMaxLifetime($seconds)
        {
            $this->_maxLifetime = max(-1, (int) $seconds);
            return $this;
        }

        public function getMaxLifetime()
        {
            if ($this->_maxLifetime < 0) {
                require_once('Application/Cache/Exception.php');
                throw new Application_Cache_Exception('Max lifetime not set');
            }

            return $this->_maxLifetime;
        }

        public function setForceCaching($flag = true)
        {
            $this->_force = (bool) $flag;

            if ($this->_force) {
                $this->_cancel = false;
            }

            return $this;
        }

        public function forceCaching()
        {
            return $this->_force;
        }

        public function cancel()
        {
            $this->_cancel = true;
            $this->_force  = false;
            return true;
        }

        public function isCancel()
        {
            return $this->_cancel;
        }

        public function addTag($tag)
        {
            $this->_tags[] = (string) $tag;
            return $this;
        }

        public function getTags()
        {
            return $this->_tags;
        }

        public function setMasterFile($path)
        {
            if (!file_exists($path)) {
                require_once('Application/Cache/Exception.php');
                throw new Application_Cache_Exception('Master file does not exist');
            }

            $this->_masterFile = $path;
            return $this;
        }

        public function getFrontendOptionsArray()
        {
            $ret = array();

            if ($this->isCancel()) {
                $ret['caching'] = false;
            }

            try {
                $lifetime = $this->getMaxLifetime();

                if ($lifetime == self::LIFETIME_UNLIMITED) {
                    $lifetime = null;
                }
                $ret['lifetime'] = $lifetime;
            }
            catch (Exception $ex) {
            }

            if (strlen($this->_masterFile) > 0) {
                $ret['master_file'] = $this->_masterFile;
            }

            return $ret;
        }
    }
?>
