<?php
    /**
     * This class is used to manage data that is stored with the client
     *
     * @author  Quentin Zervaas <quentin@recite.com.au>
     */
    class Application_Client_Data
    {
        private function __construct()
        { }

        public static function GetNamespacedPathForClient($clientId, $namespace)
        {
            $namespace = trim($namespace);

            if (strlen($namespace) == 0) {
                throw new Exception('Invalid namespace specified');
            }

            if (preg_match('|[^a-z0-9/-]|', $namespace)) {
                throw new Exception('This name space contains invalid characters');
            }

            $fullpath = sprintf(
                '%s/clients/%d/%s',
                Application::GetDataDir(),
                $clientId,
                trim($namespace, '/')
            );

            $fullpath = str_replace('/', DIRECTORY_SEPARATOR, $fullpath);

            File::CreateDirStructure($fullpath);

            if (!file_exists($fullpath)) {
                throw new Exception('Unable to create data location');
            }

            return $fullpath;
        }

        public static function RemoveNamespacedPathForClient($clientId, $namespace)
        {
            $namespace = trim($namespace);

            if (strlen($namespace) == 0) {
                throw new Exception('Invalid namespace specified');
            }

            if (preg_match('|[^a-z0-9/-]|', $namespace)) {
                throw new Exception('This name space contains invalid characters');
            }

            $fullpath = sprintf(
                '%s/clients/%d/%s',
                Application::GetDataDir(),
                $clientId,
                trim($namespace, '/')
            );

            $fullpath = str_replace('/', DIRECTORY_SEPARATOR, $fullpath);

            if (file_exists($fullpath)) {
                File::RmdirRecursive($fullpath);
            }
        }
    }
?>