<?php
    class Application_Client_Editor_Admin extends FormProcessor
    {
        public $brands;
        public $brand;
        public $obj;
        public $dumps;
        public $canAutoInstall = false;
        public $statuses;

        public function __construct($client_id = 0, $brand_id = 0)
        {
            $this->obj = new Application_Client();
            $this->obj->load($client_id);

            if ($this->obj->saved()) {
                $brand_id = $this->obj->get('brand_id');
            }
            else {
                $this->autoconfig = true;
            }

            $this->brands = array();

            foreach (Application_Brand::GetBrands() as $brand) {
                $this->brands[$brand->getId()] = array(
                    'title' => (string) $brand,
                    'dump'  => $brand->getSetting('client.default_site.auto_install')
                );
            }

            if (count($this->brands) == 0) {
                $this->addError('brand_id', 'Please create a branding');
            }

            $this->brand = new Application_Brand();
            $this->brand->load($brand_id);

            if (!$this->obj->saved() && $this->brand->saved()) {
                $this->obj->set('brand_id', $brand_id);
            }

            $this->import = 'brand';

            $this->dumps = array();

            try {
                Application_Loader::AssertCanLoadClass('Module_Backup_Dump');
                $this->canAutoInstall = true;
            }
            catch (Exception $ex) {
            }

            if ($this->canAutoInstall) {
                $dumps = Module_Backup_Dump::GetDumps(Module_Backup_Dump::GetLibraryDumpsPath());
                if( !empty( $dumps ) ){
                    foreach ($dumps as $k => $dump) {
                        $name = '';
                        try {
                            $info = $dump->getDumpInfo();
                            $name = $info->getTitle();
                        }
                        catch (Exception $ex) {
                        }

                        if (strlen($name) == 0) {
                            $name = (string) $dump;
                        }

                        $this->dumps[$dump->getName()] = $name;
                    }
                }
            }

            if ($this->obj->saved()) {
                $this->client_name = $this->obj->client_name;
                $this->status      = $this->obj->status;
            }
            else {
                $this->status = Application_Client::STATUS_ACTIVE;
            }

            $this->statuses = Application_Client::$statuses;
        }

        public function process(Zend_Controller_Request_Abstract $request)
        {
            $this->client_name = Text::sanitize($request->getPost('client_name'));
            if (strlen($this->client_name) > 0) {
                $this->obj->set('client_name', $this->client_name);
            }
            else {
                $this->addError('client_name', 'Enter the client name');
            }

            $this->brand_id = (int) $request->getPost('brand_id');

            if ($this->brand->load($this->brand_id)) {
                $this->obj->set('brand_id', $this->brand_id);
            }
            else {
                $this->addError('brand_id', 'Please select a branding');
            }

            $this->import = $request->getPost('import');

            if ($this->canAutoInstall) {
                if (!in_array($this->import, array('brand', 'force', 'none'))) {
                    $this->import = 'none';
                }

                if ($this->import == 'force') {
                    $this->dump = $request->getPost('dump');
                    if (!array_key_exists($this->dump, $this->dumps)) {
                        $this->addError('dump', 'Please select a site to auto-install');
                    }
                }
            }

            $this->url = $request->getPost('url');

            if ($this->url == 'http://')
                $this->url = '';

            if (strlen($this->url) > 0) {
                $domain = new Application_Client_Domain();

                if (!HTTP_URL::IsValidUrl($this->url)) {
                    $this->addError('url', 'Please enter a valid URL');
                }
                else if ($domain->domainExists($this->url)) {
                    $this->addError('url', 'This URL is already in use');
                }
            }

            $this->status = $request->getPost('status');

            if (!array_key_exists($this->status, $this->statuses)) {
                $this->addError('status', 'Please select the client status');
            }

            $this->autoconfig = (bool) $request->getPost('autoconfig');

            if (!$this->hasError()) {
                $new = !$this->obj->saved();

                $db = Application::GetDb();
                $db->begin();

                if ($this->canAutoInstall) {
                    if ($this->import == 'force') {
                        $this->obj->setAutoInstallSite($this->dump);
                        $this->obj->enableAutoInstall(true);
                    }
                    else if ($this->import == 'brand') {
                        $this->obj->enableAutoInstall(true);
                    }
                    else {
                        $this->obj->enableAutoInstall(false);
                    }
                }
                else {
                    $this->obj->enableAutoInstall(false);
                }

                if ($new && $this->autoconfig) {
                    $this->obj->profile->httpd_conf_generate = true;
                }

                $this->obj->status = $this->status;
                $this->obj->save();

                if ($new) {
                    if (strlen($this->url) > 0) {
                        $this->obj->domainAdd($this->url, false);
                    }
                }

                $db->commit();

                try {
                    $report = $this->obj->autoInstallDefaultSite();

                    if (!$report instanceof Module_Backup_Importer_Report) {
                        throw new Exception('Invalid report');
                    }

                    if ($report->isError()) {
                        $this->addError('dump', $report->getErrors());
                    }
                }
                catch (Exception $ex) {
                }
            }

            if (!$this->hasError()) {
                if ($new && $this->autoconfig) {
                    Application_HttpdConfig::RebuildAll();
                }
            }

            return !$this->hasError();
        }
    }
?>
