<?php
    class Application_Client_Editor_Webservices extends FormProcessor
    {
        public $brand;
        public $obj;
        public $user;

        public function __construct()
        {
            $this->obj = new Application_Client();

            foreach (Application_Brand::GetBrands(array('limit' => 1)) as $brand) {
                $this->brand = $brand;
                break;
            }

            if (!$this->brand instanceof Application_Brand) {
                throw new Exception('No brand available');
            }

            $this->obj->brand_id = $this->brand->getId();
        }

        public function process(Zend_Controller_Request_Abstract $request)
        {
            $brandId = $request->getPost('brand_id');
            if(strlen($brandId) > 0){
                $brand = new Application_Brand();
                $brand->load($brandId);
                if($brand->saved()){
                    $this->brand = $brand;
                    $this->obj->brand_id = $brand->getId();
                }
                else{
                     $this->addError('brand_id', 'Brand does not exist');
                }
            }
            
            $this->client_name = Text::sanitize($request->getPost('client_name'));
            if (strlen($this->client_name) > 0) {
                $this->obj->client_name = $this->client_name;
            }
            else {
                $this->addError('client_name', 'Client name must be specified');
            }

            $autogenUrl = $request->getPost('url_autogen');

            if (strlen($autogenUrl) == 0) {
                $this->urls = $request->getPost('urls');

                if (!is_array($this->urls)) {
                    $this->urls = array();
                }

                foreach ($this->urls as $url) {
                    if (strlen($url) == 0) {
                        continue;
                    }

                    $domain = new Application_Client_Domain();

                    if (!HTTP_URL::IsValidUrl($url)) {
                        $this->addError('url', $url . ' is not a valid URL');
                    }
                    else if ($domain->domainExists($url)) {
                        $this->addError('url', $url . ' is already in use');
                    }
                }
            }
            else {
                $this->urls = array();
            }

            $autoconfig = (bool) $request->getPost('url_autoconf');

            $createUser = (bool) $request->getPost('user_create');

            if ($createUser) {
                $this->user = new Application_User();

                $name  = $request->getPost('user_name');
                $email = $request->getPost('user_email');

                $directory = new Application_User_Directory();

                $fields = $directory->getEditableProfileFields();
                $ret = $fields->getField('email', 'email')->set($email);

                $username = Application_User_UsernameGenerator::Generate($directory, $fields, true);

                $this->user->username = $username;
                $this->user->profile->email = $email;
                $this->user->generateNewPassword();
            }

            if (!$this->hasError()) {
                $db = Application::GetDb();
                $db->begin();

                $this->obj->enableAutoInstall(true);

                if ($autoconfig) {
                    $this->obj->profile->httpd_conf_generate = true;
                }

                $this->obj->save();

                if (strlen($autogenUrl) > 0) {
                    $this->urls = array(
                        str_replace('*', 'd' . $this->obj->getId(), $autogenUrl)
                    );
                }

                $i = 0;
                foreach ($this->urls as $url) {
                    $this->obj->domainAdd($url, false);
                }

                if ($createUser) {
                    $this->user->save();

                    foreach ($this->obj->getRoles() as $role) {
                        $this->user->roleAdd($role->getId());
                        break;
                    }
                }

                $db->commit();

                try {
                    $this->obj->autoInstallDefaultSite();
                }
                catch (Exception $ex) {
                }

                if ($autoconfig) {
                    Application_HttpdConfig::RebuildAll();
                }
            }

            return !$this->hasError();
        }
    }
?>
