<?php
    class Application_Client_Settings_Editor_Admin extends FormProcessor
    {
        public $bundles;
        public $bundle;
        public $fieldLists = array();
        public $client;

        public function __construct(Application_Client $client, $bundleName)
        {
            $options = array(
                'adminMode' => true
            );

            $this->client = $client;
            Application::SetClient($this->client);

            $this->bundles = Application_Settings_Manager::GetBundles($this->client, $options);
            $this->bundleName = (string) $bundleName;

            if (!array_key_exists($bundleName, $this->bundles)) {
                return;
            }
            $this->bundle = $this->bundles[$bundleName];

            foreach ($this->bundle->getSections() as $k => $section) {
                $this->fieldLists[$k] = $section->getFieldList();

                foreach ($this->fieldLists[$k] as $field) {
                    if ($this->client->profile->has($field->getName())) {
                        $this->set($field->getName(), $this->client->getSetting($field->getName()));
                    }
                    else {
                        $this->set($field->getName(), $field->get());
                    }
                }
            }
        }

        public function process(Zend_Controller_Request_Abstract $request)
        {
            foreach ($this->fieldLists as $fieldList) {
                $processor = $fieldList->process($request);

                foreach ($processor->getFieldList() as $field) {
                    $this->client->setSetting($field->getName(), $field->get());
                    $this->set($field->getName(), $field->get());
                }

                foreach ($processor->getErrors() as $error) {
                    $field = $error['field'];

                    switch ($error['error']) {
                        case Profile_Field_FieldList_FormProcessor::ERROR_EMPTY:
                            $this->addError($field->getName(), sprintf('%s is required', $field->getTitle()));
                            break;
                        case Profile_Field_FieldList_FormProcessor::ERROR_INVALID:
                            $this->addError($field->getName(), sprintf('%s is invalid', $field->getTitle()));
                            $this->set($field->getName(), $error['value']);
                            break;
                        default:
                            $this->addError($field, 'Error in field');
                    }
                }
            }

            if (!$this->hasError()) {
                $this->client->profile->save();

                Application_Auditor::Record('Clients: Settings updated', $this->client->getId());
            }

            return !$this->hasError();
        }

        public function reset()
        {
            Application_Settings_Manager::DeployBundle($this->client, $this->bundle);
            $this->client->save();
        }
    }
?>
