<?php
    class Application_Client_Settings_Editor_Cp extends FormProcessor
    {
        public $bundle;
        public $fieldLists = array();
        public $custom = false;
        public $client;

        public function __construct($bundleName)
        {
            $this->client = Application::GetClient();

            $this->bundle = Application_Settings_Manager::GetBundle($this->client, $bundleName);
            $this->custom = $this->bundle->getName() == Application_Settings_Manager::CUSTOM_BUNDLE_NAME;

            foreach ($this->bundle->getSections() as $k => $section) {
                $this->fieldLists[$k] = $section->getFieldList();

                foreach ($this->fieldLists[$k] as $field) {
                    if ($this->custom) {
                        $this->set($field->getName(), $field->get());
                    }
                    else if ($this->client->profile->has($field->getName())) {
                        $this->set($field->getName(), $this->client->getSetting($field->getName()));
                    }
                    else {
                        $this->set($field->getName(), $field->get());
                    }
                }
            }
        }

        public function process(Zend_Controller_Request_Abstract $request)
        {
            foreach ($this->fieldLists as $fieldList) {
                $processor = $fieldList->process($request);

                foreach ($processor->getFieldList() as $field) {
                    if (!$this->custom) {
                        $this->client->setSetting($field->getName(), $field->get());
                    }
                    $this->set($field->getName(), $field->get());
                }

                foreach ($processor->getErrors() as $error) {
                    $field = $error['field'];

                    switch ($error['error']) {
                        case Profile_Field_FieldList_FormProcessor::ERROR_EMPTY:
                            $this->addError($field->getName(), sprintf('%s is required', $field->getTitle()));
                            break;
                        case Profile_Field_FieldList_FormProcessor::ERROR_INVALID:
                            $this->addError($field->getName(), sprintf('%s is invalid', $field->getTitle()));
                            $this->set($field->getName(), $error['value']);
                            break;
                        default:
                            $this->addError($field, 'Error in field');
                    }
                }
            }

            if (!$this->hasError()) {
                if ($this->custom) {
                    foreach ($this->fieldLists as $fieldList) {
                        Application_Client_Settings_Custom::SetFromFieldList($fieldList);
                        break;
                    }
                }
                else {
                    $this->client->profile->save();
                }
                Application_Auditor::Record('Clients: Settings updated', getClientId());
            }

            return !$this->hasError();
        }
    }
?>
