<?php
    class Application_Client_Site_Auth extends Zend_Controller_Plugin_Abstract
    {
        protected $_permissions;
        public $auth;

        public function __construct()
        {
            $this->auth = Zend_Auth::getInstance();
            Zend_Registry::set('auth', $this->auth);
        }

        public function getPermissions()
        {
            if (!$this->_permissions instanceof Module_Pages_Page_Permission) {
                $this->_permissions = new Module_Pages_Page_Permission();
            }

            return $this->_permissions;
        }

        /**
         * This method is used to track the current user
         * for the purpose of knowing when a control panel user is trying to edit
         * their frontend site (i.e. containers/rules) or view draft pages. Because
         * it uses sessions and cookies, we don't use this for assets
         *
         * @author  Quentin Zervaas
         */
        public function preDispatch(Zend_Controller_Request_Abstract $request)
        {
            if ($this->_isNonPersistent()) {
                return;
            }

            $tracker = Application_Auth_UserTracker::getInstance();
            $hash = $tracker->getUserHash($request);

            $request->setParam('user_hash', $hash);
            Zend_Registry::set('user_hash', $hash);

            $clientId = getClientId();

            $roles = $tracker->getControlPanelRoles($clientId, $request);

            if (!empty( $roles['roles'] ) && count($roles['roles']) > 0) {
                $cpUser = new Application_User();
                $cpUser->loadControlPanelUser($clientId, $roles['user']);

                if ($cpUser->saved()) {
                    $permissions = new Application_Cp_Site_Permissions();

                    foreach ($roles['roles'] as $role_id) {
                        $permissions->addUserRole($role_id);
                    }

                    $request->setParam('admin', true);
                    Zend_Registry::set('acl', $permissions);
                    Zend_Registry::set('user_cp', $cpUser);

                    return;
                }
            }

            $request->setParam('admin', false);
        }

        public function routeShutdown(Zend_Controller_Request_Abstract $request)
        {
            $request = $this->getRequest();
            $uri     = $request->getRequestUri();

            if ($this->_isNonPersistent()) {
                $this->auth->setStorage(new Zend_Auth_Storage_NonPersistent());
                return;
            }

            // this is a small hack to allow the session ID to be set in post data, primarily to get around
            // flash's inability to send session cookies as it should
            if (array_key_exists(session_name(), $_POST)) {
                Zend_Session::SetId($_POST[session_name()]);
            }

            // check if the user has set any page permissions at all, and if not then
            // all of this can be skipped (i.e. avoid sessions being started)

            $permissions = $this->getPermissions();

            if (!$permissions->hasProtectedPages() && Module_Forms_Form::GetFormsCount(array('types' => 'user_login')) == 0) {
                return;
            }

            $this->auth->setStorage(new Zend_Auth_Storage_Session('auth'));

            if ($this->auth->hasIdentity()) {
                $identity = $this->auth->getIdentity();
                $user = Zend_Registry::get('user');
                $user->loadRecordForClientDirectory(getClientId(), $identity['user_id']);

                if (!$user->saved()) {
                    $this->auth->clearIdentity();
                    return;
                }

                // check the age of the identity. it's more than 10 minutes old regenerate it.
                // this is primarily for persisten logins which may have old data

                if ($identity['ts_generated'] + 600 < time()) {
                    $this->auth->getStorage()->write($user->getPublicIdentityData());
                }
            }
        }

        private function _isNonPersistent()
        {
            $request = $this->getRequest();

            if ($request->getActionName() != 'backend') {
                return false;
            }

            $path = $request->getUserParam('path');

            // this is a small hack to prevent container image files from caching
            if (preg_match('|^/containers/files/|', $path)) {
                return true;
            }

            if (preg_match('|/+([^/]+)|', $path, $matches)) {
                try {
                    $driver = Module_Backend_Driver_Manager::Factory($matches[1], false);
                    return $driver instanceof Module_Backend_Driver_Abstract_NonPersistent;
                }
                catch (Exception $ex) {
                }
            }

            return false;
        }
    }
?>