<?php
    /**
     * Determine specific details about current request
     *
     * @author  Quentin Zervaas
     */
    class Application_Client_Site_Request extends Zend_Controller_Request_Http
    {
        protected $_parts;
        protected $_usedParts = array();

        public function __construct()
        {
            parent::__construct();

            $this->setParam('remote_addr', HTTP::GetRemoteAddr());
            $this->setParam('remote_host', '');
            $this->setParam('http_user_agent', $this->getServer('HTTP_USER_AGENT'));
            $this->setParam('referer', HTTP::getReferer(null));

            $path = '/' . ltrim(preg_replace('/' . preg_quote(Module_Pages_Page::FILENAME, '/') . '$/', '', $this->getPathInfo()), '/');

            $this->_parts = array_filter(preg_split('|/+|', trim($path, '/')));
        }

        public function getNextUrlPart($urlDecode = true)
        {
            if (count($this->_parts) == 0) {
                throw new Exception('No URL parts remaining');
            }

            $part = array_shift($this->_parts);
            $this->_usedParts[] = $part;

            if ($urlDecode) {
                $part = urldecode($part);
            }

            return $part;
        }

        public function getRemainingUrl($urlDecode = true)
        {
            $parts = array();

            foreach ($this->_parts as $part) {
                $parts[] = $urlDecode ? urldecode($part) : $part;
                $this->_usedParts[] = $part;
            }

            $this->_parts = array();

            return join('/', $parts);
        }

        public function hasUnusedUrlParts()
        {
            return count($this->_parts) > 0;
        }

        public function returnLastUrlPart()
        {
            if (count($this->_usedParts) > 0) {
                $keys = array_keys($this->_usedParts);
                $key  = $keys[count($keys) - 1];
                $part = $this->_usedParts[$key];
                array_unshift($this->_parts, $part);
                unset($this->_usedParts[$key]);
            }
        }

        public function addUrlPart($str)
        {
            $this->_parts[] = $str;
        }

        public function getUsedUrlPartsAsUrl($urlDecode = true)
        {
            $ret = '/' . join('/', $this->_usedParts);

            if ($urlDecode) {
                $ret = urldecode($ret);
            }

            return $ret;
        }
    }
?>
