<?php
    /**
     * Ths class is used to define some generic statuses for content. This is to help
     * with styling of content. Other classes that use statuses in some way can choose
     * either to use these statuses, or they can use their own then map them back to
     * these for display purposes
     *
     * @author  Quentin Zervaas
     * @since   Recite 2.0.0
     */
    class Application_Content_Status
    {
        const LIVE       = 'L';
        const DRAFT      = 'D';
        const APPROVED   = 'A';
        const ACTIVE     = 'AC';
        const INACTIVE   = 'I';
        const UNAPPROVED = 'U';
        const PENDING    = 'P';
        const COMPLETE   = 'C';
        const DISABLED   = 'DI';
        const CANCELLED  = 'CA';
        const DELETED    = 'X';
        const SECURE     = 'S';
        const UNKNOWN    = 'UK';
        const ERROR      = 'E';
        const YES        = 'T';
        const NO         = 'F';

        public static $statuses = array(
            self::LIVE       => array(
                'title' => 'Live'
            ),
            self::DRAFT      => array(
                'title' => 'Draft'
            ),
            self::APPROVED   => array(
                'title' => 'Approved'
            ),
            self::ACTIVE     => array(
                'title' => 'Active'
            ),
            self::INACTIVE   => array(
                'title' => 'Inactive'
            ),
            self::UNAPPROVED => array(
                'title' => 'Unapproved'
            ),
            self::PENDING    => array(
                'title' => 'Pending'
            ),
            self::COMPLETE   => array(
                'title' => 'Complete'
            ),
            self::DISABLED   => array(
                'title' => 'Disabled'
            ),
            self::CANCELLED  => array(
                'title' => 'Cancelled'
            ),
            self::DELETED    => array(
                'title' => 'Deleted'
            ),
            self::SECURE     => array(
                'title' => 'Secure'
            ),
            self::UNKNOWN => array(
                'title' => 'Unknown'
            ),
            self::ERROR => array(
                'title' => 'Error'
            ),
            self::YES => array(
                'title' => 'Success'
            ),
            self::NO => array(
                'title' => 'Failure'
            )
        );

        private static $_reflection;

        private function __construct()
        {
        }

        public static function GetStatus($code)
        {
            if (!array_key_exists($code, self::$statuses)) {
                $reflection = self::_GetReflection();

                if (!$reflection->hasConstant($code)) {
                    throw new Exception('Invalid status code');
                }

                $code = $reflection->getConstant($code);
            }

            return array(
                'code'  => $code,
                'title' => self::$statuses[$code]['title']
            );
        }

        private static function _GetReflection()
        {
            if (!self::$_reflection instanceof ReflectionClass) {
                self::$_reflection = new ReflectionClass(__CLASS__);
            }

            return self::$_reflection;
        }
    }
?>