<?php
    require_once('Application/Defines.php');
    require_once('Application/Loader.php');

    Application_Loader::Startup();

    if (Application_SystemStatus::InMaintenanceMode()) {
        Application_Cp_Site_Maintenance::SendMaintenanceMessage();
    }

    try {
        Application::startup(Application::SITE_CP);
    }
    catch (Exception $ex) {
        Application_Cp_Site_Maintenance::SendStartupErrorMessage($ex);
        exit;
    }


    $brand = new Application_Brand();
    $brand->loadRecordForDomain();

    if (!$brand->saved()) {
        Application_Cp_Site_Maintenance::SendNoBrandErrorMessage();
        exit;
    }

    Zend_Registry::set('brand', $brand);
    Zend_Registry::set('user', new Application_User());
    Zend_Registry::set('client', new Application_Client());

    $controller = Zend_Controller_Front::getInstance();
    $controller->setRouter(new Application_Cp_Site_Router());
    $controller->setRequest(new Application_Cp_Site_Request());
    $controller->setResponse(new Application_Cp_Site_Response());
    $controller->setDispatcher(new Application_Cp_Site_Dispatcher());
    $controller->setControllerDirectory(Application::getBaseDir() . '/application/cp/controllers');

    $controller->registerPlugin(new Application_Cp_Site_Auth());
    $controller->registerPlugin(new Application_Cp_Site_Startup());
    $controller->registerPlugin(new Application_Site_Profiler());

    $vr = new Zend_Controller_Action_Helper_ViewRenderer();
    $vr->setViewSuffix('tpl');
    $vr->setView(new Application_View_Cp());
    $vr->setViewScriptPathSpec('module|:module|:controller/:action.:suffix');
    $vr->setViewBasePathSpec('module|:module|:controller/:action.:suffix');
    Zend_Controller_Action_HelperBroker::addHelper($vr);

    $controller->dispatch();
?>