<?php
    /**
     * This class is used to hold details about a file that is to be served in the Control Panel
     *
     * @author  Quentin Zervaas <quentin@recite.com.au>
     * @since   Recite 2.0.0
     */
    class Application_Cp_File
    {
        protected $_src;
        protected $_path;

        public function __construct($src = null)
        {
            if (!is_null($src)) {
                if ($src instanceof Application_Module_Interface) {
                    ; // ok
                }
                else if ($src instanceof Application_Driver_Interface) {
                    ; // ok
                }
                else if ($src instanceof Application_Cp_Widget_Interface) {
                    ; // ok
                }
                else {
                    throw new Exception('Invalid file source');
                }

                $this->_src = $src;
            }
        }

        public function setPath($path)
        {
            $this->_path = $path;
            return $this;
        }

        public function getFilesystemPath()
        {
            if (strlen($this->_path) == 0) {
                throw new Exception('Path not set');
            }

            if ($this->_src instanceof Application_Module_Interface) {
                $ret = sprintf('%s/files/%s', $this->_src->getPath(), trim($this->_path, '/'));
            }
            else if ($this->_src instanceof Application_Driver_Interface) {
                $ret = sprintf('%s/files/%s', $this->_src->getDriverDirectory(), trim($this->_path, '/'));
            }
            else if ($this->_src instanceof Application_Cp_Widget_Interface) {
                $ret = sprintf('%s/files/%s', $this->_src->getPath(), trim($this->_path, '/'));
            }
            else {
                $ret = sprintf('%s/cp/htdocs/files/%s', Application::GetApplicationDir(), trim($this->_path, '/'));
            }

            if (!file_exists($ret)) {
                throw new Exception('File does not exist');
            }

            return $ret;
        }

        public function getWebPath()
        {
            $this->getFilesystemPath();

            if ($this->_src instanceof Application_Module_Interface) {
                $ret = sprintf('/files/module/%s/%s', $this->_src->getName(), trim($this->_path, '/'));
            }
            else if ($this->_src instanceof Application_Driver_Interface) {
                $ret = sprintf('/files/driver/%s/%s', $this->_src->getFullName(), trim($this->_path, '/'));
            }
            else if ($this->_src instanceof Application_Cp_Widget_Interface) {
                $ret = sprintf('/files/widget/%s/%s', $this->_src->getName(), trim($this->_path, '/'));
            }
            else {
                $ret = sprintf('%s/cp/htdocs/files/%s', Application::GetApplicationDir(), trim($this->_path, '/'));
            }

            return $ret;
        }
    }
?>
