<?php
    class Application_Cp_Site_Auth extends Zend_Controller_Plugin_Abstract
    {
        private $_authController = array(
            'module'     => 'default',
            'controller' => 'auth',
            'action'     => 'login'
        );

        private $_chooseClientController = array(
            'module'     => 'default',
            'controller' => 'auth',
            'action'     => 'choose-client'
        );

        public function __construct()
        {
            $this->auth = Zend_Auth::getInstance();
        }

        public function routeShutdown(Zend_Controller_Request_Abstract $request)
        {
            // special handler so session cache limiters aren't set for files,
            // since we're not authenticating for files anyway
            if ($this->_isNonPersistent()) {
                $this->auth->setStorage(new Zend_Auth_Storage_NonPersistent());
                return;
            }

            // this is a small hack to allow the session ID to be set in post data, primarily to get around
            // flash's inability to send session cookies as it should
            if (array_key_exists(session_name(), $_POST)) {
                Zend_Session::SetId($_POST[session_name()]);
            }

            $this->auth->setStorage(new Zend_Auth_Storage_Session('auth'));

            try {
                if (!$this->auth->hasIdentity()) {
                    throw new Exception('Identity not found');
                }

                $identity = $this->auth->getIdentity();

                if (!is_array($identity) || !array_key_exists('user_id', $identity)) {
                    throw new Exception('Invalid identity data');
                }

                $user = Zend_Registry::get('user');
                $user->load($identity['user_id']);

                if (isset($identity['client_id'])) {
                    Application::SetClientId($identity['client_id']);
                }

                try {
                    $client = Application::GetClient();

                    if ($user->saved()) {
                        $tracker = Application_Auth_UserTracker::getInstance();
                        $tracker->registerSessionActivity($user->getId(), $client->getId());
                    }
                }
                catch (Exception $ex) {
                }
            }
            catch (Exception $ex) {
                $this->auth->clearIdentity();
            }
        }

        private function _isNonPersistent()
        {
            $request = $this->getRequest();

            return $request->getModuleName() == 'default' && $request->getControllerName() == 'files';
        }

        public function preDispatch(Zend_Controller_Request_Abstract $request)
        {
            if ($this->_isNonPersistent()) {
                return;
            }

            // allow them in if they've authenticated. their specific
            // permissions will be handled throughout the application
            if ($this->auth->hasIdentity()) {

                $user = Application::GetCpUser();

                try {
                    $client = Application::GetClient();
                }
                catch (Exception $ex) {
                    $client = new Application_Client();
                }

                if (!$user->saved()) {
                    $this->auth->clearIdentity();
                }
                else if (!$client->saved()) {
                    $request->setModuleName($this->_chooseClientController['module']);
                    $request->setControllerName($this->_chooseClientController['controller']);
                    $request->setActionName($this->_chooseClientController['action']);
                }

                if ($user->saved() && $client->saved()) {
                    $identity = $this->auth->getIdentity();

                    $permissions = self::GetPermissionHandler($client, array_keys($identity['roles']['ids']));
                    Zend_Registry::set('acl', $permissions);
                }

                return;
            }

            // handle non-authenticated users

            $acl = new Zend_Acl();
            $acl->add(new Zend_Acl_Resource('auth'));
            $acl->add(new Zend_Acl_Resource('files'));

            $acl->deny();

            // open up methods for authenticating
            $acl->allow(null, 'auth');
            $acl->allow(null, 'files');

            $resource = $request->controller;

            if (!$acl->has($resource))
                $resource = null;

            if (!$acl->isAllowed(null, $resource, $request->action)) {
                $request->setModuleName($this->_authController['module']);
                $request->setControllerName($this->_authController['controller']);
                $request->setActionName($this->_authController['action']);
            }
        }

        public static function GetPermissionHandler($client, $roleIds)
        {
            $permissions = new Application_Cp_Site_Permissions($client);

            $roleIds = ArrayTools::Clean($roleIds, true);

            if (count($roleIds) > 0) {
                $options = array(
                    'directory_ids' => array(null),
                    'client_ids'    => $client->getId(),
                    'role_ids'      => $roleIds
                );

                foreach (Application_User_Role::GetRoles($options) as $role) {
                    $permissions->addUserRole($role->getId());
                }
            }

            return $permissions;
        }
    }
?>