<?php
    class Application_Cp_Site_Permissions
    {
        protected $_acl;
        protected $_role_ids = array();

        public function __construct()
        {
            $this->_acl = new Zend_Acl();
            $this->_createRoles();
        }

        public function addUserRole($role_id)
        {
            $role_id = (int) $role_id;

            if ($role_id > 0) {
                $this->_role_ids[$role_id] = $role_id;
            }
        }

        public function getUserRoles()
        {
            return array_keys($this->_role_ids);
        }

        public function isAllowed($permission)
        {
            if (!$this->_acl->has($permission)) {
                $this->_createResource($permission);
            }

            foreach ($this->_role_ids as $role_id) {
                if (!$this->_acl->hasRole($role_id)) {
                    continue;
                }

                if ($this->_acl->isAllowed($role_id, $permission)) {
                    return true;
                }
            }

            return false;
        }

        protected function _createRoles()
        {
            $options = array(
                'order'         => 'r.nleft',
                'directory_ids' => array(null),
                'client_ids'    => getClientId()
            );

            foreach (Application_User_Role::GetRoles($options) as $role) {
                $parent_id = (int) $role->parent_id;

                if (!$this->_acl->hasRole($parent_id))
                    $parent_id = null;

                $this->_acl->addRole(new Zend_Acl_Role($role->getId()), $parent_id);
            }
        }

        protected function _createResource($name)
        {
            $permission = new Application_User_Permissions_Permission();
            if (!$permission->loadForDirectoryClientTitle(null, getClientId(), $name)) {
                throw new Application_User_Permissions_Exception(sprintf('Invalid permission specified (%s)', $name));
            }

            $resource = new Zend_Acl_Resource($name);
            $this->_acl->add($resource);

            if ($permission->allow)
                $this->_acl->allow(null, $resource);
            else
                $this->_acl->deny(null, $resource);

            $permissions = $permission->getRoles();

            foreach ($permissions['defaults'] as $role_id => $type) {
                if (!$this->_acl->hasRole($role_id))
                    continue;

                if ($type == Application_User_Permissions_Permission::TYPE_ALLOW) {
                    $this->_acl->allow($role_id, $resource);
                }
                else if ($type == Application_User_Permissions_Permission::TYPE_DENY)
                    $this->_acl->deny($role_id, $resource);
            }

            return $resource;
        }
    }
?>