<?php
    class Application_Cp_Tab_Manager
    {
        const TABLE_DEFAULTS_BRANDS = 'brands_cp_tabs';

        public static $_customLayouts = array('1', '11', '12', '21', '111', '121', '1111');
        public static $_defaultLayout = '111';

        private function __construct()
        { }

        public static function GetCustomLayouts()
        {
            $ret = array();

            foreach (self::$_customLayouts as $layout) {
                $cols = strlen($layout);

                if (!array_key_exists($cols, $ret)) {
                    $ret[$cols] = array();
                }

                $ret[$cols][] = $layout;
            }

            return $ret;
        }

        public static function GetDefaultLayout()
        {
            return self::$_defaultLayout;
        }

        public static function IsValidLayout($layout)
        {
            return in_array($layout, self::$_customLayouts);
        }

        public static function GetTemplates($checkAcls = false)
        {
            $ret = array();

            foreach (Application_Cp_Tab_Template::FromFile(Application::GetApplicationDir() . '/settings/tabs.xml') as $template) {
                $ret[] = $template;
            }

            if ($checkAcls) {
                $modules = Application_Module_Manager::GetModules(Application::GetClient());
            }
            else {
                $modules = Application_Module_Manager::GetModules();
            }

            foreach ($modules as $module) {
                if ($checkAcls) {
                    if (!Application_User_Permissions_Query_Item::BuildAndCheck('module:' . $module->getName())) {
                        continue;
                    }
                }

                if ($module instanceof Application_Module_Legacy_Abstract && !$module->isHidden()) {
                    $ret[] = Application_Cp_Tab_Template::FromLegacyModule($module);
                }
                else {
                    $fullpath = sprintf('%s/settings/tabs.xml', $module->getPath());

                    try {
                        foreach (Application_Cp_Tab_Template::FromFile($fullpath) as $template) {
                            $ret[] = $template;
                        }
                    }
                    catch (Exception $ex) {
                    }
                }
            }

            usort($ret, array(__CLASS__, '__sort'));

            return $ret;
        }

        public static function __sort($a, $b)
        {
            return strcmp(strtolower($a), strtolower($b));
        }

        public static function DeleteDefaultTabs($item)
        {
            if (!$item instanceof Application_Brand) {
                throw new Exception('Invalid item to process');
            }

            $table  = self::TABLE_DEFAULTS_BRANDS;
            $column = 'brand_id';

            $db = Application::GetDb();

            $where = array(
                $db->quoteInto($column . ' = ?', $item->getId())
            );

            $db->delete($table, $where);
        }

        public static function GetDefaults($item)
        {
            if (!$item instanceof Application_Brand) {
                throw new Exception('Invalid item to process');
            }

            $table  = self::TABLE_DEFAULTS_BRANDS;
            $column = 'brand_id';

            $db = Application::GetDb();
            $select = $db->select();
            $select->from(array($table), 'data')
                   ->where($column . ' = ?', $item->getId())
                   ->order('ranking');

            $ret = array();

            foreach ($db->fetchCol($select) as $row) {
                $ret[] = json_decode($row, true);
            }

            return $ret;
        }

        public static function ProcessDefaultTabs($item, $request)
        {
            if (!$item instanceof Application_Brand) {
                throw new Exception('Invalid item to process');
            }

            $post = $request->getPost();

            if (!array_key_exists('label', $post) || !is_array($post['label'])) {
                throw new Exception('Invalid widget data');
            }

            $vals = array();

            foreach ($post['label'] as $tabId => $label) {
                if (strlen($label) == 0) {
                    throw new Exception('Label not specified');
                }

                $layout = $post['layout'][$tabId];

                if (!self::IsValidLayout($layout)) {
                    throw new Exception('Invalid tab layout');
                }

                $tab = array(
                    'label'   => $label,
                    'layout'  => $layout,
                    'columns' => array()
                );

                for ($i = 0; $i < strlen($layout); $i++) {
                    $tab['columns'][$i] = array();
                }

                foreach ($post['widget'][$tabId] as $col => $widgets) {
                    if (!array_key_exists($col, $tab['columns'])) {
                        throw new Exception('Invalid widget data');
                    }

                    foreach ($widgets as $name) {
                        $widget = Application_Cp_Widget_Manager::Factory($name);

                        $tab['columns'][$col][] = $widget->getName();
                    }
                }

                $vals[] = $tab;
            }

            self::SetDefaultTabs($item, $vals);
        }

        public static function SetDefaultTabs($item, $data)
        {
            if (!$item instanceof Application_Brand) {
                throw new Exception('Invalid item to process');
            }

            $table  = self::TABLE_DEFAULTS_BRANDS;
            $column = 'brand_id';

            $db = Application::GetDb();
            $db->begin();

            self::DeleteDefaultTabs($item);

            $ranking = 0;

            foreach ($data as $row) {
                $vals = array(
                    $column   => $item->getId(),
                    'ranking' => ++$ranking,
                    'data'    => json_encode($row)
                );

                $db->insert($table, $vals);
            }

            $db->commit();
        }
    }
?>
