<?php
    class Application_Cp_Tab_RequestHandler
    {
        protected static $_view;
        protected static $_request;
        protected static $_response;

        public static function GetFullState($selected = null)
        {
            $options  = array(
                'client_ids' => getClientId(),
                'user_ids'   => getUserId()
            );

            $tabs = Application_User_Cp_Tab::GetTabs($options);

            if (count($tabs) > 0 && !array_key_exists($selected, $tabs)) {
                foreach ($tabs as $tab) {
                    $selected = $tab->getId();
                    break;
                }
            }

            $json = array();

            foreach ($tabs as $tab) {
                $row = array(
                    'id'     => $tab->getId(),
                    'title'  => (string) $tab,
                    'layout' => $tab->getLayout()
                );

                if ($tab->getId() == $selected) {
                    $row['selected'] = true;
                    $row['columns']  = array();

                    foreach ($tab->getColumns() as $column) {
                        $col = array(
                            'span'    => $column->getSpan(),
                            'widgets' => array()
                        );

                        $request = self::GetRequest();

                        foreach ($column->getWidgets() as $widget) {
                            try {
                                $col['widgets'][] = self::GetWidget($widget);
                            }
                            catch (Exception $ex) {
                                $widget->delete();
                                continue;
                            }
                        }

                        $row['columns'][] = $col;
                    }
                }
                else {
                    $row['selected'] = false;
                }

                $json[] = $row;
            }

            return $json;
        }

        public static function GetTab(Application_User_Cp_Tab $tab)
        {
            $ret = array(
                'id'       => $tab->getId(),
                'title'    => (string) $tab,
                'selected' => true,
                'layout'   => $tab->getLayout(),
                'columns'  => array()
            );

            foreach ($tab->getColumns() as $column) {
                $col = array(
                    'span'    => $column->getSpan(),
                    'widgets' => array()
                );

                $request = self::GetRequest();

                foreach ($column->getWidgets() as $widget) {
                    try {
                        $col['widgets'][] = self::GetWidget($widget);
                    }
                    catch (Exception $ex) {
                        $widget->delete();
                        continue;
                    }
                }

                $ret['columns'][] = $col;
            }

            return $ret;
        }

        public static function GetWidget(Application_User_Cp_Widget $widget)
        {
            $driver = $widget->getDriver();
            Application_Cp_Widget_Manager::AssertDriverCanBeUsed($driver);

            $ret = self::GetDriver($driver, $widget->getParams());
            $ret['id']     = $widget->getId();
            $ret['params'] = $widget->getParams();

            return $ret;
        }

        public static function GetDriver($driver, $params = null)
        {
            $request  = self::GetRequest();
            $response = self::GetResponse();

            if (is_array($params)) {
                $_POST = array();

                try {
                    $request->setPost($params);
                }
                catch (Exception $ex) {
                }
            }

            $driver = Application_Cp_Widget_Manager::Factory($driver);

            $ret = array(
                'driver' => $driver->getName()
            );

            $view = self::GetView();
            $view->resetContainerId();

            $view->driver = $driver;


            $driver->setRequest($request);
            $driver->setResponse($response);
            $driver->setView($view);
            $driver->run();

            $ret['body'] = $response->getBody();

            try {
                $script = $driver->getScript();

                $ret['script'] = array(
                    'path' => $script->getWebPath(),
                    'name' => $script->getName()
                );
            }
            catch (Exception $ex) {
            }

            try {
                $ret['css'] = $driver->getCssPaths();
            }
            catch (Exception $ex) {
            }

            return $ret;
        }

        public static function SetView(Application_View_Cp $view)
        {
            self::$_view = $view;
        }

        public static function GetView()
        {
            if (!self::$_view instanceof Application_View_Cp) {
                throw new Exception('View not set');
            }

            return self::$_view;
        }

        public static function SetRequest(Zend_Controller_Request_Abstract $request)
        {
            self::$_request = $request;
        }

        public static function GetRequest()
        {
            if (!self::$_request instanceof Zend_Controller_Request_Abstract) {
                throw new Exception('Request not set');
            }

            return self::$_request;
        }

        public static function SetResponse(Zend_Controller_Response_Abstract $response)
        {
            self::$_response = $response;
        }

        public static function GetResponse()
        {
            if (!self::$_response instanceof Zend_Controller_Response_Abstract) {
                throw new Exception('Response not set');
            }

            return self::$_response;
        }

        public static function SetWidgetParams()
        {
            $request = self::GetRequest();

            $widgets = array();

            foreach ($request->getPost() as $k => $v) {
                if (!preg_match('/^(\d+)_(.*)$/', $k, $matches)) {
                    continue;
                }

                $widgetId = (int) $matches[1];

                if (!array_key_exists($widgetId, $widgets)) {
                    $widgets[$widgetId] = array();
                }

                $key = $matches[2];
                $widgets[$widgetId][$key] = $v;
            }

            $db = Application::GetDb();
            $db->begin();

            foreach ($widgets as $widgetId => $params) {
                $widget = new Application_User_Cp_Widget();
                $widget->loadWidget(getClientId(), getUserId(), $widgetId);

                if (!$widget->saved()) {
                    continue;
                }

                foreach ($params as $k => $v) {
                    $widget->setParam($k, $v);
                }

                $widget->save();
            }

            $db->commit();
        }

        public static function SendTabJs(Application_User_Cp_Tab $tab)
        {
            $files = array();

            foreach ($tab->getWidgetDrivers() as $driver) {
                try {
                    $files[] = $driver->getScript();
                }
                catch (Exception $ex) {

                }
            }

            $paths = array();

            foreach ($files as $file) {
                try {
                    $paths[] = $file->getFilesystemPath();
                }
                catch (Exception $ex) {
                }
            }

            $paths = array_unique($paths);

            ob_end_clean();
            header('Content-type: text/javascript');

            foreach ($paths as $path) {
                readfile($path);
            }

            exit;
        }
    }
?>