<?php
    class Application_Cp_Tab_Template
    {
        protected $_title;
        protected $_legacyModule;
        protected $_columns = array();

        public function __construct($title)
        {
            $this->_title = (string) $title;
        }

        public function __toString()
        {
            return (string) $this->_title;
        }

        public function add(Application_Cp_Tab_Template_Column $column)
        {
            $this->_columns[] = $column;
            return $this;
        }

        public function getColumns()
        {
            return $this->_columns;
        }

        public function getLayout()
        {
            if (count($this->_columns) == 0) {
                throw new Exception('No tabs found in template');
            }

            $ret = '';

            foreach ($this->_columns as $column) {
                $ret .= $column->getSpan();
            }

            return $ret;
        }

        public function setLegacyModule($module)
        {
            $this->_legacyModule = $module;
            return $this;
        }

        public function isLegacyModule()
        {
            return $this->_legacyModule instanceof Application_Module_Legacy_Abstract;
        }

        public function getLegacyModule()
        {
            if (!$this->isLegacyModule()) {
                throw new Exception('Module not set');
            }

            return $this->_legacyModule;
        }

        public function toJson()
        {
            $ret = array(
                'title'   => (string) $this,
                'layout'  => $this->getLayout(),
                'columns' => array()
            );

            foreach ($this->_columns as $column) {
                $row = array();

                foreach ($column->getWidgets() as $widget) {
                    $row[$widget->getName()] = (string) $widget;
                }

                $ret['columns'][] = $row;
            }

            return json_encode($ret);
        }

        public static function FromLegacyModule(Application_Module_Legacy_Abstract $module)
        {
            $ret = new self($module);
            $ret->setLegacyModule($module);

            return $ret;
        }

        public static function FromFile($fullpath)
        {
            if (strlen($fullpath) == 0) {
                throw new Exception('Path not set');
            }

            if (!file_exists($fullpath)) {
                throw new Exception('Path does not exist');
            }

            if (!is_file($fullpath)) {
                throw new Exception('Path is not a file');
            }

            if (!is_readable($fullpath)) {
                throw new Exception('Path is not readable');
            }

            $xml = new SimpleXmlElement($fullpath, null, true);

            $ret = array();

            foreach ($xml->tab as $node) {
                $ret[] = self::FromXmlNode($node);
            }

            return $ret;
        }

        public static function FromXmlNode($node)
        {
            $title = trim($node->title);

            if (strlen($title) == 0) {
                throw new Exception('Title not set');
            }

            $ret = new self($title);

            foreach ($node->column as $c) {
                if (isset($c['span'])) {
                    $span = (int) $c['span'];
                }
                else {
                    $span = 1;
                }

                $column = new Application_Cp_Tab_Template_Column($span);

                foreach ($c->widget as $w) {
                    try {
                        $name = sprintf('CpWidget_%s', $w['name']);

                        $driver = Application_Cp_Widget_Manager::Factory($name);

                        $column->addWidget(Application_Cp_Widget_Manager::Factory($name));
                    }
                    catch (Exception $ex) {
                    }
                }

                $ret->add($column);
            }

            return $ret;
        }
    }
?>