<?php
    abstract class Application_Cp_Theme_Abstract implements Application_Cp_Theme_Interface
    {
        const USE_LEGACY_CORE_PATH = false;

        public function getName()
        {
            return preg_replace('/^Theme_/', '', get_class($this));
        }

        public function getApplicationTitle()
        {
            return Application::NAME;
        }

        public function getApplicationVersion()
        {
            return Application::VERSION;
        }

        public function getApplicationUrl()
        {
            return 'http://www.recite.com.au';
        }

        public function getDocumentationUrl()
        {
            return 'http://docs.recite.com.au/?s=cp&v=' . Application::VERSION;
        }

        final public function getCoreFilesWebPath()
        {
            if (self::USE_LEGACY_CORE_PATH) {
                return '/files/v2';
            }
            return '/files/v3';
        }

        final public function getCoreFilesSystemPath()
        {
            if (self::USE_LEGACY_CORE_PATH) {
                return sprintf('%s/cp/htdocs/files/v2', Application::getApplicationDir());
            }
            return sprintf('%s/cp/htdocs/files/v3', Application::getApplicationDir());
        }

        final public function get3rdPartyFilesWebPath()
        {
            return '/files/3rdparty';
        }

        final public function get3rdPartyFilesSystemPath()
        {
            return sprintf('%s/cp/htdocs/files/3rdparty', Application::getApplicationDir());
        }

        public function getCssWebPath($filename)
        {
            return sprintf('/files/theme/%s/%s.css', $this->getName(), $filename);
        }

        public function _getThemeCssFullpath($filename)
        {
            $fullpath = sprintf('%s/components/%s.css', $this->getCoreFilesSystemPath(), preg_replace('/[^a-z0-9]/', '', $filename));

            if (!file_exists($fullpath)) {
                throw new Application_Cp_Theme_Exception('Invalid file');
            }

            return $fullpath;
        }

        public function getThemeCssMtime($filename)
        {
            return filemtime($this->_getThemeCssFullpath($filename));
        }

        public function getThemeCss($filename)
        {
            $fullpath = $this->_getThemeCssFullpath($filename);

            $sections = array(
                $this->getGlobalTheme(),
                $this->getHeaderTheme(),
                $this->getWidgetTheme(),
                $this->getDialogTheme()
            );


            $placeholders = array();

            foreach ($sections as $section) {
                if ($section instanceof Application_Cp_Theme_Section) {
                    $placeholders = array_merge($placeholders, $section->getPlaceHolderValues());
                }
            }
            var_dump($fullpath,$placeholders);

            return strtr(
                file_get_contents($fullpath),
                $placeholders
            );
        }


        /**
         * Get theme for global styles
         *
         * @return  Application_Cp_Theme_Section_Global
         */
        abstract public function getGlobalTheme();


        /**
         * Get theme for the header
         *
         * @return  Application_Cp_Theme_Section_Header
         */
        abstract public function getHeaderTheme();


        /**
         * Get theme for widgets
         *
         * @return  Application_Cp_Theme_Section_Widget
         */
        abstract public function getWidgetTheme();


        /**
         * Get theme for dialogs
         *
         * @return  Application_Cp_Theme_Section_Dialog
         */
        abstract public function getDialogTheme();
    }
?>
