<?php
    /**
     * The base interface that all control panel themes must implement
     *
     * @package Application_Cp_Theme
     * @author  Quentin Zervaas
     */
    interface Application_Cp_Theme_Interface
    {
        /**
         * Returns the title of the theme
         *
         * @return  string
         */
        public function __toString();


        /**
         * Returns the internal name of the theme
         *
         * @return  string
         */
        public function getName();


        /**
         * Returns the title of the application, not including version number
         *
         * @return  string
         */
        public function getApplicationTitle();


        /**
         * Returns the version of the application
         *
         * @return  string
         */
        public function getApplicationVersion();


        /**
         * Returns the URL of the application
         *
         * @return  string
         */
        public function getApplicationUrl();


        /**
         * Returns the URL where documentation lives
         *
         * @return  string
         */
        public function getDocumentationUrl();


        /**
         * Get the web path where core files (such as CSS and JavaScript) can be found.
         * This path may include the domain if required
         *
         * @return  string
         */
        public function getCoreFilesWebPath();


        /**
         * Get the file system path where core files (such as CSS and JavaScript) can be found.
         *
         * @return  string
         */
        public function getCoreFilesSystemPath();


        /**
         * Get the web path where third party files (such as JavaScript libraries) can be found.
         * This path may include the domain if required
         *
         * @return  string
         */
        public function get3rdPartyFilesWebPath();


        /**
         * Get the file system path where third party files (such as JavaScript libraries) can be found.
         *
         * @return  string
         */
        public function get3rdPartyFilesSystemPath();


        /**
         * Get the web path used to return the CSS
         *
         * @return  string
         */
        public function getCssWebPath($filename);


        /**
         * Get the CSS code for this theme
         *
         * @return  string
         */
        public function getThemeCss($filename);


        /**
         * Get the modified time for the theme CSS
         *
         * @return  int
         */
        public function getThemeCssMtime($filename);
    }
?>