<?php
    class Application_Cp_Theme_Section
    {
        protected $_values = array();

        protected $_defaultColors = array(
            'primaryDark'       => '#00518A',
            'primary'           => '#0067B0',
            'primaryLight'      => '#d2ecff',
            'icons'             => '#FFFFFF',
            'accent'            => '#F59C00',
            'accentNegative'    => '#F44336',
            'accentPositive'    => '#448AFF',
            'textPrimary'       => '#212121',
            'text'              => '#FFFFFF',
            'textSecondary'     => '#727272',
            'dividerPrimary'    => '#B6B6B6',
            'dividerSecondary'  => '#333333'
            );

        public function __construct($prefix = '')
        {
            $this->_prefix = trim($prefix, '/');

            if (strlen($this->_prefix) > 0) {
                $this->_prefix .= '/';
            }

            $this->setPrimaryDark($this->_defaultColors['primaryDark']);
            $this->setPrimary($this->_defaultColors['primary']);
            $this->setPrimaryLight($this->_defaultColors['primaryLight']);
            $this->setIcons($this->_defaultColors['icons']);
            $this->setAccent($this->_defaultColors['accent']);
            $this->setAccentNegative($this->_defaultColors['accentNegative']);
            $this->setAccentPositive($this->_defaultColors['accentPositive']);
            $this->setTextPrimary($this->_defaultColors['textPrimary']);
            $this->setText($this->_defaultColors['text']);
            $this->setTextSecondary($this->_defaultColors['textSecondary']);
            $this->setDividerPrimary($this->_defaultColors['dividerPrimary']);
            $this->setDividerSecondary($this->_defaultColors['dividerSecondary']);
            
            
            $this->setPlaceholderValue('global/logo', '/__/containers/files/logo/' . Application_Cp_Theme_Logo::WHITE . '.' . Application_Cp_Theme_Logo::EXTENSION);
        }

        public function setPlaceHolderValue($placeholder, $value)
        {
            $p = sprintf('recite(%s%s)', $this->_prefix, trim($placeholder, '/'));
            $this->_values[$p] = $value;

            $p = sprintf('reciteNoHash(%s%s)', $this->_prefix, trim($placeholder, '/'));
            $this->_values[$p] = ltrim($value, '#');

            return $this;
        }

        public function getPlaceHolderValues()
        {
            return $this->_values;
        }

        public function setPrimaryDark($color)
        {
            $this->setPlaceHolderValue('primary/dark', $color);
            return $this;
        }

        public function setPrimary($color)
        {
            $this->setPlaceHolderValue('primary', $color);
            return $this;
        }

        public function setPrimaryLight($color)
        {
            $this->setPlaceHolderValue('primary/light', $color);
            return $this;
        }

        public function setIcons($color)
        {
            $this->setPlaceHolderValue('icons', $color);
            return $this;
        }

        public function setAccent($color)
        {
            $this->setPlaceHolderValue('accent', $color);
            return $this;
        }

        public function setAccentNegative($color)
        {
            $this->setPlaceHolderValue('accent/negative', $color);
            return $this;
        }

        public function setAccentPositive($color)
        {
            $this->setPlaceHolderValue('accent/positive', $color);
            return $this;
        }

        public function setTextPrimary($color)
        {
            $this->setPlaceHolderValue('text/primary', $color);
            return $this;
        }

        public function setText($color)
        {
            $this->setPlaceHolderValue('text', $color);
            return $this;
        }

        public function setTextSecondary($color)
        {
            $this->setPlaceHolderValue('text/secondary', $color);
            return $this;
        }

        public function setDividerPrimary($color)
        {
            $this->setPlaceHolderValue('divider/primary', $color);
            return $this;
        }

        public function setDividerSecondary($color)
        {
            $this->setPlaceHolderValue('divider/secondary', $color);
            return $this;
        }
    }
?>
