<?php
    class Application_Cp_Ui_Dialog
    {
        protected $_title;
        protected $_breadcrumbs;
        protected $_actionUrl;
        protected $_body;
        protected $_controllerScript;
        protected $_dimensions = array(
            'w' => 0,
            'h' => 0,
            'm' => false
        );
        protected $_maximised = false;

        const WIDTH_AUTO    = -1;
        const WIDTH_DEFAULT = 0;

        public function __construct()
        {

        }

        public function maximise()
        {
            $this->_dimensions['m'] = true;
            return $this;
        }

        public function setTitle($title)
        {
            $this->_title = trim($title);
            return $this;
        }

        public function setBreadcrumbs($breadcrumbs)
        {
            $this->_breadcrumbs = $breadcrumbs;
        }

        public function getTitle()
        {
            if (strlen($this->_title) > 0) {
                return $this->_title;
            }

            return $this->_breadcrumbs->getTitle();
        }

        public function setActionUrl($url)
        {
            $this->_actionUrl = $url;
            return $this;
        }

        public function getActionUrl()
        {
            if (strlen($this->_actionUrl) == 0) {
                throw new Exception('Action URL not set');
            }

            return $this->_actionUrl;
        }

        public function setBody($body)
        {
            $this->_body = $body;
            return $this;
        }

        public function hasBody()
        {
            return strlen($this->_body) > 0;
        }

        public function getBody()
        {
            return $this->_body;
        }

        public function getButtons()
        {
            return array();
        }

        public function setControllerScript($webPath, $scriptName)
        {
            $this->_controllerScript = array(
                'path' => $webPath,
                'name' => $scriptName
            );

            return $this;
        }

        public function setDimensions($w, $h)
        {
            $this->setW($w);
            $this->setH($h);
            return $this;
        }

        public function setW($w)
        {
            if ($w < 0) {
                $w = self::WIDTH_AUTO;
            }
            else {
                $w = (int) $w;
            }

            $this->_dimensions['w'] = $w;
            return $this;
        }

        public function setH($h)
        {
            $this->_dimensions['h'] = max(0, (int) $h);
            return $this;
        }

        public function asArray()
        {
            $ret = array(
                'title'   => $this->getTitle(),
                'body'    => $this->_body,
                'action'  => '',
                'buttons' => array()
            );

            try {
                $ret['action'] = $this->getActionUrl();
            }
            catch (Exception $ex)
            { }

            foreach ($this->getButtons() as $button) {
                $ret['buttons'][] = array(
                    'label' => $button->getLabel(),
                    'type'  => $button->getType(),
                    'name'  => $button->getName()
                );
            }

            if (is_array($this->_controllerScript)) {
                $ret['script'] = array(
                    'path' => (string) $this->_controllerScript['path'],
                    'name' => (string) $this->_controllerScript['name']
                );
            }

            $ret['dims'] = $this->_dimensions;

            return $ret;
        }
    }
?>