<?php
    /**
     * The base interface that all widgets must implement
     *
=     * @author  Quentin Zervaas
     */
    interface Application_Cp_Widget_Interface
    {
        const SIZE_SMALL  = 's';
        const SIZE_MEDIUM = 'm';
        const SIZE_LARGE  = 'l';

        /**
         * Constructor
         */
        public function __construct();


        /**
         * A descriptive name for the widget, displayed to the user when adding widgets
         *
         * @return  string
         */
        public function __toString();


        /**
         * Get a list of modules that this widget relies on to work properly
         */
        public function getDependentModules();


        /**
         * The internal name for the widget
         *
         * @return  string
         */
        public function getName();


        /**
         * Get one or more categories for a widget
         *
         * @return  string[]
         */
        public function getCategories();


        /**
         * Return a description of the purpose of the widget. This should be
         * returned in whatever the current language is, and shouldn't contain
         * any HTML
         *
         * @return  string
         */
        public function getDescription();


        /**
         * Return the path of the driver
         *
         * @return string
         */
        public function getPath();


        /**
         * Get the size of the widget. This is used to determine auto-placing of widgets
         *
         * @return  string
         */
        public function getWidgetSize();


        /**
         * Whether or not the widget can be manually added by a user
         *
         * @return  bool
         */
        public function canBeManuallyAdded();


        /**
         * Set the active request
         *
         * @param   Application_Cp_Site_Request     $request
         */
        public function setRequest(Application_Cp_Site_Request $request);


        /**
         * Set the active response
         *
         * @param   Application_Cp_Site_Response    $response
         */
        public function setResponse(Application_Cp_Site_Response $response);


        /**
         * Set the view for the current request
         *
         * @param   Application_View_Cp     $view
         */
        public function setView(Application_View_Cp $view);


        /**
         * Run the widget
         *
         * @param   string  $path   The path of the request, allowing sub-actions to be performed
         */
        public function run($path = '');


        /**
         * Get the relative path to the script that controls this widget
         *
         * @throws  Application_Cp_Widget_Exception If there is no controller script
         * @return  Application_Cp_Widget_Script
         */
        public function getScript($path = '');


        /**
         * Get the relative path to the CSS that file that applies extra style to the widget
         *
         * @throws  Application_Cp_Widget_Exception If there is no CSS file
         * @return  string
         */
        public function getCssPaths();



        /**
         * Send a widget file. Throws an exception if the file isn't found or can't be sent
         *
         * @throws  Application_Cp_Widget_Exception
         */
        public function sendFile(Application_Cp_Site_Response $response, $path);


        /**
         * Perform any required permissions check to determine whether or not this widget
         * can even be used
         *
         * @throws  Application_User_Permissions_Exception  If the permissions check fails
         */
        public function assertPermissions();
    }
?>