<?php
    class Application_Cp_Widget_Manager
    {
        public static function GetPath()
        {
            return realpath(dirname(__FILE__) . '/../../../../../lib/widgets');
        }

        public static function GetDrivers($options = array())
        {
            $defaults = array(
                'addableOnly' => false,
                'checkPerms'  => false,
                'categories'  => array(),
                'assert'      => true
            );

            $options = ArrayTools::BuildOptions($options, $defaults);

            $ret = array();

            $fd = new File_Directory(self::GetPath());

            $categories = array();

            $options['categories'] = ArrayTools::Clean($options['categories']);
            foreach ($options['categories'] as $category) {
                $k = Text::Urlize($category);
                $categories[$k] = $k;
            }

            $checkCategories = count($categories) > 0;

            foreach ($fd->getDirs() as $dir) {
                foreach (self::GetDriversByPath($fd->getPath() . '/' . $dir) as $driver) {
                    if ($options['addableOnly'] && !$driver->canBeManuallyAdded()) {
                        continue;
                    }

                    if ($options['assert']) {
                        try {
                            self::AssertDriverCanBeUsed($driver, $options['checkPerms']);
                        }
                        catch (Exception $ex) {
                            continue;
                        }
                    }

                    if ($checkCategories) {
                        $add = false;
                        foreach ($driver->getCategories() as $category) {
                            if (array_key_exists(Text::Urlize($category), $categories)) {
                                $add = true;
                                break;
                            }
                        }

                        if (!$add) {
                            continue;
                        }
                    }

                    $ret[$driver->getName()] = $driver;
                }
            }

            return $ret;
        }

        public static function GetCategories($options = array())
        {
            $ret = array();

            foreach (self::GetDrivers($options) as $driver) {
                $categories = ArrayTools::Clean($driver->getCategories());

                foreach ($categories as $category) {
                    $key      = Text::Urlize($category);
                    $ret[$key] = $category;
                }
            }

            $ret = array_values($ret);
            sort($ret);
            return $ret;
        }

        public static function GetDriversByPath($path)
        {
            $ret = array();

            $fullpath = $path . '/controller.php';

            if (file_exists($fullpath)) {
                try {
                    $driver = self::GetDriverByPath($path);
                    $ret[$driver->getName()] = $driver;
                }
                catch (Exception $ex)
                { }
            }
            else {
                $fd = new File_Directory($path);

                foreach ($fd->getDirs() as $dir) {
                    foreach (self::GetDriversByPath($path . '/' . $dir, $dir . '_') as $k => $v) {
                        $ret[$k] = $v;
                    }
                }
            }

            return $ret;
        }

        public static function GetDriverByPath($path)
        {
            $path = rtrim($path, '/');

            $fullpath = sprintf('%s/controller.php', $path);

            if (strlen($path) == 0 || !file_exists($fullpath)) {
                throw new Application_Cp_Widget_Exception('Path does not exist: ' . $fullpath);
            }

            if (!is_file($fullpath)) {
                throw new Application_Cp_Widget_Exception('Path is not a directory: ' . $fullpath);
            }

            require_once($fullpath);

            $class = substr('CpWidget_' . str_replace('/', '_', substr(trim($fullpath, '/'), strlen(self::GetPath()))), 0, -4);

            if (!class_exists($class, false)) {
                throw new Application_Cp_Widget_Exception('Driver class not defined');
            }

            $reflection = new Zend_Reflection_Class($class);

            if (!$reflection->isSubclassOf('Application_Cp_Widget_Abstract')) {
                throw new Application_Cp_Widget_Exception('Class does not extend from Application_Cp_Widget_Abstract');
            }

            return $reflection->newInstance();
        }


        public static function Factory($name)
        {
            if ($name instanceof Application_Cp_Widget_Interface) {
                return $name;
            }

            if (!preg_match('/^CpWidget_(.*)$/', $name, $matches)) {
                throw new Application_Cp_Widget_Exception('Invalid widget');
            }

            $fullpath = sprintf('%s/%s', self::GetPath(), str_replace('_', '/', $matches[1]));

            return self::GetDriverByPath($fullpath);
        }

        public static function AssertDriverCanBeUsed(Application_Cp_Widget_Interface $driver, $checkPerms = false)
        {
            $modules = $driver->getDependentModules();

            if (!is_array($modules)) {
                throw new Exception(sprintf('%s::getDependentModules() must return an array', get_class($driver)));
            }

            $client = Application::GetClient();

            $query = Application_User_Permissions_Query::NewAnd();

            foreach ($modules as $name) {
                $module = Application_Module_Manager::Factory($name);

                if (!$module->isClientModule()) {
                    continue;
                }

                if (!$client->hasModule($module)) {
                    throw new Exception(sprintf('Client does not have access to module %s', $name));
                }

                $query->add('module:' . $name);
            }

            if ($checkPerms) {
                $query->assert();
                $driver->assertPermissions();
            }
        }
    }
?>