<?php
    /**
     * Abstract class that all drivers can extend from
     *
     * @author  Quentin Zervaas <quentin@recite.com.au>
     */
    abstract class Application_Driver_Abstract implements Application_Driver_Interface
    {
        private $_request;
        private $_response;

        /**
         * Constructor. Must not have any arguments
         */
        public function __construct()
        {

        }

        final public function getFullName()
        {
            return get_class($this);
        }

        public function getDescription()
        {
            return null;
        }


        /**
         * Return the path of the driver
         *
         * @return string
         */
        public function getPath()
        {
            $reflection = new ReflectionObject($this);

            return $reflection->getFileName();
        }


        public function getDriverDirectory()
        {
            return dirname($this->getPath());
        }


        /**
         * Get the active client
         *
         * @return  DatabaseObject_Client    The active client being used
         */
        public function getClient()
        {
            return Application::GetClient();
        }

        public function isClientDriver()
        {
            return true;
        }

        public function getDependentDrivers()
        {
            return array();
        }

        public function assertPermissions()
        { }

        public function getFileFullPath($path)
        {
            $path = preg_replace('|^[\./]+|', '', $path);

            return sprintf('%s/files/%s', $this->getDriverDirectory(), trim($path, '/'));
        }

        public function getFileWebPath($path)
        {
            $path = preg_replace('|^[\./]+|', '', $path);

            return sprintf('/files/driver/%s/%s', $this->getFullName(), $path);
        }

        public function sendFile(Application_Cp_Site_Response $response, $path)
        {
            $fullpath = realpath($this->getFileFullPath($path));

            $prefix = realpath($this->getDriverDirectory());

            if (substr($fullpath, 0, strlen($prefix)) != $prefix) {
                throw new Application_Driver_Exception('Path not in valid directory');
            }

            $options = array(
                'response' => $response
            );

            HTTP::SendFile($fullpath, $options);
        }

        public function isCpController()
        {
            return false;
        }

        public function setCpRequest(Zend_Controller_Request_Abstract $request)
        {
            $this->_request = $request;
        }

        public function getCpRequest()
        {
            return $this->_request;
        }

        public function setCpResponse(Zend_Controller_Response_Abstract $response)
        {
            $this->_response = $response;
        }

        public function getCpResponse()
        {
            return $this->_response;
        }
    }
?>