<?php
    require_once('Application/Loader/Autoload.php');

    class Application_Loader
    {
        protected static $_driverPath;
        protected static $_modulePath;

        private function __construct()
        { }

        public static function Startup()
        {
            $path = realpath(dirname(__FILE__) . '/../../../lib');

            self::$_driverPath = $path . DIRECTORY_SEPARATOR . 'drivers';
            self::$_modulePath = $path . DIRECTORY_SEPARATOR . 'modules';

            self::GetAutoloader();
        }

        public static function GetAutoloader()
        {
            return Application_Loader_Autoload::getInstance();
        }

        public static function GetPathForClassName($className)
        {
            if (strlen($className) == 0) {
                throw new Application_Loader_Exception('Class name not specified');
            }

            if (preg_match('/^Driver_/', $className)) {
                $str = str_replace('_', DIRECTORY_SEPARATOR, substr($className, 7));
                $fullpath = sprintf('%s%s%s.php', self::$_driverPath, DIRECTORY_SEPARATOR, $str);
            }
            else if (preg_match('/^Module_([^_]+)_(.*)$/', $className, $matches)) {
                $str = str_replace('_', DIRECTORY_SEPARATOR, $matches[2]);
                $fullpath = sprintf('%s%s%s%sinclude%s%s.php', self::$_modulePath, DIRECTORY_SEPARATOR, strtolower($matches[1]), DIRECTORY_SEPARATOR, DIRECTORY_SEPARATOR, $str);
            }
            else {
                $fullpath = str_replace('_', DIRECTORY_SEPARATOR, $className) . '.php';
            }

            return $fullpath;
        }

        public static function Load($className)
        {
            $fullpath = self::AssertCanLoadClass($className);

            return require_once($fullpath);
        }

        public static function AssertCanLoadClass($className)
        {
            if (class_exists($className, false)) {
                return true;
            }

            $path = self::GetPathForClassName($className);

            $paths = array();

            if (IS_WINDOWS && preg_match('|^[a-z]:|i', $path)) {
                $paths[] = $path;
            }
            else if (substr($path, 0, 1) == '/') {
                $paths[] = $path;
            }
            else {
                foreach (explode(PATH_SEPARATOR, get_include_path()) as $dir) {
                    $_path = realpath(str_replace('/', DIRECTORY_SEPARATOR, sprintf('%s/%s', $dir, $path)));

                    if (strlen($_path) > 0) {
                        $paths[] = $_path;
                    }
                }
            }

            $found = false;

            foreach ($paths as $path) {
                if (file_exists($path) && is_readable($path)) {
                    $found = true;
                    break;
                }
            }

            if (!$found) {
                throw new Application_Loader_Exception(sprintf('Unable to load class: %s', $className), $className);
            }

            return $path;
        }
    }
?>