<?php
    /**
     * This class is used for formatting dates and times according to locale settings
     *
     * @package Application
     * @author  Quentin Zervaas
     * @version Recite 2.0.0
     */
    class Application_Locale_Date
    {
        const SHORT  = 's';
        const MEDIUM = 'm';
        const LONG   = 'l';
        const FULL   = 'f';

        public static $formats = array(
            self::SHORT  => 'Short',
            self::MEDIUM => 'Medium',
            self::LONG   => 'Long',
            self::FULL   => 'Full'
        );

        private static $_reflection;

        private function __construct()
        { }

        public static function GetDateFormat($format = null)
        {
            try {
                $format = self::_GetFormat($format);
            }
            catch (Exception $ex) {
                $format = null;
            }

            switch ($format) {
                case self::SHORT:
                    $format = Zend_Date::DATE_SHORT;
                    break;

                case self::MEDIUM:
                    $format = Zend_Date::DATE_LONG;
                    break;

                case self::FULL:
                    $format = Zend_Date::DATE_FULL;
                    break;

                case self::MEDIUM:
                default:
                    $format = Zend_Date::DATE_MEDIUM;
                    break;
            }

            return $format;
        }

        public static function GetTimeFormat($format = null)
        {
            try {
                $format = self::_GetFormat($format);
            }
            catch (Exception $ex) {
                $format = null;
            }

            switch ($format) {
                case self::SHORT:
                    $format = Zend_Date::TIME_SHORT;
                    break;

                case self::MEDIUM:
                    $format = Zend_Date::TIME_LONG;
                    break;

                case self::FULL:
                    $format = Zend_Date::TIME_FULL;
                    break;

                case self::MEDIUM:
                default:
                    $format = Zend_Date::TIME_MEDIUM;
                    break;
            }

            return $format;
        }

        public static function GetDate($date, $format = null)
        {
            if (is_null($date)) {
                return null;
            }

            if (!$date instanceof Zend_Date) {
                $date = new Zend_Date($date);
            }

            return $date->get(self::GetDateFormat($format));
        }

        public static function GetTime($date, $format = null)
        {
            if (is_null($date)) {
                return null;
            }

            if (!$date instanceof Zend_Date) {
                $date = new Zend_Date($date);
            }

            return $date->get(self::GetTimeFormat($format));
        }

        public static function GetDateAndTime($date, $format = null)
        {
            if (is_null($date)) {
                return null;
            }

            if (!$date instanceof Zend_Date) {
                $date = new Zend_Date($date);
            }

            return sprintf('%s %s', $date->get(self::GetDateFormat($format)), $date->get(self::GetTimeFormat($format)));
        }

        private static function _GetFormat($format)
        {
            if (!array_key_exists($format, self::$formats)) {
                $reflection = self::_GetReflection();

                if (!$reflection->hasConstant($format)) {
                    throw new Exception('Invalid format');
                }

                $format = $reflection->getConstant($format);
            }

            return $format;
        }

        private static function _GetReflection()
        {
            if (!self::$_reflection instanceof ReflectionClass) {
                self::$_reflection = new ReflectionClass(__CLASS__);
            }

            return self::$_reflection;
        }
    }
?>