<?php
    /**
     * Application_Log_Emailer
     *
     * A writer class for Zend_Log that emails recorded log messages
     * to the specified address
     */
    class Application_Log_Emailer extends Zend_Log_Writer_Abstract
    {
        protected $_email;
        protected $_events = array();

        const RESEND_AGE = 3600;

        public function __construct()
        {
            $this->_formatter = new Zend_Log_Formatter_Simple();
        }

        public function setEmail($email)
        {
            $validator = new Zend_Validate_EmailAddress();
            if (!$validator->isValid($email))
                throw new Exception('Invalid email address specified');

            $this->_email = $email;
        }

        protected function _write($event)
        {
            $this->_events[] = $event;
        }

        public function shutdown()
        {
            if (count($this->_events) == 0 || strlen($this->_email) == 0)
                return;
            $hashArray = array();
            foreach ($this->_events as $event)
                $hashArray[] = $event['message'];

            $hash = md5(serialize($hashArray) . ',' . __FILE__);

            $fullpath = sprintf('%s/%s', File::GetSystemTmpDir(), $hash);

            $send = !file_exists($fullpath) || (time() - filemtime($fullpath) > self::RESEND_AGE);

            if ($send) {
                $subject = sprintf('Web site log messages (%d)', count($this->_events));

                $body = array();
                foreach ($this->_events as $event) {
                    $body[] = $this->_formatter->format($event);
                }

                $mail = new Application_Mail();

                $mail->addTo($this->_email)
                     ->setSubject($subject)
                     ->setBodyText(join('', $body))
                     ->send();

                file_put_contents($fullpath, $hash);
            }
        }
    }
?>