<?php
    class Application_Log_Renderer extends DataRenderer
    {
        protected $_records;

        const PREFIX = '(\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}[-+]\d{2}:\d{2}) ([A-Z]+) \((\d+)\):';

        public function __construct()
        {
            parent::__construct(Application::GetDb());

            $this->getLimitOption()->setDefault(50);
        }

        public function getData(&$arr, DataRenderer_Pager $pager)
        {
            $records = $this->getLogRecords();

            $rows = array_slice($records, $pager->offset, $pager->limit);

            foreach ($rows as $row) {
                $arr[] = self::GetRecord($row);
            }
        }

        public function getDataCount()
        {
            $records = $this->getLogRecords();
            return count($records);
        }

        public static function GetRecord($row)
        {
            $row = trim($row);

            $ret = array(
                'ts'       => '',
                'priority' => '',
                'code'     => '',
                'msg'      => ''
            );

            if (preg_match('/^' . self::PREFIX . ' (.*)$/s', $row, $matches)) {
                $ret['ts']       = $matches[1];
                $ret['priority'] = $matches[2];
                $ret['code']     = $matches[3];
                $ret['msg']      = $matches[4];
            }

            return $ret;
        }

        public function getLogRecords()
        {
            if (!is_array($this->_records)) {
                $fullpath = Zend_Registry::get('config')->debug->debug_log_file;


                if (file_exists($fullpath) && is_readable($fullpath)) {
                    $lines = file($fullpath);

                    $entries = array();

                    $current = '';
                    foreach ($lines as $line) {
                        if (preg_match('/' . self::PREFIX . '/', $line)) {
                            if (strlen($current) > 0) {
                                $entries[] = $current;
                            }
                            $current = $line;
                        }
                        else {
                            $current .= $line;
                        }
                    }

                    if (strlen($current) > 0) {
                        $entries[] = $current;
                    }

                    $this->_records = array_reverse($entries);
                }
                else {
                    $this->_records = array();
                }
            }

            return $this->_records;
        }
    }
?>
