<?php
    /**
     * This is a wrapper to the main mail sending routines,
     * used so we can audit and queue all mail that is sent
     *
     * @since   Recite 1.7.0
     * @author  Quentin Zervaas
     */
    class Application_Mail extends Zend_Mail
    {
        public function __construct()
        {
            parent::__construct('utf-8');

            $this->_setupDefaultFrom();
        }

        public static function GetDefaultFrom()
        {
            $validator = new Zend_Validate_EmailAddress();
            $email = '';
            $name  = '';

            try {
                $brand = Application::GetBrand();

                if ($brand instanceof Application_Brand && $brand->saved()) {
                    $email = trim($brand->getSetting('email.from_email'));
                    $name  = trim($brand->getSetting('email.from_name'));
                }
            }
            catch (Exception $ex) {
            }

            if (!$validator->isValid($email)) {
                $email = coalesce($_SERVER['SERVER_ADMIN']);
                $name  = '';
            }

            return new Application_Mail_Recipient($email, $name);
        }

        private function _setupDefaultFrom()
        {
            try {
                $from = self::GetDefaultFrom();
                $this->setFrom($from->getEmail(), $from->getName());
            }
            catch (Exception $ex) {
                $this->setFrom('noreply@localhost', 'Administrator');
            }
        }

        public function fromDirectory(Application_User_Directory $directory)
        {
            if ($directory instanceof Application_User_Directory && $directory->isClientDirectory()) {
                $client = $directory->getClient();
            }
            else {
                $client = null;
            }

            if ($client instanceof Application_Client && $client->saved()) {
                $this->fromClient($client);
            }
        }

        public function fromClient(Application_Client $client)
        {
            if (!$client instanceof Application_Client || !$client->saved()) {
                return;
            }

            $email  = $client->profile->get('email.from_email');
            $name   = $client->profile->get('email.from_name');

            if (strlen($email) > 0) {
                $validator = new Zend_Validate_EmailAddress();
                if ($validator->isValid($email)) {
                    $this->setFrom($email, $name);
                }
            }
        }

        public function setFrom($email, $name = '')
        {
            $this->_from = null;
            unset($this->_headers['From']);

            if (strlen($name) == 0) {
                $name = null;
            }

            return parent::setFrom($email, $name);
        }

        public function send($audit = true)
        {
            $ret = parent::send();

            if ($audit) {
                $message = new Application_Auditor_Message('Email sent');
                $message->addMemo('from', $this->getFrom())
                        ->addMemo('to', $this->getRecipients())
                        ->addMemo('subject', $this->getSubject())
                        ->record();
            }

            return $ret;
        }
    }
?>