<?php
    /**
     * Interface for all maintenance actions to implement
     *
     * @author  Quentin Zervaas
     * @since   Recite 1.7.0
     */
    interface Application_Maintenance_Interface
    {
        /**
         * Constructor. Accepts no arguments
         */
        public function __construct();

        /**
         * Return a basic description of what the action performs
         */
        public function __toString();


        /**
         * Run the action
         *
         * @param   array   $options                    Options to use in running the action
         * @throws  Application_Maintenance_Exception   If something goes wrong
         */
        public function run($options);


        /**
         * Whether or not the action can be queued for the given options
         *
         * @param   array   $options    Options to use when running the action
         * @return  bool                True if the action and params can be queued
         */
        public function accept($options);
    }
?>