<?php
    class Application_Maintenance_QueueItem
    {
        protected $_id;
        protected $_client;
        protected $_params = array();
        protected $_canDuplicate = false;
        protected $_action;
        protected $_tsCreated;
        protected $_tsRunning;

        const DUPLICATE_KEY = '__can_duplicate';

        public function __construct($action)
        {
            $this->_action = Application_Maintenance::Factory($action);
        }

        public function setId($id)
        {
            $this->_id = $id;
            return $this;
        }

        public function getId()
        {
            if (strlen($this->_id) == 0) {
                throw new Application_Maintenance_Exeption('ID not set');
            }

            return $this->_id;
        }

        public function getController()
        {
            return Application_Maintenance::Factory($this->_action);
        }

        public function setClient(Application_Client $client)
        {
            if (!$client->saved()) {
                throw new Application_Maintenance_Exception('Invalid client');
            }

            $this->_client = $client;
            return $this;
        }

        public function getClient()
        {
            if (!$this->_client instanceof Application_Client) {
                throw new Application_Maintenance_Exception('Client not set');
            }

            return $this->_client;
        }

        public function hasClient()
        {
            return $this->_client instanceof Application_Client;
        }

        public function queue()
        {
            Application_Maintenance::Queue($this);
            return $this;
        }

        public function __toString()
        {
            return (string) $this->getController();
        }

        public function setCreatedTimestamp($ts)
        {
            $this->_tsCreated = $ts;
            return $this;
        }

        public function getCreatedTimestamp()
        {
            return $this->_tsCreated;
        }

        public function setRunningTimestamp($ts)
        {
            $this->_tsRunning = $ts;
            return $this;
        }

        public function getRunningTimestamp()
        {
            return $this->_tsRunning;
        }

        public function setParams($arr)
        {
            if (!is_array($arr)) {
                $arr = array();
            }

            if (array_key_exists(self::DUPLICATE_KEY, $arr)) {
                $this->_canDuplicate = (bool) $arr[self::DUPLICATE_KEY];
                unset($arr[self::DUPLICATE_KEY]);
            }

            $this->_params = $arr;
            return $this;
        }

        public function setParam($k, $v)
        {
            $this->_params[$k] = $v;
            return $this;
        }

        public function getParams()
        {
            $ret = $this->_params;
            //$ret[self::DUPLICATE_KEY] = $this->_canDuplicate;

            return $ret;
        }

        public function setCanHaveDuplicates($flag = true)
        {
            $this->_canDuplicate = (bool) $flag;
            return $this;
        }

        public function canHaveDuplicates()
        {
            return $this->_canDuplicate;
        }

        public function getAction()
        {
            return $this->_action;
        }

        public function getActionName()
        {
            return get_class($this->_action);
        }
    }
?>
