<?php
    /**
     * Class to help with scheduled tasks on Windows
     *
     * @since   Recite 2.0.4
     * @author  Quentin Zervaas <quentin@recite.com.au>
     */
    class Application_Maintenance_Windows
    {
        private function __construct()
        { }

        public static function GetScheduledTaskXml($path = null)
        {
            if (!IS_WINDOWS) {
                throw new Exception('Unable to generate scheduled task XML');
            }

            $path = trim($path);

            if (strlen($path) == 0) {
                $path = Application_Maintenance::GetCliPath();
            }

            $tpl = <<<EOF
<?xml version="1.0" encoding="UTF-8"?>
<Task xmlns="http://schemas.microsoft.com/windows/2004/02/mit/task">
    <Actions>
        <Exec>
            <Command>%s</Command>
            <Arguments>%s</Arguments>
        </Exec>
    </Actions>

    <Triggers>
        <TimeTrigger>
            <Repetition>
                <Interval>PT1M</Interval>
                <StopAtDurationEnd>false</StopAtDurationEnd>
            </Repetition>
            <ExecutionTimeLimit>PT30M</ExecutionTimeLimit>
        </TimeTrigger>
    </Triggers>

    <Principals>
        <Principal id="Author">
            <LogonType>S4U</LogonType>
            <RunLevel>HighestAvailable</RunLevel>
        </Principal>
    </Principals>

    <Settings>
        <MultipleInstancesPolicy>Parallel</MultipleInstancesPolicy>
        <AllowHardTerminate>true</AllowHardTerminate>
        <StartWhenAvailable>false</StartWhenAvailable>
        <RunOnlyIfNetworkAvailable>true</RunOnlyIfNetworkAvailable>
        <IdleSettings>
            <StopOnIdleEnd>true</StopOnIdleEnd>
            <RestartOnIdle>false</RestartOnIdle>
        </IdleSettings>
        <AllowStartOnDemand>true</AllowStartOnDemand>
        <Enabled>true</Enabled>
        <Hidden>true</Hidden>
        <RunOnlyIfIdle>false</RunOnlyIfIdle>
        <WakeToRun>false</WakeToRun>
        <ExecutionTimeLimit>PT1H</ExecutionTimeLimit>
        <Priority>7</Priority>
    </Settings>
</Task>
EOF;

            $xml = sprintf($tpl, $path, Application_Maintenance::GetProcessScriptPath());
            return $xml;
        }
    }
?>