<?php
    class Application_Settings_Bundle
    {
        protected $_name;
        protected $_title;
        protected $_template;
        protected $_sections = array();

        public function __construct()
        { }

        public function setName($name)
        {
            $this->_name = (string) $name;
            return $this;
        }

        public function getName()
        {
            return (string) $this->_name;
        }

        public function setTitle($title)
        {
            $this->_title = (string) $title;
            return $this;
        }

        public function __toString()
        {
            $ret = (string) $this->_title;
            if (strlen($ret) == 0) {
                $ret = 'Untitled';
            }

            return $ret;
        }

        public function addSection(Application_Settings_Section $section)
        {
            $this->_sections[] = $section;
            return $this;
        }

        public function getSections()
        {
            return $this->_sections;
        }

        public function getNumSettings()
        {
            $ret = 0;

            foreach ($this->_sections as $section) {
                $ret += $section->getNumSettings();
            }

            return $ret;
        }

        public function setTemplate($tpl)
        {
            $this->_template = $tpl;
            return $this;
        }

        public function getTemplate()
        {
            return $this->_template;
        }

        public function isDefault()
        {
            return $this->_name == Application_Settings_Manager::GLOBAL_BUNDLE_NAME;
        }

        public function xmlExport($item, DOMDocument $doc, DOMElement $root, $field = 'bundle')
        {
            if (!$item instanceof Application_Brand && !$item instanceof Application_Client) {
                throw new Exception('Export item must either be a client or brand');
            }

            $node = $doc->createElement($field);

            foreach ($this->getSections() as $section) {
                $sectionNode = $doc->createElement('section');
                $sectionNode->setAttribute('title', (string) $section);

                foreach ($section->getSettings() as $setting) {
                    $key = $setting->getKey();

                    $field = $setting->getFieldListField();
                    $field->set($item->getSetting($key));
                    $type  = $field->getType();

                    $settingNode = $doc->createElement('setting');
                    $settingNode->setAttribute('key', $key);

                    $typeNode = $doc->createElement('type');
                    $typeNode->setAttribute('name', $type->getType());
                    $settingNode->appendChild($typeNode);

                    $valueNode = $doc->createElement('value');

                    $value = $type->export();

                    if ($value instanceof Profile_Field_ComplexValue) {
                        $value->exportToXmlNode($doc, $valueNode);
                    }
                    else {
                        $valueNode->appendChild($doc->createCDATASection($value));
                    }

                    $settingNode->appendChild($valueNode);

                    $sectionNode->appendChild($settingNode);
                }

                $node->appendChild($sectionNode);
            }

            $root->appendChild($node);
        }

        public static function XmlImport($item, SimpleXmlElement $node)
        {
            if (!$item instanceof Application_Brand && !$item instanceof Application_Client) {
                throw new Exception('Import item must either be a client or brand');
            }

            foreach ($node->section as $sectionNode) {
                foreach ($sectionNode->setting as $settingNode) {
                    $key = trim($settingNode['key']);

                    if (strlen($key) == 0) {
                        continue;
                    }

                    try {
                        $type = Profile_Field::Factory($settingNode->type['name']);

                        if (isset($settingNode->value['type']) && $settingNode->value['type'] == 'complex') {
                            $succ = $type->import(
                                Profile_Field_ComplexValue::FromXml($settingNode->value)
                            );
                        }
                        else {
                            $succ = $type->import((string) $settingNode->value);
                        }

                        if (!$succ) {
                            throw new Exception('Unable to import');
                        }
                    }
                    catch (Exception $ex) {
                        continue;
                    }

                    $item->setSetting($key, $type->get());
                }
            }

            $item->save();
        }

        public static function FromPath($fullpath, $target, $options = array())
        {
            if (!file_exists($fullpath)) {
                throw new Application_Settings_Exception('Unable to find settings file');
            }

            $defaults = array(
                'adminMode' => false,
                'all'       => false
            );

            $options = ArrayTools::BuildOptions($options, $defaults);

            if ($target instanceof Application_Brand) {
                $target = 'brand';
            }
            else if ($target instanceof Application_Client) {
                $target = 'client';
            }
            else if ($target instanceof Application_User) {
                $target = 'user';
            }

            $xpath = sprintf('/s:settings/s:section[@target="%s"]', $target);

            $ret = new self();

            $xml = new SimpleXmlElement($fullpath, null, true);
            $xml->registerXPathNamespace('s', 'http://www.recite.com.au/schema/settings');

            $nodes = $xml->xpath($xpath);

            if (is_array($nodes)) {
                foreach ($nodes as $node) {
                    $section = new Application_Settings_Section($node->title);

                    foreach ($node->setting as $n) {
                        $setting = Application_Settings_Setting::FromXml($n);

                        if (!$options['all']) {
                            if ($options['adminMode'] != $setting->isAdminOnly()) {
                                continue;
                            }
                        }

                        $section->add($setting);
                    }

                    if ($section->getNumSettings() > 0) {
                        $ret->addSection($section);
                    }
                }
            }

            return $ret;
        }
    }
?>