<?php
    class Application_Settings_Manager
    {
        const GLOBAL_BUNDLE_NAME      = '_global';
        const CUSTOM_BUNDLE_NAME      = '_custom';
        const CACHE_BUNDLE_NAME       = '_cache';
        const MAINTENANCE_BUNDLE_NAME = '_maintenance';

        public static $customBundles = array(
            self::CUSTOM_BUNDLE_NAME => array(
                'title' => 'Custom Settings',
                'tpl'   => 'bundle/custom.tpl'
            ),
            self::CACHE_BUNDLE_NAME => array(
                'title' => 'Cache Settings',
                'tpl'   => 'bundle/cache.tpl'
            ),
            self::MAINTENANCE_BUNDLE_NAME => array(
                'title' => 'Maintenance Settings',
                'tpl'   => 'bundle/maintenance.tpl'
            )
        );

        public static function GetBundles($target, $options = array())
        {
            $ret = array();

            $paths = self::GetBundlePaths($target);

            foreach ($paths as $path) {
                try {
                    $bundle = Application_Settings_Bundle::FromPath($path['path'], $target, $options);

                    $bundle->setName($path['name'])
                           ->setTitle($path['title']);

                    if ($bundle->getNumSettings() > 0) {
                        $ret[$bundle->getName()] = $bundle;
                    }
                }
                catch (Exception $ex)
                { }
            }

            return $ret;
        }


        public static function GetBundle($target, $name, $options = array())
        {
            if ($name == self::GLOBAL_BUNDLE_NAME) {
                $info = self::_GetGlobalBundleInfo();

                $bundle = Application_Settings_Bundle::FromPath($info['path'], $target, $options);

                $bundle->setName($info['name'])
                       ->setTitle($info['title']);

                return $bundle;
            }

            if (array_key_exists($name, self::$customBundles)) {
                if (!$target instanceof Application_Client) {
                    throw new Application_Settings_Exception('Target must be a client');
                }

                if ($name == self::CUSTOM_BUNDLE_NAME) {
                    $bundle = Application_Client_Settings_Custom::GetBundle($target);
                }
                else {
                    $bundle = new Application_Settings_Bundle();
                }

                $bundle->setName($name);
                $bundle->setTitle(self::$customBundles[$name]['title'])
                       ->setTemplate(self::$customBundles[$name]['tpl']);

                return $bundle;
            }

            try {
                $bundle = self::GetModuleBundle($name, $target, $options);

                if ($target instanceof Application_Client && !$target->hasModule($name)) {
                    throw new Application_Settings_Exception('Client does not have access to this bundle');
                }

                return $bundle;
            }
            catch (Exception $ex) {
                throw new Application_Settings_Exception($ex->getMessage());
            }
        }


        public static function GetBundlePaths($target)
        {
            $ret = array(self::_GetGlobalBundleInfo());

            if ($target instanceof Application_Brand) {
                //$modules = Application_Module_Manager::GetModules($target);
                $modules = Application_Module_Manager::GetModules();
            }
            else if ($target instanceof Application_Client) {
                $modules = Application_Module_Manager::GetModules($target);
            }
            else if ($target instanceof Application_User) {
                $modules = Application_Module_Manager::GetModules(Application::GetClient());
            }
            else {
                return array();
            }

            foreach ($modules as $module) {
                $fullpath = sprintf('%s/settings/settings.xml', $module->getPath());

                if (file_exists($fullpath) && is_file($fullpath) && is_readable($fullpath)) {
                    $ret[] = array(
                        'name'  => $module->getName(),
                        'path'  => $fullpath,
                        'title' => (string) $module
                    );
                }
            }

            return $ret;
        }

        private function _GetGlobalBundleInfo()
        {
            return array(
                'name'  => self::GLOBAL_BUNDLE_NAME,
                'path'  => sprintf('%s/settings/settings.xml', Application::GetApplicationDir()),
                'title' => 'Global Settings'
            );
        }

        public static function DeployBrandDefaults(Application_Brand $brand)
        {
            foreach (self::GetBundles($brand, array('all')) as $bundle) {
                self::DeployBundle($brand, $bundle);
            }
        }

        public static function DeployBundle($target, $bundle)
        {
            foreach ($bundle->getSections() as $section) {
                foreach ($section->getSettings() as $setting) {
                    $target->setSetting($setting->getKey(), $setting->getDefault());
                }
            }
        }

        public static function GetModuleBundle($module, $target, $options = array())
        {
            $module = Application_Module_Manager::Factory($module);

            $fullpath = sprintf('%s/settings/settings.xml', $module->getPath());

            $bundle = Application_Settings_Bundle::FromPath($fullpath, $target, $options);

            $bundle->setName($module->getName())
                   ->setTitle((string) $module);

            return $bundle;
        }

        public static function DeployModuleDefaults($module)
        {
            $db = Application::GetDb();
            $db->begin();

            foreach (Application_Brand::GetBrands() as $brand) {
                try {
                    self::DeployBundle(
                        $brand,
                        self::GetModuleBundle($module, $brand, array('all' => true))
                    );
                    $brand->save();
                }
                catch (Exception $ex) {
                }
            }

            $db->commit();
        }

        public static function DeployModuleToClient(Application_Client $client, $module)
        {
            $bundle = self::GetModuleBundle($module, $client, array('all' => true));
            self::DeployBundle($client, $bundle);
        }

        public static function GetDefaultForSetting($target, $settingName)
        {
            $setting = self::GetSetting($target, $settingName);
            return $setting->getDefault();
        }

        public static function GetSetting($target, $settingName)
        {
            foreach (self::GetBundles($target) as $bundle) {
                foreach ($bundle->getSections() as $section) {
                    foreach ($section->getSettings() as $setting) {
                        if ($setting->getKey() == $settingName) {
                            return $setting;
                        }
                    }
                }
            }

            throw new Application_Settings_Exception(sprintf('Setting with name %s not found', $settingName));
        }
    }
?>