<?php
    class Application_Settings_Setting
    {
        protected $_key;
        protected $_type;
        protected $_typeParams = array();
        protected $_default;
        protected $_label;
        protected $_help;
        protected $_adminOnly = false;

        public function __construct($key)
        {
            $this->_key = (string) $key;
        }

        public function getFieldListField()
        {
            $ret = new Profile_Field_FieldList_Field($this->_type);
            $ret->setTitle($this->_label)
                ->setName($this->_key)
                ->setHelp($this->_help)
                ->setParams($this->_typeParams)
                ->set($this->_default);

            return $ret;
        }

        public function getKey()
        {
            return $this->_key;
        }

        public function setIsAdmin($flag)
        {
            $this->_adminOnly = (bool) $flag;
            return $this;
        }

        public function isAdminOnly()
        {
            return $this->_adminOnly;
        }

        public function setType($type)
        {
            $type = Profile_Field::Factory($type);
            $this->_type = $type;
            return $this;
        }

        public function setTypeParam($k, $v)
        {
            $this->_typeParams[$k] = $v;
            return $this;
        }

        public function setDefault($default)
        {
            $this->_default = $default;
            return $this;
        }

        public function getDefault()
        {
            return $this->_default;
        }

        public function setLabel($label)
        {
            $this->_label = (string) $label;
            return $this;
        }

        public function setHelp($help)
        {
            $this->_help = trim((string) $help);
            return $this;
        }

        public static function FromXml($node)
        {
            $key = (string) $node['key'];

            if (strlen($key) == 0) {
                throw new Application_Settings_Exception('Key not specified');
            }

            $ret = new self($key);

            if (isset($node->type)) {
                if (strlen($node->type['name']) > 0) {
                    $ret->setType($node->type['name']);
                }

                foreach ($node->type->param as $param) {
                    $ret->setTypeParam((string) $param['name'], (string) $param);
                }
            }

            if (isset($node->default) && strlen($node->default) > 0) {
                $ret->setDefault((string) $node->default);
            }

            if (isset($node->label) && strlen($node->label) > 0) {
                $ret->setLabel((string) $node->label);
            }

            if (isset($node->help) && strlen($node->help) > 0) {
                $ret->setHelp((string) $node->help);
            }

            if (isset($node['admin'])) {
                $ret->setIsAdmin((bool) $node['admin']);
            }

            return $ret;
        }
    }
?>