<?php
    class Application_Site_ControllerAction extends Zend_Controller_Action
    {
        private $_autoSetContentType = true;

        public function noRender($flag = true)
        {
            $this->_helper->viewRenderer->setNoRender($flag);
        }

        protected function addStatusMessage($msg, $code = 0)
        {
            $this->getResponse()->addStatusMessage(new Application_Site_Response_StatusMessage($msg, $code));
        }

        public function addEvent($name)
        {
            $event = new Application_Site_Response_Event($name);
            $this->getResponse()->addEvent($event);

            return $event;
        }

        public function addCustomData($key, $value)
        {
            $this->getResponse()->addCustomData($key, $value);
        }

        public function noAutosetContentType()
        {
            $this->_autoSetContentType = false;
            return $this;
        }

        public function postDispatch()
        {
            if ($this->getRequest()->getPost('___iframe_requires_text')) {
                $this->getResponse()->setHeader('Content-Type', HTTP::AddContentTypeCharset('text/plain', 'UTF-8'), true);
            }
            else if ($this->_autoSetContentType) {
                $response = $this->getResponse();

                $found = false;
                foreach ($response->getHeaders() as $header) {
                    if (strtolower($header['name']) == 'content-type') {
                        $found = true;
                        break;
                    }
                }

                if (!$found) {
                    $response->setHeader('Content-Type', HTTP::AddContentTypeCharset('text/html', 'UTF-8'), true);
                }
            }
        }

        public function sendJson($data)
        {
            $this->noRender();

            $request  = $this->getRequest();
            $response = $this->getResponse();

            //if ($request->isXmlHttpRequest()) {
                $response->setHeader('Content-Type', HTTP::AddContentTypeCharset('application/json', 'UTF-8'), true);
            //}
            //else {
            //    $response->setHeader('Content-Type', HTTP::AddContentTypeCharset('text/plain', 'UTF-8'), true);
            //}

            $this->noAutosetContentType();
            $response->setBody(Zend_Json::encode($data));
        }

        public function sendDialog(Application_Cp_Ui_Dialog $dialog)
        {
            $request = $this->getRequest();
            $dialog->setBreadcrumbs($this->breadcrumbs);

            if (!$dialog->hasBody()) {
                $script = $this->_helper->viewRenderer->getViewScript();
                $dialog->setBody($this->view->render($script));
            }

            try {
                $dialog->getActionUrl();
            }
            catch (Exception $ex) {
                $dialog->setActionUrl($request->getRequestUri());
            }

            $this->addEvent('open.dialog')
                 ->addMemo('dialog', $dialog->asArray());
        }

        public function getUrl($action = null, $controller = null, $module = null)
        {
            $url  = rtrim($this->getRequest()->getBaseUrl(), '/') . '/';
            $url .= $this->_helper->url->simple($action, $controller, $module);

            return '/' . ltrim($url, '/');
        }
    }
?>
