<?php
    class Application_Site_Profiler extends Zend_Controller_Plugin_Abstract
    {
        protected static $_logger;
        protected static $_isSetup = false;

        public function preDispatch(Zend_Controller_Request_Abstract $request)
        {
            if (!self::$_isSetup) {
                $request = $this->getRequest();
                $logger  = self::GetLogger();

                $db = Zend_Registry::get('db');

                $info = array(
                    sprintf(
                        '%s %s - PHP %s, Zend Framework %s, %s',
                        Application::NAME,
                        Application::VERSION,
                        phpversion(),
                        Zend_Version::VERSION,
                        $db,
                        $db->getConfigSingle('dbname'),
                        $db->getConfigSingle('host')
                    ),
                    sprintf('AppRoot( %s )', Application::GetBaseDir()),
                    sprintf('DocRoot( %s )', $request->getServer('DOCUMENT_ROOT'))
                );

                try {
                    $client = Application::GetClient();

                    if ($client instanceof Application_Client && $client->saved()) {
                        $info[] = sprintf('Client( %d / %s )', $client->getId(), $client);
                    }
                }
                catch (Exception $ex)
                { }

                $user = Zend_Registry::get('user');

                if ($user->saved()) {
                    $info[] = sprintf('User( %d / %s )', $user->getId(), $user->username);
                }

                $logger->info($info);

                self::$_isSetup = true;
            }
        }

        public static function GetLogger()
        {
            if (!self::$_logger instanceof Zend_Log) {
                if (Application::IsProfiling()) {
                    $writer = new Zend_Log_Writer_Firebug();
                }
                else {
                    $writer = new Zend_Log_Writer_Null();
                }

                self::$_logger = new Zend_Log($writer);
            }

            return self::$_logger;
        }
    }
?>