<?php
    require_once('Zend/Controller/Response/Http.php');

    class Application_Site_Response extends Zend_Controller_Response_Http
    {
        private $_events   = array();
        private $_statuses = array();
        private $_data     = array();

        private $_sendWidgetResponse = false;

        public static function SupportsGzip()
        {
            return strpos($_SERVER['HTTP_ACCEPT_ENCODING'], 'gzip') !== false;
        }

        public function useGzip()
        {
            if (!self::SupportsGzip()) {
                return false;
            }

            foreach ($this->getHeaders() as $header) {
                if (strtolower($header['name']) == 'content-type') {
                    return strtolower(substr($header['value'], 0, 9)) == 'text/html';
                }
            }

            return false;
        }

        public function sendWidgetResponse($flag = true)
        {
            $this->_sendWidgetResponse = $flag;

            if ($flag) {
                Zend_Controller_Action_HelperBroker::getStaticHelper('viewRenderer')->setNoRender(true);
                $this->setHeader('Content-Type', HTTP::AddContentTypeCharset('application/json', 'UTF-8'), true);
            }
        }

        public function addEvent(Application_Site_Response_Event $event)
        {
            $this->sendWidgetResponse(true);
            $this->_events[] = $event;
            return $this;
        }

        public function addStatusMessage(Application_Site_Response_StatusMessage $msg)
        {
            $this->sendWidgetResponse(true);
            $this->_statuses[] = $msg;
            return $this;
        }

        public function addCustomData($key, $value)
        {
            $this->sendWidgetResponse(true);
            $this->_data[$key] = $value;
            return $this;
        }

        public function sendResponse()
        {
            if (self::UseGzip()) {
                $this->setHeader('Content-Encoding', 'gzip');
            }

            parent::sendResponse();
        }

        public function outputBody()
        {
            if (!$this->_sendWidgetResponse) {
                if ($this->useGzip()) {
                    ob_start();
                    parent::outputBody();
                    echo gzencode(ob_get_clean(), 9);
                }
                else {
                    parent::outputBody();
                }

                return;
            }

            $data = array(
                'events' => array(),
                'status' => array(),
                'data'   => $this->_data
            );

            foreach ($this->_events as $event) {
                $data['events'][] = array(
                    'name' => (string) $event,
                    'memo' => $event->getMemo()
                );
            }

            foreach ($this->_statuses as $status) {
                $data['status'][] = array(
                    'msg'  => (string) $status,
                    'type' => $status->getType()
                );
            }

            echo Zend_Json::encode($data);
        }
    }
?>