<?php
    class Application_Upgrader
    {
        private function __construct()
        {
        }

        public static function RunUpdateTasks($oldVersion, $newVersion)
        {
            $db = Application::GetDb();

            if(version_compare($oldVersion, '2.1.41') < 0) {
                if($db instanceof Db_Adapter_mysql){
                    $differences = Application_Upgrader_Database::getDifferences();
                    
                    if(isset($differences['incorrect'])){
                        foreach($differences['incorrect'] as $tableName => $table){
                            if(isset($table['incorrect'])){
                                $tableObject = Application_Installer_Database::getTable($tableName);
                                
                                foreach($table['incorrect'] as $columnName => $message){
                                    //Change field type
                                    $field = $tableObject->getField($columnName);
                                    $allowNull = $field->getNull();
                                    $default = '';
                                    try{
                                        $default = $field->getDefault();
                                    }
                                    catch(Exception $e){}
                                    
                                    $alterTableSQL = sprintf('ALTER TABLE `%s` MODIFY `%s` MEDIUMTEXT%s%s', $tableName, $columnName, ($allowNull ? '' : ' NOT NULL '), (!empty($default)) ? "DEFAULT '" . $db->quote($default) . "'" : '');
                                    echo $alterTableSQL . "\n";
                                    $stmt = $db->query($alterTableSQL);
                                }
                            }
                        }
                    }
                }
            }
            if (version_compare($oldVersion, '2.1.26') < 0) {
                try {
                    $table = new Application_Installer_Database_Schema_UsersLinks($db);
                    if(!$table->exists()){
                        $table->create();
                    }
                }
                catch (Exception $ex) {
                }
            }
            if (version_compare($oldVersion, '2.1.6') < 0) {
                try {
                    $table = new Application_Installer_Database_Schema_ApplicationContentLocks($db);
                    $table->create();
                }
                catch (Exception $ex) {
                }
            }

            if (version_compare($oldVersion, '2.1.8') < 0) {
                $db->begin();

                $table = new Application_Installer_Database_Schema_ClientsDomains($db);
                $table->createField('ranking');

                $select = $db->select();
                $select->from(Application_Client_Domain::$table, '*')
                       ->order('domain_id');

                $result  = $db->query($select);
                $clients = array();

                while ($row = $result->fetch()) {
                    if (!array_key_exists($row->client_id, $clients)) {
                        $clients[$row->client_id] = array(
                            'primary' => 0,
                            'domains' => array(),
                            'dav'     => array()
                        );
                    }

                    $primary = Text::toBoolean($row->is_primary);
                    $dav     = Text::toBoolean($row->is_dav);

                    if ($dav) {
                        $clients[$row->client_id]['dav'][] = $row->domain_id;
                    }
                    else if ($primary && $clients[$row->client_id]['primary'] == 0) {
                        $clients[$row->client_id]['primary'] = $row->domain_id;
                    }
                    else {
                        $clients[$row->client_id]['domains'][] = $row->domain_id;
                    }
                }

                $query   = 'update ' . Application_Client_Domain::$table . ' set ranking = %d where domain_id = %d';
                $queries = array();

                foreach ($clients as $clientId => $domains) {
                    $ranking = 0;

                    if ($domains['primary'] > 0) {
                        $queries[] = sprintf($query, ++$ranking, $domains['primary']);
                    }

                    foreach ($domains['domains'] as $id) {
                        $queries[] = sprintf($query, ++$ranking, $id);
                    }

                    $ranking = 0;

                    foreach ($domains['dav'] as $id) {
                        $queries[] = sprintf($query, ++$ranking, $id);
                    }
                }

                foreach ($queries as $query) {
                    $db->query($query);
                }

                $table->dropField('is_primary');

                $table = new Application_Installer_Database_Schema_BrandsDomains($db);
                $table->createField('ranking');

                $select = $db->select();
                $select->from(Application_Brand_Domain::$table, '*')
                       ->order('domain_id');

                $result = $db->query($select);
                $brands = array();

                while ($row = $result->fetch()) {
                    if (!array_key_exists($row->brand_id, $brands)) {
                        $brands[$row->brand_id] = array(
                            'primary' => 0,
                            'domains' => array()
                        );
                    }

                    $primary = Text::toBoolean($row->is_primary);

                    if ($primary && $brands[$row->brand_id]['primary'] == 0) {
                        $brands[$row->brand_id]['primary'] = $row->domain_id;
                    }
                    else {
                        $brands[$row->brand_id]['domains'][] = $row->domain_id;
                    }
                }

                $query   = 'update ' . Application_Brand_Domain::$table . ' set ranking = %d where domain_id = %d';
                $queries = array();

                foreach ($brands as $brandId => $domains) {
                    $ranking = 0;

                    if ($domains['primary'] > 0) {
                        $queries[] = sprintf($query, ++$ranking, $domains['primary']);
                    }

                    foreach ($domains['domains'] as $id) {
                        $queries[] = sprintf($query, ++$ranking, $id);
                    }
                }

                foreach ($queries as $query) {
                    $db->query($query);
                }

                $table->dropField('is_primary');

                $db->commit();
            }

            if (version_compare($oldVersion, '2.1.14') < 0) {
                $db = Application::GetDb();
                $db->begin();

                $table = new Application_Installer_Database_Schema_Clients($db);
                $table->createField('status');

                $values = array(
                    'status' => Application_Client::STATUS_ACTIVE
                );

                $where = array(
                    'status is null'
                );

                $db->update(Application_Client::$table, $values, $where);

                $pm = Application_User_Permissions_Manager::getInstance();
                foreach (Application_Client::GetClients() as $client) {
                    $pm->deployDefaults($client);
                }

                $db->commit();
            }
        }
    }
?>