<?php
    /**
     * Application_Upgrader_Database
     *
     * Manage database upgrade tasks
     *
     * @package     Application
     */
    class Application_Upgrader_Database
    {
        public static function getDifferences()
        {
            $report = array(
                'missing'     => array(),
                'incorrect'   => array(),
                'extra'       => array(),
                'errors'      => array(),
                'unsupported' => array()
            );

            $db = Application::GetDb();

            // first get all tables
            try {
                foreach ($db->listTables() as $table) {
                    $report['extra'][$table] = $table;
                }

                $tables = Application_Installer_Database::getAllTables($db);

                foreach ($tables as $section) {
                    foreach ($section as $table) {
                        $name = (string) $table;

                        unset($report['extra'][$name]);

                        if ($table->exists()) {
                            if (!$table->isCorrect()) {
                                $report['incorrect'][$name] = $table->getDifferences();
                            }
                        }
                        else {
                            $report['missing'][$name] = array(
                                'name'  => $name,
                                'class' => get_class($table)
                            );
                        }

                        $report['unsupported'] = array_merge($report['unsupported'], $table->getUnsupported());
                    }
                }
            }
            catch (Exception $ex) {
                $report['errors'][] = $ex->getMessage();
                return $report;
            }

            $report['unsupported'] = array_unique($report['unsupported']);

            ksort($report['missing']);
            ksort($report['incorrect']);
            ksort($report['extra']);

/*
            if (count($report['missing']) > 0 || count($report['incorrect'])) {
                Application_SystemStatus::EnableMaintenanceMode();
            }
*/
            return $report;
        }

        public static function getMissingTables($db)
        {
            $tables = Application_Installer_Database::getAllTables($db);
            $ret = array();

            foreach ($tables as $section) {
                foreach ($section as $table) {
                    $name = (string) $table;

                    if (!$table->exists()) {
                        $ret[$name] = array(
                            'name'  => $name,
                            'class' => get_class($table)
                        );
                    }
                }
            }

            return $ret;
        }

        public static function createTables($db, $tables)
        {
            $missing = self::getMissingTables($db);

            $queue = array();

            foreach ($tables as $table) {
                if (!array_key_exists($table, $missing)) {
                    continue;
                }

                $class = $missing[$table]['class'];
                $queue[$table] = new $class($db);
            }

            if (count($queue) == 0) {
                throw new Exception('No tables to create');
            }

            $num   = 0;
            $tries = 0;

            $db = Application::GetDb();
            $db->begin();

            while (count($queue) > 0) {
                $table = array_shift($queue);

                if ($table->create()) {
                    $tries = 0;
                    $num++;
                }
                else {
                    $tries++;
                    $queue[] = $table;
                }

                if ($tries > count($queue)) {
                    $db->rollback();
                    throw new Exception(sprintf('Unable to create table %s', $table));
                }
            }

            $db->commit();

            return $num;
        }
    }
?>