<?php
    class Application_Upgrader_Package_Registered
    {
        protected $_data;
        protected $_version = array();
        protected $_uninstalled = false;

        public function __construct($name, $version)
        {
            $this->_name    = $name;
            $this->_version = $version;
        }

        public function __toString()
        {
            return sprintf('%s-%s', $this->_name, $this->_version);
        }

        public function getVersion()
        {
            return $this->_version;
        }

        public function getName()
        {
            return $this->_name;
        }

        public function setData($data)
        {
            $this->_data = $data;
            return $this;
        }

        public function getItemPaths()
        {
            if (!is_array($this->_data)) {
                return array();
            }

            $items = coalesce($this->_data['items']);

            $ret = array();

            foreach ($items as $item) {
                $basepath = realpath(sprintf('%s/%s', rtrim(Application::GetBaseDir(), '/'), trim($item['basedir'], '/')));

                if (strlen($basepath) == 0) {
                    throw new Exception('Invalid base path');
                }

                $fullpath = sprintf('%s/%s', $basepath, trim($item['path'], '/'));

                if (!preg_match('|^' . preg_quote($basepath, '|') . '/[^/]+/|', $fullpath)) {
                    continue;
                }

                $ret[] = array(
                    'path'     => trim(sprintf('%s/%s', trim($item['basedir'], '/'), trim($item['path'], '/')), '/'),
                    'fullpath' => $fullpath,
                    'preserve' => (bool) $item['preserve']
                );
            }

            return $ret;
        }

        public function removeItems()
        {
            foreach ($this->getItemPaths() as $path) {
                if (file_exists($path['fullpath'])) {
                    File::RmdirRecursive($path['fullpath']);
                }
            }
        }

        public function unregister()
        {
            $db = Application::GetDb();

            $where = array(
                $db->quoteInto('name = ?', $this->_name)
            );

            $db->delete(Application_Upgrader_Manager::REGISTRY_TABLE, $where);
        }


        /**
         * Return packages that depend on this package
         */
        public function getDependents()
        {
            // @todo implement this
            return array();
        }

        public function uninstall()
        {
            if ($this->_uninstalled) {
                return;
            }

            $this->removeItems();
            $this->unregister();

            $this->_uninstalled = true;
        }

        public function setChannel($name)
        {
            $this->_channel = new Application_Upgrader_Channel($name);
        }

        public function getChannel()
        {
            if (!$this->_channel instanceof Application_Upgrader_Channel) {
                throw new Exception('Channel not set');
            }

            return $this->_channel;
        }

        public static function GetPreservePaths()
        {
            $ret = array();

            foreach (Application_Upgrader_Manager::GetRegisteredPackages() as $package) {
                foreach ($package->getItemPaths() as $path) {
                    if ($path['preserve']) {
                        $ret[] = $path['path'];
                    }
                }
            }

            return $ret;
        }
    }
?>