<?php
    class Application_User_Cp_Tab_Editor extends FormProcessor
    {
        public $obj;
        public $layouts;
        public $templates;

        public $client;
        public $user;

        public function __construct($tab_id = 0)
        {
            $this->client = Application::GetClient();
            $this->user   = Application::GetUser();

            $this->obj = new Application_User_Cp_Tab();
            $this->obj->loadForClientAndUser($this->client->getId(), $this->user->getId(), $tab_id);

            if ($this->obj->saved()) {
                $this->title  = $this->obj->getTitle();
                $this->layout = $this->obj->getLayout();
            }
            else {
                $this->obj->user_id   = $this->user->getId();
                $this->obj->client_id = $this->client->getId();
                $this->layout = Application_Cp_Tab_Manager::GetDefaultLayout();
            }

            $this->layouts   = Application_Cp_Tab_Manager::GetCustomLayouts();
            $this->templates = Application_Cp_Tab_Manager::GetTemplates(true);
        }

        public function process(Zend_Controller_Request_Abstract $request)
        {
            $this->title = $this->sanitize($request->getPost('title'));

            if ($this->obj->saved()) {
                if (!$this->obj->isLegacy()) {
                    $this->layout = $request->getPost('layout');
                    $this->obj->setLayout($this->layout);
                }
            }
            else if ($request->getPost('type') == 'template') {
                $template = $request->getPost('template');

                if (array_key_exists($template, $this->templates)) {
                    $this->obj->fromTemplate($this->templates[$template]);

                    if (strlen($this->title) == 0) {
                        $this->title = (string) $this->templates[$template];
                    }
                }
                else {
                    $this->addError('template', 'Please select a layout');
                }
            }
            else {
                if (strlen($this->title) == 0) {
                    $this->addError('title', 'Please enter a title');
                }
            }

            if (!$this->hasError()) {
                $this->obj->setTitle($this->title);
                $this->obj->save();
            }

            return !$this->hasError();
        }

        public function saveOrder(Zend_Controller_Request_Abstract $request)
        {
            $options = array(
                'user_ids'   => $this->user->getId(),
                'client_ids' => $this->client->getId()
            );

            $tabs = Application_User_Cp_Tab::GetTabs($options);
            $ids  = ArrayTools::Clean($request->getPost('tab'));

            if (count($tabs) == 0 || count($tabs) != count($ids)) {
                throw new Exception('Invalid data received for tab sort');
            }

            foreach ($ids as $id) {
                if (!array_key_exists($id, $tabs)) {
                    throw new Exception('Unknown tab ID passed to tab sort (' . $id . ')');
                }
            }

            $db = Application::GetDb();

            $db->begin();
            $i = 0;
            foreach ($ids as $id) {
                $tabs[$id]->ranking = ++$i;
                $tabs[$id]->save();
            }
            $db->commit();

            return true;
        }
    }
?>
