<?php
    class Application_User_Editor extends FormProcessor
    {
        public $obj;
        public $role;
        public $directory;
        public $client;
        public $statuses;
        public $fields;
        public $roles = array();

        public function __construct($userId = 0, $roleId = 0, $clientId = 0)
        {
            $this->obj    = new Application_User();
            $this->role   = new Application_User_Role();
            $this->client = new Application_Client();
            $directory    = new Application_User_Directory();

            $this->obj->loadForDirectory(null, $userId);

            $this->statuses = Application_User::$statuses;
            $this->fields = $directory->getEditableProfileFields();

            if ($this->obj->saved()) {
                $this->username = $this->obj->username;
                $this->status   = $this->obj->status;

                foreach ($this->fields as $field) {
                    $name = $field->getName();
                    $this->set($name, $this->obj->profile->get($name));
                }
            }
            else {
                $this->status = Application_User::STATUS_ACTIVE;
                $this->role->load($roleId);

                if ($this->role->saved()) {
                    $this->client = $this->role->getClient();
                }
                else {
                    $this->client->load($clientId);
                }

                if ($this->client->saved()) {
                    $this->roles = $this->client->getRoles();
                }
            }
        }

        public function process(Zend_Controller_Request_Abstract $request)
        {
            $this->username = Text::sanitize($request->getPost('username'));
            $this->password = Text::sanitize($request->getPost('password'));

            if ($this->obj->saved() && Application_Auth::DIGEST_ENABLED && $this->username != $this->obj->username && strlen($this->password) == 0) {
                $this->addError('password', 'You must enter a new password to change the username');
            }
            else if (!$this->obj->saved() && strlen($this->password) == 0) {
                $this->addError('password', 'Please enter a password');
            }

            if (!$this->obj->saved() || $this->username != $this->obj->username) {
                if (strlen($this->username) == 0) {
                    $this->addError('username', 'Please enter a username');
                }
                else if (!$this->obj->check('username', $this->username)) {
                    $this->addError('username', 'Please enter a valid username');
                }
                else if ($this->obj->userExists($this->username)) {
                    $this->addError('username', 'This username is already in use');
                }
                else {
                    $this->obj->set('username', $this->username);
                }
            }

            if (strlen($this->password) > 0) {
                $this->obj->setPassword($this->password);
            }

            $this->status = Text::sanitize($request->getPost('status'));
            if (array_key_exists($this->status, $this->statuses)) {
                $this->obj->set('status', $this->status);
            }
            else {
                $this->addError('status', 'Please select a status');
            }

            $processor = $this->fields->process($request);

            foreach ($processor->getFieldList() as $field) {
                $this->obj->profile->set($field->getName(), $field->get());
                $this->set($field->getName(), $field->get());
            }

            foreach ($processor->getErrors() as $error) {
                $field = $error['field'];

                switch ($error['error']) {
                    case Profile_Field_FieldList_FormProcessor::ERROR_EMPTY:
                        $this->addError($field->getName(), sprintf('%s is required', $field->getTitle()));
                        break;
                    case Profile_Field_FieldList_FormProcessor::ERROR_INVALID:
                        $this->addError($field->getName(), sprintf('%s is invalid', $field->getTitle()));
                        $this->set($field->getName(), $error['value']);
                        break;
                    default:
                        $this->addError($field, 'Error in field');
                }
            }

            if ($this->client->saved() && count($this->roles) > 0 && !$this->role->saved()) {
                $this->addError('role_id', 'Please select the role');
            }

            if (!$this->hasError()) {
                $new = !$this->obj->saved();

                if ($new && $this->role->saved()) {
                    $db = Application::GetDb();
                    $db->begin();
                    $this->obj->save();
                    $this->obj->roleAdd($this->role->getId());
                    $db->commit();
                }
                else {
                    $this->obj->save();
                }
            }

            return !$this->hasError();
        }
    }
?>
