<?php
    class Application_User_ItemSelector extends Components_ItemSelector_Abstract
    {
        protected $_directory;

        public function __construct()
        {

        }


        public function _getDirectory(Components_ItemSelector_Options $options)
        {
            if (!$this->_directory instanceof Application_Directory) {
                if (preg_match('/^directory_(\d+)$/', $options->getNodeId(), $matches)) {
                    $directoryId = $matches[1];
                }
                else {
                    $directoryId = 0;
                }

                try {
                    $this->_directory = DatabaseObject_Registry::Load('Application_User_Directory', $directoryId);
                }
                catch (Exception $ex) {
                    $this->_directory = new Application_User_Directory();
                }
            }

            return $this->_directory;
        }

        public function getTitle(Components_ItemSelector_Options $options)
        {
            $directory = $this->_getDirectory($options);

            if ($directory->saved()) {
                return sprintf('Directory: %s', $directory);
            }
            else {
                return 'Directories';
            }
        }

        public function getSelectedItems(Components_ItemSelector_Options $options)
        {
            $userIds = array();

            foreach ($options->getSelectedIds() as $id) {
                if (preg_match('/^(user)_(\d+)$/', $id, $matches)) {
                    $userIds[] = (int) $matches[2];
                }
                else if (preg_match('/^\d+$/', $id)) {
                    $userIds[] = (int) $id;
                }
                else {
                    continue;
                }
            }

            $ret = array();

            if (count($userIds) > 0) {
                $ret = array();

                foreach ($userIds as $userId) {
                    Application::GetLogger()->debug($userId);
                    $user = new Application_User();
                    $user->load($userId);
                    
                    if($user->saved()){
                        $userTitle = (string) $user;
                        try{
                            $userEmail = $user->getEmailAddress();
                            $userTitle .= ' (' . $userEmail . ')'; 
                        }
                        catch(Exception $e){}
                        
                        $item = new Components_ItemSelector_Item('user_' . $user->getId());
                        $item->setTitle($userTitle)
                             ->setCanBeSelected(true)
                             ->setCanBeOpened(false);
    
                        $ret[] = $item;
                    }
                }
            }

            return $ret;
        }

        public function getItems(Components_ItemSelector_Options $options)
        {
            $ret = array();

            $directory = $this->_getDirectory($options);

            if (!$directory->saved()) {
                $opts = array('client_ids' => array(getClientId()));
                $directories = Application_User_Directory::GetDirectories($opts);
                foreach ($directories as $directory){
                    $item = new Components_ItemSelector_Item('directory_' . $directory->getId());
                    $item->setTitle((string) $directory)
                         ->setCanBeSelected(false)
                         ->setCanBeOpened(true);

                    $ret[] = $item;
                }
            }
            else {
                $opts = array(
                    'directory_ids' => array($directory->getId())
                );

                $item = new Components_ItemSelector_Item_Unselectable('directory_0');
                $item->setTitle('..');

                $ret[] = $item;
                
                foreach (Application_User::getUsers($opts) as $user) {
                    $userTitle = (string) $user;
                    try{
                        $userEmail = $user->getEmailAddress();
                        $userTitle .= ' (' . $userEmail . ')'; 
                    }
                    catch(Exception $e){}
                    
                    $item = new Components_ItemSelector_Item('user_' . $user->getId());
                    $item->setTitle($userTitle)
                         ->setCanBeSelected(true)
                         ->setCanBeOpened(false);

                    $ret[] = $item;
                }
            }

            return $ret;
        }

        public function getFilters()
        {
            return array();
        }
    }
?>
