<?php
    class Application_User_Permissions_Permission_Editor extends FormProcessor
    {
        public $role;
        public $directory;
        public $client;

        public $roles;
        public $permissions;

        public function __construct($request)
        {
            $this->role = new Application_User_Role();
            $this->role->load($request->getParam('role_id'));

            if ($this->role->saved()) {
                $directory_id = $this->role->directory_id;
                $client_id    = $this->role->client_id;
            }
            else {
                $directory_id = (int) $request->getParam('directory_id');
                $client_id    = (int) $request->getParam('client_id');
            }

            $this->directory = new Application_User_Directory();
            $this->directory->load($directory_id);

            $this->client = $this->directory->getClient();
            if (!$this->client->saved()) {
                $this->client->load($client_id);
            }

            $options = array();

            if ($this->directory->saved()) {
                $options['directory_ids'] = array($this->directory->getId());
            }
            else {
                $options['directory_ids'] = array(null);
            }

            if ($this->client->saved())
                $options['client_ids'] = $this->client->getId();

            $this->roles = Application_User_Role::GetRoles($options);

            $options = array();
            if ($this->client->saved())
                $options['client_ids'] = $this->client->getId();


            if ($request->isPost() && strlen($request->getPost('clear')) > 0) {
                $this->filter = '';
            }
            else {
                $this->filter = $this->sanitize($request->get('filter'));
            }



            if (strlen($this->filter) > 0)
                $options['filter'] = $this->filter;

            $this->permissions = Application_User_Permissions_Permission::GetPermissions($this->directory->getId(), $options);
        }

        public function getPermission($permission_id, $role_id)
        {
            if (!array_key_exists($permission_id, $this->permissions))
                return false;

            $val = $this->permissions[$permission_id]->getType($role_id);

            if ($val == Application_User_Permissions_Permission::TYPE_ALLOW)
                return 1;
            else if ($val == Application_User_Permissions_Permission::TYPE_DENY)
                return -1;

            return 0;
        }

        public function process(Zend_Controller_Request_Abstract $request)
        {
            if ($request->isXmlHttpRequest())
                return $this->processAjax($request);

            $name  = $this->sanitize($request->getPost('name'));
            $name  = str_replace('.', ':', $name);
            $parts = explode(':', $name);

            $permissions = array();

            foreach ($parts as $part) {
                $part = trim($part);
                if (strlen($part) > 0)
                    $permissions[] = Text::Urlize($part);
            }

            $numPermissions = count($permissions);

            if ($numPermissions == 0) {
                $this->addError('title', 'Please enter the permission title');
                return false;
            }

            $db = Application::GetDb();
            $db->begin();

            $path = array();
            foreach ($permissions as $i => $perm) {

                $path[] = $perm;
                $title  = join(':', $path);
                $last   = $i + 1 == $numPermissions;

                $permission = new Application_User_Permissions_Permission();
                if ($permission->loadForDirectoryClientTitle($this->directory->getId(), $this->client->getId(), $title)) {
                    if ($last) {
                        $db->rollback();
                        return;
                    }

                    continue;
                }

                $permission->directory_id = $this->directory->getId();
                $permission->client_id    = $this->client->getId();
                $permission->title        = $title;

                if ($last)
                    $permission->allow = false;
                else
                    $permission->allow = $request->getPost('default') == 'y';

                $permission->save();
            }

            // now map each role to the newly created main permission created as required

            foreach ($this->roles as $role) {
                $val = $request->getPost('role_' . $role->getId());

                if ($val == 'y')
                    $permission->allow($role->getId());
                else if ($val == 'n')
                    $permission->deny($role->getId());
                else
                    continue;
            }

            // all done
            $db->commit();

            $this->filter = $name;

            return !$this->hasError();
        }

        protected function processAjax($request)
        {
            $role_id       = $request->getPost('role');
            $permission_id = $request->getPost('permission');
            $type          = $request->getPost('type');

            $permission = new Application_User_Permissions_Permission();
            if (!$permission->load($permission_id))
                $this->addError('permission_id', 'Invalid permission specified');

            $role = new Application_User_Role();
            $role->load($role_id);

            if (!$role->saved() && $role_id != 'default')
                $this->addError('role_id', 'Invalid role specified');

            if ($this->hasError())
                return false;

            if ($role->saved()) {

                if ($type == 'y')
                    $permission->allow($role->getId());
                else if ($type == 'n')
                    $permission->deny($role->getId());
                else if ($type == 'i')
                    $permission->inherit($role->getId());
            }
            else {
                $permission->allow = $type == 'y';
                $permission->save();
            }

            return true;
        }
    }
?>
