<?php
    /**
     * Holds a single permissions query to be used with the
     * Application_User_Permissions_Query class
     *
     * @author  Quentin Zervaas <quentin@recite.com.au>
     */
    class Application_User_Permissions_Query_Item
    {
        /**
         * Holds the name of the permission being checked
         *
         * @var string
         */
        protected $_permissionName;

        public function __construct($permissionName = null)
        {
            $this->setPermissionName($permissionName);
        }

        public function setPermissionName($permissionName)
        {
            if (strlen($permissionName) > 0) {
                $this->_permissionName = (string) $permissionName;
            }

            return $this;
        }

        public function __toString()
        {
            return $this->_permissionName;
        }

        public function assert()
        {
            $acl = Application::GetAclManager();

            if (!$acl->isAllowed($this->_permissionName)) {
                throw new Application_User_Permissions_Exception($this);
            }
        }

        public function check()
        {
            try {
                $this->assert();
                return true;
            }
            catch (Exception $ex) {
                return false;
            }
        }

        public static function BuildAndAssert($permissionName)
        {
            $item = new self($permissionName);
            $item->assert();
        }

        public static function BuildAndCheck($permissionName)
        {
            $item = new self($permissionName);
            return $item->check();
        }
    }
?>