<?php
    class Application_User_Profile_Editor_Cp extends FormProcessor
    {
        public $obj = null;
        public $canEditClient = false;
        public $client;

        public function __construct(Application_User $user)
        {
            $this->obj = $user;
            $this->client = Application::GetClient();

            $this->username   = $this->obj->get('username');
            $this->first_name = $this->obj->profile->first_name;
            $this->last_name  = $this->obj->profile->last_name;
            $this->email      = $this->obj->profile->email;

            $this->canEditClient = Application_User_Permissions_Query_Item::BuildAndCheck('core:client:edit');

            if ($this->canEditClient) {
                $this->client_name = $this->client->client_name;
            }
        }

        public function process(Zend_Controller_Request_Abstract $request)
        {
            if ($this->canEditClient) {
                $this->client_name = $request->getPost('client_name');

                if (strlen($this->client_name) == 0) {
                    $this->addError('client_name', 'Please enter the client name');
                }
            }

            $this->first_name = Text::sanitize($request->getPost('first_name'));
            if (strlen($this->first_name) == 0) {
                $this->addError('first_name', 'Please enter your first name');
            }

            $this->last_name = Text::sanitize($request->getPost('last_name'));
            if (strlen($this->last_name) == 0) {
                $this->addError('last_name', 'Please enter your last name');
            }

            $this->email = Text::sanitize($request->getPost('email'));

            $validator = new Zend_Validate_EmailAddress();

            if (strlen($this->email) == 0) {
                $this->addError('email', 'Please enter your email address');
            }
            else if (!$validator->isValid($this->email)) {
                $this->addError('email', 'Please enter a valid email address');
            }

            $this->password = Text::sanitize($request->getPost('password'));

            if (!$this->hasError()) {
                $this->obj->profile->first_name = $this->first_name;
                $this->obj->profile->last_name = $this->last_name;
                $this->obj->profile->email = $this->email;

                if (strlen($this->password) > 0) {
                    $this->obj->setPassword($this->password);
                }

                $db = Application::GetDb();
                $db->begin();

                $this->obj->save();

                if ($this->canEditClient && $this->client_name != $this->client->client_name) {
                    $this->client->client_name = $this->client_name;
                    $this->client->save();
                }

                $db->commit();

                $identity = $this->obj->getPublicIdentityData();
                $identity['client_id'] = $this->client->getId();

                Zend_Auth::getInstance()->getStorage()->write($identity);
            }

            return !$this->hasError();
        }
    }
?>
