<?php
    class Application_User_Renderer extends DataRenderer
    {
        public $directory;

        public function __construct(Application_User_Directory $directory)
        {
            parent::__construct($directory->getDb());

            $this->directory = $directory;

            $this->getLimitOption()
                 ->setValues(array(0   => 'All',
                                   5   =>  5,
                                   10  =>  10,
                                   20  =>  20,
                                   50  =>  50,
                                   100 => 100,
                                   500 => 500))
                 ->setDefault(50);

            $clients = Application_Client::GetClients();

            $option = $this->addOption('client');
            $option->addValue(0, 'Any client')
                    ->setDefault(0);

            foreach ($clients as $client) {
                $option->addValue($client->getId(), (string) $client);
            }

            $this->addOption('order')
                 ->setValues(array('username'  => 'Username (A-Z)',
                                   'usernamed' => 'Username (Z-A)'))
                 ->setDefault('username');
        }

        public function getData(&$arr, DataRenderer_Pager $pager)
        {
            $options = $this->getOptions();
            $options['limit'] = (int) $pager->limit;
            $options['offset'] = (int) $pager->offset;

            switch ($this->getSetting('order')) {
                case 'usernamed':
                    $order = 'lower(u.username) desc';
                    break;

                case 'username':
                default:
                    $order = 'lower(u.username)';
            }

            $options['order'] = $order;

            $arr = Application_User::GetUsers($options);
        }

        public function getDataCount()
        {
            return Application_User::GetUsersCount($this->getOptions());
        }

        protected function getOptions()
        {
            $ret = array();

            if ($this->directory->saved()) {
                $ret['directory_ids'] = $this->directory->getId();
            }
            else {
                $ret['default_directory_only'] = true;
            }

            $clientId = (int) $this->getSetting('client');

            if ($clientId > 0) {
                $ret['client_ids'] = array($clientId);
            }

            return $ret;
        }
    }
?>
