<?php
    class Application_User_UsernameGenerator
    {
        const MIN_LENGTH = 5;

        // any usernames that have been generated are stored here so they won't be re-generated for the current request
        protected static $_generated = array();

        public static function Generate(Application_User_Directory $directory, $fieldList = null, $force = false)
        {
            if (!array_key_exists($directory->getId(), self::$_generated)) {
                self::$_generated[$directory->getId()] = array();
            }

            $username = '';

            // check for any name type fields
            if (strlen($username) == 0) {
                foreach ($fieldList as $field) {
                    $type = $field->getType();

                    if (!$type instanceof Driver_forms_fields_string) {
                        continue;
                    }

                    if (preg_match('/name/i', $field->getName())) {
                        if (strlen($field->get()) > 0) {
                            $values = array_filter(preg_split('/\s+/', $field->get()));

                            if (count($values) == 1) {
                                $username = $values[0];
                            }
                            else {
                                $username = sprintf('%s%s', $values[0]{0}, $values[1]);
                            }

                            break;
                        }
                    }
                }
            }

            // check for email fields
            if (strlen($username) == 0) {
                foreach ($fieldList as $field) {
                    $type = $field->getType();

                    if (!$type instanceof Driver_forms_fields_email) {
                        continue;
                    }

                    if ($field->hasError()) {
                        continue;
                    }

                    $email = $field->get();

                    if (strlen($email) == 0) {
                        continue;
                    }

                    if (preg_match('/^([^@]+)/', $email, $matches)) {
                        $username = $matches[1];
                        break;
                    }
                }
            }

            // ensure the username is valid characters
            $username = preg_replace('/[^a-z0-9]/', '', Text::Urlize($username));

            $obj = new Application_User();

            try {
                if (strlen($username) == 0) {
                    throw new Exception('Unable to generate a username');
                }


                // if username too short then append some random numbers
                if (strlen($username) < self::MIN_LENGTH) {
                    $username .= rand(1000, 9999);
                }

                // validate the username to ensure it can be used
                $_username = $username;

                $i = 0;
                while (1) {
                    if (!in_array($username, self::$_generated[$directory->getId()]) && !$obj->userExists($username, $directory->getId())) {
                        break;
                    }

                    if (++$i > 100) {
                        throw new Exception('Unable to generate a username');
                    }

                    $username = $_username . $i;
                }
            }
            catch (Exception $ex) {
                if (!$force) {
                    throw $ex;
                }

                while (1) {
                    $username = Text_Password::create(6) . rand(1, 100);

                    if (!in_array($username, self::$_generated[$directory->getId()]) && !$obj->userExists($username, $directory->getId())) {
                        break;
                    }
                }
            }

            self::$_generated[$directory->getId()][] = $username;

            return $username;
        }
    }
?>
