<?php
    abstract class Application_View extends Zend_View_Abstract
    {
        protected $_path;
        protected $_engine;

        protected $_defaultOptions = array('trimOutput'     => true,
                                           'createCacheDir' => true);

        protected $_config;

        protected $_default_template_handler_func = array('Application_View', '__getDefaultTemplate');
        protected $_default_template_handler_path = '';

        protected $_template;

        protected $_isSetup = false;

        public function __construct($options = array())
        {
            $this->_config = Zend_Registry::get('config')->templates;

            $options = ArrayTools::buildOptions($options, $this->_defaultOptions);

            require_once('Smarty.php');

            $this->_engine = new Smarty();
            $this->_engine->compiler_file = 'Smarty/Compiler.php';
            $this->_engine->template_dir  = $this->getTemplateDir();
            $this->_engine->compile_dir   = Application::getTmpDir() . '/templates_c';
            $this->_engine->config        = $this->_config;
            $this->_engine->view          = $this;

            if ($options['createCacheDir']) {
                $this->initCompileDir();
            }

            $this->APPLICATION_VERSION = Application::version();
            $this->APPLICATION_NAME    = Application::NAME;
        }

        public function getPluginDirs()
        {
            return array('Application/View', 'Smarty/plugins');
        }

        public function getEngine()
        {
            return $this->_engine;
        }

        public function compileToTempDir($flag = true)
        {
            if ($flag) {
                $this->_engine->compile_dir = File::GetSystemTmpDir();
            }
        }

        abstract function getTemplateDir();

        public function setDefaultTemplateDirectory($path)
        {
            if (strlen($path) == 0 || !file_exists($path) || !is_dir($path)) {
                return false;
            }

            $this->_default_template_handler_path = $path;
            $this->_engine->default_template_handler_func = $this->_default_template_handler_func;
        }

        public static function loadResource($engine, $name, $path = '')
        {
            if (strlen($path) == 0) {
                $path = dirname(__FILE__) . '/View/Resources';
            }

            $path = rtrim($path, '/');

            $fullpath = $path . sprintf('/%s.php', $name);

            if (!file_exists($fullpath))
                return;

            require_once($fullpath);

            $engine->register_resource($name, array('smarty_resource_' . $name . '_source',
                                                    'smarty_resource_' . $name . '_timestamp',
                                                    'smarty_resource_' . $name . '_secure',
                                                    'smarty_resource_' . $name . '_trusted'));
        }

        public static function __getDefaultTemplate($resource_type, $resource_name, &$template_source, &$template_timestamp, &$smarty)
        {
            $defaultPath = $smarty->view->_default_template_handler_path . '/' . $resource_name;

            if (file_exists($defaultPath) && is_readable($defaultPath)) {
                $template_source = file_get_contents($defaultPath);
                $template_timestamp = time();
                return true;
            }

            return false;
        }

        public function templateExists($tpl)
        {
            $fullpath = $this->_engine->template_dir . '/' . $tpl;
            $ret = file_exists($fullpath) && is_readable($fullpath);

            if (!$ret && strlen($this->_default_template_handler_path) > 0) {
                $fullpath = $this->_default_template_handler_path . '/' . $tpl;
                $ret = file_exists($fullpath) && is_readable($fullpath);
            }

            return $ret;
        }

        public function fetchFromString($str)
        {
            $this->_runLastMinuteSetup();

            require_once($this->_engine->_get_plugin_filepath('function', 'eval'));

            $ret = smarty_function_eval(array('var' => (string) $str), $this->_engine);

            // now process any output filters
            foreach ($this->_engine->_plugins['outputfilter'] as $filter) {
                $func = $filter[0];
                $ret = $func($ret, $this->_engine);
            }

            return $ret;
        }

        public function initCompileDir()
        {
            self::_InitCompileDir($this->_engine->compile_dir);

            if (!file_exists($this->_engine->compile_dir) || !is_writable($this->_engine->compile_dir)) {
                $this->_engine->force_compile = true;
            }
        }

        public static function _InitCompileDir($dir)
        {
            File::CreateDirStructure($dir);
        }

        public function __set($key, $val)
        {
            $this->_engine->assign($key, $val);
        }

        public function __get($key)
        {
            return $this->_engine->get_template_vars($key);
        }

        public function __isset($key)
        {
            return $this->_engine->get_template_vars($key) !== null;
        }

        public function __unset($key)
        {
            $this->_engine->clear_assign($key);
        }

        public function assign($spec, $value = null)
        {
            if (is_array($spec)) {
                $this->_engine->assign($spec);
                return;
            }

            $this->_engine->assign($spec, $value);
        }

        public function clearVars()
        {
            $this->_engine->clear_all_assign();
        }

        public function render($name)
        {
            $this->_runLastMinuteSetup();

            $name = str_replace('|', ':', $name); // used to deal with colons and ZF

            if (strlen($this->_template) > 0) {
                $name = $this->_template;
                $this->_template = null;
            }
            else {
                $name = preg_replace('/^module:default:/i', 'default/', $name);
            }

            return $this->_engine->fetch(strtolower($name));
        }

        public function getVars()
        {
            return $this->_engine->get_template_vars();
        }

        public function template($name)
        {
            $this->_template = $name;
        }

        public function display($name)
        {
            echo $this->render($name);
        }

        public function _run()
        { }

        protected function _runLastMinuteSetup()
        {
            if ($this->_isSetup) {
                return;
            }

            $this->_engine->plugins_dir = $this->getPluginDirs();
            $this->_isSetup = true;
        }
    }
?>