<?php
    class Application_View_Client extends Application_View
    {
        protected static $_containerRuleClient = null;

        protected static $_function_blacklist = array(
            'eval'
        );

        protected static $_globalTemplateData = array();

        protected static $_allowedFuncs;

        protected $_activeCache;

        public function __construct($options = array())
        {
            if (!is_array($options)) {
                $options = array();
            }

            $options['createCacheDir'] = false;

            parent::__construct($options);

            self::Configure($this->_engine, $options);

            // assign identity details to template
            try {
                $user = Application::GetUser();

                $this->__set('authenticated', true);
                $this->__set('identity', Zend_Auth::getInstance()->getIdentity());
            }
            catch (Exception $ex) {
                $this->__set('authenticated', false);
            }

            $client = Application::GetClient();

            $this->__set('domain', Application::GetCurrentDomain());
            $this->__set('_domain', Application::GetCurrentDomain());
            $this->__set('SETTINGS', Application_Client_Settings_Custom::GetClientTemplateData());
            $config = Application::getConfig();
            if(isset($config->settings)){
                $this->__set('CONFIG_SETTINGS', $config->settings->toArray());
            }
        }

        public function getTemplateDir()
        {
            return '';
        }

        public function setActiveCache(Zend_Cache_Core $cache)
        {
            $this->_cache = $cache;
            return $this;
        }

        public function getActiveCache()
        {
            if (!$this->_cache instanceof Zend_Cache_Core) {
                throw new Exception('Cache not set');
            }

            return $this->_cache;
        }

        public function templateExists($tpl)
        {
            return smarty_resource_clienttemplate_load_tpl($tpl, $this->_engine);
        }

        public function getPluginDirs()
        {
            return $this->_engine->plugins_dir;
        }

        protected function _runLastMinuteSetup()
        {
            if ($this->_isSetup) {
                return;
            }

            parent::_runLastMinuteSetup();

            $home    = false;
            $subHome = false;
            if (!Module_Pages_ClientRenderer::InErrorMode()) {
                try {
                    $page = Module_Pages_ClientRenderer::GetPageRequestTree()->getPage();

                    $home    = $page->isRootRecord();
                    $subHome = !$home && $page->isSubsiteRootRecord();
                }
                catch (Exception $ex) {
                }
            }
            $this->__set('home', $home);
            $this->__set('subHome', $subHome);
        }

        /**
         * Configure
         *
         * Configure an instance of Smarty so it can be used for client sites. This includes
         * setting up of plugins directory, compilation directory and security settings.
         * Additionally, this is used by the template validator since it also needs to know
         * about the security settings and plugins.
         */
        public static function Configure($smarty, $options = array())
        {
            $smarty->client_id = getClientId();

            $smarty->template_dir = Application::getBaseDir() . '/application/client/templates';
            $smarty->compile_dir  = Application_Client_Data::GetNamespacedPathForClient($smarty->client_id, 'templates/compiled');

            self::loadResource($smarty, 'clienttemplate');
            self::loadResource($smarty, 'client');

            $smarty->plugins_dir = array(
                './Smarty/plugins',
                'Application/View/Plugins/Client',
                sprintf('%s/templates/plugins', Application_Driver_Manager::GetPath())
            );

            $smarty->default_resource_type = 'clienttemplate';
            $smarty->default_template_handler_func = 'smarty_resource_create_template';

            $smarty->loadedTemplates = array();

            $smarty->security = true;
            $smarty->security_settings['IF_FUNCS'][] = 'strlen';
            $smarty->security_settings['ALLOW_CONSTANTS'] = true;

            foreach (self::GetAllowedFuncs() as $func) {
                $smarty->security_settings['MODIFIER_FUNCS'][] = $func;
            }

            self::_InitCompileDir($smarty->compile_dir);
        }

        public static function AddGlobalTemplateData($k, $v)
        {
            self::$_globalTemplateData[$k] = $v;
        }

        public static function WriteGlobalTemplateData(Application_View_Client $view, $container = '')
        {
            if (strlen($container) == 0) {
                foreach (self::$_globalTemplateData as $k => $v) {
                    $view->assign($k, $v);
                }
            }
            else {
                $view->assign($container, self::$_globalTemplateData);
            }
        }

        public static function GetAllowedFuncs()
        {
            if (!is_array(self::$_allowedFuncs)) {
                self::$_allowedFuncs = array();

                if (Zend_Registry::IsRegistered('brand')) {
                    $setting = Application::GetBrand()->getSetting('templates.security.allowed_functions');
                }
                else {
                    $setting = Application_Brand::GetStaticSetting(Application::GetClient()->brand_id, 'templates.security.allowed_functions');
                }

                foreach (preg_split('/[^a-z0-9_]+/', $setting) as $func) {
                    $func = strtolower(trim($func));

                    if (in_array($func, self::$_function_blacklist)) {
                        continue;
                    }

                    self::$_allowedFuncs[] = $func;
                }
            }

            return self::$_allowedFuncs;
        }
    }
?>
