<?php
    /**
     * This class is used to validate a Smarty template
     */

    require_once('Smarty/Compiler.php');
    require_once('Smarty.php');

    class Application_View_Validator extends Smarty_Compiler
    {
        public function __construct()
        {
            // *** danh - 09/07/2015 - correcting for PHP 5.6 changes ***
            // new smarty uses proper constructors, and thus old class named constructors don't work.
            // original left for historic/rollback reasons
            //$this->Smarty();
            //$this->Smarty_Compiler();
            Smarty::__construct();
            Smarty_Compiler::__construct();
            // *** danh - 09/07/2015 - correcting for PHP 5.6 changes ***
        }

        public function _initCompileDir()
        {

        }

        /**
         * This functions parses and validates the passed template, returning
         * an array of erros
         *
         * @param   string  $str        The template to validate
         * @param   array   $options    Validator run-time options
         *                  +validateResources   bool   Whether or not to validate include resources (e.g. from {include file='...'})
         * @return  array               The errors found. Empty array means valid template
         */
        public function validateTemplate($str, $options = array())
        {
            $er = error_reporting(0);

            $this->errors = array();
            $this->includes = array();

            if (!is_array($options))
                $options = array();

            $options['validateResources'] = isset($options['validateResources']) ? (bool) $options['validateResources'] : true;
            $options['self'] = isset($options['self']) ? $options['self'] : '';

            // first check the syntax is good
            $compiled_content = '';
            $this->_compile_file($options['self'], $str, $compiled_content);

                // we can't stop processing after a fatal error without
                // hacking Smarty so we just hide PHP errors instead.

            // now check included resources
            if ($options['validateResources']) {
                foreach ($this->includes as $include) {
                    if (!array_key_exists('file', $include))
                        continue;

                    $resource_name = substr(substr($include['file'], 1), 0, -1); // strips the quotes off each end
                    if ($resource_name == $options['self'])
                        continue;
                    $params = array('resource_name' => $resource_name);
                    $info = $this->_fetch_resource_info($params);
                }
            }

            $this->errors = array_unique($this->errors); // remove duplicate errors

            foreach ($this->errors as $k => $v) {
                $v = str_replace('Unable to read resource:', 'Included template not found:', $v);
                $v = str_replace('[plugin] (secure mode)', '', $v);
                $this->errors[$k] = ucfirst(trim($v));
            }

            error_reporting($er);

            return $this->errors;
        }

        /**
         * Intercept resource calls so we can check them later
         *
         * @access private
         */
        function _compile_include_tag($tag_args)
        {
            $this->includes[] = $this->_parse_attrs($tag_args);
            return parent::_compile_include_tag($tag_args);
        }

        /**
         * Add the passed error to the return array
         *
         * @access private
         */
        function trigger_error($error_msg, $error_type = E_USER_WARNING)
        {
            $this->errors[] = ucfirst($error_msg);
        }

        /**
         * Handle a fatal error, such as a syntax error
         *
         * In normal Smarty, execution stops at this point, but
         * there's no way for us to do this and still return all errors
         * and validate resources, so just keep going. This may result in
         * extra irrelevant errors that would be resolved just from
         * fixing the initial fatal error
         *
         * @access private
         */
        function _trigger_fatal_error($error_msg, $tpl_file = null, $tpl_line = null,
                $file = null, $line = null, $error_type = E_USER_ERROR)
        {
            if (isset($tpl_line))
                $info = ' (line ' . $tpl_line . ')';
            else
                $info = '';

            $this->trigger_error($error_msg . $info, $error_type);

            // need a way to stop processing here ... ?
        }
    }
?>
