<?php
    /**
     * ArrayTools
     *
     * Various utility functions to manipulate arrays
     *
     * @author  Quentin Zervaas
     */
    class ArrayTools
    {
        /**
         * clean
         *
         * Clean empty and duplicate entries from an array. Additionally,
         * if the passed in value is not an array it is made into an array.
         * Values can also be turned into ints if it's expected that all
         * values be ints.
         *
         * @since   v1.0.4
         * @param   array   $arr    The array to clean
         * @param   bool    $ints   Optional. True if all array data is expected to be ints
         * @return  array           The cleaned array
         */
        public static function clean($arr, $ints = false)
        {
            if (!is_array($arr))
                $arr = array($arr);

            if ($ints) {
                $new = array();
                foreach ($arr as $v)
                    $new[] = (int) $v;

                return array_unique(array_filter($new));
            }
            return array_unique(array_filter($arr));
        }

        public static function buildOptions($options, $defaults)
        {
            if (!is_array($options))
                $options = array();

            foreach ($defaults as $k => $v) {
                if (!isset($options[$k])) {
                    $options[$k] = $v;
                }
            }

            return $options;
        }

        public static function buildOptionsRecursive($options, $defaults)
        {
            self::_buildOptionsRecursive($options, $defaults);

            return $options;
        }

        protected static function _buildOptionsRecursive(&$options, $defaults)
        {
            foreach ($defaults as $k => $v) {
                if (is_array($v)) {
                    if (!isset($options[$k]))
                        $options[$k] = array();

                    if (isset($options[$k]) && is_array($options[$k]))
                        self::_buildOptionsRecursive($options[$k], $defaults[$k]);
                }
                else if (!array_key_exists($k, $options))
                    $options[$k] = $v;
            }
        }

        public static function arrayDiffAssocRecursive($a, $b)
        {
            $diff = array();
            foreach($a as $k => $v) {
                if (is_array($v)) {
                    if (!array_key_exists($k, $b) ||!is_array($b[$k]))
                        $diff[$k] = $v;
                    else {
                        $_diff = self::arrayDiffAssocRecursive($v, $b[$k]);
                        if (count($_diff) > 0)
                            $diff[$k] = $_diff;
                    }
                }
                else if (array_key_exists($k, $b) || $b[$k] != $v)
                    $diff[$k] = $v;
            }

            return $diff;
        }

        public static function ToQueryString($arr, $filterEmpty = false)
        {
            $tuples = array();
            foreach ($arr as $k => $v) {
                if (is_array($v)) {
                    if (!$filterEmpty || count($v) > 0) {
                        foreach ($v as $_v)
                            $tuples[] = sprintf('%s[]=%s', $k, $_v);
                    }
                }
                else if (strlen($v) > 0)
                    $tuples[] = sprintf('%s=%s', $k, $v);
            }

            return join('&', $tuples);
        }

    }
?>