<?php
    class Calendar_Tools
    {
        const SECONDS_IN_DAY = 86400;

        public static function GetStartOfMonth($ts)
        {
            return mktime(0, 0, 0, date('m', $ts), 1, date('Y', $ts));
        }

        public static function GetStartOfDay($ts)
        {
            return mktime(0, 0, 0, date('m', $ts), date('j', $ts), date('Y', $ts));
        }

        public static function GetEndOfDay($ts)
        {
            return self::GetStartOfDay($ts) + 86399;
        }

        public static function GetDaysFromRange($from, $to)
        {
            $from = self::GetStartOfDay($from);
            $to = max($from, $to);

            $ret = array();

            for ($i = $from; $i <= $to; $i += self::SECONDS_IN_DAY)
                $ret[$i] = $i;

            return $ret;
        }

        public static function ProcessDate($y, $m, $d, $hr, $min, $mer)
        {
            $y = (int) $y;
            $m = max(1, min(12, (int) $m));
            $d = max(1, min(31, (int) $d));

            if (!checkDate($m, $d, $y))
                return -1;

            $hr = max(1, min(12, (int) $hr));
            $min = max(0, min(59, (int) $min));
            $mer = strtolower($mer);
            if ($mer != 'am')
                $mer = 'pm';

            if ($mer == 'am') {
                if ($hr == 12) $hr = 0;
            }
            else {
                if ($hr != 12) $hr += 12;
            }

            return mktime($hr, $min, 0, $m, $d, $y);
        }

        public static function Duration($secs)
        {
            $secs = abs($secs);

            $vals = array(
                'week'   => (int) ($secs / 86400 / 7),
                'day'    => $secs / 86400 % 7,
                'hour'   => $secs / 3600 % 24,
                'minute' => $secs / 60 % 60,
                'second' => $secs % 60
            );

            $ret = array();

            $added = false;
            $i = 0;
            $numVals = count($vals);
            foreach ($vals as $k => $v) {
                $last = ++$i == $numVals;

                if ($v > 0 || $added || ($last && !$added)) {
                    $added = true;
                    $ret[] = $v . substr($k, 0, 1);
                }
            }
            return join(' ', $ret);
        }

        public static function GetDateRange($from, $to, $group)
        {
            $ret = array();

            switch ($group) {
                case 'daily':
                    $interval = Zend_Date::DAY;
                    $format   = 'YYYY-MM-dd';
                    break;

                case 'monthly':
                default:
                    $interval = Zend_Date::MONTH;
                    $format   = 'YYYY-MM';

                    $from = substr($from, 0, 7) . '-01';
                    $to   = substr($to, 0, 7) . '-01';

                    break;
            }


            $from = new Zend_Date($from, Zend_Date::ISO_8601);
            $to   = new Zend_Date($to, Zend_Date::ISO_8601);

            while ($from->isEarlier($to) || $from->equals($to)) {
                $date = $from->toString($format);
                $ret[$date] = $date;

                $from->add(1, $interval);
            }

            return $ret;
        }
    }
?>