<?php
    abstract class DataRenderer implements Iterator
    {
        protected $_limitOption = 'limit';
        protected $_pageOption  = 'page';

        protected $db;

        public $pager;
        public $data = array();
        protected $_options = array();

        public function __construct($db)
        {
            $this->db = $db;

            $option = new DataRenderer_Option($this->_pageOption);
            $option->setDefault(1);
            $this->addOption($option);

            $option = new DataRenderer_Option($this->_limitOption);
            $option->setValues(array(0   => 'All',
                                     5   =>  5,
                                     10  =>  10,
                                     20  =>  20,
                                     50  =>  50,
                                     100 => 100,
                                     500 => 500))
                   ->setDefault(10);

            $this->addOption($option);
        }

        protected function getLimitOption()
        {
            return $this->_options[$this->_limitOption];
        }

        public function run(Zend_Controller_Request_Http $request)
        {
            $this->_buildOptions($request);
            $this->_buildData();
        }

        abstract public function getData(&$arr, DataRenderer_Pager $pager);
        abstract public function getDataCount();

        protected function _buildData()
        {
            $count = $this->getDataCount();

            if ($count < 0) {
                $limit = 0;
                $page  = 1;
            }
            else {
                $limit = $this->getSetting($this->_limitOption);
                $page  = $this->getSetting($this->_pageOption);
            }

            $this->pager = new DataRenderer_Pager();
            $this->pager->init($count, $limit, $page);
            $this->pager->setUrl($this->getPagerQueryString(false, true));

            $this->data = array();
            $this->getData($this->data, $this->pager);
        }

        protected function _buildOptions(Zend_Controller_Request_Http $request)
        {
            foreach ($this->_options as $name => $option) {
                $option->process($request);
            }
        }

        public function addOption($option)
        {
            if (!$option instanceof DataRenderer_Option)
                $option = new DataRenderer_Option($option);

            $this->_options[$option->getName()] = $option;

            return $option;
        }

        public function getOption($name)
        {
            return $this->hasSetting($name) ? $this->_options[$name] : null;
        }

        public function getSetting($name)
        {
            return $this->hasSetting($name) ? $this->_options[$name]->get() : null;
        }

        public function getValues($name)
        {
            return $this->hasSetting($name) ? $this->_options[$name]->getValues() : array();
        }

        public function hasSetting($name)
        {
            return array_key_exists($name, $this->_options);
        }

        public function getSettings($page = false)
        {
            $parts = array();

            foreach ($this->_options as $name => $option) {
                $parts[$name] = $option->get();
            }

            if (!$page)
                unset($parts[$this->_pageOption]);

            return $parts;
        }

        public function getPagerQueryString($page = false, $pagePlaceHolder = false)
        {
            $parts = $this->getSettings($page);

            if (!$page && $pagePlaceHolder)
                $parts[$this->_pageOption] = '%d';

            return ArrayTools::toQueryString($parts);
        }

        /**
         * Iterator methods
         */

        public function rewind()
        {
            reset($this->data);
        }

        public function current()
        {
            return current($this->data);
        }

        public function key()
        {
            return key($this->data);
        }

        public function next()
        {
            return next($this->data);
        }

        public function valid()
        {
            return $this->current() !== false;
        }

        public function size()
        {
            return count($this->data);
        }
    }
?>
