<?php
    class DatabaseObject_Registry
    {
        protected static $_registry = array();

        public static function ClearAll()
        {
            self::$_registry = array();
        }
        
        public static function Load($class, $id)
        {
            if (self::IsRegistered($class, $id)) {
                return self::Get($class, $id);
            }

            if (!class_exists($class)) {
                throw new DatabaseObject_Exception('Invalid class: ' . $class);
            }

            $obj = new $class();
            $obj->load($id);

            if (!$obj->saved()) {
                throw new DatabaseObject_Exception('Unable to load record');
            }

            return self::Register($obj);
        }

        public static function Register($obj)
        {
            if (!$obj instanceof DatabaseObject) {
                throw new DatabaseObject_Exception('Invalid object');
            }

            $class = self::SanitizeClassName(get_class($obj));

            if (!array_key_exists($class, self::$_registry)) {
                self::$_registry[$class] = array(
                    'ids'  => array(),
                    'keys' => array()
                );
            }

            self::$_registry[$class]['ids'][$obj->getId()] = $obj;

            return $obj;
        }

        public static function IsRegistered($class, $id)
        {
            $class = self::SanitizeClassName($class);

            if (!array_key_exists($class, self::$_registry))
                return false;

            return array_key_exists($id, self::$_registry[$class]['ids']);
        }

        public static function Unregister($class, $id)
        {
            $class = self::SanitizeClassName($class);

            if (self::IsRegistered($class, $id))
                unset(self::$_registry[$class][$id]);
        }

        public static function Get($class, $id)
        {
            if (!self::IsRegistered($class, $id)) {
                throw new DatabaseObject_Exception('Object not found in registry');
            }

            $class = self::SanitizeClassName($class);

            return self::$_registry[$class]['ids'][$id];
        }

        public static function SanitizeClassName($class)
        {
            return strtolower($class);
        }
    }
?>
