<?php
    /**
     * Db_Schema_Table_Field
     *
     * @package     Db
     */
    class Db_Schema_Table_Field
    {
        const TYPE_SERIAL      = 'serial';
        const TYPE_VARCHAR     = 'varchar';
        const TYPE_TEXT        = 'text';
        const TYPE_INT         = 'int';
        const TYPE_BOOL        = 'boolean';
        const TYPE_TIMESTAMPTZ = 'timestamptz';
        const TYPE_REAL        = 'real';
        const TYPE_DOUBLE      = 'double precision';
        const TYPE_FK          = 'fk'; // special type for dynamic typing of foreign keys

        protected $_db;
        protected $_name;
        protected $_type;
        protected $_precision;
        protected $_null;
        protected $_default;
        protected $_pk = false;
        protected $_fk = false;

        public function __construct($db, $name)
        {
            $this->_db   = $db;
            $this->_name = $name;
        }

        public function __toString()
        {
            return (string) $this->_name;
        }

        public function getName()
        {
            return $this->_name;
        }

        public function setType($type)
        {
            if ($type == self::TYPE_FK)
                $this->setFk(true);

            $this->_type = $type;
            return $this;
        }

        public function getType()
        {
            return $this->_type;
        }

        public function setPrecision($precision)
        {
            $this->_precision = $precision;
            return $this;
        }

        public function getPrecision()
        {
            if (strlen($this->_precision) == 0) {
                throw new Db_Exception('Precision is not set');
            }

            return $this->_precision;
        }

        public function setPk($flag = true)
        {
            $this->_pk = (bool) $flag;
        }

        public function isPk()
        {
            return $this->_pk;
        }

        public function setFk($flag = true)
        {
            $this->_fk = (bool) $flag;
        }

        public function isFk()
        {
            return $this->_fk;
        }

        public function setNull($flag)
        {
            $this->_null = (bool) $flag;
            return $this;
        }

        public function getNull()
        {
            return $this->_null;
        }

        public function setDefault($default)
        {
            $this->_default = $default;
            return $this;
        }

        public function getDefault()
        {
            if (strlen($this->_default) == 0) {
                throw new Db_Exception('Default is not set');
            }

            return $this->_default;
        }


        /**
         * create
         *
         * Used to create this field only in the given table
         *
         * @param   Db_Schema_Table_Abstract    $table  The table in which to create this column
         */
        public function create($table)
        {
            $db = $table->getDb();

            $manager = $db->getManagerHelper();
            $manager->fieldCreate($table, $this);
        }

        public function assertIsCorrect($db, $description)
        {
            $manager = $db->getManagerHelper();
            $manager->assertFieldIsCorrect($this, $description);
        }

        public static function CastToType($name, $type)
        {
            return $name . '::' . $type;
        }
    }
?>