<?php
    abstract class Db_Statement_Helper_Abstract implements Db_Statement_Helper_Interface
    {
        protected $_db;

        public function __construct(Db_Adapter_Abstract $db)
        {
            $this->_db = $db;
        }

        public function getRandomExpr()
        {
            return new Zend_Db_Expr('random()');
        }

        public function getTimestamp($ts)
        {
            return date('Y-m-d H:i:s', $ts);
        }

        public function unixTimestamp($val)
        {
            return is_null($val) ? null : strtotime($val);
        }

        public function getOlderThanExpression($field, $qty, $unit, $orEqual = false)
        {
            return new Zend_Db_Expr(
                sprintf(
                    "%s + interval '%d %s' %s now()",
                    $field,
                    $qty,
                    self::GetIntervalUnit($unit),
                    $orEqual ? '<=' : '<'
                )
            );
        }

        public function getYoungerThanExpression($field, $qty, $unit, $orEqual = false)
        {
            return new Zend_Db_Expr(
                sprintf(
                    "%s + interval '%d %s' %s now()",
                    $field,
                    $qty,
                    self::GetIntervalUnit($unit),
                    $orEqual ? '>=' : '>'
                )
            );
        }

        public static function GetIntervalUnit($unit)
        {
            switch ($unit) {
                case Zend_Date::SECOND:
                    return 'second';
                case Zend_Date::MINUTE:
                    return 'minute';
                case Zend_Date::HOUR:
                    return 'hour';
                case Zend_Date::DAY:
                    return 'day';
                case Zend_Date::MONTH:
                    return 'month';
                case Zend_Date::YEAR:
                    return 'year';
            }

            throw new Exception('Invalid unit type');
        }

        public function getDateString($field)
        {
            return new Zend_Db_Expr(sprintf("to_char(%s, 'yyyy-mm-dd')", $field));
        }

        public function getDateTimeString($field)
        {
            return new Zend_Db_Expr(sprintf("to_char(%s, 'yyyy-mm-dd HH24:MI:SS')", $field));
        }
    }
?>