<?php
    interface Db_Statement_Helper_Interface
    {
        /**
         * Constructor. Requires the database connection
         *
         * @param   Db_Adapter_Abstract $db
         */
        public function __construct(Db_Adapter_Abstract $db);

        /**
         * Get the expression used for generating randomness
         *
         * @return  Zend_Db_Expr
         */
        public function getRandomExpr();


        /**
         * Get a string version of a unix timestamp compatible for use in database
         *
         * @return  string
         */
        public function getTimestamp($ts);


        /**
         * Get a unix timestamp from a database column value
         */
        public function unixTimestamp($val);


        /**
         * Get an expression to check that a column is at least a certain age
         *
         * @param   string  $field  The column to check
         * @param   int     $qty    The age quantity
         * @param   string  $unit   The age unit (e.g. minutes). Possible values include:
         *                          - Zend_Date::SECOND
         *                          - Zend_Date::MINUTE
         *                          - Zend_Date::HOUR
         *                          - Zend_Date::DAY
         *                          - Zend_Date::MONTH
         *                          - Zend_Date::YEAR
         * @param   bool    $orEqual    If true then can be the same age
         *
         * @return  Zend_Db_Expr
         */
        public function getOlderThanExpression($field, $qty, $unit, $orEqual = false);


        /**
         * Get an expression to check that a column is no more than a certain age
         *
         * @param   string  $field  The column to check
         * @param   int     $qty    The age quantity
         * @param   string  $unit   The age unit (e.g. minutes). Possible values include:
         *                          - Zend_Date::SECOND
         *                          - Zend_Date::MINUTE
         *                          - Zend_Date::HOUR
         *                          - Zend_Date::DAY
         *                          - Zend_Date::MONTH
         *                          - Zend_Date::YEAR
         * @param   bool    $orEqual    If true then can be the same age
         *
         * @return  Zend_Db_Expr
         */
        public function getYoungerThanExpression($field, $qty, $unit, $orEqual = false);


        /**
         * Get a string used to fetch a date in YYYY-MM-DD format
         *
         * @param   string  $field  The field to retrieve the date from
         * @return  Zend_Db_Expr
         */
        public function getDateString($field);


        /**
         * Get a string used to fetch a date in YYYY-MM-DD HH::MM:SS format
         *
         * @param   string  $field  The field to retrieve the date from
         * @return  Zend_Db_Expr
         */
        public function getDateTimeString($field);
    }
?>