<?php
class Db_Statement_Manager_mysqli extends Db_Statement_Manager_Abstract
{
    const TEXT_INDEX_LENGTH = 255;

    public function getTableCreateSql(Db_Schema_Table_Abstract $table) {
        $ret = parent::getTableCreateSql($table);
        $ret[0] .= ' ENGINE=InnoDB';

        return $ret;
    }

    public function getFieldTypeName($type, $pk = false, $fk = false) {
        switch ($type) {
            case Db_Schema_Table_Field::TYPE_TIMESTAMPTZ:
                return 'DATETIME';

            case Db_Schema_Table_Field::TYPE_SERIAL:
                if ($fk) {
                    return 'BIGINT UNSIGNED';
                }

            case Db_Schema_Table_Field::TYPE_INT:
                if ($fk) {
                    return 'BIGINT UNSIGNED';
                }
                break;
            case Db_Schema_Table_Field::TYPE_TEXT:
                return 'MEDIUMTEXT';
        }

        return parent::getFieldTypeName($type);
    }

    public function getFieldTypeNameForComparison($type, $pk = false, $fk = false) {
        switch ($type) {
            case Db_Schema_Table_Field::TYPE_TIMESTAMPTZ:
                return 'DATETIME';

            case Db_Schema_Table_Field::TYPE_SERIAL:
                return 'BIGINT UNSIGNED';

            case Db_Schema_Table_Field::TYPE_BOOL:
                return 'TINYINT';

            case Db_Schema_Table_Field::TYPE_DOUBLE:
            case Db_Schema_Table_Field::TYPE_REAL:
                return 'DOUBLE';

            case Db_Schema_Table_Field::TYPE_INT:
                if ($fk) {
                    return 'BIGINT UNSIGNED';
                }
                break;
            case Db_Schema_Table_Field::TYPE_TEXT:
                return 'MEDIUMTEXT';
        }

        return $type;
    }

    public function getIndexableField(Db_Schema_Table_Abstract $table, $fieldName) {
        $field = $table->getField($fieldName);

        if ($field->getType() == Db_Schema_Table_Field::TYPE_TEXT) {
            $ret = sprintf('%s(%d)', $fieldName, self::TEXT_INDEX_LENGTH);
        }
        else {
            $ret = $fieldName;
        }

        return $ret;
    }

    public function fieldCreate(Db_Schema_Table_Abstract $table, Db_Schema_Table_Field $field) {
        $this->_db->begin();

        $query = sprintf(
            'ALTER TABLE %s ADD %s',
            $this->_db->quoteIdentifier($table),
            $this->getFieldCreateSql($field, true)
        );

        $this->_db->query($query);

        try {
            $default = $this->getFieldCreateDefaultValue($field);

            $values = array(
                $field->getName() => $default
            );

            $where = array(
                $this->_db->quoteIdentifier($field->getName()) . ' is null'
            );

            $this->_db->update($table, $values, $where);

            if (!$field->getNull()) {
                $query = sprintf(
                    'ALTER TABLE %s MODIFY %s %s NOT NULL',
                    $this->_db->quoteIdentifier($table),
                    $this->_db->quoteIdentifier($field),
                    $this->getFieldCreateSql($field, true, false)
                );

                $this->_db->query($query);
            }
        }
        catch (Exception $ex) {
        }

        $this->_db->commit();
    }

    public function fieldMakeNullable($table, Db_Schema_Table_Field $field) {
        $query = sprintf(
            'ALTER TABLE %s MODIFY %s %s',
            $this->_db->quoteIdentifier($table),
            $this->_db->quoteIdentifier($field),
            $this->getFieldCreateSql($field, true, false)
        );

        $this->_db->query($query);
    }

    public function fieldMakeNotNullable($table, Db_Schema_Table_Field $field) {
        $default = $this->getFieldCreateDefaultValue($field, true);

        $values = array(
            $field->getName() => $default
        );

        $where = array(
            $this->_db->quoteIdentifier($field->getName()) . ' is null'
        );

        $this->_db->update($table, $values, $where);

        $query = sprintf(
            'ALTER TABLE %s MODIFY %s %s NOT NULL',
            $this->_db->quoteIdentifier($table),
            $this->_db->quoteIdentifier($field),
            $this->getFieldCreateSql($field, true, false)
        );

        $this->_db->query($query);
    }

    public function tablePkRebuild(Db_Schema_Table_Abstract $table) {
        $items = array();

        foreach ($table->getPks() as $fieldName) {
            $items[] = $this->getIndexableField($table, $fieldName);
        }

        if (count($items) == 0) {
            return;
        }

        $query = sprintf(
            'ALTER TABLE %s DROP PRIMARY KEY, ADD PRIMARY KEY (%s)',
            $this->_db->quoteIdentifier($table->getName()),
            join(', ', $items)
        );

        $this->_db->query($query);
    }

    public function tableGetIndexes($table) {
        $query = sprintf('show indexes from %s', $table);

        $ret = array();

        foreach ($this->_db->fetchAll($query) as $row) {
            $name = $row['Key_name'];

            if (!array_key_exists($name, $ret)) {
                $ret[$name] = array(
                    'name'    => $name,
                    'columns' => array()
                );
            }

            $ret[$name]['columns'][] = $row['Column_name'];
        }

        return $ret;
    }

    public function tableDrop($table) {
        $query = sprintf("DROP TABLE %s", $this->_db->quoteIdentifier((string) $table));
        $this->_db->query($query);
    }
}
