<?php
    class Db_Statement_Manager_postgresql extends Db_Statement_Manager_Abstract
    {
        public function getFieldTypeName($type, $pk = false, $fk = false)
        {
            switch ($type) {
                case Db_Schema_Table_Field::TYPE_SERIAL:
                    if ($fk) {
                        return 'INT';
                    }
            }

            return parent::getFieldTypeName($type);
        }

        public function getFieldTypeNameForComparison($type, $pk = false, $fk = false)
        {
            switch ($type) {
                case Db_Schema_Table_Field::TYPE_INT:
                case Db_Schema_Table_Field::TYPE_SERIAL:
                    return 'INT4';

                case Db_Schema_Table_Field::TYPE_REAL:
                    return 'FLOAT4';

                case Db_Schema_Table_Field::TYPE_DOUBLE:
                    return 'FLOAT8';
            }

            return $type;
        }

        public function tablePkCreate(Db_Schema_Table_Abstract $table)
        {
            $pks = $table->getPks();

            // create the new primary key
            if (count($pks) > 0) {
                $query = sprintf(
                    'ALTER TABLE %s ADD PRIMARY KEY (%s)',
                    $this->_db->quoteIdentifier($table->getName()),
                    join(', ', $pks)
                );
                $this->_db->query($query);
            }
        }

        public function tablePkDrop(Db_Schema_Table_Abstract $table)
        {
            $name     = $table->getName();
            $sequence = $this->_db->getTablePkSequence($name);

            $query = sprintf('ALTER TABLE %s DROP CONSTRAINT %s', $this->_db->quoteIdentifier($name), $sequence);
            $this->_db->query($query);
        }

        public function tableGetIndexes($table)
        {
            $query = <<<EOF
select
    i.relname as index_name,
    a.attname as column_name
from
    pg_class t,
    pg_class i,
    pg_index ix,
    pg_attribute a
where
    t.oid = ix.indrelid
    and i.oid = ix.indexrelid
    and a.attrelid = t.oid
    and a.attnum = ANY(ix.indkey)
    and t.relkind = 'r'
    and t.relname = '%s'
order by
    t.relname,
    i.relname;
EOF;
            $query = sprintf($query, $table);

            $ret = array();

            foreach ($this->_db->fetchAll($query) as $row) {
                $idx = $row['index_name'];

                if (!array_key_exists($idx, $ret)) {
                    $ret[$idx] = array(
                        'name' => $idx,
                        'columns' => array()
                    );
                }

                $ret[$idx]['columns'][] = $row['column_name'];
            }

            return array_values($ret);
        }

        public function tableDrop($table)
        {
            $query = sprintf("DROP TABLE %s", $this->_db->quoteIdentifier((string) $table));
            $this->_db->query($query);
        }
    }
?>