<?php


class Db_Statement_MySQLi extends Db_Statement_Abstract {
    protected $_result;
    protected $_keys;
    protected $_values;
    protected $_sql;

    /**
     * This is basically duplicated from the parent class, but the code to extract bound
     * parameters is removed since there's no way for us to bind parameters using this
     * mysql interface. mysqli would be needed for this (for which there's already an
     * existing class, except php needs to be built with mysqli support), or we could
     * use pdo_mysql, which also requires different configuration
     */
    public function __construct($adapter, $sql) {
        $this->_adapter = $adapter;
        if ($sql instanceof Zend_eb_Select) {
            $sql = $sql->__toString();
        }
        $this->_prepare($sql);

        $this->_queryId = $this->_adapter->getProfiler()->queryStart($sql);
    }

    public function _prepare($sql) {
        $this->_sql = $sql;
    }

    protected function _bindParam($parameter, &$variable, $type = null, $length = null, $options = null) {
        return true;
    }

    public function closeCursor() {
        return true;
    }

    public function columnCount() {
        return mysqli_num_fields($this->_result);
    }

    public function isError() {
        if (!$this->_result) {
            return true;
        }

        return $this->errorCode() != 0;
    }

    public function errorCode() {
        return mysqli_errno($this->_adapter->getConnection());
    }

    public function errorInfo() {
        return array(
            mysqli_errno($this->_adapter->getConnection()),
            mysqli_error($this->_adapter->getConnection())
        );
    }

    public function _execute(array $params = null) {
        $this->_result = mysqli_query($this->_adapter->getConnection(), $this->_sql);

        if (!$this->_result) {
            $logger = Application::getLogger();
            if ($logger instanceof Zend_Log) {
                $logger->err(sprintf('SQL Error: %s, query = %s', mysqli_error($this->_adapter->getConnection()), $this->_sql));
            }
        }
    }

    public function fetch($style = null, $cursor = null, $offset = null) {
        if (!$this->_result)
            return false;

        $row = false;

        switch ($style) {
            case Zend_Db::FETCH_NUM:
                $row = mysqli_fetch_array($this->_result, MYSQLI_NUM);
                break;

            case Zend_Db::FETCH_BOTH:
                $row = mysqli_fetch_array($this->_result, MYSQLI_BOTH);
                break;

            case Zend_Db::FETCH_BOUND:
                $row = mysqli_fetch_array($this->_result, MYSQLI_BOTH);
                return $this->_fetchBound($row);

            case Zend_Db::FETCH_ASSOC:
                $row = mysqli_fetch_array($this->_result, MYSQLI_ASSOC);
                break;

            default:
            case Zend_Db::FETCH_OBJ:
                $row = mysqli_fetch_object($this->_result);
                break;
        }

        return $row;
    }

    public function nextRowset() {
        throw new Zend_Db_Statement_Exception(__FUNCTION__.'() is not implemented');
    }

    public function rowCount() {
        if (preg_match('/^select/i', trim($this->_sql)))
            return mysqli_num_rows($this->_result);

        return mysqli_affected_rows($this->_adapter->getConnection());
    }
}
