<?php
    class Db_Statement_PostgreSQL extends Db_Statement_Abstract
    {
        protected $_result;
        protected $_keys;
        protected $_values;
        protected $_sql;

        public function _prepare($sql)
        {
            $this->_sql = $sql;
        }

        protected function _bindParam($parameter, &$variable, $type = null, $length = null, $options = null)
        {
            return true;
        }

        public function closeCursor()
        {
            return true;
        }

        public function columnCount()
        {
            return pg_num_fields($this->_result);
        }

        public function errorCode()
        {
            return pg_result_error_field($this->_result, PGSQL_DIAG_SQLSTATE);
        }

        public function isError()
        {
            if (!$this->_result) {
                return true;
            }

            return pg_result_error_field($this->_result, PGSQL_DIAG_SQLSTATE) != 0;
        }

        public function errorInfo()
        {
            if (!$this->_result) {
                return array(
                    -1,
                    pg_last_error($this->_adapter->getConnection())
                );
            }

            return array(
                pg_result_error_field($this->_result, PGSQL_DIAG_SQLSTATE),
                pg_result_error_field($this->_result, PGSQL_DIAG_MESSAGE_DETAIL)
            );
        }

        public function _execute(array $params = null)
        {
            $this->_result = @pg_query_params($this->_adapter->getConnection(),
                                              $this->_sql,
                                              $params);

            if (!$this->_result) {
                $logger = Application::getLogger();
                $err = pg_last_error($this->_adapter->getConnection());
                if ($logger instanceof Zend_Log) {
                    $str = sprintf('SQL Error: %s, query = %s', $err, $this->_sql);

                    if (count($params) > 0) {
                        $str .= sprintf(', params = %s', Debug::Dumpret($params));
                    }

                    $logger->err($str);
                }

                throw new Db_Statement_Exception(sprintf('SQL Error: %s', $err));
            }
        }

        public function fetch($style = null, $cursor = null, $offset = null)
        {
            if (!$this->_result)
                return false;

            $row = false;

            switch ($style) {
                case Zend_Db::FETCH_NUM:
                    $row = pg_fetch_array($this->_result, null, PGSQL_NUM);
                    break;

                case Zend_Db::FETCH_BOTH:
                    $row = pg_fetch_array($this->_result, null, PGSQL_BOTH);
                    break;

                case Zend_Db::FETCH_BOUND:
                    $row = pg_fetch_array($this->_result, null, PGSQL_BOTH);
                    return $this->_fetchBound($row);

                case Zend_Db::FETCH_ASSOC:
                    $row = pg_fetch_array($this->_result, null, PGSQL_ASSOC);
                    break;

                default:
                case Zend_Db::FETCH_OBJ:
                    $row = pg_fetch_object($this->_result);
                    break;
            }

            return $row;
        }

        public function nextRowset()
        {
            throw new Zend_Db_Statement_Exception(__FUNCTION__.'() is not implemented');
        }

        public function rowCount()
        {
            if (preg_match('/^select/i', trim($this->_sql)))
                return pg_num_rows($this->_result);

            return pg_affected_rows($this->_result);
        }
    }
?>