<?php
    class File_CSV
    {
        protected $_stream;
        protected $_columnsLocked = false;
        protected $_columns;
        protected $_rows = array();
        protected $_data;
        protected $_headerAdded = false;

        public function __construct($columns = null, $locked = false)
        {
            $this->_columnsLocked = (bool) $locked;

            if (!$columns instanceof File_CSV_Row && !is_array($columns)) {
                $columns = new File_CSV_Row;
            }

            $this->_columns = $columns;
        }

        public function setStream($stream)
        {
            $this->_stream = $stream;
            return $this;
        }

        public function add($row)
        {
            if (is_resource($this->_stream)) {
                if (!$this->_headerAdded) {
                    fwrite($this->_stream, $this->_getHeadingFromRow($row) . "\n");
                    $this->_headerAdded = true;
                }

                if (!$this->_columnsLocked) {
                    foreach ($row->getKeys() as $key) {
                        if (!$this->_columns->has($key)) {
                            $this->_columns->add($key, $key);
                        }
                    }
                }

                fwrite($this->_stream, $this->_getData($row) . "\n");
            }
            else {
                if ($this->_columnsLocked) {
                    $this->_data .= $this->_getData($row) . "\n";
                }
                else {
                    foreach ($row->getKeys() as $key) {
                        if (!$this->_columns->has($key)) {
                            $this->_columns->add($key, $key);
                        }
                    }

                    $this->_rows[] = $row;
                }
            }

            return $this;
        }

        protected function _getHeadingFromRow($row)
        {
            if ($row instanceof File_CSV_Row) {
                foreach ($row->getKeys() as $key) {
                    $values[] = $this->escape($key);
                }
            }
            else foreach ($row as $name) {
                $values[] = $this->escape($name);
            }

            return join(',', $values);
        }

        public function toString()
        {
            $values = array();

            $ret = $this->_getHeadingFromRow($this->_columns) . "\n";

            if ($this->_columnsLocked) {
                return $ret . $this->_data;
            }
            else foreach ($this->_rows as $row) {
                $ret .= $this->_getData($row) . "\n";
            }

            return $ret;
        }

        public function __toString()
        {
            return $this->toString();
        }

        public function escape($val)
        {
            $val = (string) $val;
            $val = str_replace('"', '""', $val);

            return '"' . $val . '"';
        }

        public function _getData($row)
        {
            $values = array();

            if ($row instanceof File_CSV_Row) {
                foreach ($this->_columns->getKeys() as $key) {
                    $value = $row->get($key);

                    if (is_array($value)) {
                        $value = join(',', $value);
                    }

                    $values[] = $this->escape($value);
                }
            }
            else if (is_array($row)) {
                foreach ($row as $k => $v) {
                    if (is_array($value)) {
                        $v = join(',', $v);
                    }

                    $values[] = $this->escape($v);
                }
            }
            else {
                throw new Exception('Invalid row data');
            }

            return join(',', $values);
        }
    }
?>
