<?php
    class File_Directory
    {
        protected $_files = array();
        protected $_dirs = array();
        protected $_path;
        protected $_includeHidden = false;

        public function __construct($path, $includeHidden = false)
        {
            $this->_includeHidden = (bool) $includeHidden;
            $this->_setPath($path);
            $this->_load();
        }

        private function _setPath($path)
        {
            if (!file_exists($path)) {
                throw new File_Exception('Path does not exist: ' . $path);
            }

            if (!is_dir($path)) {
                throw new File_Exception('Path is not a directory: ' . $path);
            }

            $this->_path = $path;
        }

        private function _load()
        {
            $dp = @opendir($this->_path);
            if ($dp === false) {
                throw new File_Exception('Unable to read directory');
            }

            $this->_dirs = array();
            $this->_files = array();

            while (($file = readdir($dp)) !== false) {
                if ($file == '.' || $file == '..') {
                    continue;
                }

                if (!$this->_includeHidden && $file{0} == '.') {
                    continue;
                }

                if (is_dir($this->_path . '/' . $file)) {
                    $this->_dirs[] = $file;
                }
                else {
                    $this->_files[] = $file;
                }
            }

            sort($this->_dirs);
            sort($this->_files);

            closedir($dp);
        }

        public function getPath()
        {
            return $this->_path;
        }

        public function __toString()
        {
            return (string) $this->getPath();
        }

        public function getFiles()
        {
            return $this->_files;
        }

        public function getDirs()
        {
            return $this->_dirs;
        }
    }
?>
