<?php
    abstract class File_Type
    {
        protected $_fullpath;
        protected $_filename;

        public function __construct($fullpath = null, $filename = null)
        {
            if (strlen($fullpath) > 0 && file_exists($fullpath) && is_readable($fullpath)) {
                $this->_fullpath = $fullpath;

                if (strlen($filename) > 0) {
                    $this->_filename = $filename;
                }

                if (strlen($this->_filename) == 0 && strlen($this->_fullpath) > 0) {
                    $this->_filename = basename($this->_fullpath);
                }
            }
        }

        public function __toString()
        {
            return $this->getExtension();
        }

        public function isImage()
        {
            return false;
        }

        public function isText()
        {
            return false;
        }

        public function isArchive()
        {
            return $this instanceof File_Type_Archive;
        }

        public function getAsText($fullpath)
        {
            throw new File_Exception(sprintf('%s::%s() not implemented', get_class($this), __FUNCTION__));
        }

        public function getIndexableDocument($fullpath)
        {
            throw new File_Exception(sprintf('%s::%s() not implemented', get_class($this), __FUNCTION__));
        }

        public static function factory($filename = '', $mime = '', $fullpath = '')
        {
            if (strlen($filename) > 0) {
                $extension = strtolower(File::GetExtension($filename));

                $class = 'Driver_assets_types_' . $extension;
                $path  = sprintf('%s/%s.php', self::GetDriverPath(), $extension);

                if (file_exists($path) && is_file($path)) {
                    require_once($path);

                    if (class_exists($class, false)) {
                        return new $class($fullpath, $filename);
                    }
                }
            }


            return new File_Type_Unknown($fullpath, $filename);
        }

        public static function GetTypes()
        {
            $baseReflection = new ReflectionClass('File_Type');

            $ret = array();
            foreach (glob(self::GetDriverPath() . '/*.php') as $fullpath) {
                $type = substr(basename($fullpath), 0, -4);

                if (strtolower($type) != $type) {
                    continue;
                }

                $class = 'Driver_assets_types_' . $type;

                $reflection = new ReflectionClass($class);

                if ($reflection->isSubclassOf($baseReflection)) {
                    $ret[$type] = new $class;
                }
            }

            return $ret;
        }

        public static function GetDriverPath()
        {
            require_once('Application/Driver/Manager.php');
            return sprintf('%s/assets/types', Application_Driver_Manager::GetPath());
        }

        abstract public function getMimeType();
        abstract public function getDescription();
        abstract public function getExtension();
    }
?>