<?php
    class File_Zip
    {
        protected static $_zip;

        public static function CreateFromDirectory($path, $dst)
        {
            if (!file_exists($path)) {
                throw new File_Exception('Path to Zip does not exist');
            }

            try {
                $zip = File::GetBinaryPath('zip');
            }
            catch (Exception $ex) {
                $zip = '';
            }

            if (strlen($zip) > 0) {
                $cmd = sprintf(
                    '%s -qq -r %s %s',
                    escapeshellcmd($zip),
                    escapeshellarg($dst),
                    escapeshellarg(basename($path))
                );
                $retval = 0;
                $output = '';

                chdir(dirname($path));
                exec($cmd, $output, $retval);

                if ($retval != 0) {
                    throw new File_Exception('Unable to compress package');
                }
            }
            else {
                if (!class_exists('ZipArchive')) {
                    throw new File_Exception('ZipArchive support not found');
                }

                self::$_zip = new ZipArchive();
                $succ = self::$_zip->open($dst, ZIPARCHIVE::CREATE);

                if ($succ !== true) {
                    self::$_zip = null;
                    throw new File_Exception(sprintf('Unable to create ZIP: Error code %s', $succ), $succ);
                }

                if (is_file($path)) {
                    self::$_zip->addFile($path, basename($path));
                }
                else {
                    self::_Zip(dirname($path), basename($path));
                }

                self::$_zip->close();
            }
        }

        private static function _Zip($root, $path)
        {
            $dir = new DirectoryIterator($root . '/' . $path);

            foreach ($dir as $file) {
                if ($file->isDot()) {
                    continue;
                }

                $name = $file->getFilename();

                if ($file->isDir()) {
                    self::$_zip->addEmptyDir(ltrim($path . '/' . $name, '/'));
                    self::_Zip($root, $path . '/' . $file);
                }
                else {
                    self::$_zip->addFile($file->getPathName(), ltrim($path . '/' . $name, '/'));
                }
            }
        }

        public static function UnzipSupported()
        {
            if (!IS_WINDOWS) {
                try {
                    File::GetBinaryPath('unzip');
                    return true;
                }
                catch (Exception $ex) {
                }
            }

            return class_exists('ZipArchive');
        }

        public static function Unzip($path, $extractPath)
        {
            try {
                $unzip = File::GetBinaryPath('unzip');
            }
            catch (Exception $ex) {
                $unzip = '';
            }

            if (strlen($unzip) > 0) {
                $cmd = sprintf(
                    '%s -qq %s -d %s',
                    escapeshellcmd($unzip),
                    escapeshellarg($path),
                    escapeshellarg($extractPath)
                );
                $retval = 0;
                $output = '';

                exec($cmd, $output, $retval);

                if ($retval != 0) {
                    throw new File_Exception('Unable to extract package');
                }
            }
            else {
                if (!class_exists('ZipArchive', false)) {
                    throw new File_Exception('ZipArchive PHP extension is required');
                }

                $zip = new ZipArchive();
                $result = $zip->open($path);

                if ($result !== true) {
                    throw new File_Exception(sprintf('Unable to open package (error code %d)', $result));
                }

                $result = $zip->extractTo($extractPath);

                if (!$result) {
                    throw new File_Exception('Unable to extract package');
                }
            }
        }
    }
?>