<?php
    abstract class FormProcessor implements FormProcessor_Interface
    {
        protected $_errors = array();
        protected $_vals = array();
        private static $_sanitizeChain = null;

        static $_emailValidator;

        public function __construct()
        {
            // this must be left blank
        }

        abstract function process(Zend_Controller_Request_Abstract $request);

        public static function sanitize($value)
        {
            if (!self::$_sanitizeChain instanceof Zend_Filter) {
                self::$_sanitizeChain = new Zend_Filter();
                self::$_sanitizeChain->addFilter(new Zend_Filter_StringTrim())
                                     ->addFilter(new Zend_Filter_StripTags());
            }

            return self::$_sanitizeChain->filter($value);
        }

        public static function isValidEmail($email, $messages = false)
        {
            if (!self::$_emailValidator instanceof Zend_Validate_EmailAddress)
                self::$_emailValidator = new Zend_Validate_EmailAddress();

            $valid = self::$_emailValidator->isValid($email);
            if (!$messages)
                return $valid;

            return array(
                'valid'    => $valid,
                'messages' => self::$_emailValidator->getMessages()
            );
        }

        public function addError($key, $val)
        {
            if (!is_array($val)) {
                $val = array($val);
            }

            $val = array_filter($val);

            if (count($val) == 0) {
                return;
            }

            if (!array_key_exists($key, $this->_errors)) {
                $this->_errors[$key] = array();
            }

            foreach ($val as $v) {
                $this->_errors[$key][] = $v;
            }
        }

        public function getError($key)
        {
            if (!is_array($key)) {
                $key = array($key);
            }

            $errors = array();

            foreach ($key as $k) {
                if ($this->hasError($k)) {
                    $error = $this->_errors[$k];

                    if (!is_array($error))
                        $error = array($error);

                    foreach ($error as $e) {
                        $errors[] = $e;
                    }
                }
            }

            $errors = array_unique($errors);

            if (count($errors) == 0) {
                return null;
            }

            if (count($errors) == 1) {
                return $errors[0];
            }

            return $errors;
        }

        public function getErrors()
        {
            $ret = array();

            foreach ($this->_errors as $k => $error) {
                if (count($error) == 0) {
                    continue;
                }

                if (count($error) == 1) {
                    $ret[$k] = $error[0];
                }
                else {
                    $ret[$k] = $error;
                }
            }

            return $ret;
        }

        public function getErrorsByPrefix($prefix)
        {
            $ret = array();
            $regex = '/^' . preg_quote($prefix, '/') . '/';

            foreach ($this->_errors as $k => $v) {
                if (!preg_match($regex, $k))
                    continue;

                $key = preg_replace($regex, '', $k);

                if (is_array($v)) {
                    foreach ($v as $_v) {
                        $ret[] = $_v;
                    }
                }
                else {
                    $ret[] = $v;
                }
            }

            return $ret;
        }

        public function hasError($key = null)
        {
            $keys = func_get_args();

            if (count($keys) == 0) {
                return count($this->_errors) > 0;
            }

            foreach ($keys as $key) {
                if (array_key_exists($key, $this->_errors)) {
                    return true;
                }
            }

            return false;
        }

        public function getNumErrors()
        {
            return count($this->_errors);
        }

        public function __set($name, $value)
        {
            $this->_vals[$name] = $value;
        }

        public function set($name, $value)
        {
            return $this->__set($name, $value);
        }

        public function __get($name)
        {
            return $this->has($name) ? $this->_vals[$name] : null;
        }

        public function get($name)
        {
            return $this->__get($name);
        }

        public function has($name)
        {
            return array_key_exists($name, $this->_vals);
        }

        public function __isset($name)
        {
            return $this->has($name);
        }

        public function getValues()
        {
            return $this->_vals;
        }

        public function getValidFormKey($key)
        {
            return str_replace('.', '_', $key);
        }

        public function getResponseAsArray()
        {
            $ret = array(
                'success' => !$this->hasError(),
                'errors'  => array()
            );

            foreach ($this->getErrors() as $k => $v) {
                $ret['errors'][] = array(
                    'name'   => $k,
                    'errors' => $v
                );
            }

            return $ret;
        }
    }
?>