<?php
    class HTTP
    {
        /**
         * @todo    Look at using Zend_Http_Request for this function
         */
        public static function SendFile($fullpath, $options = array())
        {
            if (!file_exists($fullpath)) {
                throw new Exception('File does not exist');
            }
            else if (!is_file($fullpath)) {
                throw new Exception('Path is not a file');
            }
            else if (!is_readable($fullpath)) {
                throw new Exception('File is not readable');
            }

            if (!is_array($options)) {
                $options = array($options);
            }

            $defaults = array(
                'filename'     => '',
                'attach'       => false,
                'browsercache' => true,
                'sendlength'   => true,
                'mimeforce'    => '',
                'mimeauto'     => true,
                'sendfile'     => true,
                'response'     => null,
                'sendexpires'  => false
            );

            require_once('ArrayTools.php');
            $options = ArrayTools::buildOptions($options, $defaults);

            $parts = pathinfo($options['filename']);
            $extension = coalesce($parts['extension']);

            $headers = array();
            $responseCode = 200;

            $contentLength = filesize($fullpath);

            if ($options['browsercache']) {
                $filemtime = filemtime($fullpath);
                $hash = md5($filemtime . ',' . $contentLength);

                $etag = coalesce($_SERVER['HTTP_IF_NONE_MATCH']);

                if ($etag == '"' . $hash . '"') {
                    $responseCode = 304;
                    $options['sendfile'] = false;
                    $options['sendlength'] = false;
                }
                else {
                    $headers['Last-Modified'] = gmdate('D, d M Y H:i:s', $filemtime) . ' GMT';
                }
                $headers['Etag'] = '"' . $hash . '"';
            }

            if ($options['sendexpires']) {
                $headers['Expires'] = date('r', time() + 365 * 86400);
            }

            if ($options['sendlength']) {
                $headers['Content-Length'] = $contentLength;
            }

            if ($options['attach'])
                $disposition = 'attachment';
            else
                $disposition = 'inline';

            if (strlen($options['filename']))
                $disposition .= '; filename="' . $options['filename'] . '"';

            $headers['Content-Disposition'] = $disposition;

            $mime = $options['mimeforce'];

            if (strlen($mime) == 0 && $options['mimeauto']) {
                $mime = File::DetectMimeType($fullpath, null, $options['filename']);
            }

            if (strlen($mime) > 0) {
                $headers['Content-Type'] = self::AddContentTypeCharset($mime);
            }

            if ($options['response'] instanceof Zend_Controller_Response_Abstract) {
                $options['response']->setHttpResponseCode($responseCode);

                foreach ($headers as $k => $v) {
                    $options['response']->setHeader($k, $v, true);
                }

                $options['response']->sendHeaders();
            }
            else {
                header(self::getStatusHeader($responseCode));
                foreach ($headers as $k => $v) {
                    header($k . ': ' . $v);
                }
            }

            if ($options['sendfile']) {
                ob_end_clean();
                readfile($fullpath);
            }

            if ($options['response'] instanceof Zend_Controller_Response_Abstract) {
                $options['response']->clearHeaders();
            }
        }

        public static function SendDataAsFile(Zend_Controller_Response_Abstract $response, $data, $options)
        {
            $defaults = array(
                'filename'     => '',
                'attach'       => false,
                'sendlength'   => true,
                'sendexpires'  => false,
                'mime'         => ''
            );

            require_once('ArrayTools.php');
            $options = ArrayTools::buildOptions($options, $defaults);

            $headers = array();

            if (strlen($options['mime']) == 0) {
                $options['mime'] = File::DetectMimeType('', null, $options['filename']);
            }

            if (strlen($options['mime']) > 0) {
                $headers['Content-Type'] = self::AddContentTypeCharset($options['mime']);
            }

            if ($options['sendlength']) {
                $headers['Content-Length'] = strlen($data); // this probably isn't mb safe
            }

            if ($options['attach']) {
                $disposition = 'attachment';
            }
            else {
                $disposition = 'inline';
            }

            if (strlen($options['filename']) > 0) {
                $disposition .= '; filename="' . $options['filename'] . '"';
            }

            if ($options['sendexpires']) {
                $headers['Expires'] = date('r', time() + 365 * 86400);
            }

            $headers['Content-Disposition'] = $disposition;

            if ($response instanceof Zend_Controller_Response_Abstract) {
                foreach ($headers as $k => $v) {
                    $response->setHeader($k, $v, true);
                }

                $response->setBody($data);
            }
            else {
                foreach ($headers as $k => $v) {
                    header($k . ': ' . $v);
                }

                echo $data;
            }
        }


        public static function SendFilesCombined(Zend_Controller_Response_Abstract $response, $files, $options = array())
        {
            if (count($files) == 0) {
                throw new Exception('No files');
            }

            $defaults = array(
                'sendexpires' => false
            );

            $options = ArrayTools::BuildOptions($options, $defaults);

            foreach ($files as $fullpath) {
                $mime = File::DetectMimeType($fullpath);
            }

            if ($options['sendexpires']) {
                $response->setHeader('Expires', date('r', time() + 365 * 86400), true);
            }

            $response->setHttpResponseCode(200);
            $response->setHeader('Content-Type', self::AddContentTypeCharset($mime), true);
            $response->sendHeaders();

            ob_end_clean();

            foreach ($files as $fullpath) {
                readfile($fullpath);
            }
        }


        public static function AddContentTypeCharset($type, $charset = 'UTF-8')
        {
            $type = trim($type);
            if (!preg_match('/charset/i', $type) && substr($type, 0, 4) == 'text') {
                $type .= '; charset=' . $charset;
            }

            return $type;
        }

        public static function GetStatusHeader($code)
        {
            $msg = Zend_Http_Response::responseCodeAsText($code);

            $prefix = IS_CGI ? 'Status: ' : 'HTTP/1.1 ';

            return $prefix . $code . ' ' . $msg;
        }

        public static function SendContentTypeHeader($mime, $charset = 'UTF-8')
        {
            if (!is_null($charset) && preg_match('|^text/|i', $mime))
                $mime = self::AddContentTypeCharset($mime, $charset);

            header('Content-Type: ' . $mime, true);
        }

        public static function GetRemoteAddr()
        {
            $remote_addr = coalesce($_SERVER['REMOTE_ADDR']);

            if (isset($_SERVER['HTTP_X_FORWARDED_FOR']) && strlen($_SERVER['HTTP_X_FORWARDED_FOR']) > 0)
                $forwarded = array_filter(explode(',', coalesce($_SERVER['HTTP_X_FORWARDED_FOR'])));
            else
                $forwarded = array();

            foreach ($forwarded as $k => $ip)
                $forwarded[$k] = trim($ip);

            $proxy = '';
            if ($remote_addr == '127.0.0.1') {
                if (count($forwarded) > 0)
                    return array_shift($forwarded);
            }
            else if (count($forwarded) == 1)
                return array_shift($forwarded);

            return $remote_addr;
        }

        public static function RedirectToSSL($port = 443)
        {
            $inSSL = strtolower(coalesce($_SERVER['HTTPS'])) == 'on';

            if ($inSSL)
                return;

            $host = preg_replace('/(:\d+)$/', '', coalesce($_SERVER['HTTP_HOST']));

            $url = sprintf('https://%s%s%s',
                           $host,
                           $port != 443 ? ':' . $port : '',
                           coalesce($_SERVER['REQUEST_URI']));

            self::redirect($url);
        }



        /**
         * getReferer
         *
         * Determine referring page for the current request
         *
         * @param   string  $default    String to return if no referer found
         * @return  string              The referer
         */
        function getReferer($default = '/')
        {
            return coalesce($_SERVER['HTTP_REFERER'], $default);
        }


        /**
         * redirect
         *
         * Send a HTTP redirect header and exit the current script
         *
         * @access  public
         * @param   string  $url    The URL to redirect to
         */
        function redirect($url)
        {
            header('Location: ' . $url);
            exit;
        }


        /**
         * redirectToReferer
         *
         * Redirect back to the refering document
         *
         * @param   string  $default    Optional. Location to redirect to if referer not found
         */
        function redirectToReferer($default = '/')
        {
            self::redirect(getReferer($default));
        }

        function headerNotFound()
        {
            header('HTTP/1.0 404 Not Found');
        }
    }
?>
