<?php
    class Image_Coords
    {
        public static function Resize($w, $h, $maxW, $maxH, $padOrCrop = false)
        {
            $_maxW = min($w, $maxW);
            if ($_maxW == 0) {
                $_maxW = $w;
            }

            $_maxH = min($h, $maxH);
            if ($_maxH == 0) {
                $_maxH = $h;
            }

            if ($padOrCrop) {
                if ($maxW <= 0) {
                    $maxW = $w;
                }
                if ($maxH <= 0) {
                    $maxH = $h;
                }
            }

            $ratio = $w / $h;

            $newW = $_maxW;
            $newH = $newW / $ratio;

            if ($newH > $_maxH) {
                $newH = $_maxH;
                $newW = $newH * $ratio;
            }

            $ret = array(
                'w' => $newW,
                'h' => $newH,
                'maxW' => $_maxW,
                'maxH' => $_maxH
            );

            if ($padOrCrop) {
                $ret['maxW'] = $maxW;
                $ret['maxH'] = $maxH;
            }

            return $ret;
        }

        /**
         * DetermineCropMarks
         *
         * Based on a given aspect ratio (e.g. 4:3), determine crop marks
         * for the given width and height. It'll maximise the possible size
         * within the given width/height for that aspect ratio. Additionally,
         * the returned data will be centered on the image. The returned data
         * consists of an x-y coord (marking top-left corner of crop point),
         * with a width and height of how far to crop.
         *
         * @static
         * @param   int     $imageW     The width of the image
         * @param   int     $imageH     The height of the image
         * @param   int     $ratioW     The aspect ratio width
         * @param   int     $ratioH     The aspect ratio height
         * @return  array               The crop marks
         *           +w                 Width of the crop
         *           +h                 Height of the crop
         *           +x                 X coordinate of crop (from left)
         *           +y                 Y coordinate of crop (from top)
         */
        public static function DetermineCropMarks($imageW, $imageH, $ratioW, $ratioH)
        {
            $imageW = (int) $imageW;
            $imageH = (int) $imageH;
            $ratioW = (int) $ratioW;
            $ratioH = (int) $ratioH;

            if ($imageW <= 0 || $imageH <= 0 || $ratioW <= 0 || $ratioH <= 0) {
                return null;
            }

            $ret = array('w' => $imageW,
                         'h' => $imageH,
                         'x' => 0,
                         'y' => 0);

            // first we determine the new width and height based on the aspect ratio

            $ratio = $ratioW / $ratioH;

            $newW = $imageW;
            $newH = $imageW / $ratio;

            if ($newH > $imageH) {
                $newH = $imageH;
                $newW = $imageH * $ratio;
            }

            $newW = floor($newW);
            $newH = floor($newH);

            // now we need to center the crop in the middle of the image
            // horizontal centering (x-coord, based on width)
            $imgCenter = $imageW / 2;
            $cropCenter = $newW / 2;
            $cropX = floor($imgCenter - $cropCenter);

            // vertical centering (y-coord, based on height)
            $imgCenter = $imageH / 2;
            $cropCenter = $newH / 2;
            $cropY = floor($imgCenter - $cropCenter);

            // write all the new values
            $ret['w'] = $newW;
            $ret['h'] = $newH;
            $ret['x'] = $cropX;
            $ret['y'] = $cropY;

            return $ret;
        }

    }
?>