<?php
    class Image_Generator
    {
        public static function Create($options = array())
        {
            $defaultOptions = array('w'           => 0,
                                    'h'           => 0,
                                    'border'      => '',
                                    'background'  => '',
                                    'overlay'     => '');

            foreach ($defaultOptions as $k => $v)
                $options[$k] = coalesce($options[$k], $v);

            $options['w'] = max(1, $options['w']);
            $options['h'] = max(1, $options['h']);
            $im = ImageCreateTrueColor($options['w'], $options['h']);

            $background = Image_Colour::AllocateColorFromHexString($im, $options['background']);
            ImageFill($im, 0, 0, $background);

            if (strlen($options['overlay']) > 0) {
                $info = @GetImageSize($options['overlay']);

                $infunc = '';
                if ($info) {
                    switch ($info[2]) {
                        case 1:
                            $infunc = 'ImageCreateFromGif';
                            break;
                        case 2:
                            $infunc = 'ImageCreateFromJpeg';
                            break;
                        case 3:
                            $infunc = 'ImageCreateFromPng';
                    }
                }

                if (strlen($infunc) > 0) {
                    $overlay = $infunc($options['overlay']);

                    $overlayW = ImageSx($overlay);
                    $overlayH = ImageSy($overlay);

                    if ($overlayW > $options['w'] || $overlayH > $options['h']) {
                        $ratio = $overlayW / $overlayH;
                        $newOverlayW = $options['w'];
                        $newOverlayH = $newOverlayW / $ratio;

                        if ($newOverlayH > $options['h']) {
                            $newOverlayH = $options['h'];
                            $newOverlayW = $newOverlayH * $ratio;
                        }

                        $overlay2 = ImageCreateTrueColor($newOverlayW, $newOverlayH);
                        ImageCopyResampled($overlay2, $overlay, 0, 0, 0, 0, $newOverlayW, $newOverlayH, $overlayW, $overlayH);

                        $overlay = $overlay2;
                        $overlayW = ImageSx($overlay);
                        $overlayH = ImageSy($overlay);
                    }

                    $x = ($options['w']) / 2 - ($overlayW / 2);
                    $y = ($options['h']) / 2 - ($overlayH / 2);

                    ImageCopy($im, $overlay, $x, $y, 0, 0, $overlayW, $overlayH);
                }
            }

            if (strlen($options['border']) > 0)
                self::AddBorder($im, Image_Colour::AllocateColorFromHexString($im, $options['border']));

            return $im;
        }

        public static function AddBorder($im, $colour)
        {
            $w = ImageSx($im);
            $h = ImageSy($im);

            ImageLine($im, 0, 0, $w - 1, 0, $colour);
            ImageLine($im, 0, 0, 0, $h - 1, $colour);
            ImageLine($im, $w - 1, 0, $w - 1, $h - 1, $colour);
            ImageLine($im, 0, $h - 1, $w - 1, $h - 1, $colour);
        }
    }
?>