<?php
    class Image_Thumbnail
    {
        /**
         * Create
         *
         * Generate a thumbnail
         */
        public static function Create($fullpath, $dstPath, $options = array())
        {
            if (!file_exists($fullpath) || !is_file($fullpath)) {
                throw new Exception('Specified file not found');
            }

            if (!is_readable($fullpath)) {
                throw new Exception('Cannot read file ' . $fullpath);
            }

            $defaults = array(
                'extension'   => null, // manually override output type
                'w'           => 0,
                'h'           => 0,
                'crop'        => false,
                'pad'         => false,
                'padOpacity'  => 100,
                'padColour'   => 'ffffff',
                'filename'    => ''
            );

            $options = ArrayTools::buildOptions($options, $defaults);

            if ($options['w'] <= 0 && $options['h'] <= 0) {
                return $fullpath;
            }

            $image = basename($fullpath);

            $tsModified = (int) @filemtime($fullpath);

            $info = getImageSize($fullpath);

            $w = $info[0];
            $h = $info[1];
            $coords = Image_Coords::Resize($w, $h, $options['w'], $options['h'], $options['pad'] || $options['crop']);

            $canvasW = $coords['w'];
            $canvasH = $coords['h'];

            if ($canvasH < 1) {
                $ratio = 1 / $canvasH;
                $canvasH *= $ratio;
                $canvasW *= $ratio;
            }
            else if ($canvasW < 1) {
                $ratio = 1 / $canvasW;
                $canvasW *= $ratio;
                $canvasH *= $ratio;
            }

            if ($options['crop'] || $options['pad']) {
                $canvasW = $coords['maxW'];
                $canvasH = $coords['maxH'];
            }

            if ($canvasW == $w && $canvasH == $h) {
                return $fullpath;
            }

            switch ($info[2]) {
                case IMAGETYPE_GIF:
                    $infunc = 'ImageCreateFromGif';
                    $outfunc = 'ImageGif';
                    break;
                case IMAGETYPE_JPEG:
                    $infunc = 'ImageCreateFromJpeg';
                    $outfunc = 'ImageJpeg';
                    break;
                case IMAGETYPE_PNG:
                    $infunc = 'ImageCreateFromPng';
                    $outfunc = 'ImagePng';
                    break;
                default:
                    return false;
            }

            switch ($options['extension']) {
                case 'png':
                    $outfunc = 'ImagePng';
                    break;
                case 'jpg':
                    $outfunc = 'ImageJpeg';
                    break;
                case 'gif':
                    $outfunc = 'ImageGif';
                    break;
            }

            switch ($outfunc) {
                case 'ImagePng':
                    $extension = 'png';
                    break;
                case 'ImageJpeg':
                    $extension = 'jpg';
                    break;
                case 'ImageGif':
                    $extension = 'gif';
                    break;
                default:
                    $extension = '';
            }

            if (strlen($options['filename']) == 0) {
                $options['filename'] = sprintf('%s.%dx%d.%d.%s.%s',
                                               $image,
                                               $coords['w'],
                                               $coords['h'],
                                               $tsModified,
                                               md5(serialize($options)),
                                               $extension);
            }

            $thumbFullPath = $dstPath . '/' . $options['filename'];

            if (file_exists($thumbFullPath)) {
                return $thumbFullPath;
            }

            File::CreateDirStructure($dstPath);

            if (!is_writable($dstPath)) {
                throw new Exception(sprintf('Cannot write to directory (%s)', $dstPath));
            }

            $im = $infunc($fullpath);

            if (!$im) {
                throw new Exception('Unable to read image file');
            }

            $imDst = ImageCreateTrueColor($canvasW, $canvasH);

            $crop = array('x' => 0, 'y' => 0, 'w' => $w, 'h' => $h);

            $background = Image_Colour::AllocateColorFromHexString($imDst, $options['padColour'], 0);
            imageFill($imDst, 0, 0, $background);

            if ($options['pad']) {
                $succ = ImageCopyResampled($imDst, $im, ($coords['maxW'] - $coords['w']) / 2, ($coords['maxH'] - $coords['h']) / 2, 0, 0, $coords['w'], $coords['h'], $w, $h);
            }
            else {
                if ($options['crop']) {
                    $crop = Image_Coords::DetermineCropMarks($w, $h, $canvasW, $canvasH);
                }

                $succ = ImageCopyResampled($imDst, $im, 0, 0, $crop['x'], $crop['y'], $canvasW, $canvasH, $crop['w'], $crop['h']);
            }

            if (!$succ) {
                throw new Exception('Unable to resample image');
            }

            $outfunc($imDst, $thumbFullPath);

            return $thumbFullPath;
        }
    }
?>
