<?php
    /**
     * @package NestedTree
     */
    class NestedTree_Node extends NestedTree_NodeList
    {
        const STATE_OPEN   = 'o';
        const STATE_CLOSED = 'c';

        protected $_context;

        protected $_properties = array(
            'title'    => null,
            'left'     => null,
            'right'    => null,
            'level'    => null,
            'parentId' => null,
            'path'     => null,
            'selected' => false,
            'open'     => false,
            'additional' => array()
        );

        protected $_meta = array();

        protected $_iconPath;
        protected $_type;

        public function __construct($id, $left = null, $right = null, $level = null)
        {
            $this->setId($id);
            $this->setL($left);
            $this->setR($right);
            $this->setLevel($level);
        }

        public function __toString()
        {
            return $this->getTitle();
        }

        public function hasChildren()
        {
            return parent::hasChildren() || ($this->getR() - $this->getL() > 1);
        }

        public function setTitle($title)
        {
            $this->_properties['title'] = (string) $title;
            return $this;
        }

        public function getTitle()
        {
            return (string) $this->_properties['title'];
        }

        public function setParentId($id)
        {
            $this->_properties['parentId'] = (int) $id;
            return $this;
        }

        public function getParentId()
        {
            return $this->_properties['parentId'];
        }

        public function setL($left)
        {
            $this->_properties['left'] = (int) $left;
            return $this;
        }

        public function getL()
        {
            return $this->_properties['left'];
        }

        public function setR($right)
        {
            $this->_properties['right'] = (int) $right;
            return $this;
        }

        public function getR()
        {
            return $this->_properties['right'];
        }

        public function setLevel($level)
        {
            $this->_properties['level'] = (int) $level;
            return $this;
        }

        public function getLevel()
        {
            return $this->_properties['level'];
        }

        public function setPathElement($path)
        {
            $this->_properties['path'] = $path;
            return $this;
        }

        public function getPathElement()
        {
            return $this->_properties['path'];
        }

        public function setIsOpen($flag = true)
        {
            $this->_properties['open'] = (bool) $flag;
            return $this;
        }

        public function isOpen()
        {
            return $this->_properties['open'];
        }

        public function setIsSelected($flag = true)
        {
            $this->_properties['selected'] = (bool) $flag;
            return $this;
        }

        public function isSelected()
        {
            return $this->_properties['selected'];
        }

        public function setAdditionalField($name, $value)
        {
            $this->_properties['additional'][$name] = $value;
            return $this;
        }

        public function getAdditionalField($name)
        {
            if (!array_key_exists($name, $this->_properties['additional'])) {
                throw new NestedTree_Exception('Unknown additional field: ' . $name);
            }

            return $this->_properties['additional'][$name];
        }

        public function setMeta($k, $v)
        {
            $this->_meta[$k] = $v;
            return $this;
        }

        public function getMeta($k = null, $default = null)
        {
            if (strlen($k) == 0) {
                return $this->_meta;
            }
            else if (array_key_exists($k, $this->_meta)) {
                return $this->_meta[$k];
            }

            return $default;
        }

        public function add(NestedTree_Node $node)
        {
            $node->setParentid($this->getId());

            $context = $this->_getContext();
            $this->_children[$node->getId()] = $node;
            $node->_setContext($context);
            $context->_registerNode($node);

            return $this;
        }

        protected function _setContext(NestedTree_NodeList $nodelist)
        {
            $this->_context = $nodelist;
            return $this;
        }

        protected function _getContext()
        {
            if (!$this->_context instanceof NestedTree_NodeList) {
                throw new NestedTree_Exception('NodeList context not set');
            }

            return $this->_context;
        }

        public function setIconPath($path)
        {
            $this->_iconPath = (string) $path;
            return $this;
        }

        public function getIconPath()
        {
            if (strlen($this->_iconPath) == 0) {
                throw new NestedTree_Exception('Icon path not set');
            }

            return $this->_iconPath;
        }

        public function setNodeType($type)
        {
            $this->_type = trim($type);
            return $this;
        }

        public function getNodeType()
        {
            if (strlen($this->_type) == 0) {
                throw new NestedTree_Exception('Node type not set');
            }

            return $this->_type;
        }

        /**
         * Returns true if the current object is within the passed the node
         *
         * @param   NestedTree_Node     $node
         * @return  bool
         */
        public function isInsideOf(NestedTree_Node $node)
        {
            return $this->getL() > $node->getL() && $this->getR() < $node->getR();
        }
    }
?>
