<?php
    /**
     * @package NestedTree
     */
    class NestedTree_NodeList
    {
        protected $_children = array();
        protected $_nodes    = array();

        protected $_id;

        public function setId($id)
        {
            $this->_id = $id;
            return $this;
        }

        public function getId()
        {
            return $this->_id;
        }

        public function size()
        {
            return count($this->_nodes);
        }

        public function getNodeIds()
        {
            return array_keys($this->_nodes);
        }

        public function getNodes()
        {
            return $this->_nodes;
        }

        public function getNode($id)
        {
            if (!array_key_exists($id, $this->_nodes)) {
                throw new NestedTree_Exception('Node not found in list');
            }

            return $this->_nodes[$id];
        }

        /**
         * Return whether or not the node has children
         *
         * @return  bool
         */
        public function hasChildren()
        {
            return count($this->_children) > 0;
        }


        public function getChildren()
        {
            return $this->_children;
        }

        public function add(NestedTree_Node $node)
        {
            $this->_children[$node->getId()] = $node;
            $node->_setContext($this);
            $this->_registerNode($node);
            return $this;
        }

        public function getChildFromId($nodeId)
        {
            if (!array_key_exists($nodeId, $this->_children)) {
                throw new NestedTree_Exception('Node not found');
            }

            return $this->_children[$nodeId];
        }

        public function addToNode($nodeId, NestedTree_Node $node)
        {
            $nodeId = (int) $nodeId;

            if ($nodeId <= 0) {
                return $this->add($node);
            }

            if (!array_key_exists($nodeId, $this->_nodes)) {
                throw new NestedTree_Exception('Invalid parent node');
            }

            $this->_nodes[$nodeId]->add($node);
            return $this;
        }

        protected function _registerNode(NestedTree_Node $node)
        {
            $this->_nodes[$node->getId()] = $node;
            return $this;
        }

        public function setTreeState(NestedTree_TreeView_TreeState $treeState)
        {
            foreach ($treeState->getNodeList()->getChildren() as $child) {
                if (!array_key_exists($child->getId(), $this->_nodes)) {
                    continue;
                }
                $this->_nodes[$child->getId()]->setIsOpen(true);

                if ($child->isSelected()) {
                    $this->_nodes[$child->getId()]->setIsSelected(true);
                }
            }
        }

        public function getSelectedNodeIds()
        {
            $ret = array();

            foreach ($this->_nodes as $node) {
                if ($node->isSelected()) {
                    $ret[] = $node->getId();
                }
            }

            return $ret;
        }
    }
?>
