<?php
    /**
     * Class used to hold a series of tree left, right and level values.
     * This is primarily intended for use in querying the tree
     *
     * @author  Quentin Zervaas
     */
    class NestedTree_Nrange
    {
        const INSIDE_INCLUSIVE  = '>=';
        const INSIDE_EXCLUSIVE  = '>';
        const OUTSIDE_INCLUSIVE = '<=';
        const OUTSIDE_EXCLUSIVE = '<';
        const EITHER_SIDE       = '<>';

        public static $rangeTypes = array(
            self::INSIDE_INCLUSIVE  => 'Inside (inclusive)',
            self::INSIDE_EXCLUSIVE  => 'Inside (exclusive)',
            self::OUTSIDE_INCLUSIVE => 'Outside (inclusive)',
            self::OUTSIDE_EXCLUSIVE => 'Outside (exclusive)',
            self::EITHER_SIDE       => 'Either side'
        );

        protected $_left  = 0;
        protected $_leftType;

        protected $_right = 0;
        protected $_rightType;

        protected $_level = 0;
        protected $_levelType;

        public function __construct()
        { }

        public function setL($l, $type)
        {
            if (!array_key_exists($type, self::$rangeTypes)) {
                throw new NestedTree_Exception('Invalid range type specified');
            }

            $this->_left = (int) $l;
            $this->_leftType = $type;

            return $this;
        }

        public function getL()
        {
            if ($this->_left <= 0) {
                throw new NestedTree_Exception('Left value not specified');
            }

            return $this->_left;
        }

        public function setR($r, $type)
        {
            if (!array_key_exists($type, self::$rangeTypes)) {
                throw new NestedTree_Exception('Invalid range type specified');
            }

            $this->_right = (int) $r;
            $this->_rightType = $type;

            return $this;
        }

        public function getR()
        {
            if ($this->_right <= 0) {
                throw new NestedTree_Exception('Right value not specified');
            }

            return $this->_right;
        }

        public function setLevel($l, $type)
        {
            if (!array_key_exists($type, self::$rangeTypes)) {
                throw new NestedTree_Exception('Invalid range type specified');
            }

            $this->_level = (int) $l;
            $this->_levelType = $type;

            return $this;
        }

        public function getLevel()
        {
            if ($this->_level <= 0) {
                throw new NestedTree_Exception('Level value not specified');
            }

            return $this->_level;
        }

        public function getSqlWhereStatements(NestedTree_Abstract $tree)
        {
            $db  = $tree->getDb();
            $ret = array();

            if ($this->_left > 0) {
                $l = $this->_left;
                if ($this->_leftType == self::EITHER_SIDE) {
                    $statements = array(
                        $db->quoteInto($tree->getLeftColumn() . ' < ?', $l),
                    );

                    if ($this->_right > 0) {
                        $r = $this->_right;
                        $statements[] = $db->quoteInto($tree->getLeftColumn() . ' > ?', $r);
                    }

                    $ret[] = join(' or ', $statements);
                }
                else {
                    switch ($this->_leftType) {
                        case self::INSIDE_INCLUSIVE:
                            $op = '>=';
                            break;

                        case self::INSIDE_EXCLUSIVE:
                            $op = '>';
                            break;

                        case self::OUTSIDE_INCLUSIVE:
                            $op = '<=';
                            break;

                        case self::OUTSIDE_EXCLUSIVE:
                            $op = '<';
                            break;
                    }

                    $ret[] = $db->quoteInto($tree->getLeftColumn() . ' ' . $op . ' ?', $l);
                }
            }

            if ($this->_right > 0) {
                $r = $this->_right;

                if ($this->_rightType == self::EITHER_SIDE) {
                    $statements = array(
                        $db->quoteInto($tree->getRightColumn() . ' > ?', $r),
                    );

                    if ($this->_left > 0) {
                        $l = $this->_left;
                        $statements[] = $db->quoteInto($tree->getRightColumn() . ' < ?', $l);
                    }

                    $ret[] = join(' or ', $statements);
                }
                else {
                    switch ($this->_rightType) {
                        case self::INSIDE_INCLUSIVE:
                            $op = '<=';
                            break;

                        case self::INSIDE_EXCLUSIVE:
                            $op = '<';
                            break;

                        case self::OUTSIDE_INCLUSIVE:
                            $op = '>=';
                            break;

                        case self::OUTSIDE_EXCLUSIVE:
                            $op = '>';
                            break;
                    }

                    $ret[] = $db->quoteInto($tree->getRightColumn() . ' ' . $op . ' ?', $r);
                }
            }

            if ($this->_level > 0) {
                $l = $this->_level;

                switch ($this->_levelType) {
                    case self::INSIDE_INCLUSIVE:
                        $op = '>=';
                        break;

                    case self::INSIDE_EXCLUSIVE:
                        $op = '>';
                        break;

                    case self::OUTSIDE_INCLUSIVE:
                        $op = '<=';
                        break;

                    case self::OUTSIDE_EXCLUSIVE:
                        $op = '<';
                        break;
                }

                $ret[] = $db->quoteInto($tree->getLevelColumn() . ' ' . $op . ' ?', $l);
            }

            return $ret;
        }
    }
?>
