<?php
    class NestedTree_Path extends NestedTree_NodeList
    {
        public function __construct($separator)
        {
            $this->_separator = $separator;
        }

        public function __toString()
        {
            $parts = array();

            foreach ($this->_children as $node) {
                $parts[] = $node->getPathElement();
            }

            return $this->_separator . join($this->_separator, $parts);
        }

        public function getPortion($offset, $limit = null)
        {
            $ret = new self($this->_separator);

            foreach (array_slice($this->_children, $offset, $limit) as $node) {
                $ret->add($node);
            }

            return $ret;
        }

        public function getNode($offset)
        {
            foreach (array_slice($this->_children, $offset, 1) as $node) {
                return $node;
            }

            throw new NestedTree_Exception('Node not found');
        }

        public function getClientTemplateData($rootPath = '')
        {
            $fullPath = (string) $this;
            $rootPath = trim($rootPath, '/');

            if (!preg_match('|^/' . preg_quote($rootPath, '|') . '|', $fullPath)) {
                $rootPath = '';
            }

            $ret = array(
                'separator' => $this->_separator,
                'full' => array(
                    'path'  => (string) $this,
                    'parts' => array()
                )
            );

            foreach ($this->_children as $node) {
                $ret['full']['parts'][] = (string) $node;
            }

            if (strlen($rootPath) == 0) {
                $ret['adjusted'] = $ret['full'];
            }
            else {
                $numParts = count(explode($this->_separator, $rootPath));
                $adjusted = $this->getPortion($numParts);

                $ret['adjusted'] = array(
                    'path' => (string) $adjusted,
                    'parts' => array()
                );

                foreach ($adjusted->getChildren() as $node) {
                    $ret['adjusted']['parts'][] = (string) $node;
                }
            }

            return $ret;
        }
    }
?>
