<?php
    abstract class NestedTree_TreeView_Data_Abstract implements NestedTree_TreeView_Data_Interface
    {
        const LEVELS_AUTO = -1;
        const LEVELS_ALL  = 0;

        protected $_rootNodeId;
        protected $_nodeId;
        protected $_numLevels = self::LEVELS_AUTO;

        protected $_formatter;
        protected $_request;

        protected $_treeState;

        public function getTree()
        {
            throw new NestedTree_Exception(sprintf('%s::%s not implemented', __CLASS__, __FUNCTION__));
        }

        public function getTreeState()
        {
            if (!$this->_treeState instanceof NestedTree_TreeView_TreeState) {
                $this->_treeState = new NestedTree_TreeView_TreeState($this->getTree());
            }

            return $this->_treeState;
        }

        public function setRequest(Zend_Controller_Request_Abstract $request)
        {
            $this->_request = $request;
            return $this;
        }

        public function setRootId($nodeId)
        {
            $this->_rootNodeId = $nodeId;
            return $this;
        }

        public function getRootId()
        {
            if (strlen($this->_rootNodeId) == 0) {
                throw new NestedTree_Exception('Root node ID not set');
            }

            return $this->_rootNodeId;
        }

        public function setNodeId($nodeId)
        {
            $this->_nodeId = $nodeId;
            return $this;
        }

        public function getNodeId()
        {
            if (strlen($this->_nodeId) == 0) {
                throw new NestedTree_Exception('Node ID not set');
            }

            return $this->_nodeId;
        }

        public function setNumLevels($levels)
        {
            $this->_numLevels = max(-1, (int) $levels);
            return $this;
        }

        public function getNumLevels()
        {
            return $this->_numLevels;
        }


        public function getFormatter()
        {
            if (!$this->_formatter instanceof NestedTree_TreeView_Renderer_Interface) {
                $this->_formatter = new NestedTree_TreeView_Renderer_jsTree();
            }

            return $this->_formatter;
        }


        public function send(Zend_Controller_Response_Abstract $response)
        {
            $formatter = $this->getFormatter();
            $formatter->setRequest($this->_request);

            $body = $formatter->format($this->getNodes());

            $response->setHeader('Content-Type', HTTP::AddContentTypeCharset($formatter->getContentType(), 'UTF-8'), true);
            $response->setBody($body);
        }


        /**
         * Send the response to the move operation
         *
         * @param   Zend_Controller_Response_Abstract       $response       The response object
         * @param   NestedTree_Node_Move_Response|Exception $moveResponse   Either the response details or an exception if something went wrong
         */
        public function sendMoveResponse(Zend_Controller_Response_Abstract $response, $moveResponse)
        {
            if (!$moveResponse instanceof NestedTree_Node_Move_Response && !$moveResponse instanceof Exception) {
                $moveResponse = new Exception('Invalid response data');
            }

            $formatter = $this->getFormatter();

            $body = $formatter->formatMoveResponse($moveResponse);

            $response->setHeader('Content-Type', HTTP::AddContentTypeCharset($formatter->getMoveResponseContentType(), 'UTF-8'), true);
            $response->setBody($body);
        }
    }
?>
