<?php
    /**
     * Formatter for the DhtmlTree style of output
     *
     * @package NestedTree
     * @author  Quentin Zervaas
     */
    class NestedTree_TreeView_Renderer_DhtmlxTree extends NestedTree_TreeView_Renderer_Abstract
    {
        /**
         * Format the tree data
         *
         * @param   NestedTree_NodeList $nodeList
         * @return  str
         */
        public function format(NestedTree_NodeList $nodeList)
        {
            $data = array(
                'id'    => (int) $nodeList->getId(),
                'item' => array()
            );

            foreach ($nodeList->getChildren() as $child) {
                $data['item'][] = $this->_format($child);
            }

            return Zend_Json::encode($data);
        }

        private function _format($node)
        {
            $ret = array(
                'id'   => $node->getId(),
                'text' => $node->getTitle(),
            );

            if ($node->isSelected()) {
                $ret['select'] = true;
            }

            if ($node->hasChildren()) {
                $ret['child'] = true;

                if ($node->isOpen()) {
                    $ret['open'] = true;
                }

                $children = $node->getChildren();

                if (count($children) > 0) {
                    $ret['item'] = array();

                    foreach ($children as $child) {
                        $ret['item'][] = $this->_format($child);
                    }
                }
            }

            $meta = $node->getMeta();

            if (count($meta) > 0) {
                $ret['attributes'] = $meta;
            }

            return $ret;
        }

        public function getContentType()
        {
            return 'application/json';
        }

        /**
         *
         * @param   NestedTree_Node_Move_Response|Exception $moveResponse   Either the response details or an exception if something went wrong
         */
        public function formatMoveResponse($moveResponse)
        {
            $xml = new XML_File();

            $xml->open('moveResponse', true);

            if ($moveResponse instanceof NestedTree_Node_Move_Response) {
                $xml->addAttribute('success', 1);
                $xml->addAttribute('id', $moveResponse->getNode()->getId());

                if ($moveResponse->needsReload()) {
                    $xml->addAttribute('reload', 1);
                }
            }
            else {
                $xml->addAttribute('success', 0);
                $xml->addAttribute('errmsg', $moveResponse->getMessage());
            }

            $xml->close(); // </moveResponse>

            return (string) $xml;
        }

        public function getMoveResponseContentType()
        {
            return 'text/xml';
        }
    }
?>
