<?php
    /**
     * Formatter for the jsTree style of output
     *
     * @package NestedTree
     * @author  Quentin Zervaas
     */
    class NestedTree_TreeView_Renderer_jsTree extends NestedTree_TreeView_Renderer_Abstract
    {
        const STATE_OPEN   = 'open';
        const STATE_CLOSED = 'closed';
        const ID_FORMAT    = 'jstreenode_%s_%s';

        /**
         * Format the tree data
         *
         * @param   NestedTree_NodeList $nodeList
         * @return  str
         */
        public function format(NestedTree_NodeList $nodeList)
        {
            $data = array();

            foreach ($nodeList->getChildren() as $child) {
                $data[] = $this->_format($child, true);
            }

            return Zend_Json::encode($data);
        }

        private function _format($node, $root = false)
        {
            try {
                $type = $node->getNodeType();
            }
            catch (Exception $ex) {
                $type = 'folder';
            }

            //jsTree 3.x
            $ret = array(
                'id'  => sprintf(self::ID_FORMAT, $this->_request->get('widget'), $node->getId()),
                'text' => $node->getTitle(),
                'rel' => $type,
                'state' => array(),
                'root' => $root
            );

            //jsTree 0.9.8
            //$ret = array(
            //    'attributes' => array(
            //        'id'  => sprintf(self::ID_FORMAT, $this->_request->get('widget'), $node->getId()),
            //        'rel' => $type
            //    ),
            //    'data' => array(
            //        'title' => $node->getTitle()
            //    )
            //);

            try {
                //jsTree 0.9.8
                //$ret['data']['icon'] = $node->getIconPath();

                //jsTree 3.x
                $ret['icon'] = $node->getIconPath();
            }
            catch (Exception $ex)
            { }

            if ($node->isSelected()) {
                // jsTree 0.9.8 doesn't really support this

                //jsTree 3.x
                $ret['state']['selected'] = true;
            }

            // setup the node status
            $statuses = ArrayTools::Clean($node->getMeta('status'));

            if (count($statuses) > 0) {
                //jsTree 0.9.8
                //$ret['attributes']['class'] = 'status-' . join('', $statuses);

                //jsTree 3.x
                //$ret['class'] = 'status-' . join('', $statuses);
                $ret['li_attr'] = array('class' => sprintf('status-%s', join('', $statuses)));
            }

            if ($node->hasChildren()) {

                $children = $node->getChildren();
                //jsTree 3.x
                $ret['children'] = true;

                if (count($children) == 0 || !$node->isOpen()) {
                    //jsTree 0.9.8
                    //$ret['state'] = self::STATE_CLOSED;

                    //jsTree 3.x
                    $ret['state']['opened'] = false;
                }
                else {
                    //jsTree 0.9.8
                    //$ret['state'] = self::STATE_OPEN;

                    //jsTree 3.x
                    $ret['state']['opened'] = true;
                }

                if (count($children) > 0) {
                    $ret['children'] = array();


                    foreach ($children as $child) {
                        $ret['children'][] = $this->_format($child);
                    }
                }
            }

            return $ret;
        }

        public function getContentType()
        {
            return 'application/json';
        }

        /**
         *
         * @param   NestedTree_Node_Move_Response|Exception $moveResponse   Either the response details or an exception if something went wrong
         */
        public function formatMoveResponse($moveResponse)
        {
            $ret = array();

            if ($moveResponse instanceof NestedTree_Node_Move_Response) {
                $ret['success'] = true;
                $ret['id'] = $moveResponse->getNode()->getId();

                if ($moveResponse->needsReload()) {
                    $ret['reload'] = true;
                }
            }
            else {
                $ret['success'] = false;
                $ret['errmsg']  = $moveResponse->getMessage();
            }

            return Zend_Json::encode($ret);
        }

        public function getMoveResponseContentType()
        {
            return 'text/xml';
        }
    }
?>
