<?php
    class Profile_Field_ComplexValue implements Iterator, Countable
    {
        protected $_values = array();

        public function __construct()
        {

        }

        public function __set($k, $v)
        {
            $this->_values[$k] = $v;
        }

        public function __get($k)
        {
            if (!array_key_exists($k, $this->_values)) {
                throw new Exception('Unknown value');
            }

            return $this->_values[$k];
        }

        public function exportToXmlNode(DOMDocument $doc, DOMElement $node)
        {
            $node->setAttribute('type', 'complex');

            foreach ($this as $k => $v) {
                $n = $doc->createElement('value');

                $nameNode = $doc->createElement('name');
                $nameNode->appendChild($doc->createCDATASection($k));
                $n->appendChild($nameNode);

                $valueNode = $doc->createElement('value');

                if (is_array($v)) {
                    $complex = new self();

                    foreach ($v as $_k => $_v) {
                        $complex->__set($_k, $_v);
                    }

                    $complex->exportToXmlNode($doc, $valueNode);
                }
                else {
                    $valueNode->appendChild($doc->createCDATASection($v));
                }

                $n->appendChild($valueNode);

                $node->appendChild($n);
            }
        }

        public static function fromXml(SimpleXmlElement $xml)
        {
            if (!isset($xml->value)) {
                return;
            }

            $ret = new self();

            foreach ($xml->value as $node) {
                $name = (string) $node->name;

                if (isset($node->value['type']) && $node->value['type'] == 'complex') {
                    $ret->__set($name, self::fromXml($node->value));
                }
                else {
                    $ret->__set($name, (string) $node->value);
                }
            }

            return $ret;
        }

        public function rewind()
        {
            reset($this->_values);
        }

        public function current()
        {
            return current($this->_values);
        }

        public function key()
        {
            return key($this->_values);
        }

        public function next()
        {
            return next($this->_values);
        }

        public function valid()
        {
            return current($this->_values) !== false;
        }

        public function count()
        {
            return count($this->_values);
        }
    }
?>