<?php
    /**
     * A class to handle custom fields that can be created, mainly for client forms
     * but also potentially for other aspects of the CMS (such as the comments module)
     *
     * @author  Quentin Zervaas
     */
    class Profile_Field_CustomTypes
    {
        protected static $_types = null;
        private static $_form_prefix = '_cfm_';

        public static function getTypes()
        {
            if (self::$_types === null) {
                self::$_types = Profile_Field::GetAvailableTypes(true);

                uasort(self::$_types, array(__CLASS__, '_sortTypes'));
            }
            return self::$_types;
        }

        public static function _sortTypes($a, $b)
        {
            return strcmp(strtolower($a), strtolower($b));
        }

        public static function getType($type)
        {
            $types = self::getTypes();

            return array_key_exists($type, $types) ? $types[$type] : null;
        }

        /**
         * Build a new custom field list from the passed in request
         *
         * @param   $request    The containing the field data
         * @return  Profile_Field_FieldList
         */
        public static function processFieldList($request, $key)
        {
            $ids = array();

            foreach ($request->getPost() as $k => $v) {
                if (preg_match('/^' . self::$_form_prefix . $key . '_type_(\d+)$/', $k, $matches))
                    $ids[] = (int) $matches[1];
            }

            $ids = ArrayTools::clean($ids, true);

            $fields = new Profile_Field_FieldList();

            foreach ($ids as $id) {
                $type = $request->getPost(self::$_form_prefix . $key . '_type_' . $id);
                $type = self::getType($type);

                if (!$type instanceof Profile_Field)
                    continue;

                $field = $fields->add($type);

                $field->setTitle($request->getPost(self::$_form_prefix . $key . '_title_' . $id))
                      ->setName($request->getPost(self::$_form_prefix . $key . '_name_' . $id))
                      ->setHelp($request->getPost(self::$_form_prefix . $key . '_help_' . $id))
                      ->setProtected(Text::toBoolean($request->getPost(self::$_form_prefix . $key . '_protected_' . $id)))
                      ->setNoEdit(Text::toBoolean($request->getPost(self::$_form_prefix . $key . '_noedit_' . $id)))
                      ->setRequired(Text::toBoolean($request->getPost(self::$_form_prefix . $key . '_required_' . $id)));

                $options = $type->getOptions();

                $params  = array();

                foreach ($options as $option) {
                    $type    = $option->getType();
                    $_fields = $type->getFormFields();
                    $name    = $option->getName();

                    $k = sprintf('%s%s_option_%s_%d', self::$_form_prefix, $key, $name, $id);

                    if (count($_fields) == 0) {
                        $vals = $request->getPost($k);
                    }
                    else foreach ($_fields as $_k) {
                        $vals[$k] = $request->getPost($k . $_k);
                    }

                    $type->set($vals);

                    $params[$name] = $type->get();
                }
                $field->setParams($params);
            }

            return $fields;
        }
    }
?>
