<?php
    class Profile_Field_FieldList implements Iterator, Countable
    {


        protected $_fields = array();

        public function __construct()
        {

        }

        public function __clone()
        {
            foreach ($this->_fields as $k => $v) {
                $this->_fields[$k] = clone $v;
            }
        }

        public function count()
        {
            return count( $this->_fields );
        }

        public function add($field)
        {
            if (!$field instanceof Profile_Field_FieldList_Field) {
                $field = new Profile_Field_FieldList_Field($field);
            }

            $field->setRanking(count($this->_fields) + 1);
            $this->_fields[] = $field;

            return $field;
        }

        public function __toString()
        {
            return $this->serialize();
        }

        public function serialize()
        {
            $arr = array();

            foreach ($this->_fields as $field) {
                $type = $field->getType();
                $arr[] = array(
                    'type'     => $type->getType(),
                    'title'    => $field->getTitle(),
                    'name'     => $field->getName(),
                    'required' => $field->getRequired(),
                    'params'   => $field->getParams(),
                    'value'    => $type->get()
                );
            }

            return serialize($arr);
        }

        public function toArray()
        {
            $ret = array();

            foreach ($this->_fields as $field) {
                $ret[] = $field->toArray();
            }

            return $ret;
        }

        public static function Unserialize($str)
        {
            if (is_array($str)) {
                $arr = $str;
            }
            else {
                $arr = unserialize($str);
            }

            $ret = new self();

            if (!is_array($arr)) {
                return $ret;
            }

            foreach ($arr as $row) {
                $field = $ret->add($row['type'])
                             ->setTitle($row['title'])
                             ->setName($row['name'])
                             ->setRequired($row['required'])
                             ->setParams($row['params']);

                $field->getType()->set(coalesce($row['value']));
            }

            return $ret;
        }

        public function get($title, $type = null)
        {
            return $this->getField($title, $type);
        }

        public function getField($title, $type = null)
        {
            $key = Profile_Field::getFormKey($title);

            if ($type instanceof Profile_Field) {
                $type = $type->getType();
            }

            foreach ($this->_fields as $field) {
                if ($field->getName() != $key) {
                    continue;
                }

                if (strlen($type) == 0) {
                    return $field;
                }

                if ($type == $field->getType()->getType()) {
                    return $field;
                }
            }

            return null;
        }

        public function getFields()
        {
            $args = func_get_args();

            if (count($args) == 0)
                return $this->_fields;

            $ret = array();

            foreach ($this->_fields as $field) {
                if (in_array($field->getType()->getType(), $args))
                    $ret[] = $field;
            }

            return $ret;
        }

        public function getFieldsExcept()
        {
            $args = func_get_args();

            if (count($args) == 0)
                return $this->_fields;

            $ret = array();

            foreach ($this->_fields as $field) {
                if (!in_array($field->getType()->getType(), $args))
                    $ret[] = $field;
            }

            return $ret;
        }

        public function getEmailAddressField()
        {
            foreach ($this->_fields as $field) {
                $type = $field->getType();
                if ($type->getType() == 'email' || $type->getType() == 'html5email') {
                    return $field;
                }
            }

            throw new Profile_Field_Exception('No email address field found');
        }

        public function getFirstNameField()
        {
            foreach ($this->_fields as $field) {
                $title = $field->getTitle();
                $name  = $field->getName();

                if (!preg_match('/name/i', $title)) {
                    continue;
                }

                if (preg_match('/first|given/i', $title)) {
                    return $field;
                }
            }

            throw new Profile_Field_Exception('No first name field found');
        }

        public function getLastNameField()
        {
            foreach ($this->_fields as $field) {
                $title = $field->getTitle();
                $name  = $field->getName();

                if (!preg_match('/name/i', $title)) {
                    continue;
                }

                if (preg_match('/last|sur|family/i', $title)) {
                    return $field;
                }
            }

            throw new Profile_Field_Exception('No last name field found');
        }

        public function getBodyField()
        {
            $types = array('wysiwyg', 'text', 'html5text');

            foreach ($types as $typeName) {
                foreach ($this->_fields as $field) {
                    $type = $field->getType();

                    if ($type->getType() != $typeName) {
                        continue;
                    }

                    return $field;
                }
            }

            throw new Profile_Field_Exception('Unable to determine body field');
        }

        public function setValuesFromProfile(Profile $profile)
        {
            foreach ($this->_fields as $field) {
                $name = $field->getName();
                $type = $field->getType();

                $type->init($profile->get($name));
            }
        }

        public function setValuesFromArray($values)
        {
            foreach ($this->_fields as $field) {
                $name = $field->getName();
                $type = $field->getType();

                $type->init(coalesce($values[$name]));
            }
        }

        public function getClientTemplateData($values = null)
        {
            return Profile_Field_FieldList_Gctd::Single($this, $values);
        }

        public function toJson()
        {
            $ret = array();

            foreach ($this->_fields as $field) {
                $type = $field->getType();

                $ret[] = array(
                    'name'      => $field->getName(),
                    'title'     => $field->getTitle(),
                    'type'      => $type->getType(),
                    'typeTitle' => (string) $type,
                    'params'    => $type->getParams()
                );
            }

            return Zend_Json::Encode($ret);
        }

        public function getPairs()
        {
            $ret = array();

            foreach ($this->_fields as $field) {
                $name = $field->getName();
                $ret[$name] = $field->getType()->preview();
            }

            return $ret;
        }

        public function getErrors()
        {
            $ret = array();

            foreach ($this->_fields as $field) {
                if ($field->hasError()) {
                    $ret[$field->getName()] = $field->getErrors();
                }
            }

            return $ret;
        }

        public function process(Zend_Controller_Request_Abstract $request)
        {
            $ret = new Profile_Field_FieldList_FormProcessor($this);
            $ret->process($request);

            return $ret;
        }

        public function getSummaryFieldList($limit = 5, $force = null)
        {
            $limit = max(1, (int) $limit);
            $size  = $limit;
            $force = ArrayTools::clean($force);

            $ret = new self();
            $remaining = new self();

            $emailsFound    = 0;
            $namesFound     = 0;
            $countriesFound = 0;

            foreach ($this as $field) {
                if (!$field->getType()->canBeUsedInSummary()) {
                    continue;
                }

                if ($ret->size() >= $limit) {
                    $remaining->add($field);
                }
                else {
                    $title = $field->getTitle();

                    if (in_array($field->getName(), $force)) {
                        $ret->add($field);
                    }
                    else if (preg_match('/name/i', $title) && $namesFound < 2) {
                        $ret->add($field);
                        $namesFound++;
                    }
                    else if (preg_match('/country/i', $title) && $countriesFound < 1) {
                        $ret->add($field);
                        $countriesFound++;
                    }
                    else if (preg_match('/email/i', $title) && $emailsFound < 1) {
                        $ret->add($field);
                        $emailsFound++;
                    }
                    else {
                        $remaining->add($field);
                    }
                }
            }

            $size -= $ret->size();

            foreach ($remaining as $field) {
                if ($size <= 0) {
                    break;
                }

                $ret->add($field);
                $size--;
            }

            return $ret;
        }

        /**
         * Data backup routines
         */

        public function xmlExport(DOMDocument $doc, DOMElement $root, $field = 'fields', $exportValue = false)
        {
            $node = $doc->createElement($field);

            foreach ($this->_fields as $field) {
                $field->xmlExport($doc, $node, $exportValue);
            }

            $root->appendChild($node);
        }

        public function xmlExportProfileValues(DOMDocument $doc, DOMElement $root, Profile $profile, $field = 'profile')
        {
            $node = $doc->createElement($field);

            foreach ($this->_fields as $field) {
                $field->xmlExportProfileValues($doc, $node, $profile, 'field');
            }

            $root->appendChild($node);
        }

        public static function FromXml($xml)
        {
            $ret = new self();

            foreach ($xml->field as $field) {
                $ret->add(Profile_Field_FieldList_Field::FromXml($field));
            }

            return $ret;
        }

        public static function FromXmlForProfile($xml, Profile $profile)
        {
            foreach ($xml->field as $field) {
                Profile_Field_FieldList_Field::FromXmlForProfile($field, $profile);
            }
        }

        /**
         * Iterator methods
         */

        public function rewind()
        {
            reset($this->_fields);
        }

        public function current()
        {
            return current($this->_fields);
        }

        public function key()
        {
            return key($this->_fields);
        }

        public function next()
        {
            return next($this->_fields);
        }

        public function valid()
        {
            return $this->current() !== false;
        }

        public function size()
        {
            return count($this->_fields);
        }
    }
?>
