<?php
    class Profile_Field_FieldList_FormProcessor
    {
        const ERROR_INVALID = 'invalid';
        const ERROR_EMPTY   = 'empty';

        protected $_fields;
        protected $_errors = array();
        protected $_values = array();

        public function __construct(Profile_Field_FieldList $fields)
        {
            $this->_fields = $fields;
        }

        public function process(Zend_Controller_Request_Abstract $request)
        {
            $ret = array();

            foreach ($this->_fields as $field) {
                $vals   = array();
                $type   = $field->getType();
                $fields = $type->getFormFields();
                $key    = $field->getName();

                if (count($fields) == 0) {
                    $vals = $request->getPost($key);
                }
                else foreach ($fields as $k) {
                    $vals[$k] = $request->getPost($key . $k);
                }

                $this->_values[$key] = $vals;

                if ($type->isEmpty($vals)) {
                    $type->set($vals);
                    $this->_values[$key] = null;
                    if ($field->getRequired()) {
                        $this->addError($field, self::ERROR_EMPTY);
                    }
                }
                else if (!$type->set($vals)) {
                    $this->addError($field, self::ERROR_INVALID, $vals);
                }
            }

            return count($this->_errors) == 0;
        }

        public function hasData()
        {
            $values = array_filter($this->_values);

            foreach ($values as $value) {
                if (is_array($value)) {
                    $value = array_filter($value);
                    if (count($value) > 0) {
                        return true;
                    }
                }
                else if (strlen($value) > 0) {
                    return true;
                }
            }

            return false;
        }

        protected function addError($field, $error, $value = null)
        {
            $this->_errors[] = array(
                'field' => $field,
                'error' => $error,
                'value' => $value
            );
        }

        public function getErrors()
        {
            return $this->_errors;
        }

        public function getFieldList()
        {
            return $this->_fields;
        }

        public static function ProcessFilters($fields, $request, $name = 'filters')
        {
            $ret = new Profile_Field_Fieldlist();

            $ids = array();
            foreach ($request->getPost() as $k => $v) {
                if (!preg_match('/^' . preg_quote($name, '/') . '_(\d+)_/', $k, $matches)) {
                    continue;
                }

                $id = (int) $matches[1];
                $ids[$id] = $id;
            }

            foreach ($ids as $id) {
                $key = sprintf('%s_%d_', $name, $id);

                try {
                    $type = Profile_Field::factory($request->getPost($key .'type'));
                }
                catch (Exception $ex) {
                    continue;
                }

                $fieldName = $request->getPost($key . 'name');

                $field = $fields->getField($fieldName, $type);

                if (!$field instanceof Profile_Field_FieldList_Field) {
                    continue;
                }

                $value     = $request->getPost($key . 'value');
                $operators = $type->getComparisonOperators();
                $operator  = $request->getPost($key . 'operator');

                if (!array_key_exists($operator, $operators)) {
                    continue;
                }

                $field = $ret->add($type);
                $field->setName($fieldName)
                      ->setParam('operator', $operator)
                      ->setParam('filterValue', $value);
/*
                if (strlen($query) > 0) {
                    $field->setParam('query', $query);
                }
*/
            }

            return $ret;
        }

        public static function GetFilterList($request)
        {
            $tmp = array();

            foreach ($request->getPost() as $k => $v) {
                if (!preg_match('/^filter(\d+)_(.+)$/', $k, $matches)) {
                    continue;
                }

                $id = (int) $matches[1];

                if (!array_key_exists($id, $tmp)) {
                    $tmp[$id] = array();
                }

                $key = $matches[2];
                $tmp[$id][$key] = $v;
            }

            $filters = array();
            foreach ($tmp as $k => $filter) {
                try {
                    $type = Profile_Field::Factory($filter['type']);
                }
                catch (Exception $ex) {
                    continue;
                }

                $filter['type'] = $type;
                $filters[$k] = $filter;
            }

            return $filters;
        }
    }
?>