<?php
    class Profile_Field_FieldList_Gctd
    {
        private function __construct()
        { }

        public static function Single(Profile_Field_FieldList $fieldList, $values = array())
        {
            $ret = array();

            if ($values instanceof Profile) {
                $fieldList->setValuesFromProfile($values);
            }

            if (!is_array($values)) {
                $values = array();
            }

            foreach ($fieldList->getFields() as $field) {
                $name = $field->getName();
                $idx  = $field->getIndexName();

                if (strlen($idx) == 0) {
                    $idx = $name;
                }

                $type = $field->getType();

                $ret[$idx] = array(
                    'name'      => $name,
                    'title'     => $field->getTitle(),
                    'type'      => (string) $type->getType(),
                    'typeTitle' => (string) $type,
                    'preview'   => '',
                    'content'   => array(),
                    'required'  => $field->getRequired(),
                    'value'     => $type->get(),
                    'options'   => $type->getParams(),
                    'errors'    => $field->getErrors(),
                    'help'      => $field->getHelp()
                );

                if (array_key_exists($name, $values)) {
                    $ret[$idx]['value'] = $values[$name];

                    $_type = clone $type;
                    $_type->set($values[$name]);

                    try {
                        $ret[$idx]['preview'] = $_type->preview();
                    }
                    catch (Exception $ex) {
                    }

                    try {
                        $ret[$idx]['content'] = $_type->getClientTemplateData();
                    }
                    catch (Exception $ex) {
                    }
                }
                else {
                    try {
                        $ret[$idx]['preview'] = $type->preview();
                    }
                    catch (Exception $ex) {
                    }

                    try {
                        $ret[$idx]['content'] = $type->getClientTemplateData();
                    }
                    catch (Exception $ex) {
                    }
                }

                if (!is_array($ret[$idx]['content'])) {
                    $ret[$idx]['content'] = array();
                }
            }

            return $ret;
        }

        public static function Bulk($fieldLists)
        {
            if (!is_array($fieldLists) || count($fieldLists) == 0) {
                return array();
            }

            $ret = array();
            $typeData = array();

            foreach ($fieldLists as $k => $fieldList) {
                $row = array();

                foreach ($fieldList->getFields() as $field) {
                    $type  = $field->getType();
                    $name  = $field->getName();
                    $class = get_class($type);
                    $idx   = $field->getIndexName();

                    if (strlen($idx) == 0) {
                        $idx = $name;
                    }

                    $key = sprintf('%s,%s', $k, $idx);

                    $row[$idx] = array(
                        'name'      => $name,
                        'title'     => $field->getTitle(),
                        'type'      => (string) $type->getType(),
                        'typeTitle' => (string) $type,
                        'preview'   => '',
                        'content'   => array(),
                        'required'  => $field->getRequired(),
                        'value'     => $type->get(),
                        'options'   => $type->getParams(),
                        'errors'    => $field->getErrors(),
                        'help'      => $field->getHelp()
                    );

                    if (!array_key_exists($class, $typeData)) {
                        $typeData[$class] = array(
                            'type' => $type,
                            'vals' => array()
                        );
                    }

                    $typeData[$class]['vals'][$key] = $type->get();
                }

                $ret[$k] = $row;
            }

            foreach ($typeData as $type) {
                foreach ($type['type']->previewBulk($type['vals']) as $k => $v) {
                    list($a, $b) = explode(',', $k, 2);
                    $ret[$a][$b]['preview'] = $v;
                }

                foreach ($type['type']->getClientTemplateDataBulk($type['vals']) as $k => $v) {
                    list($a, $b) = explode(',', $k, 2);
                    $ret[$a][$b]['content'] = $v;
                }
            }

            return $ret;
        }
    }
?>