<?php
    abstract class Profile_Field_Selector extends Profile_Field
    {
        protected $_template;
        protected $_addDialogUrl;

        public function setAddDialogUrl($url)
        {
            $this->_addDialogUrl = $url;
        }

        public function getInputTemplate($name, $attrs = array())
        {
            $container = new HTML_Element('div', array('class' => 'ui-item-selector'));

            $options = array(
                'name' => $name,
                'data' => array(
                    'type' => $this->getType()
                )
            );

            if (strlen($this->_addDialogUrl) > 0) {
                $options['addUrl'] = $this->_addDialogUrl;
            }

            $data = $this->getUrlData();

            if (count($data) > 0) {
                foreach ($data as $k => $v) {
                    $name = 'param_' . $k;

                    if (is_array($v)) {
                        $name .= '[]';
                    }

                    $options['data'][$name] = $v;
                }
            }

            $controller = Zend_Controller_Front::GetInstance();
            $request    = $controller->getRequest();

            if ($request instanceof Application_Client_Site_Request) {
                $options['url'] = '/__/containers/selector';
            }
            else if ($request instanceof Application_Admin_Site_Request) {
                $options['url'] = '';
            }
            else if ($request instanceof Application_Cp_Site_Request) {
                $options['url'] = '/default/forms/selector';
            }
            else {
                throw new Exception('Invalid request type');
            }

            $limit = $this->getParam('limit');

            if (preg_match('/^\d+$/', $limit)) {
                $options['limit'] = (int) $limit;
            }
            else if ($this->getParam('multiple')) {
                $options['limit'] = 0;
            }
            else {
                $options['limit'] = 1;
            }

            $options['selected'] = $this->prepare();

            if ($this->getParam('sortable')) {
                $options['sortable'] = true;
            }

            foreach ($options['selected'] as $id) {
                $container->insert(new HTML_Element('input', array(
                    'type'  => 'hidden',
                    'name'  => $name,
                    'value' => $id
                )));
            }

            //@TODO - window._Recite should probably be promoted for the CP as well, but it
            //is front-end only right now.
            $js = sprintf('(jQuery ? jQuery : window._Recite.$)(\'#%s\').reciteItemSelector(%s);', $container->getUniqueId(), Zend_Json::encode($options));

            $script = new HTML_Element('script', array('type' => 'text/javascript'));
            $script->insert($js);

            return (string) $container . ' ' . $script;
        }

        public function getUrlData()
        {
            return array();
        }

        public function isPublic()
        {
            return false;
        }

        public function set($ids)
        {
            if (!is_array($ids)) {
                $ids = preg_split('/(\r\n|\r|\n)+/', $ids);
            }

            $selector = $this->getItemSelector();

            $options = new Components_ItemSelector_Options();
            $options->setSelectedIds($ids);

            foreach ($this->getUrlData() as $k => $v) {
                $options->setFilter($k, $v);
            }

            $ids = array();

            foreach ($selector->getSelectedItems($options) as $item) {
                $ids[] = $item->getId();
            }

            $this->_value = join("\n", $ids);

            return true;
        }

        public function init($ids)
        {
            if (!is_array($ids)) {
                $ids = preg_split('/(\r\n|\r|\n)+/', $ids);
            }

            $this->_value = join("\n", $ids);
        }

        public function prepare()
        {
            $ret = ArrayTools::Clean(preg_split('/(\r\n|\r|\n)+/', $this->get()));

            return $ret;
        }

        public function getIndexableField($name)
        {
            throw new Exception('This field type is not indexable');
        }

        /**
         *
         * @return  Components_ItemSelector_Abstract
         */
        abstract public function getItemSelector();
    }
?>
