<?php
    class XML_File
    {
        private $_indent = 0;
        private $_tabWidth = 4;
        private $_tempTagOpen = false;
        private $_tags = array();
        private $_lines = array();

        public $encoding = 'UTF-8';

        public function __construct($version = '1.0', $encoding = null, $dtd = '', $dtdTopElt = '')
        {
            if (strlen($encoding) > 0)
                $this->encoding = $encoding;

            $this->_lines[] = sprintf("<?xml version=\"%s\" encoding=\"%s\" ?>\n", $version, $this->encoding);

            if (strlen($dtd) > 0)
                $this->_lines[] = sprintf("<!DOCTYPE %s SYSTEM \"%s\">\n", $dtdTopElt, $dtd);
        }

        public function getXml()
        {
            return join('', $this->_lines);
        }

        public function __toString()
        {
            return $this->getXml();
        }

        public function open($tag, $empty = false)
        {
            if ($this->_tempTagOpen) {
                $this->_lines[] = ">\n";
                $this->_tempTagOpen = false;
            }

            $this->_tags[] = array($tag, $empty);
            $this->_lines[] = str_repeat(' ', $this->_indent * $this->_tabWidth) . "<" . $tag;

            $this->_indent++;
            $this->_tempTagOpen = true;
        }

        public function close()
        {
            $element = array_pop($this->_tags);
            $this->_indent = max(0, $this->_indent - 1);

            if ($element[1]) {
                $this->_lines[] = " />\n";
                $this->_tempTagOpen = false;
            }
            else {
                if ($this->_tempTagOpen) {
                    $this->_lines[] = ">\n";
                    $this->_tempTagOpen = false;
                }
                $this->_lines[] = str_repeat(' ', $this->_indent * $this->_tabWidth) . "</" . $element[0] . ">\n";
            }
        }

        public function addAttribute($attribute, $value, $escape = true)
        {
            $this->_lines[] = sprintf(' %s="%s"',
                                      $attribute,
                                      $escape ? self::escape($value) : $value);
        }

        public function addTagAndValue($tag, $value = null, $attrs = null, $cdata = false)
        {
            if ($this->_tempTagOpen) {
                $this->_lines[] = ">\n";
                $this->_tempTagOpen = false;
            }
            $this->_lines[] = str_repeat(' ', $this->_indent * $this->_tabWidth);
            $this->_lines[] = "<" . $tag;

            if (is_array($attrs) && count($attrs) > 0)
                $this->_lines[] = ' ' . self::arrayToXhtmlAttrs($attrs);

            $this->_lines[] = sprintf(">%s</%s>\n", self::escape($value, $cdata), $tag);
        }

        public function addValue($value, $cdata = false)
        {
            if ($this->_tempTagOpen) {
                $this->_lines[] = ">\n";
                $this->_tempTagOpen = false;
            }
            $this->_lines[] = str_repeat(' ', $this->_indent * $this->_tabWidth) . self::escape($value, $cdata) . "\n";
        }

        public function escape($str, $cdata = false)
        {
            if ($cdata)
                return sprintf('<![CDATA[%s]]>', $str);

            static $trans;
            if (!isset($trans)) {
                $trans = get_html_translation_table(HTML_ENTITIES, ENT_QUOTES);
                foreach ($trans as $key => $value)
                    $trans[$key] = '&#'.ord($key).';';
                // dont translate the '&' in case it is part of &xxx;
                $trans[chr(38)] = '&';
            }
            return preg_replace("/&(?![A-Za-z]{0,4}\w{2,3};|#[0-9]{2,4};)/","&#38;" , strtr($str, $trans));
        }

        public static function arrayToXhtmlAttrs($arr)
        {
            $pairs = array();
            foreach ($arr as $k => $v)
                $pairs[] = $k . '="' . self::escape($v) . '"';
            return join(' ', $pairs);
        }
    }
?>
