#!/usr/local/bin/php -q
<?php
    $cwd = dirname(realpath($argv[0]));
    ini_set('include_path', sprintf('%s/../../include', $cwd));
    require_once('Application/CommandLine/Bootstrap.php');

    function exitWithMessage($msg = null, $code = 1)
    {
        if (strlen($msg) > 0) {
            echo sprintf("%s\n", trim($msg));
        }

        $msg = sprintf("Usage: %s [client id] [output dir] [module1 module2 ...]\n", $GLOBALS['argv'][0]);

        $brands = Application_Brand::GetBrands();

        foreach ($brands as $brand_id => $brand) {
            $options = array('brand_ids' => $brand_id);
            $clients = Application_Client::GetClients($options);

            if (count($clients) > 0) {
                $msg .= sprintf("\n%s Clients:\n\n", $brand);
                foreach ($clients as $client_id => $client) {
                    $msg .= sprintf("\t%4d. %s\n", $client_id, $client);
                }
            }
        }

        echo $msg;
        exit($code);
    }

    $opts = new Zend_Console_Getopt(array('s' => 'site wizard option', 'modifiedSince=s' => 'modified since option'));
    $args = $opts->getRemainingArgs();
    $siteWizard = (bool) $opts->getOption('s');
    $modifiedSince = $opts->getOption('modifiedSince');
    
    $clientId  = (int) array_shift($args);
    $outputDir = array_shift($args);

    if (substr($outputDir, 0, 1) != '/') {
        $outputDir = getcwd() . '/' . $outputDir;
    }

    $parentDir = realpath(dirname($outputDir));

    if (strlen($parentDir) == 0) {
        exitWithMessage('Unable to find output directory');
    }

    $client = new Application_Client();
    $client->load($clientId);

    if (!$client->saved()) {
        exitWithMessage();
    }

    Application::SetClient($client);

    $options  = new Module_Backup_Exporter_Options();
    $options->setOutputPath($outputDir);
    
    if(!is_null($modifiedSince)){
        $options->setModifiedSince($modifiedSince);
    }

    if ($siteWizard) {
        $options->exportAsSitewizard();
    }

    $modules = array_unique(array_filter($args));

    if (count($modules) > 0) {
        foreach ($modules as $module) {
            $options->addDriver($module);
        }
    }
    else {
        $options->exportAllDrivers();
    }

    Module_Backup_Logger::SetOutputDriver(new Module_Backup_Logger_Output_Stdout());

    $exporter = new Module_Backup_Exporter();

    try {
        $report = $exporter->run($options);
    }
    catch (Exception $ex) {
        echo $ex->getMessage() . "\n";
        exit(1);
    }

    if ($report->isError()) {
        echo "-----------------------------------------------------------\n";
        echo "ERRORS:\n";

        foreach ($report->getErrors() as $error) {
            echo "\t- " . $error . "\n";
        }

        exit(1);
    }
    else {
        $dump = $report->getDump();

        echo sprintf("Dump path: %s\n", $dump->getPath());
    }
?>
