#!/usr/local/bin/php -q
<?php
    $cwd = dirname(realpath($argv[0]));
    ini_set('include_path', sprintf('%s/../../include', $cwd));
    require_once('Application/CommandLine/Bootstrap.php');

    $argc = coalesce($_SERVER['argc']);
    $argv = coalesce($_SERVER['argv']);

    $db = Application::GetDb();

    if ($argc != 3) {
        $msg = sprintf("Usage: %s [brand id] [input dir/file]\n", $argv[0]);

        $brands = Application_Brand::GetBrands();

        if (count($brands) > 0) {
            $msg .= "\nAvailable Brands:\n\n";
            foreach ($brands as $brand_id => $brand) {
                $msg .= sprintf("\t%4d. %s\n", $brand_id, $brand);
            }
            $msg .= "\n";
        }

        echo $msg;
        exit(1);
    }

    $options = array();
    $path = coalesce($argv[2]);

    if (!preg_match('|^/|', $path)) {
        $path = realpath(sprintf('%s/%s', rtrim(getcwd(), '/'), $path));
    }

    try {
        $dump = new Module_Backup_Dump($path);
    }
    catch (Exception $ex) {
        echo $ex->getMessage() . "\n";
        exit(1);
    }

    $brand = new Application_Brand();
    $brand->load($argv[1]);

    if (!$brand->saved()) {
        echo "Invalid brand\n";
        exit(1);
    }

    $options  = new Module_Backup_Importer_Options();
    $options->setDump($dump)
            ->setBrand($brand);

    Module_Backup_Logger::SetOutputDriver(new Module_Backup_Logger_Output_Stdout());

    $importer = new Module_Backup_Importer();

    try {
        $report = $importer->run($options);
    }
    catch (Exception $ex) {
        echo $ex->getMessage() . "\n";
        exit(1);
    }

    if ($report->isError()) {
        echo "-----------------------------------------------------------\n";
        echo "ERRORS:\n";

        foreach ($report->getErrors() as $error) {
            echo "\t- " . $error . "\n";
        }

        exit(1);
    }
    else {
        $client = $report->getClient();

        echo sprintf("Import complete on client: %d. %s\n", $client->getId(), $client);
    }
?>
