#!/usr/local/bin/php -q
<?php
    $cwd = dirname(realpath($argv[0]));
    ini_set('include_path', sprintf('%s/../../include', $cwd));
    require_once('Application/CommandLine/Bootstrap.php');

    
    $opts = new Zend_Console_Getopt(array('override' => 'override option'));
    $args = $opts->getRemainingArgs();
    $override = (bool) $opts->getOption('override');
    
    $argc = count($args);
    $argv = coalesce($args);

    $db = Application::GetDb();

    if ($argc != 2) {
        $msg = sprintf("Usage: %s [client id] [input dir/file]\n", $GLOBALS['argv'][0]);

        $brands = Application_Brand::GetBrands();

        foreach ($brands as $brand_id => $brand) {
            $options = array('brand_ids' => $brand_id);
            $clients = Application_Client::GetClients($options);

            if (count($clients) > 0) {
                $msg .= sprintf("\n%s Clients:\n\n", $brand);
                foreach ($clients as $client_id => $client) {
                    $msg .= sprintf("\t%4d. %s\n", $client_id, $client);
                }
            }
        }

        echo $msg;
        exit(1);
    }
    
    $options = array();
    $path = coalesce($argv[1]);

    if (!preg_match('|^/|', $path)) {
        $path = realpath(sprintf('%s/%s', rtrim(getcwd(), '/'), $path));
    }

    try {
        $dump = new Module_Backup_Dump($path);
    }
    catch (Exception $ex) {
        echo $ex->getMessage() . "\n";
        exit(1);
    }

    $client = new Application_Client();
    $client->load($argv[0]);

    if (!$client->saved()) {
        echo "Invalid client\n";
        exit(1);
    }

    $options  = new Module_Backup_Importer_Options();
    $options->setDump($dump)
            ->setClient($client)
            ->setImportPermissions(false)
            ->setImportSettings(false)
            ->setImportCustomSettings(false)
            ->setOverrideExistingContent($override);

    Module_Backup_Logger::SetOutputDriver(new Module_Backup_Logger_Output_Stdout());

    $importer = new Module_Backup_Importer();

    try {
        $report = $importer->run($options);
    }
    catch (Exception $ex) {
        echo $ex->getMessage() . "\n";
        exit(1);
    }

    if ($report->isError()) {
        echo "-----------------------------------------------------------\n";
        echo "ERRORS:\n";

        foreach ($report->getErrors() as $error) {
            echo "\t- " . $error . "\n";
        }

        exit(1);
    }
    else {
        $client = $report->getClient();

        echo sprintf("Import complete on client: %d. %s\n", $client->getId(), $client);
    }
?>
