#!/usr/local/bin/php -q
<?php
    $cwd = dirname(realpath($argv[0]));
    ini_set('include_path', sprintf('%s/../../include', $cwd));
    require_once('Application/CommandLine/Bootstrap.php');

    $availableDrivers = array(
        'assets'    => '_versionAssets',
        'templates' => '_versionTemplates',
        'pages'     => '_versionPages'
    );

    function _versionAssets()
    {
        echo "Assets...\n";

        if (!Module_Assets_Asset_Versioning::Supported()) {
            return;
        }

        $files = Module_Assets_Asset::GetFiles();

        $count = count($files);
        $i = 0;

        foreach ($files as $file) {
            echo sprintf("%d/%d\t%s\n", ++$i, $count, $file);
            Module_Assets_Asset_Versioning::Record($file);
        }
    }

    function _versionTemplates()
    {
        echo "Templates...\n";

        if (!Module_Templates_Template_Versioning::Supported()) {
            return;
        }

        $templates = Module_Templates_Template::GetTemplates(array());

        $count = count($templates);
        $i = 0;

        foreach ($templates as $tpl) {
            echo sprintf("%d/%d\t%s\n", ++$i, $count, $tpl);
            Module_Templates_Template_Versioning::Record($tpl);
        }
    }

    function _versionPages()
    {
        echo "Pages...\n";

        if (!Module_Pages_Page_Versioning::Supported()) {
            return;
        }

        $pages = Module_Pages_Page_Retriever::GetPages(array());

        $count = count($pages);
        $i = 0;

        foreach ($pages as $page) {
            echo sprintf("%d/%d\t%s\n", ++$i, $count, $page);
            Module_Pages_Page_Versioning::Record($page);
        }
    }

    function exitWithMessage($msg = null, $code = 1)
    {
        if (strlen($msg) > 0) {
            echo sprintf("%s\n", trim($msg));
        }

        $msg = sprintf("Usage: %s [client id] [driver1 driver2 ...]\n", $GLOBALS['argv'][0]);

        $brands = Application_Brand::GetBrands();

        foreach ($brands as $brand_id => $brand) {
            $options = array('brand_ids' => $brand_id);
            $clients = Application_Client::GetClients($options);

            if (count($clients) > 0) {
                $msg .= sprintf("\n%s Clients:\n\n", $brand);
                foreach ($clients as $client_id => $client) {
                    $msg .= sprintf("\t%4d. %s\n", $client_id, $client);
                }
            }
        }

        echo $msg;
        exit($code);
    }

    $opts = new Zend_Console_Getopt('s');
    $args = $opts->getRemainingArgs();

    $clientId  = (int) array_shift($args);

    $client = new Application_Client();
    $client->load($clientId);

    if (!$client->saved()) {
        exitWithMessage();
    }

    Application::SetClient($client);

    $drivers = array();

    foreach (array_unique(array_filter($args)) as $driver) {
        if (array_key_exists($driver, $availableDrivers)) {
            $drivers[] = $driver;
        }
    }

    if (count($drivers) == 0) {
        $drivers = array_keys($availableDrivers);
    }

    foreach (array_unique($drivers) as $driver) {
        $func = $availableDrivers[$driver];
        $func();
    }
?>