#!/usr/local/bin/php -q
<?php
    $cwd = dirname(realpath($argv[0]));
    ini_set('include_path', sprintf('%s/../../../include', $cwd));
    require_once('Application/CommandLine/Bootstrap.php');

    if ($argc < 3) {
        echo sprintf("Usage: %s [client id] [num events]\n", $argv[0]);
        exit(1);
    }

    $clientId = (int) $argv[1];

    $client = new Application_Client();
    $client->load($clientId);

    if (!$client->saved()) {
        echo sprintf("Invalid client specified\n");
        exit(1);
    }

    $num = (int) $argv[2];

    if ($num <= 0) {
        echo sprintf("Must specify number of events to create\n");
        exit(1);
    }

    Application::SetClient($client);

    $lipsum = new Text_Lipsum();

    $db = Application::GetDb();
    $db->begin();

    $fields = new Profile_Field_FieldList();
    $fields->add('wysiwyg')->setName('body')->setTitle('Body');

    $type = new Module_Calendar_Event_Type();
    $type->title = 'Test Type';
    $type->save();
    $type->setProfileFields($fields);

    $calendar = new Module_Calendar_Calendar();
    $calendar->title = sprintf('Autogen %s', date('Y-m-d H:i:s'));
    $calendar->save();

    $table = Module_Calendar_Event::$table;

    $helper = $db->getStatementHelper();

    $now = new Zend_Db_Expr('now()');

    for ($i = 0; $i < $num; $i++) {
        if ($i % 100 == 0) {
            if ($i > 0) {
                echo ' ' . $i . "\n";
            }
        }
        else {
            echo '.';
        }

        $start = rand(time() - 10000000, time() + 10000000);

        $values = array(
            'client_id'   => $clientId,
            'calendar_id' => $calendar->getId(),
            'title'       => $lipsum->sentence(),
            'type_id'     => $type->getId(),
            'status'      => Module_Calendar_Event::STATUS_LIVE,
            'all_day'     => new Zend_Db_Expr('true'),
            'ts_start'    => $helper->getTimestamp($start),
            'ts_created'  => $now,
            'ts_updated'  => $now
        );

        $values['ts_finish'] = $values['ts_start'];

        $db->insert($table, $values);

        $eventId = $db->lastInsertId($table, 'event_id');

        $profile = array(
            'body' => $lipsum->document('<p>', '</p>')
        );

        foreach ($profile as $k => $v) {
            $values = array(
                'event_id'      => $eventId,
                'profile_key'   => $k,
                'profile_value' => $v
            );

            $db->insert($table . '_profile', $values);
        }
    }

    $db->commit();
?>