#!/usr/local/bin/php -q
<?php
    $cwd = dirname(realpath($argv[0]));
    ini_set('include_path', sprintf('%s/../../../include', $cwd));
    require_once('Application/CommandLine/Bootstrap.php');

    if ($argc < 3) {
        echo sprintf("Usage: %s [client id] [num listings]\n", $argv[0]);
        exit(1);
    }

    $clientId = (int) $argv[1];

    $client = new Application_Client();
    $client->load($clientId);

    if (!$client->saved()) {
        echo sprintf("Invalid client specified\n");
        exit(1);
    }

    $num = (int) $argv[2];

    if ($num <= 0) {
        echo sprintf("Must specify number of listings to create\n");
        exit(1);
    }

    Application::SetClient($client);

    $lipsum = new Text_Lipsum();

    $db = Application::GetDb();
    $db->begin();

    $fields = new Profile_Field_FieldList();
    $fields->add('wysiwyg')->setName('body')->setTitle('Body');

    $type = new Module_Listings_Listing_Type();
    $type->title = 'Test Type';
    $type->save();
    $type->setProfileFields($fields);

    $group = new Module_Listings_Group();
    $group->title = sprintf('Autogen %s', date('Y-m-d H:i:s'));
    $group->save();

    $table = Module_Listings_Listing::$table;

    $now = new Zend_Db_Expr('now()');

    for ($i = 0; $i < $num; $i++) {
        if ($i % 100 == 0) {
            if ($i > 0) {
                echo ' ' . $i . "\n";
            }
        }
        else {
            echo '.';
        }

        $values = array(
            'client_id'  => $clientId,
            'group_id'   => $group->getId(),
            'title'      => $lipsum->sentence(),
            'type_id'    => $type->getId(),
            'status'     => Module_Listings_Listing::STATUS_LIVE,
            'ts_created' => $now,
            'ts_updated' => $now
        );

        $db->insert($table, $values);

        $listingId = $db->lastInsertId($table, 'listing_id');

        $profile = array(
            'body' => $lipsum->document('<p>', '</p>')
        );

        foreach ($profile as $k => $v) {
            $values = array(
                'listing_id'    => $listingId,
                'profile_key'   => $k,
                'profile_value' => $v
            );

            $db->insert($table . '_profile', $values);
        }
    }

    $db->commit();
?>