#!/usr/local/bin/php -q
<?php
    $cwd = dirname(realpath($argv[0]));
    ini_set('include_path', sprintf('%s/../../../include', $cwd));
    require_once('Application/CommandLine/Bootstrap.php');

    if ($argc < 3) {
        echo sprintf("Usage: %s [client id] [num users]\n", $argv[0]);
        exit(1);
    }

    $clientId = (int) $argv[1];

    $client = new Application_Client();
    $client->load($clientId);

    if (!$client->saved()) {
        echo sprintf("Invalid client specified\n");
        exit(1);
    }

    $num = (int) $argv[2];

    if ($num <= 0) {
        echo sprintf("Must specify number of users to create\n");
        exit(1);
    }

    Application::SetClient($client);


    $data = array(
        'firstnames' => array_merge(file('names/female.txt'), file('names/male.txt')),
        'lastnames'  => file('names/surname.txt'),
        'streets'    => array('St', 'Rd', 'Ave', 'Tce'),
        'states'     => array(
            'SA'  => array(5000, 5999),
            'NSW' => array(2000, 2999),
            'VIC' => array(3000, 3999),
            'QLD' => array(4000, 4999),
            'WA'  => array(6000, 6999),
            'TAS' => array(7000, 7999)
        )
    );

    $lipsum = new Text_Lipsum();

    $db = Application::GetDb();
    $db->begin();

    $fields = new Profile_Field_FieldList();
    $fields->add('string')->setName('first_name')->setTitle('First Name');
    $fields->add('string')->setName('last_name')->setTitle('Last Name');
    $fields->add('email')->setName('email')->setTitle('Email Address');
    $fields->add('string')->setName('address')->setTitle('Address');
    $fields->add('string')->setName('state')->setTitle('State');
    $fields->add('string')->setName('postcode')->setTitle('Postcode');
    $fields->add('string')->setName('country')->setTitle('Country');
    $fields->add('string')->setName('phone')->setTitle('Phone');

    $directory = new Application_User_Directory();
    $directory->client_id = getClientId();
    $directory->title = sprintf('Autogen %s', date('Y-m-d H:i:s'));
    $directory->send_welcome_email = false;
    $directory->setUsernameIsEmail(false);
    $directory->save();

    $directory->setEditableProfileFields($fields);

    $table = Application_User::$table;

    for ($i = 0; $i < $num; $i++) {
        if ($i % 100 == 0) {
            if ($i > 0) {
                echo ' ' . $i . "\n";
            }
        }
        else {
            echo '.';
        }

        $firstName = trim($data['firstnames'][array_rand($data['firstnames'])]);
        $lastName  = trim($data['lastnames'][array_rand($data['lastnames'])]);
        $email     = strtolower(sprintf('%s.%s@%d.example.com', $firstName, $lastName, $i));

        $values = array(
            'directory_id' => $directory->getId(),
            'username'     => strtolower(sprintf('%s%s%d', $firstName, $lastName, $i)),
            'status'       => Application_User::STATUS_ACTIVE,
            'ts_created'   => new Zend_Db_Expr('now()')
        );

        $values['ts_updated'] = $values['ts_created'];

        $values['password'] = Application_Auth::GetAuthHash($values['username'], 'test');

        $db->insert($table, $values);

        $userId = $db->lastInsertId($table, 'user_id');

        $profile = array(
            'first_name' => $firstName,
            'last_name'  => $lastName,
            'email'      => $email,
            'address'    => sprintf(
                '%d %s %s',
                rand(1, 3000),
                trim($data['firstnames'][array_rand($data['firstnames'])]),
                trim($data['streets'][array_rand($data['streets'])])
            ),
            'state'    => array_rand($data['states']),
            'postcode' => sprintf('%04d', rand($data['states'][$state][0], $data['states'][$state][1])),
            'country'  => 'Australia',
            'phone'    => sprintf('(%02d) %04d %04d', rand(2, 9), rand(1000, 9999), rand(1000, 9999))
        );

        foreach ($profile as $k => $v) {
            $values = array(
                'user_id'       => $userId,
                'profile_key'   => $k,
                'profile_value' => $v
            );

            $db->insert($table . '_profile', $values);
        }
    }

    $db->commit();

?>