#!/usr/local/bin/php -q
<?php
    $cwd = dirname(realpath($argv[0]));
    ini_set('include_path', sprintf('.:%s/../../include', $cwd));
    require_once('Application/CommandLine/Bootstrap.php');

    $argc = coalesce($_SERVER['argc']);
    $argv = coalesce($_SERVER['argv']);

    if ($argc != 3) {
        echo sprintf("Usage: %s [client ID] [index ID]\n", $argv[0]);
        exit(1);
    }

    $db = Application::GetDb();

    $client = new Application_Client();
    $client->load((int) $argv[1]);

    Application::SetClient($client);

    if (!$client->saved()) {
        echo sprintf("Invalid client, exiting\n");
        exit(1);
    }

    $index = new Module_Search_Index();
    $index->load($argv[2]);

    if (!$client->saved()) {
        echo sprintf("Invalid search index, exiting\n");
        exit(1);
    }

    $idx  = $index->getIndex();
    $size = $idx->count();

    echo sprintf("%s (%s) - %d record%s\n", $index, $index->getDriver(), $size, $size != 1 ? 's' : '');

    for ($i = 0; $i < $size; $i++) {
        $document = $idx->getDocument($i);

        echo "--------------------------------------------------------------------------------------\n";
        echo sprintf("RECORD %d:\n", $i + 1);

        $fields = $document->getFieldNames();

        foreach ($fields as $name) {
            $field = $document->getField($name);

            echo sprintf(
                "%s:%s\n" .
                "\tstored=%s, indexed=%s, tokenized=%s, binary=%s, boost=%s, encoding=%s\n",
                $field->name,
                $field->getUtf8Value(),
                $field->isStored ? 'YES' : 'no',
                $field->isIndexed ? 'YES' : 'no',
                $field->isTokenized ? 'YES' : 'no',
                $field->isBinary ? 'YES' : 'no',
                $field->boost,
                $field->encoding
            );
        }
    }
?>