<?php
    class Driver_forms_fields_country extends Profile_Field
    {
        protected static $_countries;

        public function __toString()
        {
            return 'Country';
        }

        public static function GetCountries()
        {
            if (!is_array(self::$_countries)) {
                $countries = Zend_Locale::getTranslationList('territory', Application::GetLocale(), 2);
                unset($countries['ZZ']);
                asort($countries);

                self::$_countries = $countries;
            }

            return self::$_countries;
        }

        public function getInputTemplate($name, $attrs = array())
        {
            if (!is_array($attrs)) {
                $attrs = array();
            }

            $attrs['name'] = $name;

            $countries = self::GetCountries();

            $ret = new HTML_Element('select', $attrs);

            if ($this->getParam('multiple')) {
                $ret->writeAttribute('multiple');
                $ret->writeAttribute('size', 4);
            }
            else {
                $ret->insert(new HTML_Element('option'));
            }

            $value = $this->prepareValue($this->_value);

            foreach ($countries as $cc => $label) {
                $option = new HTML_Element('option', array('value' => $cc));
                $option->insert($label);

                if ($cc == $value) {
                    $option->writeAttribute('selected');
                }

                $ret->insert($option);
            }

            return (string) $ret;
        }

        public function getBulkInputTemplate($name)
        {
            $values = $this->prepareValue($this->_value);

            $options = array(
                'type'  => 'text',
                'name'  => $name,
                'value' => $values
            );

            $ret = new HTML_Element('input', $options);

            return (string) $ret;
        }

        public function prepareValue($val)
        {
            if ($val == 'UK') {
                $val = 'GB';
            }

            if (array_key_exists($val, self::GetCountries())) {
                return $val;
            }

            if (strlen($val) > 2) {
                $countries = array_map(array('Text', 'Urlize'), self::GetCountries());

                $cc = array_search(Text::Urlize($val), $countries);

                if (strlen($cc) == 2) {
                    return $cc;
                }
            }

            return null;
        }

        public function preview()
        {
            $countries = self::GetCountries();

            $names = array();

            foreach (ArrayTools::clean(preg_split('/\n+/', parent::preview())) as $cc) {
                if (array_key_exists($cc, $countries)) {
                    $names[] = $countries[$cc];
                }
            }

            return join(', ', $names);
        }

        public function set($val)
        {
            $val = $this->prepareValue($val);

            if (array_key_exists($val, self::GetCountries())) {
                $this->_value = $val;
                return true;
            }

            return false;
        }
    }
?>