<?php
    class Driver_forms_fields_currencyname extends Profile_Field
    {
        protected static $_currencyManager;
        protected static $_currencies;

        public function __toString()
        {
            return 'Currency Name';
        }

        public static function GetCurrencyManager()
        {
            if (!self::$_currencyManager instanceof Zend_Currency) {
                self::$_currencyManager = new Zend_Currency(null, Application::GetLocale());
            }

            return self::$_currencyManager;
        }

        public static function GetCurrencies()
        {
            if (!is_array(self::$_currencies)) {
                $currencies = array();
                $currencyManager = self::GetCurrencyManager();
                $locale = Application::GetLocale();

                foreach (self::GetCurrencyManager()->getCurrencyList() as $currency => $regions) {
                    try {
                        $currencies[$currency] = $currencyManager->getName($currency, $locale);
                    }
                    catch (Exception $ex) {
                    }

                    ksort($currencies);
                }

                self::$_currencies = $currencies;
            }

            return self::$_currencies;
        }

        public function getInputTemplate($name, $attrs = array())
        {
            if (!is_array($attrs)) {
                $attrs = array();
            }

            $attrs['name'] = $name;

            $currencies = self::GetCurrencies();

            $ret = new HTML_Element('select', $attrs);

            if ($this->getParam('multiple')) {
                $ret->writeAttribute('multiple');
                $ret->writeAttribute('size', 4);
            }
            else {
                $ret->insert(new HTML_Element('option'));
            }

            $value = $this->prepareValue($this->_value);

            foreach ($currencies as $currency => $label) {
                $option = new HTML_Element('option', array('value' => $currency));
                $option->insert(
                    sprintf('%s - %s', $currency, $label)
                );

                if ($currency == $value) {
                    $option->writeAttribute('selected');
                }

                $ret->insert($option);
            }

            return (string) $ret;
        }

        public function getBulkInputTemplate($name)
        {
            $values = $this->prepareValue($this->_value);

            $options = array(
                'type'  => 'text',
                'name'  => $name,
                'value' => $values
            );

            $ret = new HTML_Element('input', $options);

            return (string) $ret;
        }

        public function prepareValue($val)
        {
            if (array_key_exists($val, self::GetCurrencies())) {
                return $val;
            }

            if (strlen($val) > 2) {
                $currencies = array_map(array('Text', 'Urlize'), self::GetCurrencies());

                $currency = array_search(Text::Urlize($val), $currencies);

                if (strlen($currency) == 3) {
                    return $currency;
                }
            }

            return null;
        }

        public function preview()
        {
            $currencies = self::GetCurrencies();

            $names = array();

            foreach (ArrayTools::clean(preg_split('/\n+/', parent::preview())) as $cc) {
                if (array_key_exists($cc, $currencies)) {
                    $names[] = $currencies[$cc];
                }
            }

            return join(', ', $names);
        }

        public function set($val)
        {
            $val = $this->prepareValue($val);

            if (array_key_exists($val, self::GetCurrencies())) {
                $this->_value = $val;
                return true;
            }

            return false;
        }
    }
?>