<?php
    class Driver_forms_fields_date extends Profile_Field
    {
        protected $_date;

        public function __construct($options = array())
        {
            parent::__construct($options);

            $this->_date = new Zend_Date(Application::GetLocale());
        }

        public function __toString()
        {
            return 'Date';
        }

        public function getInputTemplate($name, $attrs = array())
        {
            if (!is_array($attrs)) {
                $attrs = array();
            }

            $useScript = Application::GetSite() == 'cp';

            $attrs['type']  = 'text';
            $attrs['name']  = $name;
            $attrs['value'] = '';

            if (!array_key_exists('class', $attrs)) {
                $attrs['class'] = 'profile-date';
            }

            if (!$this->isEmpty($this->get())) {
                try {
                    $date = new Zend_Date(Application::GetLocale());
                    $date->set($this->get(), 'yyyy-MM-dd');
                    $attrs['value'] = $date->get(Zend_Date::DATE_MEDIUM);
                }
                catch (Exception $ex) {
                    $attrs['value'] = $this->get();
                }
            }

            $input = new HTML_Element('input', $attrs);

            $id = $input->getUniqueId();

            if ($useScript) {
                $options = array(
                    'changeMonth' => true,
                    'changeYear'  => true,
                    'dateFormat'  => 'dd/mm/yy'
                );

                $js = sprintf('jQuery(\'#%s\').datepicker(%s);', $input->getUniqueId(), Zend_Json::encode($options));

                $script = new HTML_Element('script', array('type' => 'text/javascript'));
                $script->insert($js);

                return (string) $input . ' ' . $script;
            }
            else {
                return (string) $input;
            }
        }

        public function set($val)
        {
            if ($this->isEmpty($val)) {
                $this->_value = null;
                return true;
            }

            try {
                if (preg_match('/^\d{4}-\d{2}-\d{2}$/', $val))
                    $this->_date->setDate($val, 'yyyy-MM-dd');
                else
                    $this->_date->set($val, Zend_Date::DATE_SHORT);

                $this->_value = $this->_date->toString('yyyy-MM-dd');

                return true;
            }
            catch (Zend_Date_Exception $ex) {
                $this->_value = $val;
                return false;
            }
        }

        public function prepareValue($val)
        {
            try {
                $date = new Zend_Date(Application::GetLocale());
                $date->set($val, Zend_Date::DATE_SHORT);

                return $date->get(Zend_Date::DATE_SHORT);
            }
            catch (Zend_Date_Exception $ex) {
                return $val;
            }
        }

        public function isValid($val)
        {
            $date = new Zend_Date(Application::GetLocale());

            try {
                $date->set($val, Zend_Date::DATES);
                return true;
            }
            catch (Zend_Date_Exception $ex) {
                return false;
            }
        }

        public function preview()
        {
            $date = new Zend_Date(Application::GetLocale());
            try {
                $date->setDate($this->_value, 'yyyy-MM-dd');
                return $date->get(Zend_Date::DATE_LONG);
            }
            catch (Exception $ex) {
                return '';
            }
        }

        public function getYear()
        {
            return $this->_date->get(Zend_Date::YEAR);
        }

        public function getMonth()
        {
            return $this->_date->get(Zend_Date::MONTH);
        }

        public function getDay()
        {
            return $this->_date->get(Zend_Date::DAY);
        }
    }
?>