<?php
    class Driver_forms_fields_dimensions3d extends Profile_Field
    {
        public function __toString()
        {
            return 'Dimensions (3D)';
        }

        public function getFormFields()
        {
            return array('Width', 'Height', 'Depth', 'Unit');
        }

        /**
         * @todo    Implement this method
         */
        public function getInputTemplate($name, $attrs = array())
        {

        }

        public function prepare()
        {
            return array('units' => array(''   => '',
                                          'mm' => 'mm',
                                          'cm' => 'cm',
                                          'm'  => 'm',
                                          'in' => 'in',
                                          'f'  => 'ft',
                                          'yd' => 'yd'));
        }

        public function prepareValue($val)
        {
            if (is_array($val))
                return $val;

            $val = trim($val);

            $ret = array('Width'  => '',
                         'Height' => '',
                         'Depth'  => '',
                         'Unit'   => '');

            if (strlen($val) == 0)
                return $ret;

            $data = $this->prepare();

            $parts = preg_split('/\s+/', $val, 2);
            $dims = trim(coalesce($parts[0]));

            if (strlen($dims) > 0) {
                $dims = explode('x', $dims);
                $ret['Width']  = coalesce($dims[0]);
                $ret['Height'] = coalesce($dims[1]);
                $ret['Depth']  = coalesce($dims[2]);
            }

            $unit = trim(coalesce($parts[1]));
            if (!array_key_exists($unit, $data['units']))
                $unit = '';

            $ret['Unit'] = $unit;

            return $ret;
        }

        public function set($vals)
        {
            $vals = self::_sanitize(self::prepareValue($vals));
            $w = $vals['Width'];
            $h = $vals['Height'];
            $d = $vals['Depth'];
            $u = $vals['Unit'];

            if ($this->isEmpty($vals))
                $this->_value = '';
            else
                $this->_value = trim(sprintf('%sx%sx%s %s', $w, $h, $d, $u));

            return $this->isValid($vals);
        }

        public function isEmpty($vals)
        {
            $vals = self::_sanitize($vals);

            foreach ($vals as $k => $v) {
                if (strlen($v) > 0)
                    return false;
            }

            return true;
        }

        protected static function _sanitize($vals)
        {
            $vals['Width']  = Text::sanitize(coalesce($vals['Width']));
            $vals['Height'] = Text::sanitize(coalesce($vals['Height']));
            $vals['Depth']  = Text::sanitize(coalesce($vals['Depth']));
            $vals['Unit']   = Text::sanitize(coalesce($vals['Unit']));

            return $vals;
        }

        public function isValid($vals)
        {
            $data = $this->prepare();

            $vals = self::_sanitize($vals);
            $w = $vals['Width'];
            $h = $vals['Height'];
            $d = $vals['Depth'];
            $u = $vals['Unit'];

            if (strlen($w) == 0 && strlen($h) == 0 && strlen($d) == 0 && strlen($u) == 0)
                return true;

            if (!array_key_exists($u, $data['units']))
                return false;

            return strlen($w) > 0 && preg_match('/^\d+$/', $w) &&
                   strlen($h) > 0 && preg_match('/^\d+$/', $h) &&
                   strlen($d) > 0 && preg_match('/^\d+$/', $d);
        }

        public function getIndexableField($name)
        {
            throw new Exception('This field type is not indexable');
        }
    }
?>