<?php
    class Driver_forms_fields_filetypelist extends Profile_Field
    {
        public function __toString()
        {
            return 'File Type List';
        }

        public function getInputTemplate($name, $attrs = array())
        {
            $select = new HTML_Element('select');

            $info = new HTML_Element('option');
            $info->insert('Select a file type to add...');

            $select->insert($info);

            foreach (File_Type::GetTypes() as $type) {
                $value = sprintf('*.%s', $type->getExtension());

                $option = new HTML_Element('option', array('value' => $value));
                $option->insert(sprintf('%s - %s', $value, $type->getDescription()));

                $select->insert($option);
            }

            $selectContainer = new HTML_Element('div');
            $selectContainer->insert($select);

            $input = new HTML_Element('input', array('name' => $name));
            $input->writeAttribute('value', $this->preview());

            $inputContainer = new HTML_Element('div');
            $inputContainer->insert($input);

            $js = <<<EOF
                jQuery(function($) {
                    var selector = jQuery('#%s');
                    var input    = jQuery('#%s');

                    selector.bind('change', function() {
                        var extension = $(this).val();

                        if (extension.length == 0) {
                            return;
                        }

                        var val = $.trim(input.val());

                        if (val.length == 0) {
                            input.val(extension);
                        }
                        else {
                            input.val(val + ', ' + extension);
                        }

                        selector.val('');
                    });
                });
EOF;

            $js = sprintf($js, $select->getUniqueId(), $input->getUniqueId());

            $script = new HTML_Element('script', array('type' => 'text/javascript'));
            $script->insert($js);

            return (string) $selectContainer . $inputContainer . $script;
        }

        public function isValid($val)
        {
            return true;
        }

        public function set($val)
        {
            $val = $this->prepareValue($val);

            $this->_value = join(',', $val);
            return true;
        }

        public function preview()
        {
            return join(', ', $this->prepareValue($this->get()));
        }

        public function prepareValue($value)
        {
            if (!is_array($value)) {
                $value = explode(',', $value);
            }
            else {
                $value = ArrayTools::Clean($value);
            }

            return $value;
        }
    }
?>