<?php
    /**
     * This type is like a normal list except each value is named.
     * That is, it uses an associative array instead of indexed array
     */
    class Driver_forms_fields_hash extends Profile_Field
    {
        public function __toString()
        {
            return 'Custom Hash';
        }

        public function getInputTemplate($name, $attrs = array())
        {
            if (!is_array($attrs)) {
                $attrs = array();
            }

            $attrs['name'] = $name;

            if ($this->getParam('multiple')) {
                $attrs['name'] .= '[]';
            }

            $ret = new HTML_Element('select', $attrs);

            $options = $this->getParam('values');
            $values  = $this->prepareValue($this->_value);

            if ($this->getParam('multiple')) {
                $ret->writeAttribute('multiple');
                $ret->writeAttribute('size', min(4, count($options)));
            }
            else {
                $ret->insert(new HTML_Element('option'));
            }

            foreach ($options as $value => $label) {
                $option = new HTML_Element('option', array('value' => $value));
                $option->insert($label);

                if (in_array($value, $values)) {
                    $option->writeAttribute('selected');
                }

                $ret->insert($option);
            }

            return (string) $ret;
        }

        public function getOptions()
        {
            $list = new Module_Forms_FieldList();
            $list->add(new Module_Forms_FieldList_Field('text'))->setTitle('Values');
            $list->add(new Module_Forms_FieldList_Field('bool'))->setTitle('Multiple');

            return $list;
        }

        public function set($val)
        {
            $options = $this->getParam('values');

            $val = ArrayTools::clean($val);

            if (!$this->getParam('multiple', false))
                $val = array_slice($val, 0, 1);

            $this->_value = join("\n", $val);

            foreach ($val as $v) {
                if (!array_key_exists($v, $options)) {
                    return false;
                }
            }

            return true;
        }

        public function preview()
        {
            $val = ArrayTools::clean(preg_split('/\n+/', parent::preview()));
            return join(', ', $val);
        }

        public function prepareValue($val)
        {
            $val = ArrayTools::clean(preg_split('/\n+/', $val));
            return $val;
        }

        public function getParam($k, $default = null)
        {
            $ret = parent::getParam($k, $default);

            if ($k == 'values' && !is_array($ret)) {
                $ret = ArrayTools::clean(preg_split('/\n+/', $ret));

                foreach ($ret as $k => $v)
                    $ret[$k] = trim($v);
            }

            return $ret;
        }

        public function getIndexableField($name)
        {
            return Zend_Search_Lucene_Field::Unstored(self::getFormKey($name), $this->preview());
        }

        public function isPublic()
        {
            return false;
        }
    }
?>