<?php
    class Driver_forms_fields_html5text extends Profile_Field
    {
        public function __toString()
        {
            return 'HTML5 Text (Paragraph)';
        }

        public function isCore()
        {
            return true;
        }

        public function isValid($val){
            $ret = true;
            
            $characterLimit = (int) $this->getParam('max_length_characters');
            if($characterLimit > 0){
                $ret = strlen($val) <= $characterLimit;
            }
            
            $wordLimit = (int) $this->getParam('max_length_words');
            if($ret && $wordLimit > 0){
                $ret = str_word_count($val) <= $wordLimit;
            }
            
            return $ret;
        }
        
        public function getInputTemplate($name, $attrs = array())
        {
            if (!is_array($attrs)) {
                $attrs = array();
            }

            $attrs['name']  = $name;
            
            if((int) $this->getParam('max_length_characters') > 0){
                $attrs['maxlength'] = (int) $this->getParam('max_length_characters');
            }

            if(strlen($this->getParam('placeholder_text')) > 0){
                $attrs['placeholder'] = $this->getParam('placeholder_text');
            }
            
            if (!array_key_exists('class', $attrs)) {
                $attrs['class'] = 'profile-string';
            }

            if (!array_key_exists('rows', $attrs)) {
                $attrs['rows'] = 5;
            }

            if (!array_key_exists('cols', $attrs)) {
                $attrs['cols'] = 30;
            }

            $elt = new HTML_Element('textarea', $attrs);
            $elt->insert($this->get());

            return (string) $elt;
        }
        
        public function getOptions()
        {
            $list = new Module_Forms_FieldList();
            
            $placeholderText = new Module_Forms_FieldList_Field('string');
            $placeholderText->setName('placeholder_text');
            $placeholderText->setTitle('Placeholder Text');
            $placeholderText->setHelp('Placeholder text that displays in the element. This is cleared when the element gets focus.');
            $list->add($placeholderText);
            
            $maxCharacters = new Module_Forms_FieldList_Field('html5number');
            $maxCharacters->setName('max_length_characters');
            $maxCharacters->setTitle('Max Length (Characters)');
            $maxCharacters->setParam('minimum_value', 1);
            $maxCharacters->setHelp('Maximum number of characters. Adds a limit to the field length on the website and is validated on the server.');
            $list->add($maxCharacters);
            
            $maxWords = new Module_Forms_FieldList_Field('html5number');
            $maxWords->setName('max_length_words');
            $maxWords->setTitle('Max Length (Words)');
            $maxWords->setParam('minimum_value', 1);
            $maxWords->setHelp('Maximum number of words. Validated on the server. Restrictions on the website must be done manually with Javascript.');
            $list->add($maxWords);
            
            return $list;
        }
        
        public function set($val)
        {
            $this->_value = FormProcessor::sanitize($val);

            if ($this->isValid($this->_value)) {
                return true;
            }
            return false;
        }
    }
?>
