<?php
    class Driver_forms_fields_list extends Profile_Field
    {
        public function __toString()
        {
            return 'Custom List';
        }

        public function isCore()
        {
            return true;
        }
        
        public function getFormFields()
        {
            if ($this->getParam('allow_other')) {
                if ($this->getParam('multiple')) {
                    return array('list', 'other');
                }
                else {
                    return array('type', 'list', 'other');
                }
            }

            return array();
        }

        public function getBulkInputTemplate($name)
        {
            $values = $this->prepareValue($this->_value);

            $options = array(
                'type'  => 'text',
                'name'  => $name,
                'value' => join(', ', $values)
            );

            $ret = new HTML_Element('input', $options);

            return (string) $ret;
        }

        public function getInputTemplate($name, $attrs = array())
        {
            if (!is_array($attrs)) {
                $attrs = array();
            }

            $values  = $this->prepareValue($this->_value);
            $options = $this->getParam('values');

            if ($this->getParam('allow_other')) {
                $select = $this->_getInputSelect($name, $options, $values);
                $select->setAttribute('name', $select->getAttribute('name') . 'list');

                if ($this->getParam('multiple')) {
                    $select->setAttribute('name', $select->getAttribute('name') . '[]');
                }

                $input = $this->_getInputInput($name, $options, $values);

                $container = new HTML_Element('div', $attrs);
                $container1 = new HTML_Element('div');
                $container2 = new HTML_Element('div');

                $select->writeAttribute('class', 'profile-list-list-with-other');

                if ($this->getParam('multiple')) {
                    $container1->insert($select);
                    $container->insert($container1);

                    $container2->insert($input);
                    $container->insert($container2);
                }
                else {
                    if (strlen($input->getAttribute('value')) > 0) {
                        $type = 'other';
                    }
                    else {
                        $type = 'list';
                    }

                    $radio1 = new HTML_Element('input', array(
                        'type'  => 'radio',
                        'name'  => $name . 'type',
                        'value' => 'list',
                        'class' => 'profile-list-type-list'
                    ));

                    if ($type != 'other') {
                        $radio1->setAttribute('checked', 'checked');
                    }

                    $radio2 = new HTML_Element('input', array(
                        'type'  => 'radio',
                        'name'  => $name . 'type',
                        'value' => 'other',
                        'class' => 'profile-list-type-other'
                    ));

                    if ($type == 'other') {
                        $radio2->setAttribute('checked', 'checked');
                    }


                    $container1->insert($radio1);
                    $container1->insert($select);
                    $container->insert($container1);

                    $id = $radio2->getUniqueId();

                    $label = new HTML_Element('label', array('for' => $id));
                    $label->insert('Other:');

                    $container2->insert($radio2);
                    $container2->insert($label);
                    $container2->insert($input);
                    $container->insert($container2);
                }

                return (string) $container;
            }
            else {
                $select = $this->_getInputSelect($name, $options, $values, $attrs);

                if ($this->getParam('multiple')) {
                    $select->setAttribute('name', $select->getAttribute('name') . '[]');
                }

                return (string) $select;
            }
        }

        public function _getInputSelect($name, $options, $values, $attrs = array())
        {
            if (!is_array($attrs)) {
                $attrs = array();
            }

            $attrs['name'] = $name;

            $ret = new HTML_Element('select', $attrs);

            if ($this->getParam('multiple')) {
                $ret->writeAttribute('multiple');
                $ret->writeAttribute('size', min(4, count($options)));
            }
            else {
                $ret->insert(new HTML_Element('option'));
            }

            foreach ($options as $value) {
                $option = new HTML_Element('option', array('value' => $value));
                $option->insert($value);

                if (in_array($value, $values)) {
                    $option->writeAttribute('selected');
                }

                $ret->insert($option);
            }

            return $ret;
        }

        public function _getInputInput($name, $options, $values)
        {
            $attrs = array(
                'name'  => sprintf('%sother', $name),
                'type'  => 'text',
                'class' => 'profile-list-other'
            );

            $vals = array();

            foreach ($values as $value) {
                if (!in_array($value, $options)) {
                    $vals[] = $value;
                }
            }

            $vals = ArrayTools::clean($vals);

            if (count($vals) > 0) {
                $attrs['value'] = join(', ', $vals);
            }

            $ret = new HTML_Element('input', $attrs);

            return $ret;
        }

        public function isValid($val)
        {
            return true;
        }

        public function getOptions()
        {
            $list = new Module_Forms_FieldList();
            $list->add(new Module_Forms_FieldList_Field('text'))->setTitle('Values');
            $list->add(new Module_Forms_FieldList_Field('bool'))->setTitle('Multiple');
            $list->add(new Module_Forms_FieldList_Field('bool'))->setTitle('Allow Other');

            return $list;
        }

        public function isEmpty($val)
        {
            if ($this->getParam('allow_other') && is_array($val)) {
                if ($this->getParam('multiple')) {

                }
                else {
                    if (coalesce($val['type']) == 'other') {
                        return strlen(Text::sanitize(coalesce($val['other']))) == 0;
                    }
                    else {
                        return strlen(Text::sanitize(coalesce($val['list']))) == 0;
                    }
                }
            }

            return parent::isEmpty($val);
        }

        public function set($val)
        {
            $options = $this->getParam('values');

            if ($this->getParam('allow_other') && is_array($val)) {
                if ($this->getParam('multiple')) {
                    $vals = ArrayTools::clean(coalesce($val['list']));
                    $vals[] = Text::sanitize(coalesce($val['other']));
                }
                else {
                    if (coalesce($val['type']) == 'other') {
                        $vals = Text::sanitize(coalesce($val['other']));
                    }
                    else {
                        $vals = coalesce($val['list']);
                    }
                }
            }
            else {
                $vals = $this->prepareValue($val);
            }
            
            $vals = ArrayTools::clean($vals);

            foreach ($vals as $k => $v) {
                $vals[$k] = trim($v);
            }

            if (!$this->getParam('multiple', false)) {
                $vals = array_slice($vals, 0, 1);
            }

            $this->_value = join("\n", $vals);

            if (!$this->getParam('allow_other')) {
                foreach ($vals as $v) {
                    if (!in_array($v, $options)) {
                        return false;
                    }
                }
            }

            return true;
        }

        public function preview()
        {
            $val = ArrayTools::clean(preg_split('/\n+/', parent::preview()));
            return join(', ', $val);
        }

        public function hasValue($value)
        {
            $val = ArrayTools::clean(preg_split('/\n+/', parent::preview()));

            return in_array($value, $val);
        }

        public function prepareValue($val)
        {
            if (!is_array($val)) {
                $val = preg_split('/(\r|\n)+/', $val);
            }

            return ArrayTools::clean($val);
        }

        public function getParam($k, $default = null)
        {
            $ret = parent::getParam($k, $default);

            if ($k == 'values' && !is_array($ret)) {
                $ret = ArrayTools::clean(preg_split('/\n+/', $ret));

                foreach ($ret as $k => $v)
                    $ret[$k] = trim($v);
            }

            return $ret;
        }

        public function getIndexableField($name)
        {
            return Zend_Search_Lucene_Field::Unstored(self::getFormKey($name), $this->preview());
        }
    }
?>
